/******************************************************************************/
/* BAI -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bai_cpp_id[]="@(#) $Id: bai.cpp,v 1.9 2005/02/20 14:44:57 Oleksiy Balabay Exp $"; static const char* bai_cpp_id_id=bai_cpp_id;

#include <algorithm>
#include <functional>
#include <qfile.h>

#include "bai.h"
#include "baichasing.h"
#include "battle.h"
#include "bexception.h"
#include "bxmlconfig.h"
#include "bscenarioheader.h"
#include "bdomutils.h"
#include "bapplication.h"
#include "bships.h"
#include "bevent.h"

#ifdef Q_WS_WIN
  #define MEM_FUN std::mem_fun1
#endif
#ifdef Q_WS_X11
  #define MEM_FUN std::mem_fun
#endif

const int BAIs::THINK_INTERVAL = 12;
/** \file bai.cpp
  \brief  BAI  BattleFleet (impl.)

     BAI  BattleFleet
*/

/** \class BAI
  \brief 


*/
/** \brief 
*/
BAI::BAI(BattleObject* obj, BAIs* parent, ControlModes mode)
:_obj(obj), _parent(parent), _controlMode(mode), _deleteLater(false) {
  bApp->battle()->setModified(true);
}

/** \brief 
*/
BAI::~BAI(){
  if (bApp->battle())
    bApp->battle()->setModified(true);
}

BAIs::BAIs(Battle* battle, QObject* parent, const char* name)
:BattleObserver(battle, parent, name), _doc(0) {
  if (battle) {
    connect(battle, SIGNAL(battleObjectRemoved(BattleObject*)),
      this, SLOT(slotBattleObjectRemoved(BattleObject*)));
  }
}


BAIs::~BAIs() {
  clear();
}

void BAIs::init() {
  if (_battle == 0)
    return;
  _lastThinkTime = _battle->battleInfo().modelTime;
  connect(_battle, SIGNAL(saveAI(const QString&)), this,
    SLOT(slotSaveAI(const QString&)));
  bool bs = _battle->isModified();
  loadDocument(_battle->scenarioHeader()->aiFileName());
  _battle->setModified(bs);
}

void BAIs::slotBattleInfoChanged(const BattleInfo& info) {
  if (_lastThinkTime.secsTo(info.modelTime) >= thinkIntervalSecs()) {
    _lastThinkTime = info.modelTime;
    think();
  }
}

void BAIs::think() {
  for(BAIsContainer::iterator it = _ais.begin(); it != _ais.end(); it++) {
    (*it)->think();
  }
}

void BAIs::addChasing(BattleObject* pursuier, BattleObject* evader, 
  double keepDistance) {
  clear(pursuier);
  _ais.push_back(new BAIChasing(this, pursuier, evader, keepDistance));
}

void BAIs::addChasing(const QDomElement& e) {
  BAIChasing* ai = 0;
  try {
    QString p = BDomUtils::stringAttribute(e, BAIChasing::AI_PURSUIER);
    QString ev = BDomUtils::stringAttribute(e, BAIChasing::AI_EVADER);
    BattleObject* po = _battle->ships()->ship(p);
    BattleObject* eo = _battle->ships()->ship(ev);
    if (0 == po) {
      throw BNoObjectException(__FILE__, __LINE__,
      qApp->translate("BAIs", "Pursuier does not exist:'%1'").arg(p));
    }
    if (0 == eo) {
      throw BNoObjectException(__FILE__, __LINE__,
      qApp->translate("BAIs", "Evader does not exist:'%1'").arg(ev));
    }
    ai = new BAIChasing(this, po, eo, 0);
    _ais.push_back(ai);
  }
  catch(BException& e) {
    if (ai)
      delete ai;
    bApp->log(e.message());
    bApp->log(qApp->translate("BAIs", "Connection not added"));
  }
  catch(...) {
    if (ai)
      delete ai;
    throw;
  }
}

void BAIs::clear() {
  for(BAIsContainer::iterator it = _ais.begin(); it != _ais.end(); it++) {
    delete *it;
  }
  _ais.clear();
}

class is_obj: public std::unary_function<BAI*, bool> {
public:
	is_obj(BattleObject* obj):_obj(obj){};
  bool operator()(BAI* ai) const {
	if (ai == 0)
	  return true;
	else if (ai->object() == _obj) {
	  delete ai;
	  return true;
	}
	else
      return false;
  }
private:
  BattleObject* _obj;
};

class is_ai: public std::unary_function<BAI*, bool> {
public:
	is_ai(BAI* ai):_ai(ai){};
  bool operator()(BAI* ai) const {
	if (ai == _ai) {
	  delete ai;
	  return true;
	}
	else
      return false;
  }
private:
  BAI* _ai;
};

void BAIs::clear(BattleObject* obj) {
  BAIsContainer::iterator eit = std::remove_if(_ais.begin(), _ais.end(), 
    is_obj(obj));
  _ais.erase(eit, _ais.end());
}

void BAIs::trigChasingConnection(BAIChasing* ai, const bool *add, const bool *isAuto) {
  emit chasingConnectionChanged(ai, add, isAuto);
}

void BAIs::erase(BAI* ai) {
  BAIsContainer::iterator eit = std::remove_if(_ais.begin(), _ais.end(), 
    is_ai(ai));
  _ais.erase(eit, _ais.end());
}

class BAIsConfig: public BXmlConfig {
public:
  BAIsConfig(BAIs* parent, const QString& fileName);
  virtual ~BAIsConfig() { }
private:
  void parseDocument(const QDomDocument& doc);
private:
  BAIs* _parent;
};

const QString BAIs::AIS_AIS = "AIs";
const QString BAI::AI_AI = "AI";
const QString BAI::AI_TYPE = "Type";

BAIsConfig::BAIsConfig(BAIs* parent, const QString& fileName)
:BXmlConfig(fileName), _parent(parent) {
  try {
    parseDocument(*_doc);
    delete _doc;
    _doc = 0;
  }
  catch(BConfigException& e) {
    throw addFileName(e);
  }
  catch(BException&) {
    throw;
  }
  catch(...) {
    throw BInternalException(__FILE__, __LINE__,
      qApp->translate("BXmlConfig",
      "Unknown error loading file '%1'").arg(this->fileName()));
  }
}

void BAIsConfig::parseDocument(const QDomDocument& doc) {
  QDomElement docElem = doc.documentElement();
  if (docElem.tagName() != BAIs::AIS_AIS) {
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate("BXmlConfig",
      "File '%1' is not '%2' file but '%3'").arg(fileName()).arg(
      BAIs::AIS_AIS).arg(docElem.tagName()));
  }
  QDomNode n = docElem.firstChild();
  while( !n.isNull() ) {
    QDomElement e = n.toElement(); // try to convert the node to an element.
    n = n.nextSibling();
    if( !e.isNull() ) { // the node was really an element.
      if (e.tagName() == BAI::AI_AI) {
        QString type = BDomUtils::stringAttribute(e, BAI::AI_TYPE);
        if (type == BAIChasing::AI_TYPE_CHASING) {
          _parent->addChasing(e);
        }
        else {
          throw BConfigException(__FILE__, __LINE__,
            qApp->translate("BAIsConfig", "Unsupported AI type '%1'").arg(
            type));
        }
      }
      else {
        qWarning("Unsupported tag '%s' in file '%s'", 
          (const char*)e.tagName().local8Bit(),
          (const char*)fileName().local8Bit());
      }
    }
  }
}

void BAIs::loadDocument(const QString& fileName) {
  BAIsConfig aisConfig(this, fileName);
}

template<class T>
class to_xml : public std::unary_function<T, void> {
public:
  to_xml(QDomDocument& doc):_doc(doc){}
  void operator() (const T* a) const {
    _doc.documentElement().appendChild(a->toXml(_doc));
  }
private:
  QDomDocument& _doc;
};

QDomDocument BAIs::toXml() const {
  QDomDocument doc;
  QDomProcessingInstruction pi = doc.createProcessingInstruction(
    "xml", QString("version='1.0' encoding='UTF-8'"));
  doc.insertBefore(pi, doc.documentElement());
  doc.appendChild(doc.createElement(BAIs::AIS_AIS));
  std::for_each(_ais.begin(), _ais.end(), to_xml<BAI>(doc));
  return doc;
}

void BAIs::slotSaveAI(const QString& aiFileName) {
  QFile f(aiFileName);
  if (!f.open(IO_WriteOnly)) {
    throw BOutputException(__FILE__, __LINE__, aiFileName);
  }
  QTextStream t(&f);
  t.setEncoding(QTextStream::UnicodeUTF8);
  t << toXml().toString(2);
  f.close();
}

void BAIs::setControlMode(const BAI::ControlModes mode) {
  std::for_each(_ais.begin(), _ais.end(), 
    std::bind2nd(MEM_FUN(&BAI::setControlMode), mode));
}

void BAIs::setControlMode(const BAI::ControlModes mode, BattleObject* object) {
  for(BAIsContainer::iterator it = _ais.begin(); it != _ais.end(); it++) {
    if ((*it)->object() == object) {
      (*it)->setControlMode(mode);
    }
  }
}

class delete_ai: public std::unary_function<void, bool> {
public:
  bool operator()(BAI* ai) const {
	if (ai->isMarkedToDelete()) {
	  delete ai;
	  return true;
	}
	else
      return false;
  }
};

void BAIs::slotBattleObjectRemoved(BattleObject* o) {
  std::for_each(_ais.begin(), _ais.end(), 
    std::bind2nd(MEM_FUN(&BAI::objectRemoved), o));
  BAIsContainer::iterator eit = std::remove_if(_ais.begin(), _ais.end(), 
    delete_ai());
  _ais.erase(eit, _ais.end());
}

int BAIs::thinkIntervalSecs() {
  return THINK_INTERVAL;
}

BAIsEdit::BAIsEdit(Battle* battle, QObject* parent, const char* name)
:BAIs(battle, parent, name) {
}

BAIsEdit::~BAIsEdit() {
}

void BAIsEdit::think() {
}

