/******************************************************************************/
/* BAI -   BattleFleet              */
/******************************************************************************/

#ifndef BAI_H
#define BAI_H
static const char bai_h_id[]="@(#) $Id: bai.h,v 1.7 2005/02/20 14:44:57 Oleksiy Balabay Exp $"; static const char* bai_h_id_id=bai_h_id;
/** \file bai.h
  \brief  BAI  BattleFleet

     BAI  BattleFleet.  
*/
#include <vector>
#include <qdatetime.h>
#include <qdom.h>

#include "battleobserver.h"

class BattleObject;
class BAIs;
class BAIChasing;

class BAI {
public:
  enum ControlModes {
    ControlModeManual = 0,
    ControlModeAuto = 1
  };
  BAI(BattleObject* obj, BAIs* parent, 
    const ControlModes mode = ControlModeAuto);
  virtual ~BAI();
  virtual void think() = 0;
  virtual bool objectRemoved(BattleObject* o) = 0;
  BattleObject* object() const {
    return _obj;
  }
  BAIs* parent() const {
    return _parent;
  }
  virtual QDomElement toXml(QDomDocument& doc) const {
    return doc.createElement(AI_AI);
  }
  virtual ControlModes controlMode() const {
    return _controlMode;
  }
  virtual bool setControlMode(ControlModes mode) {
    _controlMode = mode;
    return true; // ? keep stl happy (mem_fun_1)
  }
  virtual bool isMarkedToDelete() {
    return _deleteLater;
  };
  static const QString AI_AI;
  static const QString AI_TYPE;
protected:
  void setObj(BattleObject* obj) {
    _obj = obj;
  }
  void deleteLater() {
    _deleteLater = true;
  }
private:
  BattleObject* _obj;
  BAIs* _parent;
  ControlModes _controlMode;
  bool _deleteLater;
};

class BAIs: public BattleObserver {
  Q_OBJECT
public:
  BAIs(Battle* battle, QObject* parent, const char* name);
  virtual ~BAIs();
  virtual QWidget* viewWidget() const {
    return 0;
  }
  virtual void init();
  void addChasing(BattleObject* pursuier, BattleObject* evader, 
    double keepDistance = 0);
  void addChasing(const QDomElement& e);
  void clear();
  void clear(BattleObject* obj);
  void erase(BAI* ai);
  void trigChasingConnection(BAIChasing* ai, const bool *add, const bool *isAuto);
  static const QString AIS_AIS;
  void setControlMode(const BAI::ControlModes mode);
  static int thinkIntervalSecs();
public slots:
  virtual void slotBattleInfoChanged(const BattleInfo& info);
  void slotSaveAI(const QString& aiFileName);
  void slotBattleObjectRemoved(BattleObject* o);
  void setControlMode(const BAI::ControlModes mode, BattleObject* object);
signals:
  void chasingConnectionChanged(BAIChasing* ai, const bool *add, const bool *isAuto);
private:
  virtual void think();
  void loadDocument(const QString& fileName);
  QDomDocument toXml() const;
private:
  typedef std::vector<BAI*> BAIsContainer;
  BAIsContainer _ais; 
  QString _fileName;
  QDomDocument* _doc;
  static const int THINK_INTERVAL;
private:
  // TODO:        
  QDateTime _lastThinkTime;
};

class BAIsEdit: public BAIs {
  Q_OBJECT
public:
  BAIsEdit(Battle* battle, QObject* parent, const char* name);
  virtual ~BAIsEdit();
private:
  virtual void think();
};

#endif // BAI_H
