/******************************************************************************/
/* BAIChasing -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char baichasing_cpp_id[]="@(#) $Id: baichasing.cpp,v 1.14 2005/02/20 14:44:57 Oleksiy Balabay Exp $"; static const char* baichasing_cpp_id_id=baichasing_cpp_id;

#include <qsettings.h>

#include "baichasing.h"
#include "battle.h"
#include "bapplication.h"
#include "battleobject.h"
#include "bprogram.h"
#include "bunitsconsts.h"
#include "beventprogram.h"
#include "bprogramfactory.h"
#include "bprogramturnaccelerate.h"
#include "bseamap.h"
#include "bships.h"
#include "baichasingtrackspf.h"
#include "baichasingtrackscf.h"
#include "baicollisions.h"

/** \file baichasing.cpp
  \brief  BAIChasing  BattleFleet (impl.)

     BAIChasing  BattleFleet
*/

/** \class BAIChasing
  \brief 


*/
const QString BAIChasing::AI_TYPE_CHASING = "chasing";
const QString BAIChasing::AI_PURSUIER = "Pursuier";
const QString BAIChasing::AI_EVADER = "Evader";

/** \brief 
*/
BAIChasing::BAIChasing(BAIs* parent, BattleObject* pursuier, BattleObject* evader, 
  double /*distance*/):BAI(pursuier, parent), _evader(evader) {
  BattleInfo bi = bApp->battle()->battleInfo();
  BKinematicData kdP = object()->kinematicData(bi.modelTime);
  // TODO:     ,
  //   ,     
//  _old.a = kdP.a();
//  _old.dt = bi.modelTime;
//  _old.finalCourse = kdP.course();
//  _old.v = kdP.v(false);
//  _old.r = kdP.r();
//  _new = _old;
  bool add = true;
  bool isAuto = (controlMode() == ControlModeAuto);
  parent->trigChasingConnection(this, &add, &isAuto);
  _y0 = bApp->battle()->seaMap()->originalCoordinates().y0();
  _chasingTracksPF = new BAIChasingTracksPF(parent, pursuier, evader);
  _chasingTracksCF = new BAIChasingTracksCF(parent, pursuier, evader);
  _collisions = new BAICollisions(parent, pursuier, evader);
}

/** \brief 
*/
BAIChasing::~BAIChasing(){
  bool add = false;
  parent()->trigChasingConnection(this, &add, 0);
  delete _chasingTracksPF;
  delete _chasingTracksCF;
}

void BAIChasing::think(){
  if (controlMode() == ControlModeAuto) {
    bool isOnPF = _chasingTracksPF->isOn();
    bool isOnCF = _chasingTracksCF->isOn();
    bool pfPassed = false;
    bool cfPassed = false;
    if (isOnPF) {
      thinkPF();
      pfPassed = true;
    }
    else if (isOnCF) {
      thinkCF();
      cfPassed = true;
    }
    isOnPF = _chasingTracksPF->isOn();
    isOnCF = _chasingTracksCF->isOn();

    if ((!isOnPF) && (!isOnCF)) {
      if (!pfPassed) {
        thinkPF();
        isOnPF = _chasingTracksPF->isOn();
      }
      if ((!isOnPF) && (!cfPassed)) {
        thinkCF();
        isOnCF = _chasingTracksCF->isOn();
      }
    }
    if ((!isOnPF) && (!isOnCF)) {
      thinkTT();
    }
    checkCollisions();
  }
}

void BAIChasing::thinkPF() {
  bool isOn = _chasingTracksPF->isOn();
  try {
    _chasingTracksPF->think();
  }
  catch (...) {
//    if ((isOn) && (!_chasingTracksPF->isOn())) {
//      bApp->log(qApp->translate("BAIChasing",
//      "Chasing mode switched to line-of-sight chasing. "
//      "Pursuier = '%1', evader = '%2'").arg(object()->name()).arg(
//      _evader->name()));
//    }
  }
//  if ((!isOn) && (_chasingTracksPF->isOn())) {
//      bApp->log(qApp->translate("BAIChasing",
//      "Chasing mode switched to track. "
//      "Pursuier = '%1', evader = '%2'").arg(object()->name()).arg(
//      _evader->name()));
//  }
}

void BAIChasing::thinkCF() {
  bool isOn = _chasingTracksCF->isOn();
  try {
    _chasingTracksCF->think();
  }
  catch (...) {
//    if ((isOn) && (!_chasingTracksCF->isOn())) {
//      bApp->log(qApp->translate("BAIChasing",
//      "Chasing mode switched to line-of-sight chasing. "
//      "Pursuier = '%1', evader = '%2'").arg(object()->name()).arg(
//      _evader->name()));
//    }
  }
//  if ((!isOn) && (_chasingTracksCF->isOn())) {
//      bApp->log(qApp->translate("BAIChasing",
//      "Chasing mode switched to coord. track "
//      "Pursuier = '%1', evader = '%2'").arg(object()->name()).arg(
//      _evader->name()));
//  }
}

double BAIChasing::course(double xE, double yE, double xP, double yP) {
  double lx = bApp->battle()->seaMap()->mileage(xE, yP, xP, yP);
  double ly = bApp->battle()->seaMap()->mileage(xE, yE, xE, yP);
  if (xE < xP)
    lx = -lx;
  if (yE < yP)
    ly = -ly;
  double l = sqrt(lx*lx + ly*ly);
  double cosX = lx/l;
  double angle = acos(cosX);
  if (ly > 0)
    angle = -angle;
  return angle;
}

void BAIChasing::thinkTT() {
  //  ,    
  //     
  QDateTime dt = bApp->battle()->battleInfo().modelTime;
  double thinkIntervalH = BAIs::thinkIntervalSecs()/3600.0;
  QDateTime nextT = BProgram::addHours(dt, thinkIntervalH);
//  BKinematicData kdE = _evader->kinematicData(nextT);
  BKinematicData kdE = _evader->kinematicData(dt);
  BKinematicData kdP = object()->kinematicData(dt);
  double ml = bApp->battle()->seaMap()->mileage(
    kdE.x(), kdE.y(), kdP.x(), kdP.y());
  double a = object()->availableAcceleration(dt);
  if (ml < BAIChasingTracks::timeInterval(object())*kdP.v(true)) {
    a = - a;
  }
  else {
    a = a/2;
  }
  //   -   
  double angle = Pi/2 - kdP.course() + course(kdE.x(), kdE.y(), kdP.x(), kdP.y());
  if (angle < -Pi) {
    angle = 2*Pi+angle;
  }
  else if (angle > Pi) {
    angle = 2*Pi - angle;
  }
  if (fabs(angle) < (2*Pi)/36000) {
    angle = 0;
  }
  double vk = object()->availableSpeed(dt);
  if (a < 0)
    vk = 0;
  double r = object()->availableTurnRadius(dt);
  //   
  double force = fabs(a);
  parent()->postEvent(
    new BEventProgram( BProgramFactory::createTurnAccelerateProgram(
      bApp->battle(),  
      &(angle), 
      &(vk),  
      &force,
      &(r)), 
      object()->name(), dt), true, false);
}


QDomElement BAIChasing::toXml(QDomDocument& doc) const {
  QDomElement e = BAI::toXml(doc);
  e.setAttribute(AI_TYPE, AI_TYPE_CHASING);
  e.setAttribute(AI_PURSUIER, object()->name());
  e.setAttribute(AI_EVADER, evader()->name());
  return e;
}

bool BAIChasing::setControlMode(ControlModes mode) {
  BAI::setControlMode(mode);
  bool isAuto = (controlMode() == ControlModeAuto);
  parent()->trigChasingConnection(this, 0, &isAuto);
  _chasingTracksPF->setOn(controlMode() == ControlModeAuto);
  _chasingTracksCF->setOn(controlMode() == ControlModeAuto);
  return true; //? keep stl happy
}

bool BAIChasing::objectRemoved(BattleObject* o) {
  if ((o == object())||(o == evader())) {
    bool add = false;
    parent()->trigChasingConnection(this, &add, 0);
    deleteLater();
    return true;
  }
  return false;
}


void BAIChasing::checkCollisions() {
  if (_collisions->takeControl()) {
    _chasingTracksCF->setOn(false);
    _chasingTracksPF->setOn(false);
  }
}
