/******************************************************************************/
/* BAIChasingTracks -   Battlefleet  (impl.)            */
/******************************************************************************/
static const char baichasingtracks_cpp_id[]="@(#) $Id: baichasingtracks.cpp,v 1.10 2005/03/07 15:34:37 Oleksiy Balabay Exp $"; static const char* baichasingtracks_cpp_id_id=baichasingtracks_cpp_id;

#include "baichasingtracks.h"
#include "bpathfinder.h"
#include "battleobject.h"
#include "bapplication.h"
#include "bprogramfactory.h"
#include "bprogramturnaccelerate.h"
#include "bseamap.h"
#include "bunitsconsts.h"
#include "battleobserver.h"
#include "beventprogram.h"
#include "bdomutils.h"

/** \file baichasingtracks.cpp
  \brief  BAIChasingTracks  Battlefleet (impl.)

     BAIChasingTracks  Battlefleet
*/

/** \class BAIChasingTracks
  \brief 


*/
/** \brief 
*/
BAIChasingTracks::BAIChasingTracks(BattleObserver* observer, 
  BattleObject* pursuier, 
  BattleObject* evader):BAIChasingAlg(observer, pursuier, evader), _on(false) {
  _dt = timeInterval(_p);//distance*_p->minTurnRadius()/(_p->maxSpeed()/2);
  setState(NoControl);
  _id = -1;
}

/**   ,     
         . 

       -   
*/
double BAIChasingTracks::timeInterval(const BattleObject* o) {
  double ti = o->minTurnRadius()/(o->maxSpeed()/1.5);
  if (o->size() >=30) {
    if (ti < 45.0/3600)
      ti = 45.0/3600;
  }
  return ti;
}

/** \brief 
*/
BAIChasingTracks::~BAIChasingTracks(){

}

void BAIChasingTracks::think() {
  try {
    thinkInternal();
  }
  catch (...) {
    setOn(false);
//    _targetTime = QDateTime();
    throw;
  }
  setOn(true);
}

void BAIChasingTracks::thinkInternal() {
  QDateTime dt = bApp->battle()->battleInfo().modelTime;
//  float debug = _e->kinematicData(dt).course();
//  qDebug("e->course: %f", debug);
  if (state() == NoControl) {
    //    
    selectTarget(dt);
  }
  doControl();
}

void BAIChasingTracks::selectTarget(const QDateTime& dt) {
  const BObjectTracks& ots = _e->objectTracks();
  BKinematicData targetKd = _e->kinematicData(dt);
  BKinematicData pKd = _p->kinematicData(dt);

//  double maxT = maxTimeDelay(dt);
  BObjectTracks::const_reverse_iterator it = ots.rbegin();
  while (it != ots.rend()) {
    QDateTime t = (*it).first;
    BKinematicData k = (*it).second.kd;
    if (k.v() <= 0) {
      it++;
      continue;
    }
    QDateTime rezT  = BProgram::addHours(t, _dt);
    if (rezT < dt) {
      it++;
      continue;
    }
    double reqT = BProgram::hours(dt, rezT);
    bool rez = true;
    try {
      setupPrograms(dt, reqT, (*it).second.kd, pKd);
    }
    catch(...) {
      rez = false;
    }
    if (rez) {
      setState(Control);
      _id = (*it).second.id;
      return;
    }
    it++;
  }
  //       , ,
  //     
//  targetKd = _e->kinematicData(dt);
/*  if (targetKd.v() <= 0)
    return;
  QDateTime rezT = BProgram::addHours(dt, _dt);
  BKinematicData pTargetKd = _p->kinematicData(rezT);
  if (bApp->battle()->seaMap()->mileage(targetKd.x(), targetKd.y(),
    pTargetKd.x(), pTargetKd.y()) <= _e->length()/100) {
    //  .
    setState(Control);
    if (ots.size() != 0) {
      _id = ots.front().second.id+1;
    }
    else {
      _id = -1;
    }
    return;
  }
*/
//  double reqT = BProgram::hours(dt, rezT);
  //    ,   
  bool rez = true;
  try {
    setupPrograms(dt, _dt/*reqT*/, targetKd, pKd);
  }
  catch(...) {
    rez = false;
  }
  if (rez) {
    //     ,     
    BProgram* p = _e->restOfProgram(dt);
    QDateTime rezT = BProgram::addHours(dt, _dt);
    _observer->postEvent(new BEventProgram( p, _p->name(), rezT));
    setState(Control);
    if (ots.size() != 0) {
      _id = ots.front().second.id+1;
    }
    else {
      _id = -1;
    }
    return;
  }
}

void BAIChasingTracks::setupPrograms(const QDateTime& dt, const double t, 
  const BKinematicData& ekd, const BKinematicData& pkd) {
  setupPrograms(dt, t, _p, _e, _observer, ekd, pkd, bApp->battle()); 
}

void BAIChasingTracks::setupPrograms(const QDateTime& dt,
    const double t,
    BattleObject* p,
    BattleObject* e,
    BattleObserver* o,
    const BKinematicData& ekd, const BKinematicData& pkd,
    Battle* b) {
  BPathFinder::PathFinderInput in;
  double y0 = b->seaMap()->originalCoordinates().y0();
  in.a.x = BUnitsConverter::degreesToMiles(ekd.x(), y0);
  in.a.y = BUnitsConverter::degreesToMiles(ekd.y());
  in.b.x = BUnitsConverter::degreesToMiles(pkd.x(), y0);
  in.b.y = BUnitsConverter::degreesToMiles(pkd.y());
  in.a.c = ekd.course();
  in.a.v = ekd.v(false);
  in.b.c = pkd.course();
  in.b.v = pkd.v(false);
  in.pR.lk = p->lossOfSpeedCoef(p->size());
  in.pR.maxA = p->availableAcceleration(dt);
  in.pR.minA = -in.pR.maxA;
  in.pR.maxV = p->availableSpeed(dt);
  in.pR.minV = -in.pR.maxV;
  in.pR.minR = p->availableTurnRadius(dt);
  in.t = t;
  //     
  double len = p->length();
  in.a.x/=len;
  in.a.y/=len;
  in.a.v/=len;

  in.b.x/=len;
  in.b.y/=len;
  in.b.v/=len;

  in.pR.maxA/=len;
  in.pR.minA/=len;
  in.pR.maxV/=len;
  in.pR.minV/=len;
  in.pR.minR/=len;

  BFinder::PathFinderOutput pfo = findPathAlg(in);
  if (pfo.countElements == 0) {
    throw BException(__FILE__, __LINE__, "Path not found");
  }
//  BFinder::PathFinderOutput debugPfo = findPathAlg(in);

  for(int i = 0; i < pfo.countElements; i++) {
    pfo.prgs[i].a*=len;
    pfo.prgs[i].r*=len;
    pfo.prgs[i].v*=len;
  }
//  _targetTime = BProgram::addHours(dt, in.t);
//  qDebug("ECourse: %f", in.a.c);
  sendProgramsToBattle(pfo, dt);
}

void BAIChasingTracks::doControl() {
  if (state() != Control) {
    throw BException(__FILE__, __LINE__, "No target time");
  }
  checkTracks(bApp->battle()->battleInfo().modelTime);
  const BObjectTracks& ots = _e->objectTracks();
  for(BObjectTracks::const_reverse_iterator it = ots.rbegin(); 
  it != ots.rend(); it++) {
    BObjectTrackPoint tp = (*it).second;
 //   qDebug("tt %d - %d", _id, tp.id);
    if (tp.id < _id)
      continue;
    BProgram* prg = BProgramFactory::create(tp.program,
      bApp->battle());
//    QDomDocument debugDoc;
//    qDebug("SetPrg: %s - %s", (const char*)_p->name().local8Bit(),
//    (const char*)BDomUtils::elementToString(prg->toXml(debugDoc)).local8Bit());
    QDateTime ndt = BProgram::addHours((*it).first, _dt);
    _observer->postEvent(new BEventProgram(prg, _p->name(), ndt));
    _id = tp.id + 1;
//    
//    for(Tracks::const_iterator debugIt = _tracksToCheck.begin(); 
//    debugIt != _tracksToCheck.end(); debugIt++) {
//      qDebug("%s", (const char*)(*debugIt).first.toString().local8Bit());
//    }
  }
}

void BAIChasingTracks::setOn(const bool on) {
  _on = on;
  if (on == false) {
    setState(NoControl);
    _tracksToCheck.clear();
  }
}

bool BAIChasingTracks::isOn() const {
  return _on;
}

/*double BAIChasingTracks::timeTo(const QDateTime& t, const QDateTime& nextT, 
  BattleObject* p,
  BattleObject* e,
  const BKinematicData& ekd, const BKinematicData& pkd, bool* speedUp) {
  double rez = -1;
  BPathFinder::PathFinderInput in;
  Battle* b = bApp->battle();
  double y0 = b->seaMap()->originalCoordinates().y0();
  in.a.x = BUnitsConverter::degreesToMiles(ekd.x(), y0);
  in.a.y = BUnitsConverter::degreesToMiles(ekd.y());
  in.b.x = BUnitsConverter::degreesToMiles(pkd.x(), y0);
  in.b.y = BUnitsConverter::degreesToMiles(pkd.y());
  in.a.c = ekd.course();
  in.a.v = ekd.v(false);
  in.b.c = pkd.course();
  in.b.v = pkd.v(false);
  in.pR.lk = p->lossOfSpeedCoef(p->size());
  in.pR.maxA = p->availableAcceleration(t);
  in.pR.minA = -in.pR.maxA;
  in.pR.maxV = p->availableSpeed(t);
  in.pR.minV = -in.pR.maxV;
  in.pR.minR = p->availableTurnRadius(t);
  in.t = BProgram::hours(nextT, t);
  //     
  double len = p->length();
  in.a.x/=len;
  in.a.y/=len;
  in.a.v/=len;

  in.b.x/=len;
  in.b.y/=len;
  in.b.v/=len;

  in.pR.maxA/=len;
  in.pR.minA/=len;
  in.pR.maxV/=len;
  in.pR.minV/=len;
  in.pR.minR/=len;
  try {
    rez = timeToAlg(in, speedUp);
  }
  catch (...) {
    rez = -1;
  }
  return rez;
}*/

void BAIChasingTracks::setState(BAIChasingTracks::States state) {
  _state = state;
}

BAIChasingTracks::States BAIChasingTracks::state() const {
  return _state;
}

void BAIChasingAlg::sendProgramsToBattle(const BFinder::PathFinderOutput& pfo,
  const QDateTime& dt) {
  Battle* b = bApp->battle();
  //  
  QDateTime curT = dt;
  bool firstSet = true;
//  qDebug("Name: %s, Course: %f", 
//    (const char*)_p->name().local8Bit(), 
//    _p->kinematicData(dt).course());
  for(int j = 0; j < pfo.countElements; j++) {
    QDateTime rezT = BProgram::addHours(curT, pfo.prgs[j].t);
    double force = fabs(pfo.prgs[j].a);
    //    
    _observer->postEvent(
      new BEventProgram( BProgramFactory::createTurnAccelerateProgram(b,  
        &(pfo.prgs[j].angle), 
        &(pfo.prgs[j].v),  
        &force,
        &(pfo.prgs[j].r)), 
        _p->name(), rezT), firstSet, false);
    firstSet = false;
//    QString debugStr=QString("angle = %1 v=%2 a=%3 r=%4 t=%5").arg(
//      pfo.prgs[j].angle).arg(pfo.prgs[j].v).arg(pfo.prgs[j].a).arg(
//      pfo.prgs[j].r).arg(pfo.prgs[j].t);
//    qDebug("%s", (const char*)debugStr.local8Bit());

  }
}

void BAIChasingTracks::checkTracks(const QDateTime& dt) {
  QDateTime ndt = BProgram::addHours(dt, _dt);
  _tracksToCheck.push_front( std::make_pair(ndt, _e->kinematicData(dt)));
  Track trck;
  bool find = false;
  bool good = true;
  while ( (_tracksToCheck.size() != 0) && 
    (_tracksToCheck.back().first <= dt) ){
    trck = _tracksToCheck.back();
    _tracksToCheck.pop_back();
    find = true;
  }
  if (find) {
    try {
      BKinematicData kd = _p->kinematicData(_tracksToCheck.back().first);
      if (!isSimilar(kd, trck.second)) {
        good = false;
      }
    }
    catch(.../*BProgramTimeException&*/ /*e*/) {
//      qDebug("...");
    }
  }
  if (!good) {
    throw BException(__FILE__, __LINE__, "Check failed");
  }
}

bool BAIChasingTracks::isSimilar(const BKinematicData& kd1, 
  const BKinematicData& kd2) {
//  qDebug("C1=%f, C2=%f", kd1.course(), kd2.course());
  if (bApp->battle()->seaMap()->mileage2(kd1.x(), kd1.y(), kd2.x(), kd2.y()) <
    _p->length()*_p->length()/100) {
//    qDebug("Pos fault");
    return false;
  }
//  if (fabs(kd1.course() - kd2.course()) > 1.0/36000) {
//    qDebug("Course fault");
//    return false;
//  }
//  if (fabs(kd1.v(false) - kd2.v(false)) > 0.01) {
//    qDebug("Speed fault");
//    return false;
//  }
  return true;
}
