/******************************************************************************/
/* BAIChasingTracks -   Battlefleet              */
/******************************************************************************/

#ifndef BAIChasingTracks_H
#define BAIChasingTracks_H
static const char baichasingtracks_h_id[]="@(#) $Id: baichasingtracks.h,v 1.3 2005/01/15 16:44:51 Oleksiy Balabay Exp $"; static const char* baichasingtracks_h_id_id=baichasingtracks_h_id;
/** \file baichasingtracks.h
  \brief  BAIChasingTracks  Battlefleet

     BAIChasingTracks  Battlefleet
*/
class BattleObject;
class BattleObserver;
class Battle;

#include "bpathfinder.h"
#include "bkinematicdata.h"

class BAIChasingAlg {
public:
  BAIChasingAlg(BattleObserver* observer, BattleObject* pursuier, 
  BattleObject* evader):_observer(observer), _p(pursuier),
  _e(evader){};
  virtual ~BAIChasingAlg() {};
protected:
  void sendProgramsToBattle(const BFinder::PathFinderOutput& pfo,
    const QDateTime& dt);
protected:
  BattleObserver* _observer;
  BattleObject* _p;
  BattleObject* _e;
};

class BAIChasingTracks:public BAIChasingAlg {
public:
  BAIChasingTracks(BattleObserver* observer, BattleObject* pursuier, BattleObject* evader);
  virtual ~BAIChasingTracks();
  void think();
  void setOn(const bool on);
  bool isOn() const;
  static double timeInterval(const BattleObject* o);
protected:
  enum States {
    NoControl,
    Control
  };
protected:
  virtual BFinder::PathFinderOutput findPathAlg(
    const BFinder::PathFinderInput& in) = 0;
  virtual double timeToAlg(
    const BFinder::PathFinderInput& in, bool* speedUp) = 0;
//  double timeTo(const QDateTime& t, const QDateTime& nextT, 
//    BattleObject* p,
//    BattleObject* e,
//    const BKinematicData& ekd, const BKinematicData& pkd, bool* speedUp);
  void setupPrograms(const QDateTime& dt,
    const double t,
    BattleObject* p,
    BattleObject* e,
    BattleObserver* o,
    const BKinematicData& ekd, const BKinematicData& pkd,
    Battle* b);
  void setState(States state);
  States state() const;
protected:
  typedef std::pair<QDateTime, BKinematicData> Track;
  typedef std::deque<Track> Tracks;
protected:
  Tracks _tracksToCheck;
protected:
  void checkTracks(const QDateTime& dt);
  virtual bool isSimilar(const BKinematicData& kd1, 
    const BKinematicData& kd2);
private:
  void thinkInternal();
  void selectTarget(const QDateTime& dt);
  void doControl();
  void setupPrograms(const QDateTime& dt, const double t, 
    const BKinematicData& ekd, const BKinematicData& pkd);
private:
  bool _on;
  double _dt;
  double _id;
  States _state;
};

#endif // BAIChasingTracks_H
