/******************************************************************************/
/* BAICollisions -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char baicollisions_cpp_id[]="@(#) $Id: baicollisions.cpp,v 1.4 2005/02/20 14:44:57 Oleksiy Balabay Exp $"; static const char* baicollisions_cpp_id_id=baicollisions_cpp_id;

#include "baicollisions.h"
#include "bapplication.h"
#include "battle.h"
#include "bships.h"
#include "bunitsconsts.h"
#include "bseamap.h"
#include "bmathturn.h"
#include "bprogram.h"
#include "bcollisionevader.h"
//#include "bcollisionsevaderp.h"

/** \file baicollisions.cpp
  \brief  BAICollisions  BattleFleet (impl.)

     BAICollisions  BattleFleet
*/

/** \class BAICollisions
  \brief        

     :
  -     
  -     
  -      
*/
BattleObjects BAICollisions::possibleCollisions(const QDateTime& dt){ 
  BattleObjects bos;
  BShips* shps = bApp->battle()->ships();
  BSeaMap* seaMap = bApp->battle()->seaMap();
  BKinematicData kdP = _p->kinematicData(dt);
  double range2 = BAIChasingTracks::timeInterval(_p)*_p->availableSpeed(dt);
  range2 = range2*range2;
  for(BShips::const_iterator it = shps->begin(); it != shps->end(); it++) {
    if ( _p != *it ) {
      BKinematicData kd = (*it)->kinematicData(dt);
      if (seaMap->mileage2(kdP.x(), kdP.y(), kd.x(), kd.y()) < range2){
        bos.insert(*it);
      }
    }
  }
  return bos;
}
/** \brief 
*/
BAICollisions::BAICollisions(BattleObserver* observer, 
  BattleObject* pursuier, BattleObject* evader):BAIChasingAlg(observer,
  pursuier, evader) {
}

/** \brief 
*/
BAICollisions::~BAICollisions(){
}

bool BAICollisions::takeControl() {
  BattleObjects possibleCols = possibleCollisions(
    bApp->battle()->battleInfo().modelTime);
  BattleObject* realCollision;
  QDateTime colTime;
  if (checkCollisions(possibleCols, &realCollision, colTime)) {
    return takeActions(realCollision, bApp->battle()->battleInfo().modelTime);
  }

/*  BattleObjects realCollisions;
  if (checkCollisions(possibleCols, realCollisions)) {
    return takeActions(realCollisions, bApp->battle()->battleInfo().modelTime);
  }
*/
  return false;
}


bool BAICollisions::checkCollisions(const BattleObjects& possibleCollisions, 
  BattleObject** realCollision, QDateTime& colTime) {
  BSeaMap* seaMap = bApp->battle()->seaMap();
  //  ,    
  double tI = BAIChasingTracks::timeInterval(_p);
  QDateTime dt = bApp->battle()->battleInfo().modelTime;
  // ,     
  double dTI = _p->length()/(2*_p->availableSpeed(dt));//tI*_p->availableSpeed(dt)/(_p->length()/2);
  double pLength = _p->length();
  //    
  double curT = 0;
  while (curT <= tI) {
    QDateTime curDT = 
      BProgram::addHours(bApp->battle()->battleInfo().modelTime, curT);
    BKinematicData kdP = _p->kinematicData(curDT);
    //   
    for(BattleObjects::const_iterator it = possibleCollisions.begin();
    it != possibleCollisions.end(); it++) {
      BKinematicData kd = (*it)->kinematicData(curDT);
      double range2 = (pLength + (*it)->length())/2;
      range2 = range2*range2;
      // ,   ,  (l1+l2)/2
      if (seaMap->mileage2(kdP.x(), kdP.y(), kd.x(), kd.y()) < range2) {
        //  ,   
        *realCollision = *it;
        colTime = curDT;
        return true;
      }
    }
    curT+=dTI;    
  }
  return false;
}

bool BAICollisions::takeActions(BattleObject* o, const QDateTime& dt) {
  BFinder::PathFinderInput in;
  BKinematicData pkd = _p->kinematicData(dt);
  BKinematicData kd = o->kinematicData(dt);

  double y0 = bApp->battle()->seaMap()->originalCoordinates().y0();
  in.a.x = BUnitsConverter::degreesToMiles(pkd.x(), y0);
  in.a.y = BUnitsConverter::degreesToMiles(pkd.y());
  in.b.x = BUnitsConverter::degreesToMiles(kd.x(), y0);
  in.b.y = BUnitsConverter::degreesToMiles(kd.y());
  in.a.c = pkd.course();
  in.a.v = pkd.v(false);
  in.b.c = kd.course();
  in.b.v = kd.v(false);
  in.pR.lk = _p->lossOfSpeedCoef(_p->size());
  in.pR.maxA = _p->availableAcceleration(dt);
  in.pR.minA = -in.pR.maxA;
  in.pR.maxV = _p->availableSpeed(dt);
  in.pR.minV = -in.pR.maxV;
  in.pR.minR = _p->availableTurnRadius(dt);
  in.t = 0;
  BCollisionEvader::CollisionInput colIn;
  colIn.lengthA = _p->length();
  colIn.lengthB = o->length();
  colIn.widthA = _p->width();
  colIn.widthB = o->width();
  BFinder::PathFinderOutput pfo = evadeCollision(in, colIn);
  if (pfo.countElements != 0) {
    sendProgramsToBattle(pfo, dt);
//    qDebug("Evade collision with %s", (const char*)o->name().local8Bit());
    return true;
  }
  else
    return false;
}

/*bool BAICollisions::checkCollisions(const BattleObjects& possibleCollisions, 
  BattleObjects& realCollisions) {
  QDateTime dt;
  for(BattleObjects::const_iterator it = possibleCollisions.begin();
    it != possibleCollisions.end(); it++) {
    BattleObject* o = 0;
    BattleObjects bos;
    bos.insert(*it);
    if (checkCollisions(bos, &o, dt)) {
      realCollisions.insert(o);
    }
  }
  return (realCollisions.size() != 0);
}

bool BAICollisions::takeActions(const BattleObjects& realCollisions, 
  const QDateTime& dt) {

  BCollisionsEvaderP::Points ps;
  int cnt = realCollisions.size();
  if (cnt > MAX_COLLISIONS_SIZE - 1)
    cnt = MAX_COLLISIONS_SIZE - 1;
  ps.countElements = cnt + 1;

  double y0 = bApp->battle()->seaMap()->originalCoordinates().y0();
  BKinematicData pkd = _p->kinematicData(dt);
  ps.pnts[0].x = BUnitsConverter::degreesToMiles(pkd.x(), y0);
  ps.pnts[0].y = BUnitsConverter::degreesToMiles(pkd.y());
  ps.pnts[0].c = pkd.course();

  int i = 1;
  for(BattleObjects::const_iterator it = realCollisions.begin();
  it != realCollisions.end(); it++) {
    if (i > cnt)
      break;
    BKinematicData kd = (*it)->kinematicData(dt);
    ps.pnts[1].x = BUnitsConverter::degreesToMiles(kd.x(), y0);
    ps.pnts[1].y = BUnitsConverter::degreesToMiles(kd.y());
    ps.pnts[1].c = kd.course();
  }

  BFinder::PathRestrictions pR;
  pR.lk = _p->lossOfSpeedCoef(_p->size());
  pR.maxA = _p->availableAcceleration(dt);
  pR.minA = -pR.maxA;
  pR.maxV = _p->availableSpeed(dt);
  pR.minV = -pR.maxV;
  pR.minR = _p->availableTurnRadius(dt);

  BFinder::PathFinderOutput pfo = evadeCollisionP(ps, pR);
  if (pfo.countElements != 0) {
    sendProgramsToBattle(pfo, dt);
    return true;
  }
  else
    return false;
}
*/
