/******************************************************************************/
/* BApplication -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bapplication_cpp_id[]="@(#) $Id: bapplication.cpp,v 1.25 2005/04/09 16:14:25 Oleksiy Balabay Exp $"; static const char* bapplication_cpp_id_id=bapplication_cpp_id;

#include <qsettings.h>
#include <qdir.h>
#include <qsplashscreen.h>
#include <qassistantclient.h>
#include <qprogressdialog.h>
#include <qpushbutton.h>
#include <qtextcodec.h>

#include <stdio.h>
#include <stdlib.h>

#include <windows.h>

#include "bversion.h"
#include "bapplication.h"
#include "bmainwindow.h"
#include "battle.h"
#include "bexception.h"
#include "bfactory.h"
#include "bappstate.h"
#include "buiutils.h"
#include "ballonhelp.h"

/** \file bapplication.cpp
  \brief  BApplication  BattleFleet (impl.)

     BApplication  BattleFleet
*/

/** \class BApplication
  \brief 


*/
const QString SCENARIOS_DIR = "data";
const QString SCENARIO_SAVERS_DIR = "save";

const QString BApplication::_appName = "BattleFleet";
BApplication* bApp = 0;

void messageOutput(QtMsgType type, const char* msg) {
  bApp->messageOutput(type, msg);
}

/** \brief 
*/
BApplication::BApplication(int argc, char** argv):QApplication(argc, argv),
  _appState(0), _appFactory(0), _uiUtils(0), _progressDialog(0) {

  qInstallMsgHandler(::messageOutput);

  //      ,     ,
  //    QTextCodec::locale()    ru.  
  //    

  //  , , ,      . 
  //  ... ...
  QString locale = "";
  QFile f(applicationDirPath()+"/locale");
  if (f.open(IO_ReadOnly)) {
    QTextStream ts(&f);
    locale = ts.readLine();
    f.close();
  }
  QTranslator* qt = new QTranslator(0);
  if (qt->load(QString("qt_")+locale, applicationDirPath())) {
    installTranslator(qt);
  }
//  else {
//    qt->load(QString("qt_")+QTextCodec::locale(), applicationDirPath());
//    installTranslator(qt);
//  }

  QTranslator* bf = new QTranslator(0);
  if (bf->load(QString("battlefleet_")+locale, applicationDirPath())) {
    installTranslator(bf);
  }
//  else {
//    bf->load(QString("battlefleet_")+QTextCodec::locale(), applicationDirPath());
//    installTranslator(bf);
//  }

  _splash = new QSplashScreen(QPixmap::fromMimeSource("splash.jpg"));
  _splash->message(tr("Battlefleet v. %1").arg(B_VERSION),
    AlignHCenter|AlignBottom, QColor("red"));
  _splash->show();
  bApp = this;
//  _battle = 0;
  //  
  _settings = new QSettings();
  _settings->setPath("battlefleet.narod.ru", _appName, QSettings::User);
  _ballonHelp = new BallonHelp();
  _uiUtils = new BUiUtils();
  //    
  _scenariosDir = applicationDirPath()+QDir::separator()+SCENARIOS_DIR;
  _scenarioSaversDir = applicationDirPath()+
    QDir::separator()+SCENARIO_SAVERS_DIR;
  //  
  BMainWindow* mainWindow = new BMainWindow();
  setMainWidget(mainWindow);
  connect(this, SIGNAL(newScenario()), mainWindow, SIGNAL(newScenario()));
  connect(this, SIGNAL(logMessage(const QString&)),
    mainWindow, SIGNAL(log(const QString&)));
  connect(this, SIGNAL(modeChanged(const BAppModes, const BAppModes)), 
    mainWindow, SLOT(slotModeChanged(const BAppModes, const BAppModes)));
  connect(this, SIGNAL(aboutToQuit()), mainWindow, SLOT(slotSaveSettings()));
  mainWindow->show();
  connect(this, SIGNAL(lastWindowClosed()),
    this, SLOT(quit()));
  changeMode(UndefinedMode, BattleLoadData());
  _loadTimer = startTimer( 0 );
  _splashTimer = startTimer( 500 );

  //  
  _assistant = new QAssistantClient(applicationDirPath(), this, "assistant" );
  QStringList cmdLst;
  cmdLst << "-server " << "-profile"
	       << applicationDirPath() + "/" + QString("doc") + 
         QDir::separator() + QString("battlefleet.adp");
  _assistant->setArguments( cmdLst );
}

/** \brief 
*/
BApplication::~BApplication(){
  delete _appFactory;
  delete _appState;
  delete _settings;
  if (_uiUtils) {
    delete _uiUtils;
    _uiUtils = 0;
  }
}

Battle* BApplication::battle() const {
  Q_ASSERT(_appState != 0);
  return _appState->battle();
}

void BApplication::messageOutput(QtMsgType type, const char* msg) {
  switch (type) {
  case QtDebugMsg: 
    {
      emit logMessage(tr("Debug: %1").arg(QString::fromLocal8Bit(msg)));
      break;
    }
  case QtWarningMsg: 
    {
      emit logMessage(tr("Warning: %1").arg(QString::fromLocal8Bit(msg)));
      break;
    }
  case QtFatalMsg: 
    {
      fprintf(stderr, "Fatal: %s\n", msg);
      emit logMessage(tr("Fatal: %1").arg(QString::fromLocal8Bit(msg)));
      abort();
    }
  }
}

QSettings* BApplication::settings() const {
  return _settings;
}

void BApplication::timerEvent(QTimerEvent* e) {
  if ( e->timerId() == _loadTimer ) {
    killTimer(_loadTimer);
    _loadTimer = 0;
    load();
    return;
  }
  else if ( e->timerId() == _splashTimer ) {
    killTimer(_splashTimer);
    _splashTimer = 0;
    _splash->finish(0);
    delete _splash;
    return;
  }
  QApplication::timerEvent(e);
}

void BApplication::load() {
  emit newScenario();
}

QString BApplication::scenariosDir() const {
  return _scenariosDir;
}

QString BApplication::scenarioSaversDir() const {
  return _scenarioSaversDir;
}

bool BApplication::isBattleLoaded() const {
  return (_appState->battle() != 0);
}

void BApplication::quitBattle() {
  emit aboutToQuitBattle();
//  if (_battle) {
//    delete _battle;
//    _battle = 0;
//  }
//  changeMode(UndefinedMode, "");
//  emit battleChanged(_battle);
}

BFactory* BApplication::createAppFactory(const BAppModes& appMode) const {
  switch (appMode) {
  case UndefinedMode:
    return new BFactoryUndefined();
  case PlayMode:
    return new BFactoryPlay();
  case EditMode:
    return new BFactoryEdit();
  case GameMode:
    return new BFactoryGame();
  default:
    return new BFactoryUndefined();
  }
}

void BApplication::changeMode(const BAppModes& appMode,
  const BattleLoadData& loadData) {
  if (appMode != UndefinedMode) {
    changeMode(UndefinedMode, loadData);
    while (hasPendingEvents())
      processEvents();
  }
  if (!_appFactory) {
    _appFactory = createAppFactory(appMode);
  }
  if (!_appState) {
    _appState = _appFactory->createAppState();
  }
  BAppModes oldMode = _appState->appMode();
  try {
    if (_appState->appMode() != appMode) {
      delete _appFactory;
      _appFactory = 0;
      _appFactory = createAppFactory(appMode);
    }
    delete _appState;
    _appState = 0;
    _appState = _appFactory->createAppState();
    // _appState    , .. init()
    //  bApp->battle()
    _appState->init(loadData);
    if (_appState->appMode() != UndefinedMode) {
      checkProgressDialog(80, 
        QProgressDialog::tr("User interface initialization..."));
    }
  }
  catch (BException& e) {
    //  Undefined   
    emit logMessage(e.message());
    delete _appState;
    delete _appFactory;
    _appState = 0;
    _appFactory = 0;
    _appFactory = createAppFactory(UndefinedMode);
    _appState = _appFactory->createAppState();
  }
  catch (...) {
    emit logMessage(tr("Unknown error loading scenario '%1'").arg(
      loadData.fileName));
    delete _appState;
    delete _appFactory;
    _appState = 0;
    _appFactory = 0;
    _appFactory = createAppFactory(UndefinedMode);
    _appState = _appFactory->createAppState();
  }
  if (_appState->appMode() != UndefinedMode) {
    _progressDialog->setCancelButtonText(QString::null);
  }
  emit modeChanged(_appState->appMode(), oldMode);
}

void BApplication::startNewBattle(const BattleLoadData& loadData, 
  const BAppModes& appMode) {
  resetProgressDialog();
  if (isBattleLoaded()) {
    quitBattle();
  }
  checkProgressDialog(1, 
    QProgressDialog::tr("Reseting the environment..."));
  changeMode(appMode, loadData);
  delete _progressDialog;
  _progressDialog = 0;
}

QString BApplication::extendFileName(const QString& fileName) {
  if (fileName.isEmpty()) {
    qWarning("Empty file name requested");
    return "";
  }
  if (QDir::isRelativePath(fileName)) {
    return applicationDirPath()+"/"+fileName;
  }
  else {
    return fileName;
  }
}

QString BApplication::compressFileName(const QString& fileName) {
  if (fileName.isEmpty()) {
    qWarning("Empty file name requested");
    return "";
  }
  if (QDir::isRelativePath(fileName)) {
    return fileName;
  }
  else {
    if (fileName.find(applicationDirPath()) == 0)
      return fileName.mid(applicationDirPath().length()+1);
    else
      return fileName;
  }
}

void BApplication::log(const QString& message) {
  emit logMessage(message);
}

BFactory* BApplication::appFactory() const {
  return _appFactory;
}

bool BApplication::event(QEvent* e) {
  if (e->type() == BChangeModeEventNumber) {// change mode
    BChangeModeEvent* cme = (BChangeModeEvent*)e;
    startNewBattle(cme->loadData(), cme->appMode());
    return true;
  }
  else {
    return QApplication::event(e);
  }
}

BUiUtils* BApplication::uiUtils() const {
  return _uiUtils;
}

void BApplication::setUiUtils(BUiUtils* uiUtils) {
  if (_uiUtils) {
    delete _uiUtils;
  }
  _uiUtils = uiUtils;
}

void BApplication::showHelp() {
  showHelp(0);
}

void BApplication::showHelp(QWidget* w) {
  if (_widgets.find(w) == 0)
    w = 0;
  if ( w ) {
    QString debug = _widgets[w];
    ShellExecuteA(w->winId(), 0, _widgets[w].local8Bit(), 0, 0, 0);
//    _assistant->showPage( QString( _widgets[w] ) );
  }
  else
	  _assistant->showPage( applicationDirPath()+"/"+"doc/index.html" );
}

void BApplication::addWidgetHelp(QWidget* w, const QString& url) {
  _widgets.insert( std::make_pair(w, url));
}

void BApplication::removeWidgetHelp(QWidget* w) {
  _widgets.erase(w);
}

void BApplication::resetProgressDialog() {
  delete _progressDialog;
  _progressDialog = new QProgressDialog(
    QProgressDialog::tr("Previous scenario cleanup..."),
    tr("Cancel"), 100, mainWidget(), "progressDialog", true,
    WType_Dialog|WStyle_Customize|WStyle_Dialog|WStyle_Title);
  _progressDialog->setCaption(QProgressDialog::tr("Loading..."));
}

QProgressDialog* BApplication::progressDialog() {
  return _progressDialog;
}

void BApplication::checkProgressDialog(const int progress, 
  const QString& labelText) {
  if (progressDialog() == 0)
    return;
  if (progressDialog()->wasCanceled()) {
    throw BConfigException(__FILE__, __LINE__,
      QProgressDialog::tr("Scenario load process canceled by user"));
  };
  progressDialog()->setProgress(progress);
  if (!labelText.isEmpty()) {
    progressDialog()->setLabelText(labelText);
  }
  processEvents();
}
