/******************************************************************************/
/* BApplication -   BattleFleet              */
/******************************************************************************/

#ifndef BApplication_H
#define BApplication_H
static const char bapplication_h_id[]="@(#) $Id: bapplication.h,v 1.15 2005/02/20 14:44:57 Oleksiy Balabay Exp $"; static const char* bapplication_h_id_id=bapplication_h_id;
/** \file bapplication.h
  \brief  BApplication  BattleFleet

     BApplication  BattleFleet
*/
#include <qapplication.h>
#include <map>


class QSettings;
class BAppState;
class BFactory;
class BUiUtils;
class QSplashScreen;
class QAssistantClient;
class QProgressDialog;
class BallonHelp;

#include "battle.h"
#include "bcustomeventconsts.h"

enum BAppModes {
  UndefinedMode = -1,
  PlayMode = 0,
  EditMode = 1,
  GameMode = 2
};

struct BPlayer {
  enum Types {
    OperatorPlayer = 0,
    ComputerPlayer,
    NetPlayer
  };
  Types type;
  QString name;
};

class BChangeModeEvent: public QCustomEvent {
public:
  BChangeModeEvent(const BattleLoadData& loadData, const BAppModes& appMode)
    :QCustomEvent(BChangeModeEventNumber), _loadData(loadData), _appMode(appMode) {
  }
  BattleLoadData loadData() const {
    return _loadData;
  }
  const BAppModes appMode() const {
    return _appMode;
  }
private:
  BattleLoadData _loadData;
  BAppModes _appMode;
};

class BZoomEvent: public QCustomEvent {
public:
  enum Zooms {
    ZoomIn,
    ZoomOut,
    FullView
  };

  BZoomEvent(const Zooms zoom):QCustomEvent(BZoomEventNumber), _zoom(zoom){
  }
  Zooms zoom() const {
    return _zoom;
  }
private:
  Zooms _zoom;
};

class BApplication: public QApplication {
  Q_OBJECT
public:
  BApplication(int argc, char** argv);
  QSettings* settings() const;
  virtual ~BApplication();
  QString scenariosDir() const;
  QString scenarioSaversDir() const;
  bool isBattleLoaded() const;
  void quitBattle();
  void startNewBattle(const BattleLoadData& loadData, const BAppModes& appMode);
  QString extendFileName(const QString& fileName);
  QString compressFileName(const QString& fileName);
  BFactory* appFactory() const;
  Battle* battle() const;
  virtual bool event(QEvent* e);
  BUiUtils* uiUtils() const;
  void setUiUtils(BUiUtils* uiUtils);
  void showHelp();
  void showHelp(QWidget* w);
  void addWidgetHelp(QWidget* w, const QString& url);
  void removeWidgetHelp(QWidget* w);
  QProgressDialog* progressDialog();
  void checkProgressDialog(const int progress, 
    const QString& labelText = "");
  BallonHelp* ballonHelp() const {
    return _ballonHelp;
  }
public slots:
  void log(const QString& message);
protected:
  virtual void timerEvent(QTimerEvent* e);
  ///     Main Event Loop
  void load();
private:
  void messageOutput(QtMsgType type, const char* msg);
  void changeMode(const BAppModes& appMode, const BattleLoadData& loadData);
  BFactory* createAppFactory(const BAppModes& appMode) const;
  void resetProgressDialog();
private:
  QSettings* _settings;
  int _loadTimer;
  int _splashTimer;
  static const QString _appName;
  QString _scenariosDir;
  QString _scenarioSaversDir;
  BAppState* _appState;
  BFactory* _appFactory;
  BUiUtils* _uiUtils;
  QSplashScreen* _splash;
  QAssistantClient* _assistant;
  typedef std::map<QWidget*, QString> HelpUrls;
  HelpUrls _widgets;
  friend void messageOutput(QtMsgType type, const char* msg);
  QProgressDialog* _progressDialog;
  BallonHelp* _ballonHelp;
signals:
  void newScenario();
  void aboutToQuitBattle();
  void modeChanged(const BAppModes newMode, const BAppModes oldMode);
  void logMessage(const QString& message);
};

extern BApplication* bApp;

#endif // BApplication_H
