/******************************************************************************/
/* Battle -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char battle_cpp_id[]="@(#) $Id: battle.cpp,v 1.21 2005/03/07 15:34:37 Oleksiy Balabay Exp $"; static const char* battle_cpp_id_id=battle_cpp_id;

#include <qfileinfo.h>
#include <qdir.h>
#include <qprogressdialog.h>

#include "battle.h"
#include "bship.h"
#include "bships.h"
#include "bscenarioheader.h"
#include "bseamap.h"
#include "bevent.h"
#include "bapplication.h"
#include "beventsparser.h"

/** \file battle.cpp
  \brief  Battle  BattleFleet (impl.)

     Battle  BattleFleet
*/
const unsigned int BATTLE_TIMER_TIMEOUT = 50;

struct BattleSpeedCoefs {
  BattleSpeedCoefs(double s, BattleSpeeds i, int dT)
    :speed(s), index(i), dTMSecs(dT) {}
  double speed;
  BattleSpeeds index;
  int dTMSecs;
};

const BattleSpeedCoefs battleSpeedsCoefs[] = { 
  BattleSpeedCoefs(1, RealtimeSpeed, BATTLE_TIMER_TIMEOUT),
  BattleSpeedCoefs(2, SlowSpeed, BATTLE_TIMER_TIMEOUT*2),
  BattleSpeedCoefs(10, NormalSpeed, BATTLE_TIMER_TIMEOUT*10),
  BattleSpeedCoefs(30, FastSpeed, BATTLE_TIMER_TIMEOUT*30),
  BattleSpeedCoefs(60, UltraFastSpeed, BATTLE_TIMER_TIMEOUT*60)
};

/** \class Battle
  \brief 


*/
/** \brief 
*/
Battle::Battle(const BattleLoadData& loadData)
  :QObject(0, "battle"), _events(0), _scenarioHeader(0), _loadData(loadData), 
  _ships(0), _seaMap(0), _modified(false)  {
  try {
    _scenarioHeader = new BScenarioHeader(_loadData.fileName);
    _battleInfo.modelTime = _loadData.startDate;//_scenarioHeader->beginDate();
    bApp->checkProgressDialog(10, 
      QProgressDialog::tr("Loading ships..."));
    loadShips();
    bApp->checkProgressDialog(15, 
      QProgressDialog::tr("Loading seamap..."));
    loadSeaMap();
    bApp->checkProgressDialog(20, 
      QProgressDialog::tr("Loading events..."));
    loadEvents();
    _battleInfo.speed = 10;
    _battleInfo.speedIndex = NormalSpeed;
    _battleInfo.state = NotStarted;
    connect(&_battleTimer, SIGNAL(timeout()), this, SLOT(timeoutSlot()));
  }
  catch(...) {
    clearBattle();
    throw;
  }
}

/** \brief 
*/
Battle::~Battle(){
  clearBattle();
}

void Battle::clearBattle() {
  if (_seaMap) {
    delete _seaMap;
    _seaMap = 0;
  }
  if (_ships) {
    delete _ships;
    _ships = 0;
  }
  if (_scenarioHeader) {
    delete _scenarioHeader;
    _scenarioHeader = 0;
  }
  if (_events) {
    delete _events;
    _events = 0;
  }
}

void Battle::loadShips() {
  _ships = new BShips();
  QStringList l = _scenarioHeader->shipsFileNames();
  for (QStringList::const_iterator itl = l.begin(); itl != l.end(); itl++) {
    _ships->addShipsFromFile(*itl);
  }
  _ships->initAfterParse();
}

void Battle::loadSeaMap() {
  _seaMap = new BSeaMap(_scenarioHeader->mapFileName());
}

BSeaMap* Battle::seaMap() const {
  return _seaMap;
}

const BScenarioHeader* Battle::scenarioHeader() const {
  return _scenarioHeader;
}

void Battle::loadEvents() {
  _events = new BEvents(this);

}

void Battle::slotPlay() {
  startBattleTimer();
}

void Battle::slotPause() {
  if (Paused == _battleInfo.state ) {
    startBattleTimer();
  }
  else if (Running == _battleInfo.state) {
    stopBattleTimer();
  }
}

void Battle::timeoutSlot() {
//  QTime t = QTime::currentTime();
//  int ms = BATTLE_TIMER_TIMEOUT;//_systemTime.msecsTo(t);
//  if (ms < 0) {
//    ms = 24*3600 + ms;
//  }
//  ms = ms * _battleInfo.speed;
//  _systemTime = _systemTime.addMSecs(BATTLE_TIMER_TIMEOUT);
  _battleTimer.stop();
  QDate md = _battleInfo.modelTime.date();
  QTime mt = _battleInfo.modelTime.time();
  QTime newmt = mt.addMSecs(battleSpeedsCoefs[_battleInfo.speedIndex].dTMSecs);
  if (newmt < mt)
    md = md.addDays(1);
  _battleInfo.modelTime = QDateTime(md, newmt);
  _battleInfo.flags = BattleFlags(0);
  _events->processEvents(_battleInfo.modelTime);
  processCollisions();
  emit battleInfoChanged(_battleInfo);
  //          UI
  while (bApp->hasPendingEvents()) {
    bApp->processEvents();
  }
  if (_battleInfo.state == Running)
    _battleTimer.start(BATTLE_TIMER_TIMEOUT);
}

void Battle::startBattleTimer() {
  _battleInfo.state = Running;
//  _systemTime = QTime::currentTime();
  _battleTimer.start(BATTLE_TIMER_TIMEOUT);
  emit battleInfoChanged(_battleInfo);
}

void Battle::stopBattleTimer() {
  _battleInfo.state = Paused;
  _battleTimer.stop();
  emit battleInfoChanged(_battleInfo);
}

const BattleInfo& Battle::battleInfo() const {
  return _battleInfo;
}

BShips* Battle::ships() const {
  return _ships;
}

void Battle::slotSetSpeed(const BattleSpeeds battleSpeed) {
  bool restart = false;
  if (_battleInfo.state == Running) {
    _battleTimer.stop();
    restart = true;
  }
  _battleInfo.speed = battleSpeedsCoefs[battleSpeed].speed;
  _battleInfo.speedIndex = battleSpeed;
  //   ,      
  //    
  int dt = _scenarioHeader->beginDate().secsTo(_battleInfo.modelTime)*1000 +
    _battleInfo.modelTime.time().msec();
  int timeIncr = battleSpeedsCoefs[battleSpeed].dTMSecs;
  QDate d = _battleInfo.modelTime.date();
  QTime t = _battleInfo.modelTime.time();
  int i = timeIncr - dt%timeIncr;
  if (i != timeIncr)
    t = t.addMSecs( i );
  if (t < _battleInfo.modelTime.time())
    d = d.addDays(1);
  _battleInfo.modelTime = QDateTime(d,t);
  if (restart)
    _battleTimer.start(BATTLE_TIMER_TIMEOUT);
}

/* BattleSpeeds Battle::speedToBattleSpeeds(const double speed) {
  if ((speed > 0.999) && (speed < 1.001))
    return RealtimeSpeed;
  else if ((speed > 1.999) && (speed < 2.001))
    return SlowSpeed;
  else if ((speed > 9.999) && (speed < 10.001))
    return NormalSpeed;
  else if ((speed > 29.999) && (speed < 30.001))
    return FastSpeed;
  else if ((speed > 59.999) && (speed < 60.001))
    return UltraFastSpeed;
  else {
    throw BInternalException(__FILE__, __LINE__,
      tr("Invalid battle speed: '%1'").arg(speed));
  }
}

double Battle::speedFromBattleSpeeds(const BattleSpeeds battleSpeed) {
  switch (battleSpeed) {
  case RealtimeSpeed: 
    return 1;
  case SlowSpeed: 
    return 2;
  case NormalSpeed: 
    return 10;
  case FastSpeed: 
    return 30;
  case UltraFastSpeed: 
    return 60;
  default:
    return 1;
  }
}
*/

void Battle::postEvent(BEvent* event) {
  _events->postEvent(event);
  setModified(true);
}

void Battle::init() {
  bApp->checkProgressDialog(30, 
    QProgressDialog::tr("Parsing events..."));
  BEventsParser evParser(scenarioHeader()->eventsFileName(), this);
  evParser.parse();
//  evParser.postEvents();
  bApp->checkProgressDialog(60, 
      QProgressDialog::tr("Processing events..."));
  _events->processEvents(battleInfo().modelTime);
  _ships->removeUnusedShips();
  setModified(false);
}

void Battle::save() {
  _scenarioHeader->setEndDate(battleInfo().modelTime);
  scenarioHeader()->save(scenarioHeader()->fileName());
  seaMap()->save(scenarioHeader()->mapFileName());
  _events->save(scenarioHeader()->eventsFileName());
//  ships()->save(scenarioHeader()->shipsOnMapFileName());
  emit saveAI(scenarioHeader()->aiFileName());
  setModified(false);
}

void Battle::setModified(bool modified) {
  _modified = modified;
  if (modified == true) {
//    bool debug = true;
  }
}

bool Battle::isModified() const {
  return _modified;
}

void Battle::saveAs(const QString& fileName) {
  QString fn = bApp->extendFileName(fileName);
  QFileInfo fi(fn);
  QString ext = fi.extension();
  if (ext.isEmpty()) {
    fn = fn + ".xml";
    fi.setFile(fn);
  }
  QString baseName = fi.baseName(true);
  QString dirPath = fi.dirPath(true);
  QDir d(dirPath);
  if (d.exists(baseName)) {
    d.rmdir(baseName);
  }
  d.mkdir(baseName);
  _scenarioHeader->setFileName(fn);
  _scenarioHeader->setMapFileName(dirPath+"/"+baseName+"/"+
    BScenarioHeader::DEF_MAP_FILE_NAME);
  _scenarioHeader->setEventsFileName(dirPath+"/"+baseName+"/"+
    BScenarioHeader::DEF_EVENTS_FILE_NAME);
  _scenarioHeader->setAIFileName(dirPath+"/"+baseName+"/"+
    BScenarioHeader::DEF_AI_FILE_NAME);
//  _scenarioHeader->setShipsOnMapFileName(dirPath+"/"+baseName+"/"+
//    BScenarioHeader::DEF_SHIPS_ON_MAP_FILE_NAME);
  save();
}

BCollisions* Battle::collisionsDetector() const {
  throw BInternalException(__FILE__, __LINE__,
    QString("Unexpected call to Battle::collisionsDetector"));
}
