/******************************************************************************/
/* Battle -   BattleFleet              */
/******************************************************************************/

#ifndef Battle_H
#define Battle_H
static const char battle_h_id[]="@(#) $Id: battle.h,v 1.18 2004/10/18 15:02:05 Oleksiy Balabay Exp $"; static const char* battle_h_id_id=battle_h_id;
/** \file battle.h
  \brief  Battle  BattleFleet

     Battle  BattleFleet
*/
#include <qobject.h>
#include <qdatetime.h>
#include <qtimer.h>

class QCanvas;

class BException;
class BScenarioHeader;
class BShips;
class BSeaMap;
class BEvents;
class BEvent;
class BEventClear;
class BEventsListView;
class BEventGroupRemove;
class BEventGroupAdd;
class BattleObject;
class BCollisions;

enum BattleStates {
  NotStarted = 0,
  Running,
  Paused,
  Finished
};

enum BattleFlags {
  ///     ( )
  BFShips = 0x1,
  ///   
  BFSide = 0x2,
  ///   
  BFGrouping =  0x4,
  ///     
  BFEvent = 0x8
};

enum BattleSpeeds {
  RealtimeSpeed = 0,
  SlowSpeed = 1,
  NormalSpeed = 2,
  FastSpeed = 3,
  UltraFastSpeed = 4
};

struct BattleInfo {
  BattleStates state;
  double speed;
  BattleSpeeds speedIndex;
  QDateTime modelTime;
  unsigned int flags;
};

struct BattleLoadData {
  QString fileName;
  QDateTime startDate;
};

class Battle: public QObject {
  Q_OBJECT
public:
  Battle(const BattleLoadData& loadData);
  virtual ~Battle();
  BSeaMap* seaMap() const;
  const BattleInfo& battleInfo() const;
  BShips* ships() const;
  const BScenarioHeader* scenarioHeader() const;
//  static BattleSpeeds speedToBattleSpeeds(const double speed)
//    ;
//  static double speedFromBattleSpeeds(const BattleSpeeds battleSpeed);
  virtual void postEvent(BEvent* event);
  virtual void init();
  virtual void save();
  virtual void saveAs(const QString& fileName);
  void addBattleInfoFlags(BattleFlags flags, bool clear = false) {
    if (clear)
      _battleInfo.flags = flags;
    else
      _battleInfo.flags |= flags;
  }
  bool isModified() const;
  void setModified(bool modified);
  virtual BCollisions* collisionsDetector() const;
public slots:
  void slotPlay();
  void slotPause();
  void slotSetSpeed(const BattleSpeeds battleSpeed);
signals:
  void battleInfoChanged(const BattleInfo& info);
  void battleObjectRemoved(BattleObject* o);
  void battleObjectAdded(BattleObject* o);
protected:
  BEvents* _events;
  BScenarioHeader* _scenarioHeader;
protected:
  virtual void processCollisions() {};
private:
  void clearBattle();
  void loadShips();
  void loadSeaMap(); 
  void loadEvents(); 
  void startBattleTimer();
  void stopBattleTimer();

  void trigBattleObjectRemoved(BattleObject* o) {
    emit battleObjectRemoved(o);
  }
  void trigBattleObjectAdded(BattleObject* o) {
    emit battleObjectAdded(o);
  }
private slots:
  void timeoutSlot();
private:
  BattleLoadData _loadData;
  BShips* _ships;
  BSeaMap* _seaMap;
  BattleInfo _battleInfo;
  QTimer _battleTimer;
//  QTime _systemTime;
  bool _modified;
  friend class BEventClear;
  friend class BEventsListView;
  friend class BEventGroupRemove;
  friend class BEventGroupAdd;
signals:
  void saveAI(const QString& aiFileName);
};

#endif // Battle_H
