/******************************************************************************/
/* BattleGame -     (impl.)            */
/******************************************************************************/
static const char battlegame_cpp_id[]="@(#) $Id: battlegame.cpp,v 1.2 2005/01/05 11:47:22 Oleksiy Balabay Exp $"; static const char* battlegame_cpp_id_id=battlegame_cpp_id;

#include <algorithm>

#include "battlegame.h"
#include "bcollisions.h"
#include "beventcollision.h"
#include "battleedit.h"

/** \file battlegame.cpp
  \brief  BattleGame   (impl.)

     BattleGame  
*/

/** \class BattleGame
  \brief 


*/
/** \brief 
*/
BattleGame::BattleGame(const BattleLoadData& loadData)
:Battle(loadData){
  _collisions = new BCollisions(this, this, "collisions");
  _collisions->init();
}

/** \brief 
*/
BattleGame::~BattleGame(){

}

class new_collisions: public std::unary_function<BCollisions::Collision, void> {
public:
  new_collisions(const BCollisions::Collisions& oldCols,
    BCollisions::Collisions& newCols):_oldCols(oldCols), _newCols(newCols) {
  }
  ~new_collisions(){

  }
  void operator()(const BCollisions::Collision col) {
    typedef BCollisions::Collisions::const_iterator CIT;
    bool find = false;
    for(CIT it = _oldCols.begin(); it != _oldCols.end(); it++) {
      BattleObject* a = (*it).first;
      BattleObject* b = (*it).second;
      if ( ((a == col.first)&&(b == col.second))||
        ((a == col.second)&&(b == col.first)) ) {
        find = true;
      }
    }
    if (!find)
      _newCols.push_back(col);
  }
  const BCollisions::Collisions& newCols() const {
    return _newCols;
  }
private:
  const BCollisions::Collisions& _oldCols;
  BCollisions::Collisions& _newCols;
};

class new_event_collision: public std::unary_function<BCollisions::Collision, void> {
public:
  new_event_collision(BCollisions* cols, Battle* battle):_battle(battle),
  _cols(cols) {
  }
  void operator()(const BCollisions::Collision col) const {
    BEventCollision* e = new BEventCollision(col.first->name(),
      col.second->name(), _battle, _battle->battleInfo().modelTime);
    _cols->postEvent(e, false, false);
  }
private:
  Battle* _battle;
  BCollisions* _cols;

};

void BattleGame::processCollisions() {
  BCollisions::Collisions cols = _collisions->checkObjects(
    battleInfo().modelTime);
/*  for(BCollisions::Collisions::iterator debug = cols.begin(); 
  debug != cols.end(); debug++) {
    qDebug("Date: %s. Collision %1 with %2",
      (const char*)this->battleInfo().modelTime.toString().local8Bit(),
      (const char*)( (*debug).first->name()).local8Bit(),
      (const char*)( (*debug).second->name()).local8Bit());
  }
*/
 //        
  BCollisions::Collisions newCols;
  new_collisions nc(_currentCollisions, newCols);
  std::for_each(cols.begin(), cols.end(), nc);
  new_event_collision nec(_collisions, this);
  std::for_each(newCols.begin(), newCols.end(), nec);
  //   
  _currentCollisions = cols;
}

void BattleGame::customEvent(QCustomEvent* e) {
  if (e->type() == BattleEventNumber) {
    BattleEvent* be = (BattleEvent*)e;
    postEvent(be);
  }
  else {
    Battle::customEvent(e);
  }
}

void BattleGame::postEvent(BattleEvent* event) {
  _events->postEvent(event->event());
  addBattleInfoFlags(BattleFlags(0), true);
  _events->processEvents(battleInfo().modelTime);
  setModified(true);
  emit battleInfoChanged(battleInfo());
}
