/******************************************************************************/
/* BattleObject -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char battleobject_cpp_id[]="@(#) $Id: battleobject.cpp,v 1.17 2005/01/24 03:50:43 Oleksiy Balabay Exp $"; static const char* battleobject_cpp_id_id=battleobject_cpp_id;

#include <qobjectlist.h>

#include "battleobject.h"
#include "bexception.h"
#include "bprogram.h"

/** \file battleobject.cpp
  \brief  BattleObject  BattleFleet (impl.)

     BattleObject  BattleFleet
*/

const int BattleObject::OBJECT_TRACKS_MAX = 10;

BProgramTimeException::BProgramTimeException(const QString& sourceCodeFile, const int sourceCodeLine, 
    const BProgram* program, const QDateTime& objectProgramTime, 
    const QDateTime& newProgramTime, const BattleObject* o) :
  BException(sourceCodeFile, sourceCodeLine, 
    qApp->translate("BProgramTimeException",
    "Error settin program with time '%1' after program with time '%2' "
    "for object '%3'").arg(newProgramTime.toString()).arg(
    objectProgramTime.toString()).arg(o->name())), _program(program),
    _objectProgramTime(objectProgramTime),
    _newProgramTime(newProgramTime), _o(o){};

const QString BattleObject::BO_STATE = "State";
/** \class BattleObject
  \brief  

      BShip  BShipGroup   
  Composite.

     BProgram   Strategy.

   Memento     Xml  
    state()  setState().   
  Memento  .

       :
  <ul>
    <li> , .. ,    
            .  
      <ul>
        <li>name() - </li> 
        <li>vesselType() -  </li>
        <li>size() -  (  BattleFleet)</li> 
        <li>width() - .     , 
             </li>
        <li>minTurnRadius() -  .     , 
             </li>
        <li>maxSpeed() -  . </li>
        <li>maxAcceleration() -   . 
          /(*).    3     
          1/3 .      3  
           2/3 </li>
      </ul>
    </li>
    <li>      
       ,    ,
       setProgram() - programData()  kinematicData()</li> 
    <li>  Composite   - find(), isGroup(),
      objs(), parent(), insertChild(), removeChild()</li> 
    <li>image() -   .    side()</li> 
    <li>   . flagShip()  setFlagShip().
          .</li> 
    <li>     Xml - toXml(). 
           -    state()
        setState()</li>
  </ul>



*/
/** \brief 
*/
BattleObject::BattleObject(BattleObject* parent) {
  _maxAcceleration = 0;
  _maxSpeed = 0;
  _program = 0;
  _side = NO_SIDE;
  _minTurnRadius = 0;
  _width = 0;
  _length = 0;
  _size = 0;
  _vesselType = UndefinedType;
  _flagShip = 0;
  _parentControl = false;
  setParent(parent);
  _flags = 0;
}

/** \brief 
*/
BattleObject::~BattleObject() {
}

QString BattleObject::name() const {
  return _name;
}

int BattleObject::side() const {
  return _side;
}

void BattleObject::setSide(const int side) {
  _side = side;
}

double BattleObject::maxSpeed() const {
  return _maxSpeed;
}

double BattleObject::maxAcceleration() const {
  return _maxAcceleration;
}

BattleObjects BattleObject::objs(BSelectionTypes selType) const {
  if (selType == AllSelectionType) {
    BattleObjects bos;
    bos.insert(const_cast<BattleObject*>(this));
    return bos;
  }
  else if (selType == LeafsOnlySelectionType) {
    BattleObjects bos;
    bos.insert(const_cast<BattleObject*>(this));
    return bos;
  }
  else
    return BattleObjects();
}

BattleObject* BattleObject::parent() const {
  return _parent;
}

double BattleObject::width() const {
  return _width;
}

double BattleObject::length() const {
  return _length;
}

int BattleObject::size() const {
  return _size;
}

BattleObject::BVesselType BattleObject::vesselType() const {
  return _vesselType;
}

QImage BattleObject::image() const {
  return QImage::fromMimeSource("shipbase.png");
}

void BattleObject::setFlagShip(BattleObject* flagShip) {
  _flagShip = flagShip;
}

BattleObject* BattleObject::flagShip() const {
  return _flagShip;
}

void BattleObject::insertChild(BattleObject* /* obj */) {
  throw BInternalException(__FILE__, __LINE__,
    "BattleObject::insertChild call");
}

void BattleObject::removeChild(BattleObject* /* obj */) {
  throw BInternalException(__FILE__, __LINE__,
    "BattleObject::removeChild call");
}

void BattleObject::setParent(BattleObject* parent) {
  _parent = parent;
}

bool BattleObject::isGroup() const {
  return false;
}

BattleObject* BattleObject::find(const QString& objectName) const {
  if (objectName == name()) 
    return const_cast<BattleObject*>(this);
  else
    return 0;
}

double BattleObject::minTurnRadius() const {
  return _minTurnRadius;
}

double BattleObject::lossOfSpeedCoef(const int size) {
  if (size <= 5) {
    return 1/1.466;
  }
  else if (size <= 35) {
    return 1/1.34;
  }
  else if (size <= 65) {
    return 1/1.36;
  }
  else {
    return 1/1.33;
  }
}

double BattleObject::availableSpeed(const QDateTime& /* t */) const {
  return maxSpeed();
}

double BattleObject::availableTurnRadius(const QDateTime& /* t */) const {
  return minTurnRadius();
}

double BattleObject::availableAcceleration(const QDateTime& /* t */) const {
  return maxAcceleration();
}

QDomElement BattleObject::state(QDomDocument& doc) const {
  QDomElement e = doc.createElement(BO_STATE);
  return e;
}

void BattleObject::setState(QDomElement& /* element */) {
  _objectTracks.clear();
}

void BattleObject::setName(const QString& n) {
  _name = n;
}

bool BattleObject::parentControl() const {
  return _parentControl;
}

void BattleObject::setParentControl(bool isOn) {
  _parentControl = isOn;
}

const BObjectTracks& BattleObject::objectTracks() const {
  return _objectTracks;
}

BProgram* BattleObject::restOfProgram(const QDateTime& t1) const {
  if (_program == 0) 
    throw BInternalException(__FILE__, __LINE__, "Call to restOfProgram "
    "when program is 0");
  return _program->restOfProgram(t1);
}

void BattleObject::setFlags(unsigned int flags, unsigned int mask) {
  _flags = (_flags & (~mask)) | (flags & mask);
}

unsigned int BattleObject::flags() const {
  return _flags;
}
