/******************************************************************************/
/* BattleObject -   BattleFleet              */
/******************************************************************************/

#ifndef BattleObject_H
#define BattleObject_H
static const char battleobject_h_id[]="@(#) $Id: battleobject.h,v 1.16 2005/01/24 03:50:43 Oleksiy Balabay Exp $"; static const char* battleobject_h_id_id=battleobject_h_id;
/** \file battleobject.h
  \brief  BattleObject  BattleFleet

     BattleObject  BattleFleet
*/
#include <qstring.h>
#include <qimage.h>
#include <qptrlist.h>
#include <qdom.h>

#include <set>

#include "bkinematicdata.h"
#include "bexception.h"

class QFile;
class BProgram;
class BattleObject;
class QDateTime;

const int NO_SIDE = -1;

typedef std::set<BattleObject*> BattleObjects;

class BProgramTimeException: public BException {
public:
  BProgramTimeException(const QString& sourceCodeFile, const int sourceCodeLine, 
    const BProgram* program, const QDateTime& objectProgramTime, 
    const QDateTime& newProgramTime, const BattleObject* o);
  const BProgram* program() const { return _program; };
  const QDateTime objectProgramTime() const { return _objectProgramTime; }
  const QDateTime newProgramTime() const { return _newProgramTime; }
  const BattleObject* battleObject() const { return _o; }
  virtual ~BProgramTimeException() {}
private:
  const BProgram* _program;
  const QDateTime& _objectProgramTime;
  const QDateTime& _newProgramTime;
  const BattleObject* _o;
};

class BattleObject {
  friend class BShipGroup;
public:
  enum BVesselType {
    BattleshipType = 0,
    CoastDefenseBattleshipType = 1,
    ArmoredCruiserType = 2,
    ProtectedCruiserType = 3,
    UnprotectedCruiserType = 4,
    GunBoatType = 5,
    DestroyerType = 6,
    TorpedoBoatType = 7,
    UndefinedType = 8
  };
  enum BSelectionTypes {
    TreeSelectionType = 0,
    AllSelectionType,
    LeafsOnlySelectionType
  };
  enum Flags {
    IsLineAhead = 0x1
  };
public:
  BattleObject(BattleObject* parent);
  virtual ~BattleObject();
  QString name() const;
  int side() const;
  virtual void setSide(const int side);
  virtual BKinematicData kinematicData(const QDateTime& t) const = 0;
  virtual BProgramData programData(const QDateTime& t) const = 0;
  double maxSpeed() const;
  double maxAcceleration() const;
  virtual void setProgram(BProgram* program, const QDateTime& t) = 0;
  virtual BProgram* restOfProgram(const QDateTime& t1) const;
  virtual void parse(const QDomElement& element) = 0;
  virtual BattleObjects objs(BSelectionTypes selType = TreeSelectionType) const;
  BattleObject* parent() const;
  double width() const;
  double length() const;
  int size() const;
  BVesselType vesselType() const;
  virtual QImage image() const;
  virtual void setFlagShip(BattleObject* flagShip);
  BattleObject* flagShip() const;
  virtual void insertChild(BattleObject* obj);
  virtual void removeChild(BattleObject* obj);
  virtual bool isGroup() const;
  virtual BattleObject* find(const QString& objectName) const;
  double minTurnRadius() const;
  virtual QDomElement toXml(QDomDocument& doc, 
    bool elementOnly = false) const = 0;
  virtual QDomElement state(QDomDocument& doc) const;
  virtual void setState(QDomElement& element);
  virtual void setName(const QString& n);
  virtual bool parentControl() const;
  virtual void setParentControl(bool isOn);
  static const QString BO_STATE;
  static double lossOfSpeedCoef(const int size);
  virtual double availableSpeed(const QDateTime& t) const;
  virtual double availableTurnRadius(const QDateTime& t) const;
  virtual double availableAcceleration(const QDateTime& t) const;
  virtual const BObjectTracks& objectTracks() const;
  virtual void setFlags(unsigned int flags, unsigned int mask);
  virtual unsigned int flags() const;
protected:
  void setParent(BattleObject* obj);
protected:
  const BProgram* _program;
  QString _name;
  int _side;
  double _minTurnRadius;
  double _maxSpeed;
  double _maxAcceleration;
  double _width;
  double _length;
  BVesselType _vesselType;
  int _size;
  BattleObject* _flagShip;
  BattleObject* _parent;
  bool _parentControl;
  BObjectTracks _objectTracks;
  static const int OBJECT_TRACKS_MAX;
private:
  unsigned int _flags;
};

#endif // BattleObject_H
