/******************************************************************************/
/* BCanvasConnection -     (impl.)            */
/******************************************************************************/
static const char bcanvasconnection_cpp_id[]="@(#) $Id: bcanvasconnection.cpp,v 1.5 2005/01/24 03:50:43 Oleksiy Balabay Exp $"; static const char* bcanvasconnection_cpp_id_id=bcanvasconnection_cpp_id;

#include <qpainter.h>
#include <algorithm>
#include <math.h>
#include <qwmatrix.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qapplication.h>

#include "bcanvasconnection.h"
#include "baichasing.h"
#include "bcentral.h"

/** \file bcanvasconnection.cpp
  \brief  BCanvasConnection   (impl.)

     BCanvasConnection  
*/
const int Margin = 3;
const int CONNECTION_LAYER = 4000;

void drawActiveHandle(QPainter &painter, const QPoint &center)
{
  painter.save();
    painter.setPen(Qt::black);
    painter.setBrush(Qt::cyan);
    painter.drawRect(center.x() - Margin, center.y() - Margin,
                     2 * Margin + 1, 2 * Margin + 1);
  painter.restore();
}

/** \class BCanvasConnection
  \brief 


*/
BCanvasConnection::BCanvasConnection(QCanvasItem* from, QCanvasItem* to,
  QCanvas *canvas): QCanvasLine(canvas), _from(from), _to(to) {
  setPoints((int)from->x(), (int)from->y(), (int)to->x(), (int)to->y());
  setPen(Qt::black);
  setBrush(Qt::black);
  setZ(CONNECTION_LAYER);
}

/** \brief 
*/
BCanvasConnection::~BCanvasConnection(){
  hide();
}

void BCanvasConnection::drawShape(QPainter &painter)
{
    const int Extra = Margin;
    QCanvasLine::drawShape(painter);
    int x1 = startPoint().x();
    int y1 = startPoint().y();
    int x2 = endPoint().x();
    int y2 = endPoint().y();

    double l = sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
    QWMatrix m1(1,0,0,1,x2,y2);
    double cosa = (x2-x1)/l;
    double sina = (y2-y1)/l;
    QWMatrix m2(cosa, sina,
                -sina,cosa,0,0);
    QWMatrix m3 = (m2*m1);
    QPointArray pa(3);
    pa[0] = m3.map(QPoint(0, 0));
    pa[1] = m3.map(QPoint(-2*Extra, +Extra));
    pa[2] = m3.map(QPoint(-2*Extra, -Extra));
    painter.drawPolygon(pa);
    if (isSelected()) {
        drawActiveHandle(painter, startPoint());
        drawActiveHandle(painter, endPoint());
    }
}

QPointArray BCanvasConnection::areaPoints() const
{
    const int Extra = Margin + 1;
    QPointArray points(6);
    QPoint pointA = startPoint();
    QPoint pointB = endPoint();

    if (pointA.x() > pointB.x())
      std::swap(pointA, pointB);

    points[0] = pointA + QPoint(-Extra, -Extra);
    points[1] = pointA + QPoint(-Extra, +Extra);
    points[3] = pointB + QPoint(+Extra, +Extra);
    points[4] = pointB + QPoint(+Extra, -Extra);
    if (pointA.y() > pointB.y()) {
        points[2] = pointA + QPoint(+Extra, +Extra);
        points[5] = pointB + QPoint(-Extra, -Extra);
    } else {
        points[2] = pointB + QPoint(-Extra, +Extra);
        points[5] = pointA + QPoint(+Extra, -Extra);
    }
    return points;
}

void BCanvasConnection::updatePoints() {
  setPoints((int)_from->x(), (int)_from->y(), (int)_to->x(), (int)_to->y());
}

void BCanvasConnection::popup(const QPoint /*p*/) {
}

BCanvasChasingConnection::~BCanvasChasingConnection(){
  hide();
}

BCanvasChasingConnection::BCanvasChasingConnection(BCentral* central,
  BAIChasing* ai, BCanvasShip* from, BCanvasShip* to, QCanvas *canvas)
  :BCanvasConnection( from, to, canvas), _ai(ai), _central(central) {
  updateMode();
}

void BCanvasChasingConnection::popup(const QPoint /*p*/) {
  QPopupMenu menu(_central);
  int id = menu.insertItem(qApp->translate("BCanvasChasingConnection", 
      "Auto Control"));
  menu.setItemChecked(id, _ai->controlMode() == BAI::ControlModeAuto);
  int rez = menu.exec(QCursor::pos());
  if (rez == id) {
    BAI::ControlModes cm = _ai->controlMode()==BAI::ControlModeAuto?
      BAI::ControlModeManual:BAI::ControlModeAuto;
    if (isSelected()) {
      std::vector<BAI*> ais = _central->selectedConnections();
      for(std::vector<BAI*>::iterator it = ais.begin(); it != ais.end(); it++) {
        BAIChasing* cai = dynamic_cast<BAIChasing*>(*it);
        if (cai) {
          cai->setControlMode(cm);
        }
      }
    }
    else {
      _ai->setControlMode(cm);
    }
  }
}

void BCanvasChasingConnection::updateMode() {
  QPen p = pen();
  p.setColor((_ai->controlMode() == BAI::ControlModeAuto)?Qt::black:Qt::gray);
  setPen(p);
  QBrush b = brush();
  b.setColor(p.color());
  setBrush(b);
}
