/******************************************************************************/
/* BCanvasShip -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bcanvasship_cpp_id[]="@(#) $Id: bcanvasship.cpp,v 1.16 2005/01/21 16:25:02 Oleksiy Balabay Exp $"; static const char* bcanvasship_cpp_id_id=bcanvasship_cpp_id;

#include <qpixmap.h>
#include <qpainter.h>
#include <qimage.h>
#include <qbitmap.h>

#include <math.h>

#include "bcanvasship.h"
#include "bship.h"
#include "bseamap.h"
#include "battle.h"
#include "bunitsconsts.h"

const int SHIP_LAYER = 1000;

const QColor SIDE_ONE_COLOR = QColor(255,0,0);
const QColor SIDE_TWO_COLOR = QColor(0,0,0);
const QColor NO_SIDE_COLOR = QColor(0,0,255);
const int SIDE = 6;

/** \file bcanvasship.cpp
  \brief  BCanvasShip  BattleFleet (impl.)

     BCanvasShip  BattleFleet
*/

int BCanvasShip::_z = SHIP_LAYER;
/** \class BCanvasShip
  \brief 


*/
/** \brief 
*/
BCanvasShip::BCanvasShip(const BCoordTransformer* coordTransformer,
  BShip* ship, const Battle* battle, QCanvas* canvas)
  :QCanvasEllipse(canvas), _ship(ship), _battle(battle),
  _coordTransformer(coordTransformer) {
  if (!ship) {
    throw BPtrException(__FILE__, __LINE__, "ship", "BShip");
  }
  if (!battle) {
    throw BPtrException(__FILE__, __LINE__, "battle", "Battle");
  }
  w1 = _coordTransformer->sizeFromMap(ship->length());
  h1 = _coordTransformer->sizeFromMap(ship->width());
  if (w1 < 7) {
    w1 = 7;
  }
  if (h1 < 5) {
    h1 = 5;
  }
  preparePixmap();
  setZ(_z++);
  setAnimated(true);
  _side = _ship->side();
}

/** \brief 
*/
BCanvasShip::~BCanvasShip(){
  hide();
}

void BCanvasShip::drawShape(QPainter& p) {
//  qDebug("Draw %d %d Time %s", x(), y(),
//    (const char*)_battle->battleInfo().modelTime.toString("hh:mm:ss.zzz").
//    local8Bit());

  p.save();
  int x1 = (int)x();
  int y1 = (int)y();
  p.translate(x(), y());
  p.rotate(_angle);
//  qDebug("A:%d", _angle);
  x1 = -_pm.width();
  y1 = -_pm.height();
  x1 = x1/2;
  y1 = y1/2;
//  if (0 == x1) 
//    x1 = -1;
  if (0 == y1)
    y1 = -1;
  p.drawPixmap(x1, y1, _pm);
  p.restore();
  if (isActive()) {
    p.setBrush(QColor(Qt::green));
    p.setPen(Qt::black);
    drawHandle(p, QPoint((int)x(), (int)y()));
//    QCanvasEllipse::drawShape(p);
  }
  else if (isSelected()) {
    p.setBrush(QColor(Qt::cyan));
    p.setPen(Qt::black);
    drawHandle(p, QPoint((int)x(), (int)y()));
//    QCanvasEllipse::drawShape(p);
  }
}

void BCanvasShip::setAngle(const double angle) {
  _angle = angle;
}

void BCanvasShip::advance(int phase) {
  if (1 == phase) {
    QCanvasEllipse::advance(phase);
    return;
  }
  Q_ASSERT(_advanceDateTime.isValid());
  BKinematicData kd = _ship->kinematicData(_advanceDateTime);
  QPoint pnt = _coordTransformer->pointFromMap(BMapPoint(kd.x(), kd.y()));
  setVelocity(pnt.x() - x(), pnt.y() - y());
  setAngle(BUnitsConverter::angleToDegrees(kd.course()));
  _advanceDateTime = QDateTime();//invalidate
  if (_side != _ship->side()) {
    preparePixmap();
    _side = _ship->side();
    canvas()->setChanged(boundingRect());
  }
  if (pnt.x() == x() && pnt.y() == y()) {
    canvas()->setChanged(boundingRect());
  }
}

BShip* BCanvasShip::ship() const {
  return _ship;
}

void BCanvasShip::setAdvanceDateTime(const QDateTime& dt) {
  _advanceDateTime = dt;
}

void BCanvasShip::preparePixmap() {
  if ((w1 == 7) && (h1 == 5)) {
    _pm = QPixmap(h1, w1);
    _pm.fill(white);
    QPainter pmp(&_pm);
    pmp.setPen(NoPen);
    pmp.setBrush( (_ship->side()==1)?SIDE_ONE_COLOR:
      (_ship->side()==2?SIDE_TWO_COLOR:NO_SIDE_COLOR ));
    QPointArray pa(5);
    pa.setPoint(0, QPoint(h1/2, 0));
    pa.setPoint(1, QPoint(h1-1, w1/2));
    pa.setPoint(2, QPoint(h1-1, w1-1));
    pa.setPoint(3, QPoint(0, w1-1));
    pa.setPoint(4, QPoint(0, w1/2));
    pmp.drawPolygon(pa);
    _pm.setMask(_pm.createHeuristicMask());
  }
  else {
    _pm = QPixmap(_ship->image().smoothScale( h1, w1, QImage::ScaleFree));
    _pm.setMask(_pm.createHeuristicMask());
  }
  int d = (int)sqrt(w1*w1+h1*h1)+1;
  if ( (d & 1) == 0 )
    d++;
  setSize(d, d);
}

void BCanvasShip::drawHandle(QPainter &painter, const QPoint &center) {
  int r = width()/2;
  int h = SIDE/2;
  int dx = (int)sqrt(r*r-h*h);
  if (dx%2 == 1) dx-=1;
  QPoint c = center + QPoint(-dx, -h);
  painter.drawRect(c.x(), c.y(), SIDE, SIDE);
  c = center + QPoint(dx-2*h, -h);
  painter.drawRect(c.x(), c.y(), SIDE, SIDE);
  c = center + QPoint(-h, -dx);
  painter.drawRect(c.x(), c.y(), SIDE, SIDE);
  c = center + QPoint(-h, dx-2*h);
  painter.drawRect(c.x(), c.y(), SIDE, SIDE);
}

