/******************************************************************************/
/* BCentral -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bcentral_cpp_id[]="@(#) $Id: bcentral.cpp,v 1.28 2005/02/15 16:43:34 Oleksiy Balabay Exp $"; static const char* bcentral_cpp_id_id=bcentral_cpp_id;

#include "bcentral.h"
#ifdef Q_WS_WIN
#pragma warning(disable: 4786)
#endif
#include <math.h>
#include <algorithm>
#include <qwhatsthis.h>

#include "bapplication.h"
#include "bscenarioheader.h"
#include "bcanvas.h"
#include "bseamap.h"
#include "battle.h"
#include "buiutils.h"
#include "bscales.h"
#include "bcanvasship.h"
#include "bship.h"
#include "bships.h"
#include "bobservercentral.h"
#include "boceditstate.h"
#include "baichasing.h"
#include "bcanvasconnection.h"
#include "bcentraltooltip.h"

const int MARGINE_TOP = 0;
const int MARGINE_LEFT = 0;
const int MARGINE_RIGHT = 0;
const int MARGINE_BOTTOM = 0;
const double ONE_SECOND = 1/(60.0*60);
const int SELECTION_LAYER = 3000;
const int MAX_CONTROLLED = 2000;

/** \file bcentral.cpp
  \brief  BCentral  BattleFleet (impl.)

     BCentral  BattleFleet
*/

/** \class BCentral
  \brief 


*/
/** \brief 
*/
BCentral::BCentral(BObserverCentral* observer, 
  QWidget* parent, const char* name)
  :QCanvasView(parent, name), _coordTransformer(0), _battle(0), 
  _canvasShips(new BCanvasShips()), _observer(observer) {
  setVScrollBarMode(QScrollView::AlwaysOff);
  setHScrollBarMode(QScrollView::AlwaysOff);
  setResizePolicy(QScrollView::AutoOneFit);
//  setFocusPolicy(StrongFocus);
  setFocusProxy(parent);
//  viewport()->setFocusPolicy(StrongFocus);
  viewport()->setFocusProxy(parent);
  viewport()->setMouseTracking(true);
  
  _activeCanvasShip = 0;

  _showCCCs = false;
//  _activeShip = 0;
  _toolTip = new BCentralToolTip(this);
  QWhatsThis::add(this, tr("Battle map. Displays scales and vessels"));
}

/** \brief 
*/
BCentral::~BCentral(){
//  delete _canvasShips;
  if (canvas()) {
    delete canvas();
  }
  delete _toolTip;
//  if (_coordTransformer)
//    delete _coordTransformer;
}

void BCentral::contentsMousePressEvent(QMouseEvent* e) {
  if (!canvas())
    return;
  if (!_battle)
    return;
  _observer->currentState(e)->contentsMousePressEvent(e);
}

void BCentral::contentsMouseReleaseEvent(QMouseEvent* e) {
  if (!canvas())
    return;
  if (!_battle)
    return;
  _observer->currentState(e)->contentsMouseReleaseEvent(e);
}

void BCentral::contentsMouseMoveEvent(QMouseEvent* e) {
  if (!canvas())
    return;
  if (!_battle)
    return;
  _observer->currentState(e)->contentsMouseMoveEvent(e);
}

void BCentral::battleChanged(Battle* battle) {
  _battle = battle;
  if (_battle) {
    showNewCanvas(_battle->seaMap()->originalCoordinates());
  }
  else {
    deleteCanvas();
    setActiveObject(0);
  }
}

/*template<class T, class V>
class ptr_0_second: public std::unary_function<std::pair<T*,V*>, void> {
public:
  void operator()(std::pair<T*,V*>& p) {
    if (p.second != 0) {
      delete p.second;
      p.second = 0;
    }
  }
};
*/

void BCentral::deleteCanvas() {
//  QCanvas* c = canvas();
//  setCanvas(0);
//  if (c != 0)
//    delete c;
  _activeCanvasShip = 0;
  //     Pursuior-Evader,   
  //   AIs
//  std::for_each(_cCCs.begin(), _cCCs.end(), ptr_0_second<BAIChasing, 
//    BCanvasChasingConnection>());
  for(BCCCs::iterator it = _cCCs.begin(); it != _cCCs.end(); it++) {
    if (it->second != 0) {
      delete it->second;
      it->second = 0;
    }
  }
  _canvasShips->clear();

  QSize sz = viewport()->size();
  if (canvas()) {
    QCanvasItemList list = canvas()->allItems();
    QCanvasItemList::Iterator it = list.begin();
    for (; it != list.end(); ++it) {
  	  if ( *it )
	      delete *it;
    }
    if (canvas()->size() != sz) {
      canvas()->resize(sz.width(), sz.height());
    }
  }
  else {
    setCanvas(new BCanvas(sz.width(), sz.height()));
  }
};

void BCentral::addBattleObject(BattleObject* bo, const QDateTime& modelTime) {
  BKinematicData kd = bo->kinematicData(modelTime);
  BCanvasShip* ci = 0;
  if (dynamic_cast<BShip*>(bo) != 0) {
   ci = new BCanvasShip(_coordTransformer.get(),  dynamic_cast<BShip*>(bo), _battle, canvas());
  }
  if (ci == 0) {
    throw BInternalException(__FILE__, __LINE__,
      "BCentral::addBattleObject call for non-ship");
  }
  QPoint pnt = 
    _coordTransformer->pointFromMap(BMapPoint(kd.x(), kd.y()));
  ci->setX(pnt.x());
  ci->setY(pnt.y());
  ci->setAngle(BUnitsConverter::angleToDegrees(kd.course()));

  if (_observer->activeObject() == bo) {
    _activeCanvasShip = ci;
    ci->setActive(true);
  }
  if (_observer->selectedObjects().find(bo) != 
    _observer->selectedObjects().end()) {
    ci->setSelected(true);
  }

  if ((abs((int)ci->x()) < MAX_CONTROLLED) && (abs((int)ci->y()) < MAX_CONTROLLED)) {
    ci->show();
  }
  _canvasShips->push_back(ci);
}

class update_cccs: public std::unary_function<std::pair<BAIChasing*, 
  BCanvasChasingConnection*>, void> {
public:
  update_cccs(BCentral* central, bool isVisible)
    :_central(central), _isVisible(isVisible) {
  };
  void operator()(std::pair<BAIChasing*const, BCanvasChasingConnection*>& p) {
    BCanvasChasingConnection* ccc = p.second;
    if (ccc == 0) {
      ccc = p.second = _central->createCCC(p.first);
    }
//    else {
    if (ccc->from()->isVisible() && ccc->to()->isVisible()) {
      ccc->updatePoints();
      if (_isVisible)
        ccc->show();
      else
        ccc->hide();
    }
    else {
      ccc->hide();
      ccc->updatePoints();
    }
//    }
  }
private:
  BCentral* _central;
  bool _isVisible;
};

void BCentral::showNewCanvas(const BCoordinates& mapCo) {
  //  ...
  deleteCanvas();
  if (0 == _battle) {
    return;
  }
  //  ...
  QSize sz = canvas()->size();
//  setCanvas( new BCanvas(sz.width()-2,sz.height()-2));
  BSeaMap* seaMap = _battle->seaMap();
  BCoordinates canvasCo(MARGINE_LEFT, MARGINE_TOP,
    sz.width()-MARGINE_LEFT-MARGINE_RIGHT,
    sz.height()-MARGINE_TOP-MARGINE_BOTTOM);
//  if (_coordTransformer) {
//    delete _coordTransformer;
//  }
  _coordTransformer = std::auto_ptr<BCoordTransformer>(
    seaMap->createTransformer(mapCo, canvasCo, canvas()->size()) );
  canvas()->setBackgroundPixmap( seaMap->background(*_coordTransformer));
  typedef BShips::iterator SIT;
  BShips* ships = _battle->ships();
  QDateTime mt = _battle->battleInfo().modelTime;
  for (SIT it = ships->begin(); it != ships->end(); it++) {
    BShip* ship = it.data();
    addBattleObject(ship, mt);
  }
  updateChasingConnections();
//  QCanvasRectangle* cr = new QCanvasRectangle(0,0,sz.width(),sz.height(),
//    canvas());
//  cr->setPen(QPen(QColor("red"), 2));
//  cr->show();
//  cr = new QCanvasRectangle(MARGINE_LEFT, MARGINE_TOP,
//    sz.width()-MARGINE_LEFT-MARGINE_RIGHT,
//    sz.height()-MARGINE_TOP-MARGINE_BOTTOM, canvas());
//  cr->setPen(QPen(QColor("green"), 1));
//  cr->show();
//  QRect r;
  const BCoordinates cc = _coordTransformer->currentCoordinates();
//  BMapPoint bl(cc.x0(), cc.y0());
//  r.setBottomLeft(_coordTransformer->pointFromMap(bl));
//  BMapPoint tr(cc.x0() + cc.w(), cc.y0() + cc.h());
//  r.setTopRight(_coordTransformer->pointFromMap(tr));
//  cr = new QCanvasRectangle(r, canvas());
//  cr->setPen(QPen(QColor("blue"), 1));
//  cr->setZ(60);
//  cr->show();
  BScales scales(_coordTransformer.get(), canvas());
  scales.drawLines();
  scales.drawScale();

  //     
  int hScrollSize = scrollBarSize(Horizontal);
  int hScrollPos = mapToScrollBar(Horizontal, cc.x0());
//  double testX = mapFromScrollBar(Horizontal, hScrollPos);
  int vScrollSize = scrollBarSize(Vertical);
  int vScrollPos = mapToScrollBar(Vertical, cc.y0());
//  double testY = mapFromScrollBar(Vertical, vScrollPos);

  emit visiblePart(Horizontal, hScrollPos, hScrollSize);
  emit visiblePart(Vertical, vScrollPos, vScrollSize);

}

void BCentral::viewportResizeEvent(QResizeEvent* /* e */) {
//  if (_resizeTimer.isActive())
//    return;
//  else
//    _resizeTimer.singleShot(1000, this, SLOT(slotResizeTimer()));
  if (0 == _battle) {
    deleteCanvas();
    return;
  }
  if (_coordTransformer.get() != 0) {
    showNewCanvas(_coordTransformer->currentCoordinates());
  }
  else {
    showNewCanvas(_battle->seaMap()->originalCoordinates());
  }
}

void BCentral::zoomIn() {
  const BCoordinates cc = _coordTransformer->currentCoordinates();
  BCoordinates newCo(cc.x0()+cc.w()/4, cc.y0()+cc.h()/4, cc.w()/2, cc.h()/2);
  showNewCanvas(newCo);
}

void BCentral::zoomOut() {
  const BCoordinates cc = _coordTransformer->currentCoordinates();
  BCoordinates newCo(cc.x0()-cc.w()/2, cc.y0()-cc.h()/2,
    cc.w()*2, cc.h()*2);
  showNewCanvas(newCo);
}

void BCentral::fullView() {
  showNewCanvas(_coordTransformer->originalCoordinates());
}

void BCentral::contentsMovedSlot(Orientation oR, int pos) {
  const BCoordinates cc = _coordTransformer->currentCoordinates();
  if (Horizontal == oR) {
    double x0 = mapFromScrollBar(oR, pos);
//    if (fabs(x0 - cc->x0()) < ONE_SECOND*60)
//      return;
    BCoordinates newCo(x0, cc.y0(), cc.w(), cc.h());
    showNewCanvas(newCo);
  }
  else {
    double y0 = mapFromScrollBar(oR, pos);
//    if (fabs(y0 - cc->y0()) < ONE_SECOND*60)
//      return;
    BCoordinates newCo(cc.x0(), y0, cc.w(), cc.h());
    showNewCanvas(newCo);
  }
}

int BCentral::scrollBarSize(const Orientation oR) const {
  const BCoordinates cc = _coordTransformer->currentCoordinates();
  const BCoordinates oc = _coordTransformer->originalCoordinates();
  if (Horizontal == oR) {
    double minX = QMIN(cc.x0(), oc.x0());
    double maxX = QMAX(cc.x0()+cc.w(), oc.x0() + oc.w());
    int hScrollSize = int(canvas()->width()*(maxX-minX)/cc.w());
    return hScrollSize;
  }
  else {
    double minY = QMIN(cc.y0(), oc.y0());
    double maxY = QMAX(cc.y0()+cc.h(), oc.y0() + oc.h());
    int vScrollSize = int(canvas()->height()*(maxY-minY)/cc.h());
    return vScrollSize;
  }
  
}

int BCentral::mapToScrollBar(const Orientation oR, 
  const double /* coordinate */) const {
  int size = scrollBarSize(oR);
  const BCoordinates cc = _coordTransformer->currentCoordinates();
  const BCoordinates oc = _coordTransformer->originalCoordinates();
  if (Horizontal == oR) {
    double minX = QMIN(cc.x0(), oc.x0());
    double maxX = QMAX(cc.x0()+cc.w(), oc.x0() + oc.w());
    return int ( size*(cc.x0() - minX)/(maxX-minX));
  }
  else {
    double minY = QMIN(cc.y0(), oc.y0());
    double maxY = QMAX(cc.y0()+cc.h(), oc.y0() + oc.h());
//    return int ( size*(cc->y0() - minY)/(maxY-minY));
    return int ( size*(maxY - (cc.y0()+cc.h()))/(maxY-minY));
  }
}

double BCentral::mapFromScrollBar(const Orientation oR, 
  const int coordinate) const {
  int size = scrollBarSize(oR);
  const BCoordinates cc = _coordTransformer->currentCoordinates();
  const BCoordinates oc = _coordTransformer->originalCoordinates();
  if (Horizontal == oR) {
    double minX = QMIN(cc.x0(), oc.x0());
    double maxX = QMAX(cc.x0()+cc.w(), oc.x0() + oc.w());
    double x0 = coordinate*(maxX - minX)/size+minX;
    return x0;
  }
  else {
    double minY = QMIN(cc.y0(), oc.y0());
    double maxY = QMAX(cc.y0()+cc.h(), oc.y0() + oc.h());
//    double y0 = coordinate*(maxY - minY)/size+minY;
    double y0 = maxY - coordinate*(maxY - minY)/size - cc.h();
    return y0;
  }
}


void BCentral::slotBattleInfoChanged(const BattleInfo& battleInfo) {
  typedef BCanvasShips::iterator CSIT;
  QDateTime mt = battleInfo.modelTime;
  for (CSIT it = _canvasShips->begin(); it != _canvasShips->end(); it++) {
    (*it)->setAdvanceDateTime(mt);
    BShip* ship = (*it)->ship();
    BKinematicData kd = ship->kinematicData(mt);
    QPoint pnt = 
      _coordTransformer->pointFromMap(BMapPoint(kd.x(), kd.y()));
    if (!((abs(pnt.x()) < MAX_CONTROLLED) && (abs(pnt.y()) < MAX_CONTROLLED))) {
      (*it)->hide();
    }
  }
  updateChasingConnections();

  canvas()->advance();
  bool shouldUpdate = false;
  for (CSIT it1 = _canvasShips->begin(); it1 != _canvasShips->end(); it1++) {
    (*it1)->setAdvanceDateTime(mt);
    QPoint pnt( (int)(*it1)->x(), (int)(*it1)->y() );
    if ((abs(pnt.x()) < MAX_CONTROLLED) && (abs(pnt.y()) < MAX_CONTROLLED)) {
      (*it1)->show();
      shouldUpdate = true;
    }
  }
  if (shouldUpdate) {
    updateChasingConnections();
    canvas()->advance();
  }
}

void BCentral::setActiveObject(BattleObject* o) {
  if (0 == o) {
    if (_activeCanvasShip)
      _activeCanvasShip->setActive(false);
    _activeCanvasShip = 0;
    canvas()->update();
    return;
  }
  BCanvasShip* newO = findObject(o);
  
  if (_activeCanvasShip)
    _activeCanvasShip->setActive(false);
  if (newO)
    newO->setActive(true);
  _activeCanvasShip = newO;
  canvas()->update();

}

BCanvasShip* BCentral::findObject(BattleObject* bo) {
  if (bo && bo->isGroup())
    return 0;
  for(BCanvasShips::iterator it = _canvasShips->begin(); 
    it != _canvasShips->end(); it++) {
    if ((*it)->ship() == bo) {
      return *it;
    }
  }
  throw BInternalException(__FILE__, __LINE__,
    tr("Error finding canvas item for battle object '%1'").arg(bo->name()));
}

void BCentral::setSelectedObjects(const BattleObjects& objects) {
  for(BCanvasShips::iterator it = _canvasShips->begin(); 
  it != _canvasShips->end(); it++) {
    (*it)->setSelected(objects.find((*it)->ship()) != objects.end());
  }
  canvas()->update();
}

void BCentral::slotShowChasingConnections(bool visible) {
  if (visible == _showCCCs)
    return;
  _showCCCs = visible;
  updateChasingConnections();

  canvas()->update();
}

void BCentral::slotChasingConnectionChanged(
  BAIChasing* ai, const bool *add, const bool *isAuto) {
  if (add != 0) {
    if (*add) {
      BCCCs::iterator it = _cCCs.find(ai);
      if (it != _cCCs.end()) {
        delete it->second;
        it->second = createCCC(ai);
      }
      else {
        _cCCs.insert(
          std::pair<BAIChasing*, BCanvasChasingConnection*>(ai,createCCC(ai)));
      }
    }
    else {
      BCCCs::iterator it = _cCCs.find(ai);
      if (it != _cCCs.end()) {
        delete it->second;
        _cCCs.erase(it);
      }
    }
  }
  if (isAuto) {
    BCCCs::iterator it = _cCCs.find(ai);
    it->second->updateMode();
  }
  updateChasingConnections();

  canvas()->update();
}

class is_obj: public std::unary_function<BCanvasShip*, bool> {
public:
	is_obj(BattleObject* obj):_obj(obj){};
  bool operator()(BCanvasShip* shp) const {
	if (shp == 0)
	  return false;
	else 
    return (shp->ship() == _obj);
  }
private:
  BattleObject* _obj;
};

///
BCanvasChasingConnection* BCentral::createCCC(BAIChasing* ai) {
  BCanvasChasingConnection * cCC = 0;
  //   CanvasShip
  BCanvasShips::iterator it = std::find_if(_canvasShips->begin(),
    _canvasShips->end(), is_obj(ai->object()));
  BCanvasShip* from, *to;
  from = to = 0;
  if (it != _canvasShips->end())
    from = *it;
  it = std::find_if(_canvasShips->begin(), _canvasShips->end(), 
    is_obj(ai->evader()));
  if (it != _canvasShips->end())
    to = *it;
  if (from && to) {
     cCC = new BCanvasChasingConnection(this, ai, from, to, canvas());
  }
  else {
    throw BInternalException(__FILE__, __LINE__,
      tr("Not found canvas ship"));
  }
  return cCC;
}

void BCentral::deselectAllConnections() {
  QCanvasItemList l = canvas()->allItems();
  bool f = false;
  for (QCanvasItemList::iterator it = l.begin(); it != l.end(); it++) {
    BCanvasConnection* cc = dynamic_cast<BCanvasConnection*>(*it);
    if (cc) {
      f = true;
      cc->setSelected(false);
    }
  }
  if (f)
    canvas()->update();
}

std::vector<BAI*> BCentral::selectedConnections() const {
  std::vector<BAI*> rez;
  for(BCCCs::const_iterator it = _cCCs.begin(); it != _cCCs.end(); it++) {
    if (it->second->isSelected())
      rez.push_back(it->first);
  }
  return rez;
}

BAI* BCentral::connection(BCanvasConnection* /*cc*/) const {
  for(BCCCs::const_iterator it = _cCCs.begin(); it != _cCCs.end(); it++) {
    if (it->second->isSelected())
      return (it->first);
  }
  throw BInternalException(__FILE__, __LINE__, "No AI for connection");
}

class canvas_ship_eq: public std::unary_function<BCanvasShip*, bool> {
public:
  canvas_ship_eq(BattleObject* o):_o(o){};
  bool operator()(BCanvasShip* s) const {
    return s->ship() == _o;
  }
private:
  BattleObject* _o;
};

///    BCentralEdit
void BCentral::removeBattleObject(BattleObject* bo) {
  BattleObjects bos = bo->objs(BattleObject::LeafsOnlySelectionType);
  for(BattleObjects::iterator it = bos.begin(); it != bos.end(); it++) {
    BCanvasShips::iterator it1 = std::find_if(_canvasShips->begin(),
      _canvasShips->end(), canvas_ship_eq(*it));
    if (it1 != _canvasShips->end()) {
      delete (*it1);
      _canvasShips->erase(it1);
    }
  }
  canvas()->update();
}

void BCentral::addObjectOrGroup(BattleObject* bo, const QDateTime& modelTime) {
  BattleObjects bos = bo->objs(BattleObject::LeafsOnlySelectionType);
  for(BattleObjects::iterator it = bos.begin(); it != bos.end(); it++) {
    addBattleObject(*it, modelTime);
  }
}

void BCentral::updateChasingConnections() {
  //      P-E
  std::for_each(_cCCs.begin(), _cCCs.end(), update_cccs(this, _showCCCs));
}

/**    ,     ,  
       View
*/
QRect BCentral::tip(const QPoint& pos, QString* txt) {
  if (!canvas()) {
    return QRect(0,0,-1,-1);
  }
  QCanvasItemList list = canvas()->collisions(pos);
    //viewportToContents(pos));
  if (list.isEmpty()) 
    return QRect(0,0,-1,-1);
  else {
    //      ?
    QCanvasItemList::Iterator it = list.begin();
    for(; it != list.end(); ++it) {
      BCanvasShip* shp = dynamic_cast<BCanvasShip*>(*it);
      if (shp) {
        *txt = shp->ship()->name();
        QRect r = shp->boundingRect();
//        QRect r;
//        r.setTopLeft(pos);
//        r.setSize(QSize(2,2));

//        r.setTopLeft(contentsToViewport(r.topLeft()));
        return r;
      }
    }
    return QRect(0,0,-1,-1);
  }
}
