/******************************************************************************/
/* BCentral -   BattleFleet              */
/******************************************************************************/

#ifndef BCentral_H
#define BCentral_H
static const char bcentral_h_id[]="@(#) $Id: bcentral.h,v 1.18 2005/01/21 16:25:02 Oleksiy Balabay Exp $"; static const char* bcentral_h_id_id=bcentral_h_id;
/** \file bcentral.h
  \brief  BCentral  BattleFleet

     BCentral  BattleFleet
*/

#include <qcanvas.h>
#ifdef Q_WS_WIN
#pragma warning(disable: 4786)
#endif
#include <memory>
#include <set>
#include <vector>

class Battle;
class BCoordinates;
struct BattleInfo;
class BShip;
class BattleObject;
class BCanvasShip;
typedef std::list<BCanvasShip*> BCanvasShips;
class BCoordTransformer;
class QDateTime;
class BObserverCentral;
class BCanvasChasingConnection;
class BCanvasConnection;
class BAIChasing;
class update_cccs;
class BAI;
class BCentralToolTip;

typedef std::set<BattleObject*> BattleObjects;

class BCentral: public QCanvasView {
  Q_OBJECT
public:
  BCentral(BObserverCentral* observer, QWidget* parent, const char* name = 0);
  virtual ~BCentral();
  void battleChanged(Battle* battle);
  void zoomIn();
  void zoomOut();
  void fullView();
//  BattleObject* activeObject() const;
  BCoordTransformer* coordTransformer() const {
    return _coordTransformer.get();
  }
  void showNewCanvas(const BCoordinates& mapCo);
  void removeBattleObject(BattleObject* bo);
  void addObjectOrGroup(BattleObject* bo, const QDateTime& modelTime);
  void updateChasingConnections();
  QRect tip(const QPoint& pos, QString* txt);
public slots:
  void setActiveObject(BattleObject* o);
  void setSelectedObjects(const BattleObjects& objects);
  void contentsMovedSlot(Orientation, int pos);
  virtual void slotBattleInfoChanged(const BattleInfo& battleInfo);
protected:
  virtual void contentsMousePressEvent(QMouseEvent* e);
  virtual void contentsMouseReleaseEvent(QMouseEvent* e);
  virtual void contentsMouseMoveEvent(QMouseEvent* e);
  virtual void viewportResizeEvent(QResizeEvent* e);
  BCanvasShip* findObject(BattleObject* bo);
  void addBattleObject(BattleObject* bo, const QDateTime& modelTime);
private:
  void deleteCanvas();
  int scrollBarSize(const Orientation ) const;
  int mapToScrollBar(const Orientation, const double coordinate) const;
  double mapFromScrollBar(const Orientation, const int coordinate) const;
protected:
  std::auto_ptr<BCoordTransformer> _coordTransformer;
  Battle* _battle;
  std::auto_ptr<BCanvasShips> _canvasShips;
  BObserverCentral* _observer;
private:
  // ship control
  QCanvasItem* _activeCanvasShip;
//  BattleObject* _activeShip;
  BCentralToolTip* _toolTip;

signals:
  void visiblePart(Orientation, int pos, int range);
//  void activeObjectChanged(BattleObject* o);

// Chasing connections processing
private:
  // BCanvasChasingConnections -- BCCCs
  typedef std::map<BAIChasing*, BCanvasChasingConnection*> BCCCs; 
  BCCCs _cCCs;
  bool _showCCCs;
public slots:
  void slotShowChasingConnections(bool visible);
  void slotChasingConnectionChanged(BAIChasing* ai, const bool *add, const bool *isAuto);
private:
  friend class update_cccs;
  BCanvasChasingConnection* createCCC(BAIChasing* ai);
public:
  void deselectAllConnections();
  std::vector<BAI*> selectedConnections() const;
  BAI* connection(BCanvasConnection* cc) const;
};

#endif // BCentral_H
