/******************************************************************************/
/* BCollisions -     (impl.)            */
/******************************************************************************/
static const char bcollisions_cpp_id[]="@(#) $Id: bcollisions.cpp,v 1.2 2005/01/16 15:33:39 Oleksiy Balabay Exp $"; static const char* bcollisions_cpp_id_id=bcollisions_cpp_id;

#include "bcollisions.h"
#include "battle.h"
#include "bships.h"
#include "bseamap.h"
#include "bunitsconsts.h"

#include "bcollisionsalg.h"
/** \file bcollisions.cpp
  \brief  BCollisions   (impl.)

     BCollisions  
*/

/** \class BCollisions
  \brief 


*/
/** \brief 
*/
BCollisions::BCollisions(Battle* battle, QObject* parent, const char* name)
:BattleObserver(battle, parent, name) {
}

/** \brief 
*/
BCollisions::~BCollisions(){

}

BCollisions::Collisions BCollisions::checkObjects(const QDateTime& t) {
  BattleObjects objs = _battle->ships()->objs(BattleObject::LeafsOnlySelectionType);
  Collisions cols;
  cols.reserve(10);
  for( BattleObjects::const_iterator it = objs.begin(); it != objs.end(); it++) {
    BattleObjects::const_iterator i1 = it; i1++;
    Collisions oCols = checkObjectImpl(t, *it, i1, objs.end());
    for(Collisions::iterator ci = oCols.begin(); ci != oCols.end(); ci++) {
      cols.push_back(*ci);
    }
  }
  return cols;
}

BCollisions::Collisions BCollisions::checkObject(const QDateTime& t, 
  BattleObject* a) {
  BattleObjects objs = _battle->ships()->objs(BattleObject::LeafsOnlySelectionType);
  objs.erase(a);
  return checkObjectImpl(t, a, objs.begin(), objs.end()) ;
}

bool BCollisions::checkPair(const QDateTime& t,
  BattleObject* a, BattleObject* b) {
  double y0 = _battle->seaMap()->originalCoordinates().y0();
  return checkPairImpl(t, a, b, y0);
}

BCollisions::Collisions BCollisions::checkObjectImpl(const QDateTime& t, 
  BattleObject* a, BattleObjects::const_iterator b,
  BattleObjects::const_iterator e) {
  Collisions cols;
  double y0 = _battle->seaMap()->originalCoordinates().y0();
  for( BattleObjects::const_iterator it = b; it != e; it++) {
    if (checkPairImpl(t, a, *it, y0)) {
      cols.push_back(Collision(a, *it));
    }
  }
  return cols;
}

bool BCollisions::checkPairImpl(const QDateTime& t,
  BattleObject* a, BattleObject* b, double y0) {
  BKinematicData kdA = a->kinematicData(t);
  BKinematicData kdB = b->kinematicData(t);
  BCollisionsData d;
  d.xA = BUnitsConverter::degreesToMiles(kdA.x(), y0);
  d.xB = BUnitsConverter::degreesToMiles(kdB.x(), y0);
  d.yA = BUnitsConverter::degreesToMiles(kdA.y());
  d.yB = BUnitsConverter::degreesToMiles(kdB.y());
  d.lA = a->length();
  d.wA = a->width();
  d.lB = b->length();
  d.wB = b->width();
  d.cA = kdA.course();
  d.cB = kdB.course();

/*  double debug = d.lA;
  d.xA/=debug;
  d.xB/=debug;
  d.yA/=debug;
  d.yB/=debug;
  d.lA/=debug;
  d.lB/=debug;
  d.wA/=debug;
  d.wB/=debug;
*/
  double s2 = (d.xB-d.xA)*(d.xB-d.xA)+(d.yB-d.yA)*(d.yB-d.yA);
  double r2 = d.lA+d.lB; r2 = r2*r2;

  if (s2 < r2) {
    return collides(d);
  }
  else
    return false;
}
