/******************************************************************************/
/* BControlMove -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bcontrolmove_cpp_id[]="@(#) $Id: bcontrolmove.cpp,v 1.12 2005/02/20 14:44:58 Oleksiy Balabay Exp $"; static const char* bcontrolmove_cpp_id_id=bcontrolmove_cpp_id;

#include <qlayout.h>
#include <qgroupbox.h>
#include <qvalidator.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qslider.h>
//#include <qdial.h>
#include <qcheckbox.h>

#include "bcontrolmove.h"
#include "bcoordinateedit.h"
#include "battle.h"
#include "bapplication.h"
#include "battleobject.h"
#include "beventprogram.h"
#include "bprogramfactory.h"
#include "bunitsconsts.h"
#include "battleobserver.h"
#include "bmainwindow.h"
#include "bshipgroup.h"
#include "bseamap.h"

/** \file bcontrolmove.cpp
  \brief  BControlMove  BattleFleet (impl.)

     BControlMove  BattleFleet
*/

/** \class BControlMove
  \brief 


*/
/** \brief 
*/
BControlMove::BControlMove(BattleObserver* observer, 
  QWidget* parent, const char* name, WFlags fl )
  :BControlMoveBase(parent, name, fl), _observer(observer) {
//  _course = new BCoordinateEdit(0, _courseGroup);
//  _courseGroupLayout->addWidget(_course);
//  _course->show();
//  _course->setMinimumHeight(100, 25);
 // _course->setSizeHint(minimumSizeHint());
//  _course->updateGeometry();
//  updateGeometry();
  connect(_course, SIGNAL(valueChanged(int, int, int)),
    this, SLOT(slotCourseChange(int, int, int)));
  _speedValidator = new QDoubleValidator(_speed);
  _speed->setValidator(_speedValidator);
  setActiveObject(0);
  setFocusPolicy(NoFocus);
  connect(&_timer, SIGNAL(timeout()), this, SLOT(slotTimeout()));
  _timer.start(1000);
}

/** \brief 
*/
BControlMove::~BControlMove(){

}

void BControlMove::slotCourseChange(int deg) {
  _course->setValue(deg, 0, 0);
}

void BControlMove::slotSpeedChange(int knots_10) {
  double v = /* _speedValidator->top() - */ knots_10/10.0;
  _speed->setText(QString::number(v, 'f', 1));
}

void BControlMove::slotCourseChange(int deg, int /* min */, int /* sec */) {
  _courseDial->setValue(deg);
}

void BControlMove::slotSpeedChange(const QString& knots) {
  _speedSlider->setValue((/* _speedValidator->top() - */ (int)knots.toDouble())*10);
}

void BControlMove::setActiveObject(BattleObject* o) {
  _o = o;
  if (o == 0) {
//    _courseGroup->setEnabled(false);
//    _speedGroup->setEnabled(false);
    _objectName->setEnabled(false);
    _orderGroup->setEnabled(false);
    _setButton->setEnabled(false);
    _objectName->setTitle(tr("None"));

    double maxSpeed = 10;
    QString maxSpeedS = QString::number(maxSpeed, 'f', 1);
    double minSpeed = 0;
    QString minSpeedS = QString::number(minSpeed, 'f', 1);
    _maxSpeedLabel->setText(maxSpeedS);
    _minSpeedLabel->setText(minSpeedS);
    _speedSlider->setMinValue(int(minSpeed*10));
    _speedSlider->setMaxValue(int(maxSpeed*10));
    return;
  }
  _objectName->setEnabled(true);
  _objectName->setTitle(o->name());
//  _courseGroup->setEnabled(true);
//  _speedGroup->setEnabled(true);
  _setButton->setEnabled(true);
  _orderGroup->setEnabled(BShipGroup::isTurnInSuccessionEnabled(
    o, bApp->battle()->battleInfo().modelTime, 
    BUnitsConverter::longitudeCoef(
    bApp->battle()->seaMap()->originalCoordinates().y0())));
  BProgramData pd = o->programData(bApp->battle()->battleInfo().modelTime);
  double maxSpeed = pd.availableSpeed();
  QString maxSpeedS = QString::number(maxSpeed, 'f', 1);
  double minSpeed = 0;
  QString minSpeedS = QString::number(minSpeed, 'f', 1);
  _maxSpeedLabel->setText(maxSpeedS);
  _minSpeedLabel->setText(minSpeedS);
  _speedValidator->setTop(maxSpeedS.toDouble());
  _speedValidator->setBottom(minSpeedS.toDouble());
  _speedValidator->setDecimals(1);
  _speedSlider->setMinValue(int(minSpeed*10));
  _speedSlider->setMaxValue(int(maxSpeed*10));
  _speed->setText(QString::number(pd.kinematicData().v(false), 'f', 1));
  _courseDial->setValue(0);
  _course->setValue(0);
  _course->setDegreesRange(-180, 180);
}

void BControlMove::slotSet() {
  if (_o == 0)
    return;
  Battle* b = bApp->battle();
  QDateTime dt = b->battleInfo().modelTime;
  // 
  double sp = _speed->text().toDouble();
  if (sp > _speedValidator->top()) {
    _speed->setText(QString::number(_speedValidator->top(), 'f', 1));
    sp = _speedValidator->top();
  }
  else if (sp < _speedValidator->bottom()) {
    _speed->setText(QString::number(_speedValidator->bottom(), 'f', 1));
    sp = _speedValidator->bottom();
  }
  //  
  double angle = BUnitsConverter::angleFromDegrees(_course->value());
  //   ,   ,   
  //  
  BattleObjects bos = _o->objs(BattleObject::LeafsOnlySelectionType);
  for(BattleObjects::iterator it = bos.begin(); it != bos.end(); it++) {
    emit changeControlMode(BAI::ControlModeManual, *it);
  }
  //  
  if (_o->isGroup()) {
    double a = _o->availableAcceleration(dt);
    double r = _o->availableTurnRadius(dt);
    if (_orderGroup->selected() == _simOrder) {
      _observer->postEvent(
        new BEventProgram(BProgramFactory::createTurnSimultaneousProgram(
        b, &angle, &sp, &a, &r), _o->name(), dt), 
        _clearCommandsCheckBox->isChecked(), false);
    }
    else {
      _observer->postEvent(
        new BEventProgram( BProgramFactory::createTurnInSuccessionProgram(
        b, &angle, &sp, &a, &r), _o->name(), dt), 
        _clearCommandsCheckBox->isChecked(), false);
    }
  }
  else {
    _observer->postEvent(
      new BEventProgram( BProgramFactory::createTurnAccelerateProgram(
      b, &angle, &sp), _o->name(), dt), 
      _clearCommandsCheckBox->isChecked(), false);
  }
}

void BControlMove::mousePressEvent(QMouseEvent* /*e*/) {
  qApp->postEvent(qApp->activeWindow(), new BFocusEvent());
}

void BControlMove::slotTimeout() {
  if (_o) {
    bool orderGroupEnabled = BShipGroup::isTurnInSuccessionEnabled(
      _o, bApp->battle()->battleInfo().modelTime, 
      BUnitsConverter::longitudeCoef(
      bApp->battle()->seaMap()->originalCoordinates().y0()));
    if (!orderGroupEnabled) {
      _simOrder->setChecked(true);
    }
    _orderGroup->setEnabled(orderGroupEnabled);
  }
}
