/****************************************************************************
** $Id: bdomhandler.cpp,v 1.2 2005/01/21 16:25:02 Oleksiy Balabay Exp $
**
** Implementation of QDomDocument and related classes.
**
** Created : 000518
**
** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the xml module of the Qt GUI Toolkit.
**
** This file may be distributed under the terms of the Q Public License
** as defined by Trolltech AS of Norway and appearing in the file
** LICENSE.QPL included in the packaging of this file.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding valid Qt Enterprise Edition licenses may use this
** file in accordance with the Qt Commercial License Agreement provided
** with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about Qt Commercial License Agreements.
** See http://www.trolltech.com/qpl/ for QPL licensing information.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
/******************************************************************************/
/* BDomHandler -     (impl.)            */
/******************************************************************************/
static const char bdomhandler_cpp_id[]="@(#) $Id: bdomhandler.cpp,v 1.2 2005/01/21 16:25:02 Oleksiy Balabay Exp $"; static const char* bdomhandler_cpp_id_id=bdomhandler_cpp_id;

#include "bdomhandler.h"

/** \file bdomhandler.cpp
  \brief  BDomHandler   (impl.)

     BDomHandler  
*/

/** \class BDomHandler
  \brief   QDom, .    

       DTD, Entities  Notations - 
    QDom  public .


*/
/**************************************************************
 *
 * BDomHandler
 *
 **************************************************************/

BDomHandler::BDomHandler( QDomDocument* adoc, bool namespaceProcessing )
{
    doc = adoc;
    if (doc)
      node = *doc;
    cdata = FALSE;
    nsProcessing = namespaceProcessing;
}

BDomHandler::~BDomHandler()
{
}

bool BDomHandler::endDocument()
{
//    // ### is this really necessary? (rms)
//    if ( node != doc )
//	return FALSE;
    return TRUE;
}

bool BDomHandler::startDTD( const QString& /*name*/, const QString& /*publicId*/, const QString& /*systemId*/ )
{
//    doc->doctype()->name = name;
//    doc->doctype()->publicId = publicId;
//    doc->doctype()->systemId = systemId;
    return TRUE;
}

bool BDomHandler::startElement( const QString& nsURI, const QString&, const QString& qName, const QXmlAttributes& atts )
{
    // tag name
    QDomNode n;
    if ( nsProcessing ) {
	n = doc->createElementNS( nsURI, qName );
    } else {
	n = doc->createElement( qName );
    }
    node.appendChild( n );
    node = n;

    // attributes
    for ( int i=0; i<atts.length(); i++ )
    {
	if ( nsProcessing ) {
	    node.toElement().setAttributeNS( atts.uri(i), atts.qName(i), atts.value(i) );
	} else {
	    node.toElement().setAttribute( atts.qName(i), atts.value(i) );
	}
    }

    return TRUE;
}

bool BDomHandler::endElement( const QString&, const QString&, const QString& )
{
    if ( node.isDocument() )
	return FALSE;
    node = node.parentNode();

    return TRUE;
}

bool BDomHandler::characters( const QString&  ch )
{
    // No text as child of some document
    if ( node.isDocument() )
	return FALSE;

    if ( cdata ) {
	node.appendChild( doc->createCDATASection( ch ) );
    } else if ( !entityName.isEmpty() ) {
//	QDomEntity* e = new QDomEntity( doc, 0, entityName,
//		QString::null, QString::null, QString::null );
//	e->value = ch;
//	doc->doctype()->appendChild( e );
//	node->appendChild( doc->createEntityReference( entityName ) );
    } else {
	node.appendChild( doc->createTextNode( ch ) );
    }

    return TRUE;
}

bool BDomHandler::processingInstruction( const QString& target, const QString& data )
{
    node.appendChild( doc->createProcessingInstruction( target, data ) );
    return TRUE;
}

bool BDomHandler::skippedEntity( const QString& name )
{
    node.appendChild( doc->createEntityReference( name ) );
    return TRUE;
}

bool BDomHandler::fatalError( const QXmlParseException& exception )
{
    errorMsg = exception.message();
    errorLine =  exception.lineNumber();
    errorColumn =  exception.columnNumber();
    return QXmlDefaultHandler::fatalError( exception );
}

bool BDomHandler::startCDATA()
{
    cdata = TRUE;
    return TRUE;
}

bool BDomHandler::endCDATA()
{
    cdata = FALSE;
    return TRUE;
}

bool BDomHandler::startEntity( const QString &name )
{
    entityName = name;
    return TRUE;
}

bool BDomHandler::endEntity( const QString & )
{
    entityName = QString::null;
    return TRUE;
}

bool BDomHandler::comment( const QString& ch )
{
    node.appendChild( doc->createComment( ch ) );
    return TRUE;
}

bool BDomHandler::unparsedEntityDecl( const QString &/*name*/, const QString &/*publicId*/, 
const QString &/*systemId*/, const QString &/*notationName*/ )
{
//    QDomEntity* e = new QDomEntity( doc, 0, name,
//	    publicId, systemId, notationName );
//    doc->doctype()->appendChild( e );
    return TRUE;
}

bool BDomHandler::externalEntityDecl( const QString &name, const QString &publicId, const QString &systemId )
{
    return unparsedEntityDecl( name, publicId, systemId, QString::null );
}

bool BDomHandler::notationDecl( const QString & /*name*/, const QString & /*publicId*/, 
const QString & /*systemId*/ )
{
//    QDomNotation* n = new QDomNotation( doc, 0, name, publicId, systemId );
//    doc->doctype()->appendChild( n );
    return TRUE;
}
