/******************************************************************************/
/* BDomUtils -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bdomutils_cpp_id[]="@(#) $Id: bdomutils.cpp,v 1.8 2005/01/10 18:20:57 Oleksiy Balabay Exp $"; static const char* bdomutils_cpp_id_id=bdomutils_cpp_id;

#include "bdomutils.h"

const QString DATE_YEAR = "Y";
const QString DATE_MONTH = "M";
const QString DATE_DAY = "D";
const QString DATE_HOURS = "h";
const QString DATE_MINUTES = "m";
const QString DATE_SECONDS = "s";
const QString DATE_MSECONDS = "ms";

/** \file bdomutils.cpp
  \brief  BDomUtils  BattleFleet (impl.)

     BDomUtils  BattleFleet
*/

/** \class BDomUtils
  \brief 


*/
QString BDomUtils::elementToString(const QDomElement& e) {
  QDomDocument doc("");
  doc.appendChild(e.cloneNode(true));
  QString s = doc.toString();
  return s;
}

int BDomUtils::intAttribute(const QDomElement& e, 
  const QString& attribute) {
  if (e.hasAttribute(attribute)) {
    bool ok = false;
    int rez = e.attribute(attribute).toInt(&ok);
    if (!ok) {
      throw BConfigException(__FILE__, __LINE__,
        qApp->translate("BDomUtils",
        "Attribute '%1' has not integer value '%2' "
        "in element '%3'").arg(
        attribute).arg(
        e.attribute(attribute)).arg(
        elementToString(e)));
    }
    return rez;
  }
  else {
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate("BDomUtils",
      "Attribute '%1' does not exist "
      "in element '%2'").arg(
      attribute).arg(
      elementToString(e)));
  }
}

QString BDomUtils::stringAttribute(const QDomElement& e, 
  const QString& attribute) {
  if (e.hasAttribute(attribute)) {
    return e.attribute(attribute);
  }
  else {
    QString ee = qApp->translate("BDomUtils",
      "Attribute '%1' does not exist "
      "in element '%2'").arg(
      attribute).arg(
      elementToString(e));
    throw BConfigException(__FILE__, __LINE__, ee);
  }
}

void BDomUtils::elementDoesNotExist(const QString& elementName)
  {
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate("BDomUtils",
      "Element '%1' does not exist").arg(elementName));
}

void BDomUtils::attributeDoesNotExist(const QString& elementName,
  const QString& attributeName) {
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate("BDomUtils",
      "Attribute '%1' does not exist in element '%2'").arg(
      attributeName).arg(elementName));
}

QColor BDomUtils::colorFromString(const QString& color) {
  QColor c(color);
  if (c.isValid())
    return c;
  else {
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate("BDomUtils",
      "Color '%1' is invalid").arg(color));
  }
}

QDateTime BDomUtils::dateTimeFromXml(const QDomElement& e) 
  {
  int year = BDomUtils::intAttribute(e, DATE_YEAR);
  int month = BDomUtils::intAttribute(e, DATE_MONTH);
  int day = BDomUtils::intAttribute(e, DATE_DAY);
  int hours = BDomUtils::intAttribute(e, DATE_HOURS);
  int minutes = BDomUtils::intAttribute(e, DATE_MINUTES);
  int seconds = 0;
  if (e.hasAttribute(DATE_SECONDS)) {
    seconds = BDomUtils::intAttribute(e, DATE_SECONDS);
  }
  int mseconds = 0;
  if (e.hasAttribute(DATE_MSECONDS)) {
    mseconds = BDomUtils::intAttribute(e, DATE_MSECONDS);
  }
  QDateTime dateTime = QDateTime (QDate(year, month, day), 
    QTime(hours, minutes, seconds).addMSecs(mseconds));
  if (!dateTime.isValid()) {
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate("BDomUtils",
      "Element '%1' contains invalid date: '%2'").arg(e.tagName()).arg(
      BDomUtils::elementToString(e)));
  }
  return dateTime;
}

double BDomUtils::doubleAttribute(const QDomElement& e, 
  const QString& attribute) {
  if (e.hasAttribute(attribute)) {
    bool ok = false;
    double rez = e.attribute(attribute).toDouble(&ok);
    if (!ok) {
      throw BConfigException(__FILE__, __LINE__,
        qApp->translate("BDomUtils",
        "Attribute '%1' has not double value '%2' "
        "in element '%3'").arg(
        attribute).arg(
        e.attribute(attribute)).arg(
        elementToString(e)));
    }
    return rez;
  }
  else {
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate("BDomUtils",
      "Attribute '%1' does not exist "
      "in element '%2'").arg(
      attribute).arg(
      elementToString(e)));
  }
}

QDomElement BDomUtils::dateTimeToXml(const QDateTime& dt, 
  const QString& elementName, QDomDocument& doc) {
  QDomElement el = doc.createElement(elementName);
  el.setAttribute(DATE_YEAR, dt.date().year());
  el.setAttribute(DATE_MONTH, dt.date().month());
  el.setAttribute(DATE_DAY, dt.date().day());
  el.setAttribute(DATE_HOURS, dt.time().hour());
  el.setAttribute(DATE_MINUTES, dt.time().minute());
  int sec = dt.time().second();
  if (sec)
    el.setAttribute(DATE_SECONDS, sec);
  int msec = dt.time().msec();
  if (msec)
    el.setAttribute(DATE_MSECONDS, msec);
  return el;
}
