/******************************************************************************/
/* BEvent -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bevent_cpp_id[]="@(#) $Id: bevent.cpp,v 1.18 2005/01/21 16:25:02 Oleksiy Balabay Exp $"; static const char* bevent_cpp_id_id=bevent_cpp_id;

#include <qdom.h>
#include <qfile.h>

#include "bevent.h"
#include "beventclear.h"
#include "bdomutils.h"

#include "beventprogram.h"
#include "bapplication.h"
#include "battle.h"
#include "battleobject.h"
#include "bships.h"
#include "beventsparser.h"
#include "beventseamap.h"
#include "beventssaver.h"

/** \file bevent.cpp
  \brief  BEvent  BattleFleet (impl.)

     BEvent  BattleFleet
*/

/** \class BEvent
  \brief 


*/
const QString BEvent::BE_EVENT = "Event";
const QString BEvent::BE_EVENT_TYPE = "Type";

const QString BEvent::EVENT_FLAGS = "Flags";
const QString BEvent::EVENT_FLAG_PLAYER = "Player";
const QString BEvent::EVENT_FLAG_SAVED_USER = "SavedUser";

BNoObjectException::BNoObjectException(const QString& sourceCodeFile, 
  const int sourceCodeLine, const QString& objectName)
  :BException(sourceCodeFile, sourceCodeLine, 
  qApp->translate("BNoObjectException",
  "No object named '%1'").arg(objectName)), _objectName(objectName) {
}

/** \brief .
      ,     
  .    ,      parse()
*/
BEvent::BEvent(const QString& objectName, const QDateTime& dt) throw(){
  setDate(dt);
  setObjectName(objectName);
  setEventFlags(UserEventFlag);
}

/** \brief 
*/
BEvent::~BEvent(){

}

/**      object()
*/
QDomElement BEvent::toXml(QDomDocument& doc) const {
  QDomElement e = doc.createElement(BE_EVENT);
  e.setAttribute(EVENT_FLAGS, eventFlagToString(eventFlags()));
  return e;
}

BattleObject* BEvent::object() const {
  BattleObject* bo = bApp->battle()->ships()->find(_objectName);
  if (bo == 0) {
    throw BNoObjectException(__FILE__, __LINE__,
      qApp->translate("BEvent", 
        "Object does not exist:'%1'").arg(_objectName));
  }
  return bo;
}

/** \brief     .

        xml.

    \a EVENT_FLAG_PLAYER  \a EVENT_FLAG_SAVED_USER.
        ,   
\a EVENT_FLAG_PLAYER.   

*/
unsigned int BEvent::eventFlagFromString(const QString& eventFlag) {
  if (eventFlag.contains(EVENT_FLAG_PLAYER))
    return PlayerEventFlag;
  else if (eventFlag.contains(EVENT_FLAG_SAVED_USER))
    return SavedUserEventFlag;
  else 
    return PlayerEventFlag;
}

/** \brief    

        xml.  
    \a SavedUserEventFlag,      
  .     \a UserEventFlag,    
    \a EVENT_FLAG_PLAYER.

      \a PlayerEventFlag,    
    \a EVENT_FLAG_PLAYER.     
  \a PlayerEventFlag + \a SystemEventFlag,    
  .

*/
QString BEvent::eventFlagToString(const unsigned int eventFlag) {
  if ((eventFlag & EVENT_FLAG_MASK) == PlayerEventFlag) {
    if ((eventFlag & SystemEventFlag) == SystemEventFlag) {
      throw BInternalException(__FILE__, __LINE__,
        "Internal error. Try to save player system event");
    }
    else
      return EVENT_FLAG_PLAYER;
  }
  else if ((eventFlag & EVENT_FLAG_MASK) == SavedUserEventFlag)
    return EVENT_FLAG_SAVED_USER;
  else if ((eventFlag & EVENT_FLAG_MASK) == UserEventFlag)
    return EVENT_FLAG_PLAYER;
  else {
    qWarning("Unknown flag: %d", eventFlag);
    return EVENT_FLAG_PLAYER;
  }
}


BEvents::BEvents(Battle* battle): _battle(battle) {
//  QDomProcessingInstruction pi = _history.createProcessingInstruction(
//    "xml", QString("version='1.0' encoding='UTF-8'"));
//  _history.insertBefore(pi, _history.documentElement());
//  _history.appendChild(_history.createElement(BEventsParser::SOM_EVENTS));
  _saver = new BEventsSaver();
}

BEvents::~BEvents() {
  clear();
  delete _saver;
}

void BEvents::clear() {
  for(BEventList::iterator it = _events.begin(); it != _events.end(); it++) {
    BEvent* sh = *it;
    delete sh;
  }
  _events.clear();
}

void BEvents::processEvents(const QDateTime& modelDate) {
  const int PROGRESS_LOW = 60;
  const int PROGRESS_HIGH = 79;
  int eventsProcessed = 0;
  int eventsPer1Progress = 1;
  bool showProgress = bApp->progressDialog();
  if (showProgress) {
    int total = 0;
    BEventList::const_iterator it = _events.begin();
    while ( (it != _events.end()) && ((*it)->date() <= modelDate)) {
      total++;
      it++;
    }
    eventsPer1Progress = total/(PROGRESS_HIGH - PROGRESS_LOW)+1;
  }

  while ((_events.first() != 0) && (_events.first()->date() <= modelDate)) {
//    qDebug("Process events started");
//    for (BEventList::iterator it = _events.begin(); it != _events.end(); it++) {
//      qDebug("BEvent: %s Object name: %s, descr: %s", 
//      (const char*)(*it)->date().toString().local8Bit(),
//      (const char*)(*it)->objectName().local8Bit(),
//      (const char*)(*it)->description().local8Bit());
//    }
    BEvent* ev = _events.first();
    if (ev->date() > modelDate) {
      break;
    }
    _battle->addBattleInfoFlags(BFEvent);
    bool isClear = (dynamic_cast<BEventClear*>(ev) != 0);
    bool isSeaMap = (dynamic_cast<BEventSeaMap*>(ev) != 0);
    try {
//    if ( event->eventType() != BEvent::SystemEventType) {
//      _history.documentElement().appendChild(wrapEvent(event, _history));
//    }
      //   ,   Player + System
      unsigned int evFlag = ev->eventFlags();
      if ( (evFlag & BEvent::AskBeforeExecuteEventFlag) != 0) {
        evFlag = evFlag & (~BEvent::AskBeforeExecuteEventFlag);
        ev->setEventFlags(evFlag);
        _battle->slotPause();
        return;
      }
      if (!( ((evFlag & BEvent::SystemEventFlag) == BEvent::SystemEventFlag) &&
        ((evFlag & BEvent::EVENT_FLAG_MASK) == BEvent::PlayerEventFlag))) {
        ev->exec();
        //    ,      
        QDomElement el;
        if ((!isClear) && (!isSeaMap)) {
          el = wrapEvent(ev, _history);
        }
        if ((!isClear) && (!isSeaMap)) {
          //_history.documentElement().appendChild(el);
          _saver->add(BDomUtils::elementToString(el));
        }
      }
    }
    catch (BException& e) {
      QDomDocument doc;
      bApp->log(e.message());
      bApp->log(qApp->translate("BEvents",
        "Unknown error executing command:\n%1").arg(
        BDomUtils::elementToString(ev->toXml(doc))));
    }
    catch ( ... ) {
      QDomDocument doc;
      bApp->log(qApp->translate("BEvents",
        "Unknown error executing command:\n%1").arg(
        BDomUtils::elementToString(ev->toXml(doc))));
    }
    delete (ev);
    _events.removeRef(ev);
    if (showProgress) {
      eventsProcessed++;
      if (eventsProcessed%eventsPer1Progress == 0) {
        bApp->checkProgressDialog(
          PROGRESS_LOW+eventsProcessed/eventsPer1Progress);
      }
    }
  }
}

int BEvents::BEventList::compareItems(QCollection::Item item1,QCollection::Item item2) {
  BEvent* e1 = (BEvent*)item1;
  BEvent* e2 = (BEvent*)item2;
  if (e1->date() == e2->date())
    return 0;
  else if (e1->date() < e2->date()) 
    return -1;
  else
    return 1;
}

void BEvents::postEvent(BEvent* event) {
  //   ,   Player + System
  unsigned int evFlag = event->eventFlags();
  if ( ((evFlag & BEvent::SystemEventFlag) == BEvent::SystemEventFlag) &&
    ((evFlag & BEvent::EVENT_FLAG_MASK) == BEvent::PlayerEventFlag)) {
    delete event;
    return;
  }
  if (!event->date().isValid()) {
    event->setDate(_battle->battleInfo().modelTime);
  }
  QDateTime dt = event->date();
  //       ,    
  //  ,     
//  if (dt.time().msec() != 0) {
//    event->setDate(QDateTime(dt.date(), 
//      QTime(dt.time().hour(), dt.time().minute(), dt.time().second()).addSecs(1)));
//  }
  //    Clear   ,   
  BEventClear* ecl = dynamic_cast<BEventClear*>(event);
  if (ecl /*&& ecl->date() <= _battle->battleInfo().modelTime*/) {
    ecl->exec();
    delete ecl;
    return;
  }
  //   -     ,
  //     
  if ((_events.count() > 0)&&(_events.last()->date() <= dt)) {
    _events.append(event);
  }
  else {
    //    ,    .
    // FUCKING VC6.0 - std::list    .
    BEventList::iterator it = _events.begin();
    int i = 0;
    while ( (it != _events.end()) && ((*it)->date() <= dt) ) {
      it++; i++;
    }
    _events.insert(i, event);
  }
//  _events.inSort(event);
//    if ( event->eventType() != BEvent::SystemEventType) {
//      _history.documentElement().appendChild(wrapEvent(event, _history));
//    }
}

void BEvents::save(const QString& fileName) {
//  QFile f(fileName);
//  if (!f.open(IO_WriteOnly)) {
//    throw BOutputException(__FILE__, __LINE__, fileName);
//  }
//  QTextStream t(&f);
//  t.setEncoding(QTextStream::UnicodeUTF8);
//  t << toXml().toString(2);
//  f.close();
  _saver->beginSave(fileName);
  for(BEventList::iterator it = _events.begin(); it != _events.end(); it++) {
    BEvent* e = *it;
    unsigned int f = e->eventFlags();
    if (! ( ((f& BEvent::EVENT_FLAG_MASK) == BEvent::PlayerEventFlag) &&
      ((f & BEvent::SystemEventFlag) != BEvent::SystemEventFlag) ) ) {
      (*it)->setEventFlags(BEvent::SavedUserEventFlag);
    }
    _saver->addToSave(BDomUtils::elementToString(wrapEvent(*it, _history)));
  }
  _saver->endSave();
}

//QDomDocument BEvents::toXml() {
//  QDomDocument doc;
//  QDomProcessingInstruction pi = doc.createProcessingInstruction(
//    "xml", QString("version='1.0' encoding='UTF-8'"));
//  doc.insertBefore(pi, doc.documentElement());
//  doc.appendChild(doc.createElement(BE_EVENTS));
//  for(BEventList::iterator it = _events.begin(); it != _events.end(); it++) {
//    (*it)->setEventFlags(BEvent::SavedUserEventFlag);
//    _history.documentElement().appendChild(wrapEvent(*it, _history));
//  }
//  return _history;
//}

QDomElement BEvents::wrapEvent(BEvent* ev, QDomDocument& doc) {
  QDomElement date = BDomUtils::dateTimeToXml(ev->date(), 
    BEventsParser::SOM_DATE, doc);
  QDomElement boXml = doc.createElement(BEventsParser::SOM_OBJECT);
  boXml.setAttribute(BEventsParser::SOM_NAME, ev->objectName());
  date.appendChild(boXml);
  boXml.appendChild(ev->toXml(doc));
  return date;
}

void BEvents::clear(const QString& objectName, const QDateTime& dateFrom) {
  for (BEventList::iterator it = _events.begin(); it != _events.end(); it++) {
    BEvent* e = *it;
    if ((e->date() > dateFrom) && (objectName == e->objectName())) {
//      it++;
      delete e;
      _events.removeRef(e);
    }
  }
}

/** \class BEvents

       .     
BEvent::BEventFlags.   ,     
 .
  
        UserEventFlag.

    ,  ,   
   .     
SystemEventFlag .

      BEventClear ,   
   .

          
         .

          .
     UserEventFlag,     PlayerEventFlag.
PlayerEventFlag   .     UserEventFlag
    SystemEventFlag,    (.. 
  PlayerEventFlag, . BEvent::eventFlagToString()).
  
     ,      ,
      SavedUserEventFlag,  SystemEventFlag
.     PlayerEventFlag  - 
   .

     ,      
,      .     
     PlayerEventFlag (..    
 ),       .    
    ,    
   SystemEventFlag.   
         (postEvent()).

    ,      
,      .     
     ,      
   SavedUserEventFlag

     ,     
   UserEventFlag    .

         ,   
     SavedUserEventFlag.  
,   UserEventFlag,       
SystemEventFlag   . 

*/
