/******************************************************************************/
/* BEvent -   BattleFleet              */
/******************************************************************************/

#ifndef BEvent_H
#define BEvent_H
static const char bevent_h_id[]="@(#) $Id: bevent.h,v 1.10 2005/01/21 16:25:03 Oleksiy Balabay Exp $"; static const char* bevent_h_id_id=bevent_h_id;
/** \file bevent.h
  \brief  BEvent  BattleFleet

     BEvent  BattleFleet
*/
#include <list>
#include <qdatetime.h>

#include "bxmlconfig.h"
#include "bexception.h"

class QDomElement;
class BEvents;
class BEventsSaver;
class Battle;
class BattleObject;
class BEventsListView;


const QString BE_EVENT_OBJECT = "Object";

class BNoObjectException: public BException {
public:
  BNoObjectException(const QString& sourceCodeFile, const int sourceCodeLine, 
    const QString& objectName);
  virtual ~BNoObjectException() {};
  QString objectName() const {
    return _objectName;
  };
private:
  QString _objectName;
};

class BEvent {
  friend class BEvents;
public:
  /** \brief  

      . UserEventType, PlayerEventType,
  SavedUserEventType  -,  SystemEventType
     
  */
  enum BEventFlags {
    ///      
    UserEventFlag = 0x0,
    ///      
    PlayerEventFlag = 0x1,
    ///      ,    
    SavedUserEventFlag = 0x2,
    ///    
    EVENT_FLAG_MASK = 0x3,
    ///      
    SystemEventFlag = 0x4,
    ///    
    AskBeforeExecuteEventFlag = 0x8
  };
  BEvent(const QString& objectName = "", const QDateTime& dt = QDateTime()) throw();
  virtual ~BEvent();
  unsigned int eventFlags() const {
    return _flag;
  }
  void setEventFlags(unsigned int flag) {
    _flag = flag;
  }
  QDateTime date() const throw() {
    return _date;
  }
  BattleObject* object() const;
  QString objectName() const {
    return _objectName;
  }
  virtual void exec() = 0;
  virtual void parse(const QDomElement& element, Battle* battle) = 0; 
  virtual QDomElement toXml(QDomDocument& doc) const;
  virtual QString description() const = 0;
public:
  // BEvent  xml    BShips::save()
  static const QString BE_EVENT;
  static const QString BE_EVENT_TYPE;
  static const QString EVENT_FLAGS;
  static const QString EVENT_FLAG_PLAYER;
  static const QString EVENT_FLAG_SAVED_USER;
  static unsigned int eventFlagFromString(const QString& eventFlag);
  static QString eventFlagToString(const unsigned int eventFlag);
protected:
  void setDate(const QDateTime& date) {
    _date = date;
  }
  void setObjectName(const QString& objectName) {
    _objectName = objectName;
  }
private:
  QDateTime _date;
  QString _objectName;
  unsigned int _flag;
};

class BEvents {
public:
  BEvents(Battle* battle);
  virtual ~BEvents();
  void processEvents(const QDateTime& modelDate);
  void postEvent(BEvent* event);
  void save(const QString& fileName);
  void clear(const QString& objectName, const QDateTime& dateFrom);
// protected:
//  virtual QDomDocument toXml();
private:
  void clear();
  QDomElement wrapEvent(BEvent* ev, QDomDocument& doc);
private:
  class BEventList: public QPtrList<BEvent> {
  public:
    BEventList() {};
    virtual ~BEventList() {};
  protected:
    virtual int compareItems(QCollection::Item item1, QCollection::Item item2);
  };
  BEventList _events;
  QDomDocument _history;
  Battle* _battle;
  BEventsSaver* _saver;
  friend class BEventsListView;
};

#endif // BEvent_H
