/******************************************************************************/
/* BEventGroup -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char beventgroup_cpp_id[]="@(#) $Id: beventgroup.cpp,v 1.13 2005/01/21 16:25:03 Oleksiy Balabay Exp $"; static const char* beventgroup_cpp_id_id=beventgroup_cpp_id;

#include "beventgroup.h"
#include "battleobject.h"
#include "bshipgroup.h"
#include "bdomutils.h"
#include "bexception.h"
#include "battle.h"
#include "bships.h"
#include "bscenarioheader.h"
#include "bapplication.h"
#include "bships.h"

/** \file beventgroup.cpp
  \brief  BEventGroup  BattleFleet (impl.)

     BEventGroup  BattleFleet
*/

/** \class BEventGroup
  \brief 


*/
/** \brief 
*/
BEventGroup::BEventGroup(const QString& objectName, const QDateTime& dt) throw ()
  :BEvent(objectName, dt)  {

}

/** \brief 
*/
BEventGroup::~BEventGroup(){

}

const QString BEventGroupCreate::BE_EVENT_TYPE_GROUP_CREATE = "CreateGroup";
const QString BEventGroupCreate::SOM_SIDE = "Side";
const QString BEventGroupAdd::BE_EVENT_TYPE_GROUP_ADD = "AddToGroup";
const QString BEventGroupAdd::OBJECT = "Object";
const QString BEventGroupAdd::GROUP = "Group";
const QString BEventGroupRemove::BE_EVENT_TYPE_GROUP_REMOVE = "RemoveGroup";
const QString BEventGroupRemove::OBJECT = "Object";
const QString BEventGroupRename::BE_EVENT_TYPE_GROUP_RENAME = "RenameGroup";
const QString BEventGroupRename::OLD_NAME = "OldName";
const QString BEventGroupRename::NEW_NAME = "NewName";

/** \brief 
*/
BEventGroupCreate::BEventGroupCreate(const QString& parent, 
  const QDateTime& dt) throw() :BEventGroup(parent, dt) {
  _battle = 0;
}

BEventGroupCreate::BEventGroupCreate(const QString& parent,
  Battle* battle, const QString& name, const int side,
  const BattleObjects& members, const QDateTime& dt):BEventGroup(parent, dt),
  _battle(battle) {
  if ((side == 1) || (side == 2) || (side == NO_SIDE)) {
    _side = side;
  }
  else {
    throw BRangeException(__FILE__, __LINE__, NO_SIDE, 2, side);
  }
  QDomDocument doc;
  QDomElement e = BEvent::toXml(doc);
  e.setAttribute(BEvent::BE_EVENT_TYPE, BE_EVENT_TYPE_GROUP_CREATE);
  if ((_side == 1) || (_side == 2)) {
    QString side = _battle->scenarioHeader()->sideHeader(_side)->name().upper();
    e.setAttribute(SOM_SIDE, side);
  }
  QDomElement gr = doc.createElement(BShipGroup::SHP_SHIP_GROUP);
  gr.setAttribute(BShipGroup::SHP_SHIP_GROUP_NAME, name);
  for(BattleObjects::const_iterator it = members.begin(); it != members.end(); it++) {
    gr.appendChild((*it)->toXml(doc));
  }
  e.appendChild(gr);  
  parse(e, battle);
}

/** \brief 
*/
BEventGroupCreate::~BEventGroupCreate(){
}

void BEventGroupCreate::exec() {
  BattleObject* bo = 0;
  if (_groupEl.tagName() == BShipGroup::SHP_SHIP_GROUP)
    bo = parseGroup(_groupEl, object());
  else if (_groupEl.tagName() == BShip::SHP_SHIP)
    bo = parseShip(_groupEl, object());
  if (NO_SIDE != _side)
    bo->setSide(_side);
  _battle->addBattleInfoFlags(BFShips);
  _battle->addBattleInfoFlags(BFGrouping);
  // ToDO: !
  _battle->ships()->initAfterParse();
}

void BEventGroupCreate::parse(const QDomElement& element, Battle* battle) {
  setEventFlags(eventFlagFromString(
    BDomUtils::stringAttribute(element, EVENT_FLAGS)));
  _battle = battle;
  int currentSide = NO_SIDE;
  if (element.hasAttribute(SOM_SIDE)) {
    QString sideName = 
      BDomUtils::stringAttribute(element, SOM_SIDE).upper();
    QString side1 = _battle->scenarioHeader()->sideHeader(1)->name().upper();
    QString side2 = _battle->scenarioHeader()->sideHeader(2)->name().upper();
    if ( ( side1 != sideName) && ( side2 != sideName) ) {
      throw BConfigException(__FILE__, __LINE__,
        qApp->translate("BEventGroupCreate", 
        "Bad side '%1'. Should be '%2' or '%3'").arg(sideName).arg(
        side1).arg(side2));
    }
    currentSide = (side1 == sideName)?1:2;
  }
  _side = currentSide;

  bool groupFound = false;
  QDomNode n = element.firstChild();
  while( !n.isNull() ) {
    QDomElement e = n.toElement(); // try to convert the node to an element.
    n = n.nextSibling();
    if( !e.isNull() ) { // the node was really an element.
      if ((e.tagName() == BShipGroup::SHP_SHIP_GROUP) ||
          (e.tagName() == BShip::SHP_SHIP)) {
        _groupEl = e.cloneNode(true).toElement();
//        qDebug("%s", (const char*)BDomUtils::elementToString(_groupEl)
//          .local8Bit());
        groupFound = true;
        break;
      }
    }
  }
  if (!groupFound) {
    BDomUtils::elementDoesNotExist(BShipGroup::SHP_SHIP_GROUP);
  }
}

BattleObject* BEventGroupCreate::parseShip(const QDomElement& e, 
  BattleObject* parent) {
  QString name = BDomUtils::stringAttribute(e, BShip::SHP_SHIP_NAME);
  BShip* ship = dynamic_cast<BShip*>(_battle->ships()->find(name));
  if (0 == ship) {
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate("BEventGroupCreate", "Unknown ship '%1'").arg(name));
  }
  // insertChild       
  parent->insertChild(ship);
  return ship;
}

BattleObject* BEventGroupCreate::parseGroup(const QDomElement& e,  
  BattleObject* parent) {
  QString groupName = 
    BDomUtils::stringAttribute(e, BShipGroup::SHP_SHIP_GROUP_NAME);
  if (bApp->battle()->ships()->find(groupName)) {
    throw BInternalException(__FILE__, __LINE__,
      qApp->translate("BEventGroupCreate", 
      "Group '%1' already exists").arg(groupName));
  }
  BShipGroup* group = new BShipGroup(0, groupName);
//  group->setSide(_currentSide);
  parent->insertChild(group);
  QDomNode n = e.firstChild();
  while( !n.isNull() ) {
    QDomElement e1 = n.toElement(); // try to convert the node to an element.
    n = n.nextSibling();
    if( !e1.isNull() ) { // the node was really an element.
      if (e1.tagName() == BShipGroup::SHP_SHIP_GROUP) {
        parseGroup(e1, group);
      }
      else if (e1.tagName() == BShip::SHP_SHIP) {
        parseShip(e1, group);
      }
      else {
        qWarning("Unsupported tag '%s' ", 
          (const char*)e1.tagName().local8Bit());
      }
    }
  }
  return group;
}

QDomElement BEventGroupCreate::toXml(QDomDocument& doc) const {
  QDomElement e = BEvent::toXml(doc);
  e.setAttribute(BEvent::BE_EVENT_TYPE, BE_EVENT_TYPE_GROUP_CREATE);
  if ((_side == 1) || (_side == 2)) {
    QString side = _battle->scenarioHeader()->sideHeader(_side)->name().upper();
    e.setAttribute(SOM_SIDE, side);
  }
  e.appendChild(_groupEl);
  return e;
}

QString BEventGroupCreate::description() const {
  //ToDo:   
  QString name;
  if (_groupEl.tagName() == BShipGroup::SHP_SHIP_GROUP)
    name = BDomUtils::stringAttribute(_groupEl, BShipGroup::SHP_SHIP_GROUP_NAME);
  else if (_groupEl.tagName() == BShip::SHP_SHIP)
    name = BDomUtils::stringAttribute(_groupEl, BShip::SHP_SHIP_NAME);
  return qApp->translate("BEventGroupCreate", "Create group '%1'").arg(name);
}

/** \brief 
*/
BEventGroupAdd::BEventGroupAdd(const QString& objectName, 
  const QDateTime& dt) throw():BEventGroup(objectName, dt) {
  _battle = 0;
}

BEventGroupAdd::BEventGroupAdd(const QString& objectName, 
  const QString& groupName, Battle* battle, const QDateTime& dt) throw()
  :BEventGroup(objectName, dt), _battle(battle), _groupName(groupName) {
}

/** \brief 
*/
BEventGroupAdd::~BEventGroupAdd(){
}

void BEventGroupAdd::parse(const QDomElement& element, Battle* battle) 
  {
  setEventFlags(eventFlagFromString(
    BDomUtils::stringAttribute(element, EVENT_FLAGS)));
  _battle = battle;
  setObjectName(BDomUtils::stringAttribute(element, OBJECT));
  _groupName = BDomUtils::stringAttribute(element, GROUP);
}

void BEventGroupAdd::exec() {
  BattleObject* o = object();
  BattleObject* g = _battle->ships()->find(_groupName);
  if (g == 0) {
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate("BEventGroupAdd", "Group '%1' does not exist").arg(
      _groupName));
  }
  const BShipGroup* unused = _battle->ships()->unUsedShips();
  bool needUpdate = false;
  if (unused) {
    if (unused->find(o->name()))
      needUpdate = true;
  }
  g->insertChild(o);
  if (needUpdate) {
    _battle->ships()->initAfterParse();
    _battle->trigBattleObjectAdded(o);
  }
  _battle->addBattleInfoFlags(BFShips);
  _battle->addBattleInfoFlags(BFGrouping);
}

QDomElement BEventGroupAdd::toXml(QDomDocument& doc) const {
  QDomElement e = BEvent::toXml(doc);
  e.setAttribute(BEvent::BE_EVENT_TYPE, BE_EVENT_TYPE_GROUP_ADD);
  e.setAttribute(OBJECT, objectName());
  e.setAttribute(GROUP, _groupName);
  return e;
}

QString BEventGroupAdd::description() const {
  return qApp->translate("BEventGroupCreate",
    "Add object '%1' to group '%2'").arg(objectName()).arg(_groupName);
}

/** \brief 
*/
BEventGroupRemove::BEventGroupRemove(const QDateTime& dt) throw()
:BEventGroup("", dt) {
  _battle = 0;
}

BEventGroupRemove::BEventGroupRemove(const QString& objectName, 
  Battle* battle, const QDateTime& dt) throw()
  :BEventGroup(objectName, dt), _battle(battle) {
}

/** \brief 
*/
BEventGroupRemove::~BEventGroupRemove(){
}

void BEventGroupRemove::parse(const QDomElement& element, Battle* battle) {
  setEventFlags(eventFlagFromString(
    BDomUtils::stringAttribute(element, EVENT_FLAGS)));
  _battle = battle;
  setObjectName(BDomUtils::stringAttribute(element, OBJECT));
}

void BEventGroupRemove::exec() {
  if (_battle->ships()->unUsedShips() != 0) {
    _battle->trigBattleObjectRemoved(object());
    _battle->ships()->setUnused(object());
  }
  else {
    //    ,  ,    
    if (!object()->isGroup()) {
      return;
    }
    //     
    if (objectName() == _battle->ships()->name()) {
      throw BInternalException(__FILE__, __LINE__,
      qApp->translate("BEventGroupRemove", 
      "Requested to remove top level group"));
    }
    //    ,     
    //   ,    
    BattleObjects bos = object()->objs(BattleObject::LeafsOnlySelectionType);
    for(BattleObjects::iterator it = bos.begin(); it != bos.end(); it++) {
      BattleObject* bo = *it;
//      BEvent* rEvent = new BEventGroupAdd(bo->name(), 
//        _battle->ships()->name(), _battle, date());
//      rEvent->setEventFlags(eventFlags() || SystemEventFlag);
//      _battle->postEvent(rEvent);
      _battle->ships()->insertChild(bo);
    }
    _battle->trigBattleObjectRemoved(object());
    _battle->ships()->setUnused(object());
    _battle->addBattleInfoFlags(BFGrouping);
  }
  _battle->addBattleInfoFlags(BFShips);
}

QDomElement BEventGroupRemove::toXml(QDomDocument& doc) const {
  QDomElement e = BEvent::toXml(doc);
  e.setAttribute(BEvent::BE_EVENT_TYPE, BE_EVENT_TYPE_GROUP_REMOVE);
  e.setAttribute(OBJECT, objectName());
  return e;
}

QString BEventGroupRemove::description() const {
  return qApp->translate("BEventGroupCreate",
    "Remove object '%1' from its group").arg(objectName());
}


/** \brief 
*/
BEventGroupRename::BEventGroupRename(const QDateTime& dt) throw()
  :BEventGroup("", dt) {
}

BEventGroupRename::BEventGroupRename(const QString& oldName, 
  const QString& newName, Battle* battle, 
  const QDateTime& dt) throw()
  :BEventGroup(oldName, dt), _newName(newName), _battle(battle) {
}

/** \brief 
*/
BEventGroupRename::~BEventGroupRename(){
}

void BEventGroupRename::parse(const QDomElement& element, Battle* battle) {
  setEventFlags(eventFlagFromString(
    BDomUtils::stringAttribute(element, EVENT_FLAGS)));
  _battle = battle;
  setObjectName(BDomUtils::stringAttribute(element, OLD_NAME));
  _newName = BDomUtils::stringAttribute(element, NEW_NAME);
}

void BEventGroupRename::exec() {
  object()->setName(_newName);
  _battle->addBattleInfoFlags(BFGrouping);
}

QDomElement BEventGroupRename::toXml(QDomDocument& doc) const {
  QDomElement e = BEvent::toXml(doc);
  e.setAttribute(BEvent::BE_EVENT_TYPE, BE_EVENT_TYPE_GROUP_RENAME);
  e.setAttribute(OLD_NAME, objectName());
  e.setAttribute(NEW_NAME, _newName);
  return e;
}

QString BEventGroupRename::description() const {
  return qApp->translate("BEventGroupCreate",
    "Rename object '%1' to '%2'").arg(objectName()).arg(_newName);
}
