/******************************************************************************/
/* BEventGroup -   BattleFleet              */
/******************************************************************************/

#ifndef BEventGroup_H
#define BEventGroup_H
static const char beventgroup_h_id[]="@(#) $Id: beventgroup.h,v 1.7 2004/08/08 15:20:36 Oleksiy Balabay Exp $"; static const char* beventgroup_h_id_id=beventgroup_h_id;
/** \file beventgroup.h
  \brief  BEventGroup  BattleFleet

     BEventGroup  BattleFleet
*/
#include <qdom.h>

#include <set>

#include "bevent.h"

typedef std::set<BattleObject*> BattleObjects;

class BEventGroup: public BEvent {
public:
  BEventGroup(const QString& objectName = "", 
    const QDateTime& dt = QDateTime()) throw();
  virtual ~BEventGroup();
};

class BEventGroupCreate: public BEventGroup {
public:
  BEventGroupCreate(const QString& parent = "", 
    const QDateTime& dt = QDateTime()) throw();
  BEventGroupCreate(const QString& parent,
    Battle* battle, const QString& name, const int side,
    const BattleObjects& members, const QDateTime& dt= QDateTime());
  virtual ~BEventGroupCreate();
  virtual void exec();
  virtual void parse(const QDomElement& element, Battle* battle); 
  virtual QDomElement toXml(QDomDocument& doc) const;
  virtual QString description() const;
public:
  static const QString BE_EVENT_TYPE_GROUP_CREATE;
  static const QString SOM_SIDE;
private:
  BattleObject* parseGroup(const QDomElement& e, BattleObject* parent);
  BattleObject* parseShip(const QDomElement& e, BattleObject* parent);
private:
  QDomElement _groupEl;
  Battle* _battle;
  int _side;
};

class BEventGroupAdd: public BEventGroup {
public:
  BEventGroupAdd(const QString& objectName = "", 
    const QDateTime& dt = QDateTime()) throw();
  BEventGroupAdd(const QString& objectName, const QString& groupName, 
    Battle* battle, const QDateTime& dt = QDateTime()) throw();
  virtual ~BEventGroupAdd();
  virtual void exec();
  virtual void parse(const QDomElement& element, Battle* battle); 
  virtual QDomElement toXml(QDomDocument& doc) const;
  virtual QString description() const;
public:
  static const QString BE_EVENT_TYPE_GROUP_ADD;
  static const QString OBJECT;
  static const QString GROUP;
private:
  Battle* _battle;
  QString _groupName;
};

class BEventGroupRemove: public BEventGroup {
public:
  BEventGroupRemove(const QDateTime& dt = QDateTime()) throw();
  BEventGroupRemove(const QString& objectName, Battle* battle, 
    const QDateTime& dt = QDateTime()) throw();
  virtual ~BEventGroupRemove();
  virtual void exec();
  virtual void parse(const QDomElement& element, Battle* battle); 
  virtual QDomElement toXml(QDomDocument& doc) const;
  virtual QString description() const;
public:
  static const QString BE_EVENT_TYPE_GROUP_REMOVE;
  static const QString OBJECT;
private:
  Battle* _battle;
};

class BEventGroupRename: public BEventGroup {
public:
  BEventGroupRename(const QDateTime& dt = QDateTime()) throw();
  BEventGroupRename(const QString& oldName, 
    const QString& newName, Battle* battle, 
    const QDateTime& dt= QDateTime()) throw();
  virtual ~BEventGroupRename();
  virtual void exec();
  virtual void parse(const QDomElement& element, Battle* battle); 
  virtual QDomElement toXml(QDomDocument& doc) const;
  virtual QString description() const;
public:
  static const QString BE_EVENT_TYPE_GROUP_RENAME;
  static const QString OLD_NAME;
  static const QString NEW_NAME;
private:
  QString _newName;
  Battle* _battle;
};

#endif // BEventGroup_H
