/******************************************************************************/
/* BEventsParser -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char beventsparser_cpp_id[]="@(#) $Id: beventsparser.cpp,v 1.13 2005/01/05 11:47:23 Oleksiy Balabay Exp $"; static const char* beventsparser_cpp_id_id=beventsparser_cpp_id;

#include <qfile.h>
#include <qxml.h>
#include <qfileinfo.h>

#include "beventsparser.h"
#include "bdomutils.h"
#include "bevent.h"
#include "battle.h"
#include "bships.h"
#include "beventprogram.h"
#include "beventgroup.h"
#include "beventstate.h"
#include "beventclear.h"
#include "beventcollision.h"
#include "beventshandler.h"
#include "btmpfiles.h"
#include "bapplication.h"

/** \file beventsparser.cpp
  \brief  BEventsParser  BattleFleet (impl.)

     BEventsParser  BattleFleet
*/
const QString BEventsParser::SOM_DATE = "Date";
const QString BEventsParser::SOM_OBJECT = "Object";
const QString BEventsParser::SOM_NAME = "Name";
const QString BEventsParser::SOM_EVENTS = "Events";

const unsigned int BEventsParser::PROGRESS_LOW = 40;
const unsigned int BEventsParser::PROGRESS_HIGH = 59;
const unsigned int BEventsParser::AVERAGE_EVENT_SIZE = 300;
/** \class BEventsParser
  \brief 


*/
/** \brief 
*/
BEventsParser::BEventsParser(const QString& fileName, Battle* battle) 
  : _battle(battle), _eventsProcessed(0), _eventsPer1Progress(0) {
  _origFileName = fileName;
  _fileName = BTmpFiles::instance()->createFileCopy(fileName);
  QFileInfo fi(_fileName);
  _eventsPer1Progress = 
    (fi.size()/AVERAGE_EVENT_SIZE)/(PROGRESS_HIGH - PROGRESS_LOW)+1;
}

/** \brief 
*/
BEventsParser::~BEventsParser(){
  QFile f(_fileName);
  f.remove();
}

void BEventsParser::parse(){
  try {
    QFile f(_fileName);
    if (!f.open(IO_ReadOnly)) {
      throw BInputException(__FILE__, __LINE__, _fileName);
    }
    QXmlSimpleReader reader;
    // ,  .   Dom
  	reader.setFeature( "http://xml.org/sax/features/namespaces", FALSE );
  	reader.setFeature( "http://xml.org/sax/features/namespace-prefixes", TRUE );
    reader.setFeature( "http://trolltech.com/xml/features/report-whitespace-only-CharData", FALSE );

    BEventsHandler hnd( this );
    reader.setContentHandler( &hnd );
    reader.setErrorHandler( &hnd );
    reader.setLexicalHandler( &hnd );
    reader.setDeclHandler( &hnd );
    reader.setDTDHandler( &hnd );

    if ( !reader.parse( &f ) ) {
      throw BConfigException(__FILE__, __LINE__,
        qApp->translate("BEventsParser", 
        "Error in line '%1', column '%2': '%3'").arg(hnd.errorLine).arg(
        hnd.errorColumn).arg(hnd.errorMsg));
    }
  }
  catch(BConfigException& e) {
    throw BFileException(e, _origFileName);
  }
  catch (BException&) {
    throw;
  }
  catch(...) {
    throw BInternalException(__FILE__, __LINE__,
      qApp->translate("BEventsParser",
      "Error loading file '%1'").arg(_origFileName));
  }
}

void BEventsParser::parseDate(const QDomElement& element) {
  _date = BDomUtils::dateTimeFromXml(element);
  QDomNode n = element.firstChild();
  while( !n.isNull() ) {
    QDomElement e = n.toElement(); // try to convert the node to an element.
    n = n.nextSibling();
    if( !e.isNull() ) { // the node was really an element.
      try {
        if ( e.tagName() == SOM_OBJECT ) {
          parseObject(e);
        }
        else {
          qWarning("Unsupported tag '%s' in file '%s'", 
            (const char*)e.tagName().local8Bit(),
            (const char*)_fileName.local8Bit());
        }
      }
      catch (BConfigException& ex) {
        throw BConfigException(ex.sourceCodeFile(),
          ex.sourceCodeLine(), ex.message()+"\n"+qApp->translate("BEventsParser",
          "Error in element %1").arg(BDomUtils::elementToString(e)));
      }
      catch (BException&) {
        throw;
      }
      catch (...) {
        throw BInternalException(__FILE__, __LINE__,
          qApp->translate("BEventsParser",
          "Unknown error parsing element %1").arg(
          BDomUtils::elementToString(e)));
      }
    }
  }
}

void BEventsParser::parseObject(const QDomElement& element) {
  QString objectName = BDomUtils::stringAttribute(element, SOM_NAME);
  QDomNode n = element.firstChild();
  while( !n.isNull() ) {
    QDomElement e = n.toElement(); // try to convert the node to an element.
    n = n.nextSibling();
    if( !e.isNull() ) { // the node was really an element.
      if (e.tagName() == BEvent::BE_EVENT) {
//        parseEvent(e, obj);
        parseEvent(e, objectName);
      }
      else {
        qWarning("Unsupported tag '%s' in file '%s'", 
          (const char*)element.tagName().local8Bit(),
          (const char*)_fileName.local8Bit());
      }
    }
  }
}

void BEventsParser::parseEvent(const QDomElement& element, 
  const QString& objectName) {
  _eventsProcessed++;
  if (_eventsProcessed%_eventsPer1Progress == 0) {
    bApp->checkProgressDialog(
      PROGRESS_LOW+_eventsProcessed/_eventsPer1Progress);
  }
  QString evType = BDomUtils::stringAttribute(element, BEvent::BE_EVENT_TYPE);
  BEvent* event = 0;
  if (BEventProgram::BE_EVENT_TYPE_PROGRAM == evType) {
    event = new BEventProgram(objectName, _date);
  }
  else if (BEventGroupCreate::BE_EVENT_TYPE_GROUP_CREATE == evType) {
    event = new BEventGroupCreate(objectName, _date);
  }
  else if (BEventGroupAdd::BE_EVENT_TYPE_GROUP_ADD == evType) {
    event = new BEventGroupAdd(objectName, _date);
  }
  else if (BEventGroupRename::BE_EVENT_TYPE_GROUP_RENAME == evType) {
    event = new BEventGroupRename(_date);
  }
  else if (BEventState::BE_EVENT_TYPE_STATE == evType) {
    event = new BEventState(objectName, _date);
  }
  else if (BEventClear::BE_EVENT_TYPE_CLEAR == evType) {
    event = new BEventClear(objectName, _date);
  }
  else if (BEventCollision::BE_EVENT_TYPE_COLLISION == evType) {
    event = new BEventCollision(objectName, _date);
  }
  else if (BEventGroupRemove::BE_EVENT_TYPE_GROUP_REMOVE == evType) {
    event = new BEventGroupRemove(_date);
  }
  else if (BEventSide::BE_EVENT_TYPE_SIDE == evType) {
    event = new BEventSide(objectName, _date);
  }
  else {
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate("BEventParser", "Unsupported event type: '%1'").arg(
      evType));
  }
  event->parse(element, _battle);
  _battle->postEvent(event);
}
