/******************************************************************************/
/* BEventsSaver -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char beventssaver_cpp_id[]="@(#) $Id: beventssaver.cpp,v 1.3 2004/10/06 10:25:05 Oleksiy Balabay Exp $"; static const char* beventssaver_cpp_id_id=beventssaver_cpp_id;

#include "beventssaver.h"
#include "bexception.h"
#include "beventsparser.h"
#include "btmpfiles.h"

/** \file beventssaver.cpp
  \brief  BEventsSaver  BattleFleet (impl.)

     BEventsSaver  BattleFleet
*/

/** \class BEventsSaver
  \brief   .

     ,     .
        add().

   beginSave()     .  
  -   ,   addToSave().  
    endSave(). 

     beginSave()   endSave(). 
   beginSave()  endSave()  add()    
    BInternalException().

     -      QDom,   , 
        QDom'.

*/
/** \brief 
*/
BEventsSaver::BEventsSaver():_tmpFile(temporaryFileName()) {
  if (!_tmpFile.open(IO_WriteOnly)) {
    throw BOutputException(__FILE__, __LINE__, _tmpFile.name());
  }
  _os.setDevice(&_tmpFile);
  _os.setEncoding(QTextStream::UnicodeUTF8);
  _os << "<?xml version='1.0' encoding='UTF-8'?>"<< "\n";
  _os << "<" << BEventsParser::SOM_EVENTS << ">\n";
}

/** \brief 
*/
BEventsSaver::~BEventsSaver(){
  _tmpFile.close();
  _tmpFile.remove();
}

void BEventsSaver::add(const QString& s) {
  _os << s <<"\n";
}

void BEventsSaver::beginSave(const QString& fileName) {
  _tmpFile.flush();
  _tmpFile.close();
  copyTextFile(_tmpFile.name(), fileName);
}

void BEventsSaver::addToSave(const QString& s) {
  if ( (_df.state() & IO_Open) != IO_Open) {
    throw BInternalException(__FILE__, __LINE__, 
      QString("Output file descriptor already open"));
  }
  if ( _ds.device() == 0) {
    throw BInternalException(__FILE__, __LINE__, 
      QString("Output stream not open"));
  }
  _ds << s << "\n";
}

void BEventsSaver::endSave() {
  if ( (_df.state() & IO_Open) != IO_Open) {
    throw BInternalException(__FILE__, __LINE__, 
      QString("Output file descriptor already open"));
  }
  if ( _ds.device() == 0) {
    throw BInternalException(__FILE__, __LINE__, 
      QString("Output stream not open"));
  }
  _ds << "</"+BEventsParser::SOM_EVENTS << ">\n";
  _ds.unsetDevice();
  _df.close();

  if (!_tmpFile.open(IO_WriteOnly | IO_Append)) {
    throw BOutputException(__FILE__, __LINE__, _tmpFile.name());
  }
}

QString BEventsSaver::temporaryFileName() {
  BTmpFiles* t = BTmpFiles::instance();
  return t->createFileName("bftmp.xml");
}

void BEventsSaver::copyTextFile(const QString& s, const QString& d) {
  if ( (_df.state() & IO_Open) == IO_Open) {
    throw BInternalException(__FILE__, __LINE__, 
      QString("Output file descriptor already open"));
  }
  if ( _ds.device() != 0) {
    throw BInternalException(__FILE__, __LINE__, 
      QString("Output stream already open"));
  }

  _df.setName(d);
  _df.remove();
  QFile sf(s);
  if (!sf.open(IO_ReadOnly)) {
    throw BInputException(__FILE__, __LINE__, s);
  }
  if (!_df.open(IO_WriteOnly)) {
    throw BOutputException(__FILE__, __LINE__, d);
  }
  
  QTextStream ss(&sf);
  ss.setEncoding(QTextStream::UnicodeUTF8);
  _ds.setDevice(&_df);
  _ds.setEncoding(QTextStream::UnicodeUTF8);
  while (!ss.atEnd()) {
    _ds << ss.readLine() << "\n";
  }
}
