/******************************************************************************/
/* BEventState -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char beventstate_cpp_id[]="@(#) $Id: beventstate.cpp,v 1.8 2005/01/21 16:17:59 Oleksiy Balabay Exp $"; static const char* beventstate_cpp_id_id=beventstate_cpp_id;

#include "beventstate.h"
#include "battleobject.h"
#include "bdomutils.h"
#include "bscenarioheader.h"
#include "battle.h"
#include "bapplication.h"

/** \file beventstate.cpp
  \brief  BEventState  BattleFleet (impl.)

     BEventState  BattleFleet
*/

/** \class BEventState
  \brief 


*/
const QString BEventState::BE_EVENT_TYPE_STATE = "State";
const QString BEventSide::BE_EVENT_TYPE_SIDE = "Side";
const QString BEventSide::SIDE = "SideName";
/** \brief 
*/
BEventState::BEventState(const QString& objectName, 
  const QDateTime& dt) throw():BEvent(objectName, dt) {
//  setEventType(SystemEventType);
}

/** \brief 
*/
BEventState::~BEventState(){
}

void BEventState::exec() {
  object()->setState(_el);
  bApp->battle()->addBattleInfoFlags(BFSide);
}

void BEventState::parse(const QDomElement& element, Battle* /* battle */) {
  setEventFlags(eventFlagFromString(
    BDomUtils::stringAttribute(element, EVENT_FLAGS)));
  bool stateFound = false;
  QDomNode n = element.firstChild();
  while( !n.isNull() ) {
    QDomElement e = n.toElement(); // try to convert the node to an element.
    n = n.nextSibling();
    if( !e.isNull() ) { // the node was really an element.
      if (e.tagName() == BattleObject::BO_STATE) {
        _el = e;
        stateFound = true;
        break;
      }
    }
  }
  if (!stateFound) {
    BDomUtils::elementDoesNotExist(BattleObject::BO_STATE);
  }
}

QDomElement BEventState::toXml(QDomDocument& doc) const {
  QDomElement e = BEvent::toXml(doc);
  e.setAttribute(BEvent::BE_EVENT_TYPE, BE_EVENT_TYPE_STATE);
  e.appendChild(_el);
  return e;
}

QString BEventState::description() const {
  return qApp->translate("BEventState",
    "Restore state for '%1'").arg(objectName());
}

/** \brief 
*/
BEventSide::BEventSide(const QString& objectName, 
  const QDateTime& dt) throw()
  :BEvent(objectName, dt), _side(NO_SIDE) {

}

BEventSide::BEventSide(const QString& objectName, int side,
  const QDateTime& dt):BEvent(objectName, dt), _side(side) {
}

/** \brief 
*/
BEventSide::~BEventSide(){
}

void BEventSide::exec() {
  object()->setSide(_side);
  bApp->battle()->addBattleInfoFlags(BFSide);
}

void BEventSide::parse(const QDomElement& element, Battle* battle) {
  setEventFlags(eventFlagFromString(
    BDomUtils::stringAttribute(element, EVENT_FLAGS)));
  if (!element.hasAttribute(SIDE)) {
    _side = NO_SIDE;
    return;
  }
  QString sideName = BDomUtils::stringAttribute(element, SIDE).upper();
  QString side1 = battle->scenarioHeader()->sideHeader(1)->name().upper();
  QString side2 = battle->scenarioHeader()->sideHeader(2)->name().upper();
  if ( ( side1 != sideName) && ( side2 != sideName) ) {
      throw BConfigException(__FILE__, __LINE__,
        qApp->translate("BEventSide", 
        "Bad side '%1'. Should be '%2' or '%3'").arg(sideName).arg(
        side1).arg(side2));
    }
  _side = (side1 == sideName)?1:2;
}

QDomElement BEventSide::toXml(QDomDocument& doc) const {
  QDomElement e = BEvent::toXml(doc);
  e.setAttribute(BEvent::BE_EVENT_TYPE, BE_EVENT_TYPE_SIDE);
  if (_side != NO_SIDE) {
    e.setAttribute(SIDE, 
      bApp->battle()->scenarioHeader()->sideHeader(_side)->name().upper());
  }
  return e;
}

QString BEventSide::description() const {
  return qApp->translate("BEventSide",
    "Change side for '%1' to '%2'").arg(objectName()).arg(
    _side == NO_SIDE ? qApp->translate("BEventSide", "Neutral"):
    bApp->battle()->scenarioHeader()->sideHeader(_side)->name().upper());
}

