/******************************************************************************/
/* BException -   BattleFleet              */
/******************************************************************************/

#ifndef BException_H
#define BException_H
static const char bexception_h_id[]="@(#) $Id: bexception.h,v 1.6 2005/01/21 16:25:03 Oleksiy Balabay Exp $"; static const char* bexception_h_id_id=bexception_h_id;
/** \file bexception.h
  \brief  BException  BattleFleet

     BException  BattleFleet
*/
#include <qstring.h>
#include <qapplication.h>
#include <qdatetime.h>

//#define B_MESSAGE(message) (qApp->translate("BException", "File %1. Line: %2. %3").arg(__FILE__).arg(__LINE__).arg(message))
   
class BException {
public:
  BException(const QString& sourceCodeFile, const int sourceCodeLine, 
    const QString& message):_sourceCodeFile(sourceCodeFile),
    _sourceCodeLine(sourceCodeLine), _message(message){
  };
  virtual ~BException(){
  };
  virtual const QString message() const {
#ifdef QT_DEBUG
    return qApp->translate("BException", "File: %1, Line: %2. \n%3").arg(
      sourceCodeFile()).arg(sourceCodeLine()).arg(_message);
#else
    return _message;
#endif
  }
  const QString sourceCodeFile() const {
    return _sourceCodeFile;
  }
  int sourceCodeLine() const {
    return _sourceCodeLine;
  }
private:
  const QString _sourceCodeFile;
  const int _sourceCodeLine;
protected:
  QString _message;
};

class BConfigException: public BException {
public:
  BConfigException(const QString& sourceCodeFile, const int sourceCodeLine, 
    const QString& message)
    :BException(sourceCodeFile, sourceCodeLine, message) {
  }
  virtual ~BConfigException() {
  }
};

class BOutputException: public BConfigException {
public:
  BOutputException(const QString& sourceCodeFile, const int sourceCodeLine, 
    const QString& fileName)
    :BConfigException(sourceCodeFile, sourceCodeLine, 
    qApp->translate("BOutputException", "Error writing to file '%1'").arg(
      fileName)) {
  }
  virtual ~BOutputException() {
  }
};

class BInputException: public BConfigException {
public:
  BInputException(const QString& sourceCodeFile, const int sourceCodeLine, 
    const QString& fileName)
    :BConfigException(sourceCodeFile, sourceCodeLine, 
    qApp->translate("BOutputException", "Error reading from file '%1'").arg(
      fileName)) {
  }
  virtual ~BInputException() {
  }
};

class BFileException: public BException {
public:
  BFileException(const BConfigException& e, const QString& configFile,
    const int configFileLine = -1, const int configFileColumn = -1)
    :BException(e.sourceCodeFile(), e.sourceCodeLine(), e.message()){
    _message = qApp->translate("BFileException",
      "%1\nError processing file '%2'").arg(_message).arg(configFile);
    if ((-1 != configFileLine) || (-1 != configFileColumn)) {
      _message = qApp->translate("BFileException",
      "%1 Line '%2', Column '%3'").arg(_message).arg(configFileLine).arg(
      configFileColumn);
    }
  }
  virtual ~BFileException() {
  }
};

class BInternalException: public BException {
public:
  BInternalException(const QString& sourceCodeFile, const int sourceCodeLine, 
    const QString& message)
    :BException(sourceCodeFile, sourceCodeLine, message){
  }
  virtual const QString message() const {
    return qApp->translate("BException", "File: %1, Line: %2. \n%3").arg(
      sourceCodeFile()).arg(sourceCodeLine()).arg(_message);
  }
  virtual ~BInternalException() {
  }
};

class BRangeException: public BInternalException {
public:
  BRangeException(const QString& sourceCodeFile, const int sourceCodeLine, 
    const int lowLimit, const int highLimit, const int value)
    :BInternalException(sourceCodeFile, sourceCodeLine, ""){
    _message = qApp->translate("BRangeException",
      "Range limits: from %1 to %2. Value %3 requested").arg(lowLimit).arg(
      highLimit).arg(value);
    }
  virtual ~BRangeException() {
  };
};

class BTimeException: public BInternalException {
public:
  BTimeException(const QString& sourceCodeFile, const int sourceCodeLine, 
    const QDateTime& t, const QString& message)
    :BInternalException(sourceCodeFile, sourceCodeLine, ""){
    _message = qApp->translate("BTimeException",
      "Incorrect time: %1. %2").arg(t.toString()).arg(message);
    }
  virtual ~BTimeException() {
  };
};

class BPtrException: public BInternalException {
public:
  BPtrException(const QString& sourceCodeFile, const int sourceCodeLine, 
    const QString& name, const QString& type)
    :BInternalException(sourceCodeFile, sourceCodeLine, ""){
    _message = qApp->translate("BPtrException",
      "Pointer %1 of type %2 used when 0").arg(name).arg(type);
    }
  virtual ~BPtrException() {
  };
};


#endif // BException_H
