/******************************************************************************/
/* BGroupView -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bgroupview_cpp_id[]="@(#) $Id: bgroupview.cpp,v 1.12 2005/02/20 14:44:58 Oleksiy Balabay Exp $"; static const char* bgroupview_cpp_id_id=bgroupview_cpp_id;

#include <qdragobject.h>
#include <qwhatsthis.h>

#include <algorithm>
#include <iterator>
#include <vector>

#include "bgroupview.h"
#include "battleobject.h"
#include "xpms/xpms.h"
#include "bapplication.h"
#include "battle.h"
#include "bships.h"
#include "buistate.h"
#include "bmainwindowtexts.h"

/** \file bgroupview.cpp
  \brief  BGroupView  BattleFleet (impl.)

     BGroupView  BattleFleet
*/

/** \class BGroupView
  \brief       


*/
const char* B_GROUP_VIEW = 
  QT_TRANSLATE_NOOP("BGroupView", "Ships tree");

BGroupView::BGroupView(BUiState* uiState, QWidget* parent, const char* name)
  :QDockWindow(parent, name), _uiState(uiState) {
  QWhatsThis::add(this, tr(
    "<b>%2</b><br><br>"
    "Use this window to arrange ships in groups and to select ships or groups. "
    "To create new group use <img source=\"group.png\"> "
    "button in <b>%1</b> toolbar. You can drag'n'drop selected ships and groups").arg(qApp->translate("BMainWindowState",
    shipsToolBar_N)).arg(tr(B_GROUP_VIEW)));
  setCloseMode(Always);
  setResizeEnabled(true);
  setMovingEnabled(true);
  setCaption(tr(B_GROUP_VIEW));
//  init();
//  _view->show();
//  connect(_view, SIGNAL(currentChanged(QListViewItem*)), 
//    this, SLOT(slotViewCurrentChanged(QListViewItem*))); 
}


BGroupView::~BGroupView() {
}

BGroupViewList* BGroupView::createGroupViewList() {
  return new BGroupViewList(this, "groupviewlist");
}

void BGroupView::init() {
  _view = createGroupViewList();
  setWidget(_view);
  _view->initView(bApp->battle()->ships(), 
    bApp->battle()->ships()->unUsedShips());
  slotSelectionChanged(BUiSelectionInfo(_uiState->activeObject(), 
    &_uiState->selectedObjects()));
  _view->show();
  connect(_view, SIGNAL(selectionChanged()), 
    this, SLOT(slotViewSelectionChanged())); 
  connect(_view, SIGNAL(itemDeleted(const QString&)), 
    this, SLOT(slotViewItemDeleted(const QString&))); 
  connect(_view, SIGNAL(itemDropped(const QString&, const QString&)), 
    this, SLOT(slotViewItemDropped(const QString&, const QString&))); 
}

void BGroupView::updateView(const BattleInfo& bi) {
  if ( ((bi.flags & BFGrouping) != 0) || ((bi.flags & BFShips) != 0)
    || ((bi.flags & BFSide) != 0)) {
    _view->updateView(bApp->battle()->ships(), 
      bApp->battle()->ships()->unUsedShips());
    slotSelectionChanged(BUiSelectionInfo(_uiState->activeObject(), 
      &_uiState->selectedObjects()));
  }
}

void BGroupView::slotSelectionChanged(const BUiSelectionInfo& selectionInfo) {
  _view->setSelectedObjects(selectionInfo);
//  _view->setActiveObject(selectionInfo.activeObject);
}

/*void BGroupView::slotViewCurrentChanged(QListViewItem* item) {
  qDebug("slotCurrentChanged called");
  if (item) {
    qDebug("Item: %s", (const char*)item->text(0).local8Bit());
  }
  if (_view->dragState())
    return;
  if ((item == 0) || (item == _view->firstChild())){
    _uiState->setActiveObjectAsSelection(0);
  }
  else {
    BattleObject* o = bApp->battle()->ships()->find(item->text(0));
//    if (o->isGroup()) {
//      _uiState->addSelectedObjects(o->objs(
//        BattleObject::LeafsOnlySelectionType));
//    }
    _uiState->setActiveObjectAsSelection(o);
  }
}*/

/*template<class _II1, class _II2, class _OI> inline
	_OI b_set_difference(_II1 _F1, _II1 _L1, _II2 _F2, _II2 _L2,
		_OI _X)
	{for (; _F1 != _L1 && _F2 != _L2; )
		if (*_F1 < *_F2)
			*_X++ = *_F1++;
		else if (*_F2 < *_F1)
			++_F2;
		else
			++_F1, ++_F2;
	return _X; }
*/
void BGroupView::slotViewSelectionChanged() {
  if (_view->dragState())
    return;
  //   selected
  QListViewItemIterator it(_view, QListViewItemIterator::Selected);
  BattleObjects bos;
  BShips* shp = bApp->battle()->ships();
  while (it.current()) {
    BattleObject* o = shp->find(it.current()->text(0));
    if (!o) {
      throw BInternalException(__FILE__, __LINE__, 
        QString("Object '%1' not found").arg(it.current()->text(0)));
    }
    bos.insert(o);
    it++;
  }
  //        .
  typedef std::vector<BattleObject*> BOV;
  BOV newSelected;
  std::set_difference<BattleObjects::const_iterator,
    BattleObjects::const_iterator, std::back_insert_iterator<BOV> >(
    bos.begin(), bos.end(), _uiState->selectedObjects().begin(),
    _uiState->selectedObjects().end(), 
    std::back_inserter(newSelected));
  BattleObjects ns;
  for(BOV::iterator i = newSelected.begin();  i!=newSelected.end(); i++) {
    ns.insert(*i);
  }
  BOV deSelected;
  std::set_difference<BattleObjects::const_iterator,
    BattleObjects::const_iterator, std::back_insert_iterator<BOV> >(
    _uiState->selectedObjects().begin(),
    _uiState->selectedObjects().end(), bos.begin(), bos.end(), 
    std::back_inserter(deSelected));
  BattleObjects ds;
  for(BOV::iterator ii = deSelected.begin();  ii!=deSelected.end(); ii++) {
    ds.insert(*ii);
  }
  BSelectionEvent* se = new BSelectionEvent(ns, ds);
  QApplication::postEvent(_uiState, se);
//  _uiState->changeSelection(ns, ds);
}

void BGroupView::slotViewItemDeleted(const QString& name) {
  _uiState->removeObject(name);
}

void BGroupView::slotViewItemDropped(const QString& dropped, 
  const QString& target) {
//  BShips* ships = bApp->battle()->ships();
  //     -  
  QStringList l = QStringList::split("\t", dropped);
  //  
  BattleObjects os;
  for(QStringList::const_iterator it = l.begin(); it != l.end(); it++) {
    BattleObject* bo = bApp->battle()->ships()->find(*it);
    if (bo == 0)
    throw BInternalException(__FILE__, __LINE__,
      QString("Object '%1' specified in dropped list, "
      "but not found in battle objects").arg(*it));
    else
      os.insert(bo);
  }
  QString target1 = target;
  if (target == _view->firstChild()->text(0)) {
    target1 = bApp->battle()->ships()->name();
  }
  BRegroupEvent* re = new BRegroupEvent(os, target1);
  QApplication::postEvent(_uiState, re);
/*  BattleObject* t = ships->find(target);
  if (!t->isGroup())
    t = t->parent();
  {  // it - redefinition, ...
    for(BattleObjects::iterator it = os.begin(); it != os.end(); it++) {
      _uiState->moveObject(*it, t);
    }
  }*/

//  slotSelectionChanged(BUiSelectionInfo(_uiState->activeObject(), 
//    &_uiState->selectedObjects()));
}

BGroupViewList::BGroupViewList(QWidget *parent, const char *name)
  :QListView(parent, name), oldCurrent(0), dropItem(0), _dragState(false),
  _needRestart(false) {
  addColumn(tr("Name"));
  setColumnWidthMode(0, Maximum);
  folderClosed = new QPixmap( folder_closed_xpm );
  folderOpen = new QPixmap( folder_open_xpm );
  setSelectionMode(Extended);
  autoopen_timer = new QTimer( this );
  setAcceptDrops( TRUE );
  viewport()->setAcceptDrops( TRUE );

  connect( autoopen_timer, SIGNAL( timeout() ),
	  this, SLOT( openFolder() ) );
}

BGroupViewList::~BGroupViewList() {
  delete folderClosed;
  delete folderOpen;
}

void BGroupViewList::setOpen(QListViewItem* item, bool open) {
  if ( open )
  	item->setPixmap(0, *folderOpen);
  else
	  item->setPixmap( 0, *folderClosed );
  QListView::setOpen(item, open);
}

void BGroupViewList::initView(const BattleObject* topObject,
  const BattleObject* skip) {
  BattleObjects bo = topObject->objs();
  BGroupViewListItem* root = new BGroupViewListItem(this, 
    /*topObject->name()*/ tr("All ships"));
  root->setExpandable(true);
  root->setPixmap(0, *folderClosed);
  root->setSelectable(false);

  for (BattleObjects::iterator it = bo.begin(); it != bo.end(); it++) {
    if (*it != skip) {
      BGroupViewListItem* item = new BGroupViewListItem(root, (*it)->name());
      parseBattleObject(item, *it);
    }
  }
}

void BGroupViewList::parseBattleObject(QListViewItem* parent,
  const BattleObject* battleObject) {
  parent->setDragEnabled(dragSupported());
  BattleObjects bo = battleObject->objs();
  if (!battleObject->isGroup()) {
    ((BGroupViewListItem*)parent)->setSide(battleObject->side(), battleObject);
    return;
  }
  parent->setExpandable(true);
  parent->setPixmap(0, *folderClosed);
  for (BattleObjects::iterator it = bo.begin(); it != bo.end(); it++) {
    BGroupViewListItem* item = new BGroupViewListItem(
      parent, (*it)->name());
    parseBattleObject(item, *it);
  }
}

QDragObject* BGroupViewList::dragObject() {
  if (!dragSupported())
    return 0;
  //   
  QListViewItemIterator it(this, QListViewItemIterator::Selected);
  QString s;
  while (it.current()) {
    s+=(*it)->text(0)+"\t";
    it++;
  }
  BGroupViewListItem* itm = (BGroupViewListItem*)currentItem();
  QDragObject* d = new QTextDrag(s, this);
  d->setPixmap(*itm->pixmap(0));
  return d;
}

void BGroupViewList::updateView(const BattleObject* topObject, 
  const BattleObject* skip) {
  QListViewItem* root = this->firstChild();
  updateBattleObject(root, topObject,  skip);
}

void BGroupViewList::updateBattleObject(QListViewItem* listObject, 
  const BattleObject* battleObject, const BattleObject* skip) {
  if ( (listObject->text(0) != battleObject->name()) && 
    (this->firstChild() != listObject) ) {
    throw BInternalException(__FILE__, __LINE__,
      tr("List item name '%1' != battle object name '%2'").arg(
        listObject->text(0)).arg(battleObject->name()));
  }
  ((BGroupViewListItem*)listObject)->setSide(battleObject->side(), battleObject);
  BattleObjects bo = battleObject->objs();
  for (BattleObjects::iterator it = bo.begin(); it != bo.end(); it++) {
    if (*it != skip) {
      QListViewItem* item = findItem(listObject, (*it)->name());
      if (item != 0) {
        updateBattleObject(item, *it, skip);
      }
      else {
        createBattleObject(listObject, *it, skip);
      }
    }
  }
  QListViewItem* itm = listObject->firstChild();
  while (itm != 0) {
    bool found = false;
    for (BattleObjects::iterator it = bo.begin(); it != bo.end(); it++) {
      if (*it != skip) {
        if ( (*it)->name() == itm->text(0) ) {
          found = true;
          break;
        }
      }
    }
    if (!found) {
      QListViewItem* itm1 = itm;
      itm = itm->nextSibling();
      delete itm1;
    }
    else {
      itm = itm->nextSibling();
    }
  }
}

void BGroupViewList::createBattleObject(QListViewItem* parent, 
  const BattleObject* battleObject, const BattleObject* skip) {
  if (battleObject == skip)
    return;
  BGroupViewListItem* item = new BGroupViewListItem(
    parent, battleObject->name());
  parseBattleObject(item, battleObject);
}

QListViewItem* BGroupViewList::findItem(QListViewItem* parent, 
  const QString& name) {
  QListViewItem* itm = parent->firstChild();
  while (itm != 0) {
    if ( itm->text(0) == name) {
      return itm;
    }
    itm = itm->nextSibling();
  }
  return 0;
}

/*
void BGroupViewList::setActiveObject(const BattleObject* o) {
  if (o == 0) {
    clearSelection();
    setCurrentItem(0);
    QListViewItemIterator it(this);
    while (it.current()) {
      BGroupViewListItem* itm = (BGroupViewListItem*)(it.current());
      itm->setSelectionState(BGroupViewListItem::NotSelected);
      it.current()->repaint();
      ++it;
    }
    return;
  }
  QString name = o->name();
  QListViewItemIterator it(this);
  while (it.current()) {
    if (it.current()->text(0) == name) {
      ensureItemVisible(it.current());
      setCurrentItem(it.current());
      BGroupViewListItem* itm = (BGroupViewListItem*)(it.current());
      itm->setSelectionState(BGroupViewListItem::Active);
      return;
    }
    else {
      BGroupViewListItem* itm = (BGroupViewListItem*)(it.current());
      if (itm->selectionState() == BGroupViewListItem::Active) {
        itm->setSelectionState(BGroupViewListItem::NotSelected);
      }
    }
    it.current()->repaint();
    ++it;
  }
  throw BInternalException(__FILE__, __LINE__,
    tr("Battle object '%1' not found in items list").arg(name));
}
*/

void BGroupViewList::setSelectedObjects(const BUiSelectionInfo& selectionInfo) {
  BattleObject* o = selectionInfo.activeObject;
  const BattleObjects& objs = *selectionInfo.selectedObjects;
  if (objs.size() == 0) {
    clearSelection();
    setCurrentItem(0);
    QListViewItemIterator it(this);
    while (it.current()) {
      BGroupViewListItem* itm = (BGroupViewListItem*)(it.current());
      itm->setSelectionState(BGroupViewListItem::NotSelected);
      ++it;
      it.current()->repaint();
    }
    return;
  }
  std::set<QString> objNames;
  QString name = o->name();
  for(BattleObjects::const_iterator cit=objs.begin(); cit!=objs.end(); cit++) {
    objNames.insert((*cit)->name());
  }
  QListViewItemIterator it(this);
  while (it.current()) {
    BGroupViewListItem* itm = (BGroupViewListItem*)(it.current());
    if (itm->text(0) == name) {
      ensureItemVisible(itm);
      setCurrentItem(itm);
      itm->setSelected(true);
      itm->setSelectionState(BGroupViewListItem::Active);
    }
    else if (objNames.find(itm->text(0)) != objNames.end()) {
      itm->setSelected(true);
      itm->setSelectionState(BGroupViewListItem::Selected);
    }
    else {
      itm->setSelected(false);
      itm->setSelectionState(BGroupViewListItem::NotSelected);
    }
    it.current()->repaint();
    ++it;
  }
}


void BGroupViewList::keyPressEvent(QKeyEvent* e) {
  if ( e->isAutoRepeat() ) {
    QListView::keyPressEvent(e);
    return;
  }
  if ( e->key() == Qt::Key_Delete) {
    if (currentItem() == 0) {
      QListView::keyPressEvent(e);
      return;
    }
    if (currentItem() == firstChild()) {
      QListView::keyPressEvent(e);
      return;
    }
    emit itemDeleted(currentItem()->text(0));
    e->accept();
    return;
  }
  QListView::keyPressEvent(e);
}

void BGroupViewList::openFolder() {
  autoopen_timer->stop();
  if ( dropItem && !dropItem->isOpen() ) {
  	dropItem->setOpen( true );
	  dropItem->repaint();
  }
}

static const int autoopenTime = 750;

bool BGroupViewList::canDecode(const QMimeSource* e, 
  QListViewItem* i) const {
  if (i == 0)
    return false;
  if (!QTextDrag::canDecode(e)) {
    return false;
  }
  QString s;
  //     
  QTextDrag::decode(e, s);
  QStringList l = QStringList::split("\t", s);
  //     .      
  //   ,  
  BattleObjects os;
  for(QStringList::const_iterator it = l.begin(); it != l.end(); it++) {
    BattleObject* bo = bApp->battle()->ships()->find(*it);
    if (bo == 0)
      return false;
    else
      os.insert(bo);
  }
  //    - ,    
  if (i == this->firstChild())
    return true;
  //      
  BattleObject* t = bApp->battle()->ships()->find(i->text(0));
  if (t == 0) {
    throw BInternalException(__FILE__, __LINE__,
      QString("Object '%1' specified as target, but not found in battle objects").
      arg(i->text(0)));
  }
  // ,       
  if (os.find(t) != os.end()) {
//    qDebug("find");
    return false;
  }
  is_parent_in ipn(os, bApp->battle()->ships(), true);
  bool rez = ipn.operator ()(t);
//  qDebug("rez = %d", rez);
  return !rez;
}

QString BGroupViewList::decode(const QMimeSource* e) const {
  QString s;
  QTextDrag::decode(e, s);
  return s;
}

bool BGroupViewList::dragState() const {
  return _dragState;
}

void BGroupViewList::contentsDragEnterEvent( QDragEnterEvent *e )
{
  if (!dropSupported()) {
    e->ignore();
    return;
  }
  _dragState = true;
  if (bApp->battle()->battleInfo().state == Running) {
    bApp->battle()->slotPause();
    _needRestart = true;
  }
  QListViewItem *i = itemAt( contentsToViewport(e->pos()) );
  QString decodedValue;
  if ( !canDecode(e, i) ) {
	  e->ignore();
	  return;
  }
  oldCurrent = currentItem();
  if ( i ) {
	  dropItem = i;
	  autoopen_timer->start( autoopenTime );
  }
}


void BGroupViewList::contentsDragMoveEvent( QDragMoveEvent *e )
{
  if (!dropSupported()) {
    e->ignore();
    return;
  }
  QPoint vp = contentsToViewport( ( (QDragMoveEvent*)e )->pos() );
  QListViewItem *i = itemAt( vp );

  if ( !canDecode(e, i) ) {
//    qDebug("ign");
	  e->ignore();
	  return;
  }

  if ( i ) {
	  setSelected( i, TRUE );
	  e->accept();
	  if ( i != dropItem ) {
	    autoopen_timer->stop();
	    dropItem = i;
	    autoopen_timer->start( autoopenTime );
    }
	  switch ( e->action() ) {
	  case QDropEvent::Copy:
      break;
	  case QDropEvent::Move:
      e->acceptAction();
      break;
	  case QDropEvent::Link:
      e->acceptAction();
      break;
	  default:
      ;
    }
  } 
  else {
	  e->ignore();
	  autoopen_timer->stop();
	  dropItem = 0;
  }
}

void BGroupViewList::contentsDragLeaveEvent( QDragLeaveEvent * /*e*/)
{
  if (!dropSupported()) {
    return;
  }
  _dragState = false;
  if (_needRestart) {
    bApp->battle()->slotPlay();
    _needRestart = false;
  }
  autoopen_timer->stop();
  dropItem = 0;

  setCurrentItem( oldCurrent );
  setSelected( oldCurrent, TRUE );
}

void BGroupViewList::contentsDropEvent( QDropEvent *e )
{
  if (!dropSupported()) {
    e->ignore();
    return;
  }
  _dragState = false;
  if (_needRestart) {
    bApp->battle()->slotPlay();
    _needRestart = false;
  }
  autoopen_timer->stop();
  QListViewItem *item = itemAt( contentsToViewport(e->pos()) );

  if ( !canDecode(e, item) ) {
    e->ignore();
    return;
  }
  if ( item ) {
    e->accept();
//    qDebug("drop at item %s", (const char*)item->text(0).local8Bit());
    emit itemDropped(decode(e), item->text(0));
  } 
  else
    e->ignore();
}

BGroupViewListItem::BGroupViewListItem( QListViewItem *parent, 
  QString label1 ):QListViewItem(parent, label1) {
  _side = -10;
}

BGroupViewListItem::BGroupViewListItem( QListView *parent, 
  QString label1 ):QListViewItem(parent, label1) {
  _side = -10;
}

void BGroupViewListItem::paintCell( QPainter *p, const QColorGroup &cg,
  int column, int width, int alignment ) {
  QColorGroup _cg( cg );
//  QColor hl = _cg.highlight();

  if (selectionState() == Active) {
    _cg.setColor( QColorGroup::Highlight, Qt::green );
    _cg.setColor( QColorGroup::Background, Qt::green );
    _cg.setColor( QColorGroup::Base, Qt::green );
    _cg.setColor( QColorGroup::Text, Qt::black );
    _cg.setColor( QColorGroup::Foreground, Qt::black );
    _cg.setColor( QColorGroup::HighlightedText, Qt::black );
  }
  else if (selectionState() == Selected) {
    _cg.setColor( QColorGroup::Highlight, Qt::cyan );
    _cg.setColor( QColorGroup::Background, Qt::cyan );
    _cg.setColor( QColorGroup::Base, Qt::cyan );
    _cg.setColor( QColorGroup::Text, Qt::black );
    _cg.setColor( QColorGroup::Foreground, Qt::black );
    _cg.setColor( QColorGroup::HighlightedText, Qt::black );
  }
  else {
    _cg.setColor( QColorGroup::Highlight, Qt::white );
    _cg.setColor( QColorGroup::Background, Qt::white );
    _cg.setColor( QColorGroup::Base, Qt::white );
    _cg.setColor( QColorGroup::Text, Qt::black );
    _cg.setColor( QColorGroup::Foreground, Qt::black );
    _cg.setColor( QColorGroup::HighlightedText, Qt::black );
  }


  QListViewItem::paintCell( p, _cg, column, width, alignment );
  
//  _cg.setColor( QColorGroup::Highlight, hl );
}

int BGroupViewListItem::side() const {
  return _side;
}

void BGroupViewListItem::setSide(int sd, const BattleObject* bo) {
  if (bo->isGroup())
    return;
  if (sd != _side)
    setPixmap(0, bo->image().smoothScale(16,16, QImage::ScaleMin));
}

BGroupViewPlay::BGroupViewPlay(BUiState* uiState, 
  QWidget* parent, const char* name):BGroupView(uiState, parent, name) {
}

BGroupViewPlay::~BGroupViewPlay() {
}

BGroupViewList* BGroupViewPlay::createGroupViewList() {
  return new BGroupViewListPlay(this, "groupviewlistplay");
}

BGroupViewListPlay::BGroupViewListPlay(QWidget *parent, const char *name)
:BGroupViewList(parent, name) {
}

BGroupViewListPlay::~BGroupViewListPlay() {
}

void BGroupViewListPlay::keyPressEvent(QKeyEvent* e) {
  QListView::keyPressEvent(e);
}
