/******************************************************************************/
/* BGroupView -   BattleFleet              */
/******************************************************************************/

#ifndef BGroupView_H
#define BGroupView_H
static const char bgroupview_h_id[]="@(#) $Id: bgroupview.h,v 1.7 2005/01/05 11:47:23 Oleksiy Balabay Exp $"; static const char* bgroupview_h_id_id=bgroupview_h_id;
/** \file bgroupview.h
  \brief  BGroupView  BattleFleet

     BGroupView  BattleFleet
*/
#include <qlistview.h>
#include <qstring.h>
#include <qdockwindow.h>
#include <set>

class BattleObject;
class BUiState;
class QTimer;
struct BUiSelectionInfo;
struct BattleInfo;
class BFactory;
class BFactoryPlay;

typedef std::set<BattleObject*> BattleObjects;

class BGroupViewList: public QListView {
  Q_OBJECT
public:
  BGroupViewList(QWidget *parent = 0, const char *name = 0);
  virtual ~BGroupViewList();
  void initView(const BattleObject* topObject, const BattleObject* skip);
  virtual void setOpen(QListViewItem* item, bool open);
  void updateView(const BattleObject* topObject, const BattleObject* skip);
  void setSelectedObjects(const BUiSelectionInfo& selectionInfo);
  bool dragState() const;
public:
  QPixmap* folderClosed;
  QPixmap* folderOpen;
protected:
  virtual QDragObject* dragObject();
  virtual void keyPressEvent(QKeyEvent* e);
  virtual bool dragSupported() const {
    return true;
  }
  virtual bool dropSupported() const {
    return true;
  }
private:
  void parseBattleObject(QListViewItem* parent, 
    const BattleObject* battleObject);
  void updateBattleObject(QListViewItem* listObject, 
    const BattleObject* battleObject, const BattleObject* skip);
  void createBattleObject(QListViewItem* parent, 
    const BattleObject* battleObject, const BattleObject* skip);
  QListViewItem* findItem(QListViewItem* parent, const QString& name);
protected:
  virtual void contentsDragEnterEvent( QDragEnterEvent *e );
  virtual void contentsDragMoveEvent( QDragMoveEvent *e );
  virtual void contentsDragLeaveEvent( QDragLeaveEvent *e );
  virtual void contentsDropEvent( QDropEvent *e );
protected slots:
  void openFolder();
private:
  bool canDecode(const QMimeSource* e, QListViewItem* i) const;
  QString decode(const QMimeSource* e) const;
private:
  QListViewItem *oldCurrent;
  QListViewItem *dropItem;
  QTimer* autoopen_timer;
  bool _dragState;
  bool _needRestart;
signals:
  void itemDeleted(const QString& name);
  void itemDropped(const QString& dropped, const QString& target);
};

class BGroupViewListItem: public QListViewItem {
public:
  enum SelectionStates {
    NotSelected,
    Selected,
    Active
  };
  BGroupViewListItem( QListView *parent, QString label1 );
  BGroupViewListItem( QListViewItem *parent, QString label1 );
  virtual void paintCell( QPainter *p, const QColorGroup &cg,
    int column, int width, int alignment );
  SelectionStates selectionState() const {
    return _selectionState;
  }
  void setSelectionState(const SelectionStates state) {
    _selectionState = state;
  }
  int side() const;
  void setSide(int sd, const BattleObject* bo);
private:
  SelectionStates _selectionState;
  int _side;
};

extern const char* B_GROUP_VIEW;

class BGroupView: public QDockWindow {
  Q_OBJECT
  friend class BFactory;
protected:
  BGroupView(BUiState* uiState, QWidget* parent, const char* name);
public:
  virtual ~BGroupView();
  void updateView(const BattleInfo& bi);
public slots:
  virtual void slotSelectionChanged(const BUiSelectionInfo& selectionInfo);
protected:
  virtual BGroupViewList* createGroupViewList();
  void init();
private slots:
//  void slotViewCurrentChanged(QListViewItem* item);
  void slotViewSelectionChanged();
  void slotViewItemDeleted(const QString& name);
  void slotViewItemDropped(const QString& dropped, const QString& target);
private:
  BGroupViewList* _view;
  BUiState* _uiState;
};

class BGroupViewPlay: public BGroupView {
  Q_OBJECT
  friend class BFactoryPlay;
protected:
  BGroupViewPlay(BUiState* uiState, QWidget* parent, const char* name);
public:
  virtual ~BGroupViewPlay();
protected:
  virtual BGroupViewList* createGroupViewList();
};

class BGroupViewListPlay: public BGroupViewList {
  Q_OBJECT
public:
  BGroupViewListPlay(QWidget *parent = 0, const char *name = 0);
  virtual ~BGroupViewListPlay();
protected:
  virtual void keyPressEvent(QKeyEvent* e);
  virtual bool dragSupported() const {
    return false;
  }
  virtual bool dropSupported() const {
    return false;
  }
};

#endif // BGroupView_H
