/******************************************************************************/
/* BKinematicData -   BattleFleet              */
/******************************************************************************/

#ifndef BKinematicData_H
#define BKinematicData_H
static const char bkinematicdata_h_id[]="@(#) $Id: bkinematicdata.h,v 1.5 2005/01/28 17:06:08 Oleksiy Balabay Exp $"; static const char* bkinematicdata_h_id_id=bkinematicdata_h_id;
/** \file bkinematicdata.h
  \brief  BKinematicData  BattleFleet

     BKinematicData  BattleFleet
*/
#include <deque>
#include <qdatetime.h>
#include <qdom.h>

class BKinematicData {
public:
  BKinematicData(double x = 0, double y = 0, double course = 0, double v = 0, 
    double vTurnCompensated = 0, double a = 0,
    double r = 0);
  double x() const { return _x; } // degrees
  double y() const { return _y; } // degrees
  double course() const { return _course; } // radians
  double v(bool withTurnCompensation = true) const { // miles per hour
    return withTurnCompensation?_vTurnCompensated:_v; 
  }
  double a() const { return _a; } // miles per hour2
  double r() const { return _r; } // miles
private:
  double _x;
  double _y;
  double _course;
  double _v;
  double _vTurnCompensated;
  double _a;
  double _r;
};

class BProgramData {
public:
  BProgramData(const BKinematicData& kinematicData, double availableTurnRadius,
    double availableSpeed, double availableAcceleration, 
    double lossOfSpeedCoef):_kinematicData(kinematicData),
    _availableTurnRadius(availableTurnRadius), _availableSpeed(availableSpeed),
    _availableAcceleration(availableAcceleration),
    _lossOfSpeedCoef(lossOfSpeedCoef) {
  }
  const BKinematicData& kinematicData() const { return _kinematicData; }
  double availableTurnRadius() const { return _availableTurnRadius; }
  double availableSpeed() const { return _availableSpeed; }
  double availableAcceleration() const { return _availableAcceleration; }
  double lossOfSpeedCoef() const { return _lossOfSpeedCoef; }
private:
  const BKinematicData _kinematicData;
  double _availableTurnRadius;
  double _availableSpeed;
  double _availableAcceleration;
  double _lossOfSpeedCoef;
};

typedef struct BObjectTrackPoint {
  int id;
  BKinematicData kd;
  QDomElement program;
} BObjectTrackPoint;

typedef std::pair<QDateTime, BObjectTrackPoint> BObjectTrack;
typedef std::deque<BObjectTrack> BObjectTracks;

#endif // BKinematicData_H
