/******************************************************************************/
/* BMainWindow -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bmainwindow_cpp_id[]="@(#) $Id: bmainwindow.cpp,v 1.18 2005/01/21 16:25:03 Oleksiy Balabay Exp $"; static const char* bmainwindow_cpp_id_id=bmainwindow_cpp_id;

#include <qsettings.h>
#include <qstatusbar.h>

#include "bmainwindow.h"
#include "bapplication.h"
//#include "battle.h"
#include "bfactory.h"
#include "bmainwindowstate.h"
#include "blog.h"

const QString BMainWindow::S_MAIN_WINDOW = "MainWindow";
const QString S_MAIN_WINDOW_POS = "Pos";
const QString S_MAIN_WINDOW_POS_X = "X";
const QString S_MAIN_WINDOW_POS_Y = "Y";
const QString S_MAIN_WINDOW_SIZE = "Size";
const QString S_MAIN_WINDOW_SIZE_WIDTH = "Width";
const QString S_MAIN_WINDOW_SIZE_HEIGHT = "Height";

/** \file bmainwindow.cpp
  \brief  BMainWindow  BattleFleet (impl.)

     BMainWindow  BattleFleet
*/

/** \class BMainWindow
  \brief 

*/
/** \brief 
*/
BMainWindow::BMainWindow():QMainWindow(0, "bmainwindow", 
  WType_TopLevel|WDestructiveClose), _state(0), _log(0) {
  setCaption( tr( "BattleFleet" ) );
  resize( QSize(600, 531).expandedTo(minimumSizeHint()) );
  clearWState( WState_Polished );
  QPixmap pm = QPixmap::fromMimeSource("battlefleet.png");
//  pm.setMask(pm.createHeuristicMask());
  setIcon(pm);
  loadWindowPosSize();
  setDockMenuEnabled(true);
  _log = new BLog(this, "log");
  addDockWindow(_log, DockBottom, true);
  setDockEnabled(_log, DockTop, false);
  setDockEnabled(_log, DockLeft, false);
  setDockEnabled(_log, DockRight, false);
  connect(this, SIGNAL(log(const QString&)), 
    this, SLOT(slotLog(const QString&)));
 
  /*
 
  connect(_central, SIGNAL(mouseOver(const int, const int, const QString&)),
    this, SLOT(slotMouseOver(const int, const int, const QString&)));

*/
}

/** \brief 
*/
BMainWindow::~BMainWindow(){
  saveWindowPosSize();
}

void BMainWindow::loadWindowPosSize() {
  QSettings* settings = bApp->settings();
  int x, y, w, h;
  settings->beginGroup(S_MAIN_WINDOW);
    settings->beginGroup(S_MAIN_WINDOW_POS);
      x = settings->readNumEntry(S_MAIN_WINDOW_POS_X, 0);
      y = settings->readNumEntry(S_MAIN_WINDOW_POS_Y, 0);
    settings->endGroup();
    settings->beginGroup(S_MAIN_WINDOW_SIZE);
      w = settings->readNumEntry(S_MAIN_WINDOW_SIZE_WIDTH, 800);
      h = settings->readNumEntry(S_MAIN_WINDOW_SIZE_HEIGHT, 600);
    settings->endGroup();
  settings->endGroup();
// TODO:    
//  QDesktopWidget* desktop = QApplication::desktop();
  resize(w,h);
  move(x,y);
}

void BMainWindow::saveWindowPosSize() {
//  int curX = x();
  QSettings* settings = bApp->settings();
  settings->beginGroup(S_MAIN_WINDOW);
    settings->beginGroup(S_MAIN_WINDOW_POS);
      settings->writeEntry(S_MAIN_WINDOW_POS_X, x());
      settings->writeEntry(S_MAIN_WINDOW_POS_Y, y());
    settings->endGroup();
    settings->beginGroup(S_MAIN_WINDOW_SIZE);
      settings->writeEntry(S_MAIN_WINDOW_SIZE_WIDTH, width());
      settings->writeEntry(S_MAIN_WINDOW_SIZE_HEIGHT, height());
    settings->endGroup();
  settings->endGroup();
}


void BMainWindow::slotModeChanged(const BAppModes /* newMode */, const BAppModes /* oldMode */) {
  if (_state) {
    delete _state;
  };
  _state = bApp->appFactory()->createMainWindowState(bApp->battle(), this);
  _log->setCaption(qApp->translate("BLog", B_LOG));
}

void BMainWindow::closeEvent(QCloseEvent* e) {
  if (!_state) {
//    saveWindowPosSize();
    e->accept();
    return;
  }
  if (!_state->wantToQuitBattle()) {
    e->ignore();
    return;
  }
//  saveWindowPosSize();
  e->accept();
  if (_state) {
    delete _state;
    _state = 0;
  }
}

BLog* BMainWindow::logDockWindow() const {
  return _log;
}

void BMainWindow::slotLog(const QString& message) {
  if (_log)
    _log->logText(message);
}

void BMainWindow::slotSaveSettings() {
  saveWindowPosSize();
}

void BMainWindow::mousePressEvent(QMouseEvent* /*e*/) {
  if (focusProxy())
    focusProxy()->setFocus();
}

void BMainWindow::customEvent(QCustomEvent* e) {
  if (e->type() == BFocusEventNumber) {
    if (focusProxy())
      focusProxy()->setFocus();
  }
}
