/******************************************************************************/
/* BMainWindowState -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bmainwindowstate_cpp_id[]="@(#) $Id: bmainwindowstate.cpp,v 1.28 2005/04/09 16:14:26 Oleksiy Balabay Exp $"; static const char* bmainwindowstate_cpp_id_id=bmainwindowstate_cpp_id;

/** \file bmainwindowstate.cpp
  \brief  BMainWindowState  BattleFleet (impl.)

     BMainWindowState  BattleFleet
*/
#include <qstatusbar.h>
#include <qmenubar.h>
#include <qaction.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qsettings.h>
#include <qprogressdialog.h>
#include <qfile.h>
#include <qwhatsthis.h>

#include "bmainwindow.h"
#include "bmainwindowstate.h"
#include "bapplication.h"
#include "bversion.h"
#include "blog.h"
#include "boptionsdialog.h"
#include "buiutils.h"
#include "battle.h"
#include "bscenarioselection.h"
#include "bfactory.h"
#include "buistate.h"
#include "bpropertyview.h"
#include "bgroupview.h"
#include "bshipgroup.h"
#include "buistate.h"
#include "beventsview.h"
#include "bai.h"
#include "bmainwindowtexts.h"
#include "ballonhelp.h"

#include "xpms/play.xpm"
#include "xpms/pause.xpm"

static const char* const imageNew_data[] = { 
"22 22 7 1",
". c None",
"# c #000000",
"b c #2f2f2f",
"c c #5f5f5f",
"d c #7f877f",
"e c #bfbfbf",
"a c #ffffff",
"......................",
"....##########........",
"....#aaaaaaa#b#.......",
"....#aaaaaaa#cb#......",
"....#aaaaaaa#dcb#.....",
"....#aaaaaaa#edcb#....",
"....#aaaaaaa#aedcb#...",
"....#aaaaaaa#######...",
"....#aaaaaaaaaaaaa#...",
"....#aaaaaaaaaaaaa#...",
"....#aaaaaaaaaaaaa#...",
"....#aaaaaaaaaaaaa#...",
"....#aaaaaaaaaaaaa#...",
"....#aaaaaaaaaaaaa#...",
"....#aaaaaaaaaaaaa#...",
"....#aaaaaaaaaaaaa#...",
"....#aaaaaaaaaaaaa#...",
"....#aaaaaaaaaaaaa#...",
"....#aaaaaaaaaaaaa#...",
"....###############...",
"......................",
"......................"};

static const char* const imageOpen_data[] = { 
"22 22 5 1",
". c None",
"# c #000000",
"c c #7f7f00",
"a c #ffff00",
"b c #ffffff",
"......................",
"......................",
"......................",
"............####....#.",
"...........#....##.##.",
"..................###.",
".................####.",
".####...........#####.",
"#abab##########.......",
"#babababababab#.......",
"#ababababababa#.......",
"#babababababab#.......",
"#ababab###############",
"#babab##cccccccccccc##",
"#abab##cccccccccccc##.",
"#bab##cccccccccccc##..",
"#ab##cccccccccccc##...",
"#b##cccccccccccc##....",
"###cccccccccccc##.....",
"##cccccccccccc##......",
"###############.......",
"......................"};

static const char* const imageSave_data[] = { 
"22 22 5 1",
". c None",
"# c #000000",
"a c #7f7f00",
"b c #bfbfbf",
"c c #cfb7cf",
"......................",
".####################.",
".#aa#bbbbbbbbbbbb#bb#.",
".#aa#bbbbbbbbbbbb#bb#.",
".#aa#bbbbbbbbbcbb####.",
".#aa#bbbccbbbbbbb#aa#.",
".#aa#bbbccbbbbbbb#aa#.",
".#aa#bbbbbbbbbbbb#aa#.",
".#aa#bbbbbbbbbbbb#aa#.",
".#aa#bbbbbbbbbbbb#aa#.",
".#aa#bbbbbbbbbbbb#aa#.",
".#aaa############aaa#.",
".#aaaaaaaaaaaaaaaaaa#.",
".#aaaaaaaaaaaaaaaaaa#.",
".#aaa#############aa#.",
".#aaa#########bbb#aa#.",
".#aaa#########bbb#aa#.",
".#aaa#########bbb#aa#.",
".#aaa#########bbb#aa#.",
".#aaa#########bbb#aa#.",
"..##################..",
"......................"};

static const char* B_ABOUT_MESSAGE = QT_TRANSLATE_NOOP(
  "BMainWindow", "<h2>Battlefleet v. %1</h2><p>"
  "Naval warfare at the turn of the century: 1895 - 1905<p>"
  "Based on the War Times Journal Battlefleet<p>"
  "Visit the application site: <b>battlefleet.narod.ru</b><p>"
  "You can use this application for free but the following<br>"
  "License Agreement requirements should be fulfilled<br>"
  "Qt Book Edition Non-Commercial License Agreement<br>"
  "See file LICENSE.NON_COMMERCIAL for details");

const char* BALLON_HELP_DOCK_WINDOWS_TEXT = QT_TRANSLATE_NOOP("BMainWindowState", 
  "You can use menu items or toolbar buttons to control the battle. "
  "Also some useful docked windows exist in application<br><br>"
  "To show or hide toolbars and docked windows call the right mouse button "
  "menu in the application menu bar<br><br>"
  "Use <b>What's this?</b> menu or toolbar button to read more about "
  "each command");

const QString DOCK_WINDOWS = "DockWindows";

/** \class BMainWindowState
  \brief 


*/
/** \brief 
*/
BMainWindowState::BMainWindowState(Battle* battle, 
  BMainWindow* mainWindow, const char* name)
  :BattleObserver(battle, mainWindow, name), _mainWindow(mainWindow),
  imageNew( (const char **) imageNew_data ),
  imageOpen( (const char **) imageOpen_data ),
  imageSave( (const char **) imageSave_data ),
  _propertyView(0), _eventsView(0), 
  aiShowCsAction(0), aiHideCsAction(0),
  aiControlActionGroup(0), shipsGroupAction(0),
  _central(0) {
  _timeLabel = 0;
//  _log = 0;
  _uiState = new BUiState(this, this, "uistate");
  _ais = bApp->appFactory()->createAIs(bApp->battle(), this);
  _uiState->setAIs(_ais);
}

/** \brief 
*/
BMainWindowState::~BMainWindowState(){
  clear();
  _mainWindow->setFocusProxy(0);
}

void BMainWindowState::init() {
  clear();
  initBase();
//  loadSettings();
  update();
  QTimer::singleShot(3000, this, SLOT(showBallonHelp()));
}

void BMainWindowState::clear() {
  _mainWindow->statusBar()->removeWidget(_timeLabel);
  delete _timeLabel;
  _timeLabel = 0;
  _mainWindow->statusBar()->clear();
  if (_mainWindow->centralWidget()) {
    delete _mainWindow->centralWidget();
  }
  _mainWindow->menuBar()->clear();
  {
    QPtrList<QDockWindow> dockWindows = _mainWindow->dockWindows();
    dockWindows.removeRef(_mainWindow->logDockWindow());
    dockWindows.setAutoDelete(true);
  }
}

void BMainWindowState::update() {
  ;
}

void BMainWindowState::initBase() {
}

void BMainWindowState::loadSettings() {
  QSettings* settings = bApp->settings();
  QString dockWindowsState;
  settings->beginGroup(BMainWindow::S_MAIN_WINDOW);
    dockWindowsState = settings->readEntry(DOCK_WINDOWS+stateName(), 
    defaultSettingsString());
  settings->endGroup();
  QTextStream st(&dockWindowsState, IO_ReadOnly);
  st >> (*_mainWindow);
}

void BMainWindowState::initFileGroup() {
  fileNewAction = new QAction( this, "fileNewAction" );
  fileNewAction->setIconSet( QIconSet( imageNew ) );
  fileOpenAction = new QAction( this, "fileOpenAction" );
  fileOpenAction->setIconSet( QIconSet( imageOpen ) );
//  fileSaveAction = new QAction( this, "fileSaveAction" );
//  fileSaveAction->setIconSet( QIconSet( imageSave ) );
//  fileSaveAsAction = new QAction( this, "fileSaveAsAction" );
  fileExitAction = new QAction( this, "fileExitAction" );

  fileToolBar = new QToolBar( tr(fileToolBar_N), _mainWindow, DockTop ); 
  fileToolBar->setCloseMode(QDockWindow::Undocked);

  fileNewAction->addTo( fileToolBar );
  fileOpenAction->addTo( fileToolBar );
//  fileSaveAction->addTo( fileToolBar );

  fileToolBar->hide();

  QPopupMenu* fileMenu = new QPopupMenu( _mainWindow );
  fileNewAction->addTo( fileMenu );
  fileOpenAction->addTo( fileMenu );
//  fileSaveAction->addTo( fileMenu );
//  fileSaveAsAction->addTo( fileMenu );
  fileMenu->insertSeparator();
  fileExitAction->addTo( fileMenu );
  _mainWindow->menuBar()->insertItem( tr( fileMenu_N ), fileMenu, -1, 0 );

  // signals and slots connections
  connect( fileNewAction, SIGNAL( activated() ), this, SLOT( fileNew() ) );
  connect( fileOpenAction, SIGNAL( activated() ), this, SLOT( fileOpen() ) );
  connect( fileExitAction, SIGNAL( activated() ), this, SLOT( fileExit() ) );
  fileNewAction->setText( tr( fileNewAction_TT ) );
  fileNewAction->setMenuText( tr( fileNewAction_N ) );
  fileNewAction->setAccel( CTRL+Key_N );
  fileNewAction->setStatusTip( tr(fileNewAction_ST) );
  fileNewAction->setWhatsThis( tr(fileNewAction_WT) );

  fileOpenAction->setText( tr( fileOpenAction_TT ) );
  fileOpenAction->setMenuText( tr( fileOpenAction_N ) );
  fileOpenAction->setAccel( CTRL+Key_O );
  fileOpenAction->setStatusTip( tr( fileOpenAction_ST ) );
  fileOpenAction->setWhatsThis( tr( fileOpenAction_WT ) );

  fileExitAction->setText( tr( fileExitAction_TT ) );
  fileExitAction->setMenuText( tr( fileExitAction_N ) );
  fileExitAction->setAccel( QString::null );
  fileExitAction->setStatusTip( tr( fileExitAction_ST ) );
  fileExitAction->setWhatsThis( tr( fileExitAction_WT ) );

  connect(_mainWindow, SIGNAL(newScenario()), this, SLOT(fileOpen()));
}

void BMainWindowState::initHelpGroup() {
  helpContentsAction = new QAction( this, "helpContentsAction" );
  helpIndexAction = new QAction( this, "helpIndexAction" );
  helpAboutAction = new QAction( this, "helpAboutAction" );

  helpToolBar = new QToolBar( tr(helpToolBar_N), _mainWindow, DockTop ); 
  helpToolBar->setCloseMode(QDockWindow::Undocked);
  helpToolBar->hide();
  QWhatsThis::whatsThisButton( helpToolBar );

  QPopupMenu* helpMenu = new QPopupMenu( _mainWindow );
  helpContentsAction->addTo( helpMenu );
  helpIndexAction->addTo( helpMenu );
  helpMenu->insertSeparator();
  helpAboutAction->addTo( helpMenu );
  _mainWindow->menuBar()->insertItem( tr( helpMenu_N ) , helpMenu, -1, 6 );

  // signals and slots connections
  connect( helpIndexAction, SIGNAL( activated() ), this, SLOT( helpIndex() ) );
  connect( helpContentsAction, SIGNAL( activated() ), this, SLOT( helpContents() ) );
  connect( helpAboutAction, SIGNAL( activated() ), this, SLOT( helpAbout() ) );

  helpContentsAction->setText( tr( helpContentsAction_TT ) );
  helpContentsAction->setMenuText( tr( helpContentsAction_N ) );
  helpContentsAction->setAccel( Key_F1 );
  helpContentsAction->setStatusTip( tr( helpContentsAction_ST ) );
  helpContentsAction->setWhatsThis( tr( helpContentsAction_WT ) );

  helpIndexAction->setText( tr( helpIndexAction_TT ) );
  helpIndexAction->setMenuText( tr( helpIndexAction_N ) );
  helpIndexAction->setAccel( QString::null );
  helpIndexAction->setStatusTip( tr( helpIndexAction_ST ) );
  helpIndexAction->setWhatsThis( tr( helpIndexAction_WT ) );

  helpAboutAction->setText( tr( helpAboutAction_TT ) );
  helpAboutAction->setMenuText( tr( helpAboutAction_N ) );
  helpAboutAction->setAccel( QString::null );
  helpAboutAction->setStatusTip( tr( helpAboutAction_ST ) );
  helpAboutAction->setWhatsThis( tr( helpAboutAction_WT ) );
}

/* void BMainWindowState::initLogGroup() {
  if (_mainWindow->logDockWindow() != 0) {
    connect(_mainWindow, SIGNAL(log(const QString&)), 
      this, SLOT(slotLog(const QString&)));
    return;
  }
  _log = new BLog(_mainWindow, "log");
  _mainWindow->addDockWindow(_log, DockBottom, true);
  _mainWindow->setDockEnabled(_log, DockTop, false);
  _mainWindow->setDockEnabled(_log, DockLeft, false);
  _mainWindow->setDockEnabled(_log, DockRight, false);
  _mainWindow->setLogDockWindow(_log);
  connect(_mainWindow, SIGNAL(log(const QString&)), 
    this, SLOT(slotLog(const QString&)));
}
*/

void BMainWindowState::fileNew() {
  if (!wantToQuitBattle()) {
    return;
  }
  openScenario(BScenarioSelection::NewPage);
}

void BMainWindowState::fileOpen() {
  if (!wantToQuitBattle()) {
    return;
  }
  openScenario(BScenarioSelection::OpenPage);
}

void BMainWindowState::fileExit() {
  _mainWindow->close();
//  if (!wantToQuitBattle()) {
//    return;
//  }
//  saveSettings();
//  bApp->quit();
}

void BMainWindowState::helpIndex() {
  bApp->showHelp();
}

void BMainWindowState::helpContents() {
  bApp->showHelp();
}

void BMainWindowState::helpAbout() {
  QMessageBox::about(_mainWindow, "Battlefleet", tr(B_ABOUT_MESSAGE).arg(B_VERSION));
}

/*void BMainWindowState::slotLog(const QString& message) {
  if (_log)
    _log->logText(message);
}
*/

bool BMainWindowState::wantToQuitBattle() {
  if (!_battle)
    return true;
  bool isRunning = (_battle->battleInfo().state == Running);
  if (isRunning) {
    _battle->slotPause();
    update();
  }
  if (QMessageBox::Yes == QMessageBox::question(_mainWindow, 
    tr("Stop current battle"),
    tr("Do you really want to stop current battle?"), 
    QMessageBox::Yes|QMessageBox::Default, QMessageBox::No|QMessageBox::Escape)) {
    return true;
  }
  else {
    if (isRunning) {
      _battle->slotPause();
      update();
    }
    return false;
  }
}

void BMainWindowState::saveSettings() {
  QString s;
  QTextStream st(&s, IO_WriteOnly);
  st << (*_mainWindow);
  QSettings* settings = bApp->settings();
  settings->beginGroup(BMainWindow::S_MAIN_WINDOW);
    settings->writeEntry(DOCK_WINDOWS+stateName(), s);
  settings->endGroup();
//  QFile debugF("C:/"+stateName()+".txt");
//  debugF.open(IO_WriteOnly);
//  QTextStream stDebug(&debugF);
//  stDebug << s;
}

void BMainWindowState::initToolsGroup() {
  toolsOptionsAction = new QAction( this, "toolsOptionsAction" );
  QPopupMenu* ToolsMenu = new QPopupMenu( _mainWindow );
  toolsOptionsAction->addTo( ToolsMenu );
  toolsOptionsAction->setText( tr( toolsOptionsAction_TT ) );
  toolsOptionsAction->setStatusTip( tr( toolsOptionsAction_ST ) );
  toolsOptionsAction->setWhatsThis( tr( toolsOptionsAction_WT ) );
  _mainWindow->menuBar()->insertItem( tr( ToolsMenu_N), ToolsMenu, -1, 5 );
  connect( toolsOptionsAction, SIGNAL( activated() ), this, SLOT( toolsOptions() ) );
}

void BMainWindowState::toolsOptions() {
  BOptionsDialog od(_mainWindow, "options_dialog");
  if (QDialog::Accepted != od.exec()) {
    return;
  }
  bApp->uiUtils()->setPresentationUnits(od.presentationsUnits());
}

/*void BMainWindowState::newScenario() {
  bool isRunning = (_battle->battleInfo().state == Running);
  if (isRunning) {
    _battle->slotPause();
    update();
  }
  BScenarioSelection selectScenario(_mainWindow);
  selectScenario.exec();
  if (selectScenario.result() == QDialog::Accepted) {
    BattleLoadData lD;
    lD.fileName = selectScenario.scenarioFileName();
    lD.startDate = selectScenario.scenarioStartDate();
    BChangeModeEvent* e = new BChangeModeEvent(
      lD, 
      selectScenario.applicationMode());
    qApp->postEvent(bApp, e);
  }
  else {
    bool exit = selectScenario.exit();
    if (exit)
      qApp->quit();
    else {
      if (isRunning) {
        _battle->slotPause();
        update();
      }
    }
  }
}*/

void BMainWindowState::openScenario(int pg) {
  bool isRunning = _battle?(_battle->battleInfo().state == Running):false;
  if (isRunning) {
    _battle->slotPause();
    update();
  }
  BScenarioSelection selectScenario(_mainWindow, BScenarioSelection::BPages(pg));
  selectScenario.exec();
  if (selectScenario.result() == QDialog::Accepted) {
    BattleLoadData lD;
    lD.fileName = selectScenario.scenarioFileName();
    lD.startDate = selectScenario.scenarioStartDate();
    BChangeModeEvent* e = new BChangeModeEvent(
      lD, 
      selectScenario.applicationMode());
    qApp->postEvent(bApp, e);
  }
  else {
    bool exit = selectScenario.exit();
    if (exit)
      qApp->quit();
    else {
      if (isRunning) {
        _battle->slotPause();
        update();
      }
    }
  }
}

void BMainWindowState::initViewGroup() {
  viewZoomInAction = new QAction( this, "viewZoomInAction" );
  viewZoomOutAction = new QAction( this, "viewZoomOutAction" );
  viewFullViewAction = new QAction( this, "viewFullViewAction" );
  viewZoomInAction->setText( tr( viewZoomInAction_TT ) );
  viewZoomInAction->setAccel( Key_Plus );
  viewZoomInAction->setStatusTip( tr( viewZoomInAction_ST ) );
  viewZoomInAction->setWhatsThis( tr( viewZoomInAction_WT ) );

  viewFullViewAction->setText( tr( viewFullViewAction_TT ) );
  viewFullViewAction->setStatusTip( tr( viewFullViewAction_ST ) );
  viewFullViewAction->setWhatsThis( tr( viewFullViewAction_WT ) );

  viewZoomOutAction->setText( tr( viewZoomOutAction_TT) );
  viewZoomOutAction->setAccel( Key_Minus );
  viewZoomOutAction->setStatusTip( tr( viewZoomOutAction_ST) );
  viewZoomOutAction->setWhatsThis( tr( viewZoomOutAction_WT) );
  viewZoomInAction->setIconSet( QIconSet(
    QPixmap::fromMimeSource("zoomin.png")));
  viewZoomOutAction->setIconSet( QIconSet( 
    QPixmap::fromMimeSource("zoomout.png")));
  viewFullViewAction->setIconSet( QIconSet(
    QPixmap::fromMimeSource("fullview.png")));

  viewModeActionGroup = new QActionGroup(this, "viewModeActionGroup");
//  viewModeActionGroup->setUsesDropDown(true);
  connect(viewModeActionGroup, SIGNAL(selected(QAction*)),
    this, SLOT(viewMode(QAction*)));
  viewModeSelectAction = new QAction(QPixmap::fromMimeSource("select.png"), tr(viewModeSelectAction_TT), 0, 
    viewModeActionGroup, "selectMode");
  viewModeSelectAction->setToggleAction(true);
  viewModeSelectAction->setStatusTip(tr(viewModeSelectAction_ST));
  viewModeSelectAction->setWhatsThis(tr(viewModeSelectAction_WT));

  viewModeConnectAction = new QAction(QPixmap::fromMimeSource("connect.png"), tr(viewModeConnectAction_TT), 0, 
    viewModeActionGroup, "connectMode");
  viewModeConnectAction->setToggleAction(true);
  viewModeConnectAction->setStatusTip(tr(viewModeConnectAction_ST));
  viewModeConnectAction->setWhatsThis(tr(viewModeConnectAction_WT));

  viewModeScaleAction = new QAction(QPixmap::fromMimeSource("scale.png"), tr(viewModeScaleAction_TT), 
    0, viewModeActionGroup, "scaleMode");
  viewModeScaleAction->setToggleAction(true);
  viewModeScaleAction->setOn(true);
  viewModeScaleAction->setStatusTip(tr(viewModeScaleAction_ST));
  viewModeScaleAction->setWhatsThis(tr(viewModeScaleAction_WT));

  viewToolBar = new QToolBar( tr(viewToolBar_N) , _mainWindow, DockTop ); 
  viewToolBar->setCloseMode(QDockWindow::Undocked);
  viewZoomInAction->addTo( viewToolBar );
  viewZoomOutAction->addTo( viewToolBar );
  viewFullViewAction->addTo( viewToolBar );
  viewToolBar->addSeparator();
  viewModeActionGroup->addTo( viewToolBar );
  viewToolBar->hide();

  QPopupMenu* ViewMenu = new QPopupMenu( _mainWindow );
  viewZoomInAction->addTo( ViewMenu );
  viewZoomOutAction->addTo( ViewMenu );
  viewFullViewAction->addTo( ViewMenu );
  ViewMenu->insertSeparator();
  viewModeActionGroup->addTo( ViewMenu );
  _mainWindow->menuBar()->insertItem( tr(ViewMenu_N), ViewMenu, -1, 2 );
  connect( viewZoomInAction, SIGNAL( activated() ), this, SLOT( viewZoomIn() ) );
  connect( viewZoomOutAction, SIGNAL( activated() ), this, SLOT( viewZoomOut() ) );
  connect( viewFullViewAction, SIGNAL( activated() ), this, SLOT( viewFullView() ) );
}

void BMainWindowState::viewZoomIn() {
  BZoomEvent* ze = new BZoomEvent(BZoomEvent::ZoomIn);
  qApp->postEvent(_central, ze);
}

void BMainWindowState::viewZoomOut() {
  BZoomEvent* ze = new BZoomEvent(BZoomEvent::ZoomOut);
  qApp->postEvent(_central, ze);
}

void BMainWindowState::viewFullView() {
  BZoomEvent* ze = new BZoomEvent(BZoomEvent::FullView);
  qApp->postEvent(_central, ze);
}

void BMainWindowState::initCentralGroup() {
  BPlayer player;
  player.type = BPlayer::OperatorPlayer;
  _central = bApp->appFactory()->createCentral(_uiState, _battle, &player);
  _central->viewWidget()->reparent(_mainWindow, QPoint(0,0), false);
  _mainWindow->setCentralWidget(_central->viewWidget());
  _central->viewWidget()->show();
  connect(_central, SIGNAL(mouseOver(const int, const int, const QString&)),
    this, SLOT(slotMouseOver(const int, const int, const QString&)));
  connect(_uiState, SIGNAL(selectionChanged(const BUiSelectionInfo&)),
    _central, SLOT(slotSelectionChanged(const BUiSelectionInfo&)));
//  connect(_uiState, SIGNAL(selectedObjectsChanged(
//    const BattleObjects&, const BattleObjects&)),
//    _central, SLOT(slotSelectedObjectsChanged(
//    const BattleObjects&, const BattleObjects&)));
  _mainWindow->setFocusProxy(_central->viewWidget());
}

void BMainWindowState::slotMouseOver(const int /* canvasX */, const int /* canvasY */, 
  const QString& message) {
  _mainWindow->statusBar()->message(message, 5000);
}

void BMainWindowState::viewMode(QAction* action) {
  if (action == viewModeSelectAction) {
    _uiState->setEditorMode(BUiState::SelectMode);
    if (aiHideCsAction) {
      aiHideCsAction->setEnabled(true);
    }
//    if (_central)
//      _central->viewWidget()->setCursor(ArrowCursor);
  }
  else if (action == viewModeScaleAction) {
    _uiState->setEditorMode(BUiState::ScaleMode);
    if (aiHideCsAction) {
      aiHideCsAction->setEnabled(true);
    }
//    if (_central)
//      _central->viewWidget()->setCursor(CrossCursor);
  }
  else if (action == viewModeConnectAction) {
    _uiState->setEditorMode(BUiState::ConnectMode);
    if (aiHideCsAction) {
      aiHideCsAction->setEnabled(false);
    }
    if (aiShowCsAction) {
      aiShowCsAction->setOn(true);
    }
//    if (_central)
//      _central->viewWidget()->setCursor(ArrowCursor);
  }
}

void BMainWindowState::initShipsGroup() {
  shipsGroupAction = new QAction( this, "shipsGroupAction" );
  shipsGroupAction->setIconSet( 
    QIconSet( QPixmap::fromMimeSource("group.png") ) );
  shipsUngroupAction = new QAction( this, "shipsUngroupAction" );
  shipsUngroupAction->setIconSet( 
    QIconSet( QPixmap::fromMimeSource("ungroup.png") ) );
  shipsGroupAction->setText( tr( shipsGroupAction_TT ) );
  shipsGroupAction->setAccel( CTRL+Key_G );
  shipsGroupAction->setStatusTip( tr( shipsGroupAction_ST ) );
  shipsGroupAction->setWhatsThis( tr( shipsGroupAction_WT ) );

  shipsUngroupAction->setText( tr( shipsUngroupAction_TT ) );
  shipsUngroupAction->setAccel( CTRL+Key_U );
  shipsUngroupAction->setStatusTip( tr( shipsUngroupAction_ST ) );
  shipsUngroupAction->setWhatsThis( tr( shipsUngroupAction_WT ) );

  shipsToolBar = new QToolBar( tr(shipsToolBar_N), _mainWindow, DockTop ); 
  shipsToolBar->setCloseMode(QDockWindow::Undocked);

  shipsGroupAction->addTo( shipsToolBar );
  shipsUngroupAction->addTo( shipsToolBar );

  shipsToolBar->hide();

  QPopupMenu* shipsMenu = new QPopupMenu( _mainWindow );
  shipsGroupAction->addTo( shipsMenu );
  shipsUngroupAction->addTo( shipsMenu );
//  shipsMenu->insertSeparator();
  _mainWindow->menuBar()->insertItem( tr( shipsMenu_N ), shipsMenu, -1, 3 );

  // signals and slots connections
  connect( shipsGroupAction, SIGNAL( activated() ), this, SLOT( shipsGroup() ) );
  connect( shipsUngroupAction, SIGNAL( activated() ), 
    this, SLOT( shipsUngroup() ) );
}

void BMainWindowState::shipsGroup() {
  _uiState->slotGroup();
}

void BMainWindowState::shipsUngroup() {
  _uiState->slotUngroup();
}

void BMainWindowState::initPropertyView() {
  _propertyView = new BPropertyView(this, _mainWindow, "propertyview");
  _propertyView->hide();
  _mainWindow->addDockWindow(_propertyView, DockLeft, true);
  _mainWindow->addDockWindow(_propertyView, DockRight, true);
  _mainWindow->setDockEnabled(_propertyView, DockTop, false);
  _mainWindow->setDockEnabled(_propertyView, DockBottom, false);
  connect(_uiState, SIGNAL(selectionChanged(const BUiSelectionInfo&)),
    this, SLOT(slotSelectionChanged(const BUiSelectionInfo&)));
  _propertyView->updateView(_battle->seaMap());
  _propertyView->hide();
}

void BMainWindowState::slotSelectionChanged(const BUiSelectionInfo& 
  selectionInfo) {
  if (_propertyView == 0)
    return;
  if (selectionInfo.activeObject) {
    if (selectionInfo.activeObject->isGroup())
      _propertyView->updateView(dynamic_cast<BShipGroup*>(selectionInfo.activeObject));
    else
      _propertyView->updateView(dynamic_cast<BShip*>(selectionInfo.activeObject));
  }
  else {
    _propertyView->updateView(_battle->seaMap());
  }
  //  ui
  update();
}

void BMainWindowState::initGroupView() {
  _groupView = bApp->appFactory()->createGroupView(
    _uiState, _mainWindow);
  _mainWindow->addDockWindow(_groupView, DockLeft, true);
  _mainWindow->addDockWindow(_groupView, DockRight, true);
  _mainWindow->setDockEnabled(_groupView, DockTop, false);
  _mainWindow->setDockEnabled(_groupView, DockBottom, false);
  connect(_uiState, SIGNAL(selectionChanged(const BUiSelectionInfo&)),
    _groupView, SLOT(slotSelectionChanged(const BUiSelectionInfo&)));
  _groupView->hide();
}

void BMainWindowState::initEventsView() {
  _eventsView = bApp->appFactory()->createEventsView(_mainWindow);
  _mainWindow->addDockWindow(_eventsView, DockLeft, true);
  _mainWindow->addDockWindow(_eventsView, DockRight, true);
  _mainWindow->setDockEnabled(_eventsView, DockTop, true);
  _mainWindow->setDockEnabled(_eventsView, DockBottom, true);
  _eventsView->hide();
}

void BMainWindowState::aiConnections(QAction* action) {
  if (action == aiShowCsAction) {
    _uiState->setShowChasingConnections(true);
  }
  else if (action == aiHideCsAction) {
    _uiState->setShowChasingConnections(false);
  }
}
/// ,  UIState  
void BMainWindowState::initAIs(bool initControl) {
  try {
    _ais->init();
  }
  catch(BException& e) {
    bApp->log(e.message());
    bApp->log(tr("AI state is not restored"));
  }
  catch(...) {
    bApp->log(tr("Unknown error while restoring AI state"));
    bApp->log(tr("AI state is not restored"));
  }
//    AI
  aiCsActionGroup = new QActionGroup(this, "aiCsActionGroup");
  connect(aiCsActionGroup, SIGNAL(selected(QAction*)),
    this, SLOT(aiConnections(QAction*)));
  aiShowCsAction = new QAction(
    QPixmap::fromMimeSource("show_connections.png"), 
    tr(aiShowCsAction_TT), 0, 
    aiCsActionGroup, "show_connections");
  aiShowCsAction->setToggleAction(true);
  aiShowCsAction->setStatusTip(tr(aiShowCsAction_ST));
  aiShowCsAction->setWhatsThis(tr(aiShowCsAction_WT));

  aiHideCsAction = new QAction(
    QPixmap::fromMimeSource("hide_connections.png"), 
    tr(aiHideCsAction_TT), 0, 
    aiCsActionGroup, "hide_connections");
  aiHideCsAction->setToggleAction(true);
  aiHideCsAction->setOn(true);
  aiHideCsAction->setStatusTip(tr(aiHideCsAction_ST));
  aiHideCsAction->setWhatsThis(tr(aiHideCsAction_WT));

  if (initControl)
    initAIsControl();
  aiToolBar = new QToolBar( tr(aiToolBar_N), _mainWindow, DockTop ); 
  aiToolBar->setCloseMode(QDockWindow::Undocked);

  aiCsActionGroup->addTo( aiToolBar );
  if (initControl) {
    aiToolBar->addSeparator();
    aiControlActionGroup->addTo( aiToolBar );
  }

  aiToolBar->hide();
  QPopupMenu* aiMenu = new QPopupMenu( _mainWindow );
  aiCsActionGroup->addTo( aiMenu );
  if (initControl) {
    aiMenu->insertSeparator();
    aiControlActionGroup->addTo( aiMenu );
  }
  _mainWindow->menuBar()->insertItem( tr( aiMenu_N ), aiMenu, -1, 4 );

}

void BMainWindowState::aiControls(QAction* action) {
  if (action == aiAutoControlAction) {
    _ais->setControlMode(BAI::ControlModeAuto);
  }
  else if (action == aiManualControlAction) {
    _ais->setControlMode(BAI::ControlModeManual);
  }
}

void BMainWindowState::initAIsControl() {
  aiControlActionGroup = new QActionGroup(this, "aiControlActionGroup");
  connect(aiControlActionGroup, SIGNAL(selected(QAction*)),
    this, SLOT(aiControls(QAction*)));
  aiAutoControlAction = new QAction(
    QPixmap::fromMimeSource("auto.png"), 
    tr(aiAutoControlAction_TT), 0, 
    aiControlActionGroup, "auto_control");
  aiAutoControlAction->setToggleAction(true);
  aiAutoControlAction->setStatusTip(tr(aiAutoControlAction_ST));
  aiAutoControlAction->setWhatsThis(tr(aiAutoControlAction_WT));
  aiManualControlAction = new QAction(
    QPixmap::fromMimeSource("manual.png"), 
    tr(aiManualControlAction_TT), 0, 
    aiControlActionGroup, "manual_control");
  aiManualControlAction->setToggleAction(true);
  aiManualControlAction->setOn(true);
  aiManualControlAction->setStatusTip(tr(aiManualControlAction_ST));
  aiManualControlAction->setWhatsThis(tr(aiManualControlAction_WT));
}

void BMainWindowState::initBattleGroup() {

  battlePlayAction = new QAction( this, "battlePlayAction" );
  battlePauseAction = new QAction( this, "battlePauseAction" );
  battlePlayAction->setIconSet( QIconSet(QPixmap(play_xpm)));
  battlePauseAction->setIconSet( QIconSet(QPixmap(pause_xpm)));
  
  battlePlayAction->setText( tr( battlePlayAction_TT ) );
  battlePlayAction->setMenuText( tr( battlePlayAction_N ) );
  battlePlayAction->setStatusTip( tr( battlePlayAction_ST ) );
  battlePlayAction->setWhatsThis( tr( battlePlayAction_WT ) );

  battlePauseAction->setText( tr( battlePauseAction_TT) );
  battlePauseAction->setMenuText( tr( battlePauseAction_N ) );
  battlePauseAction->setStatusTip( tr( battlePauseAction_ST ) );
  battlePauseAction->setWhatsThis( tr( battlePauseAction_WT ) );
  
  QPopupMenu* BattleMenu = new QPopupMenu( _mainWindow );
  battlePlayAction->addTo( BattleMenu );
  battlePauseAction->addTo( BattleMenu );
  _mainWindow->menuBar()->insertItem( tr( BattleMenu_N ), BattleMenu, -1, 1 );

  battleToolBar = new QToolBar( tr( battleToolBar_N ), _mainWindow, DockTop ); 

  battlePlayAction->addTo( battleToolBar );
  battlePauseAction->addTo( battleToolBar );

  battleSpeedActionGroup = new QActionGroup(this);

  battleRealtimeAction = new QAction(this, "battleRealtimeAction");
  battleRealtimeAction->setText(tr( "Realtime (x1)" ));
  battleRealtimeAction->setToggleAction(true);
  battleSpeedActionGroup->add(battleRealtimeAction);

  battleSlowAction = new QAction(this, "battleSlowAction");
  battleSlowAction->setText(tr( "Slow (x2)")); 
  battleSlowAction->setToggleAction(true);
  battleSpeedActionGroup->add(battleSlowAction);

  battleNormalAction = new QAction(this, "battleNormalAction");
  battleNormalAction->setText(tr( "Normal (x10)")); 
  battleSpeedActionGroup->add(battleNormalAction);
  battleNormalAction->setToggleAction(true);
  battleNormalAction->setOn(true);

  battleFastAction = new QAction(this, "battleFastAction");
  battleFastAction->setText(tr( "Fast (x30)" )); 
  battleFastAction->setToggleAction(true);
  battleSpeedActionGroup->add(battleFastAction);

  battleUltraFastAction = new QAction(this, "battleUltraFastAction");
  battleUltraFastAction->setText(tr("Ultra Fast (x60)")); 
  battleUltraFastAction->setToggleAction(true);
  battleSpeedActionGroup->add(battleUltraFastAction);

//  battleSpeedActionGroup->addTo(battleToolBar);
  BattleMenu->insertSeparator();
  battleSpeedActionGroup->addTo(BattleMenu);
  battleSpeedActionGroup->setExclusive(true);

  connect(battleSpeedActionGroup, SIGNAL(selected(QAction*)),
    this, SLOT(battleSpeedChanged(QAction*)));

  speedCB = new QComboBox( FALSE, battleToolBar, "speedCB" );
  speedCB->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)5, 0, 0, speedCB->sizePolicy().hasHeightForWidth() ) );
  speedCB->clear();
  speedCB->insertItem( tr( "Realtime (x1)" ) );
  speedCB->insertItem( tr( "Slow (x2)" ) );
  speedCB->insertItem( tr( "Normal (x10)" ) );
  speedCB->insertItem( tr( "Fast (x30)" ) );
  speedCB->insertItem( tr( "Ultra Fast (x60)" ) );
  speedCB->show();
  battleToolBar->hide();

  connect( battlePauseAction, SIGNAL( activated() ), this, SLOT( battlePause() ) );
  connect( battlePlayAction, SIGNAL( activated() ), this, SLOT( battlePlay() ) );
  connect( speedCB, SIGNAL( activated(int) ), this, SLOT( battleSpeedChanged(int) ) );
}

void BMainWindowState::battlePlay() {
  _battle->slotPlay();
  update();
}

void BMainWindowState::battlePause() {
  _battle->slotPause();
  update();
}

void BMainWindowState::battleSpeedChanged(int item) {
  _battle->slotSetSpeed(BattleSpeeds(item));
  _central->viewWidget()->setFocus();
  update();
}

void BMainWindowState::battleSpeedChanged(QAction* a) {
  int item;
  if (a ==battleRealtimeAction)
    {
      item = 0;
    }
  else if (a == battleSlowAction)
    {
      item = 1;
    }
  else if (a == battleNormalAction)
    {
      item = 2;
    }
  else if (a == battleFastAction)
    {
      item = 3;
    }
  else if (a == battleUltraFastAction)
    {
      item = 4;
    }
  else
    {
      throw BInternalException(__FILE__, __LINE__,
        tr("Unknown action"));
    }
  battleSpeedChanged(item);
}


void BMainWindowState::initStatusBar() {
  _timeLabel = new QLabel(QDateTime::currentDateTime().toString(), 0);
  _timeLabel->setMinimumSize(_timeLabel->sizeHint());

  _mainWindow->statusBar()->setSizeGripEnabled(false);
  _mainWindow->statusBar()->addWidget(_timeLabel, 0, true);
  _timeLabel->setText("");
  _timeLabel->show();
}

void BMainWindowState::showBallonHelp() {
  if (bApp->ballonHelp()->shouldBeShown(BallonHelp::BALLON_HELP_DOCK_WINDOWS)) {
    bApp->ballonHelp()->setShown(BallonHelp::BALLON_HELP_DOCK_WINDOWS);
    QWhatsThis::display(tr(BALLON_HELP_DOCK_WINDOWS_TEXT, 
      "BALLON_HELP_DOCK_WINDOWS_TEXT"), 
      QPoint(_mainWindow->x()+ _mainWindow->width()/2, 
      _mainWindow->y()+ _mainWindow->height()/3));
  }
}