/******************************************************************************/
/* BMainWindowState -   BattleFleet              */
/******************************************************************************/

#ifndef BMainWindowState_H
#define BMainWindowState_H
static const char bmainwindowstate_h_id[]="@(#) $Id: bmainwindowstate.h,v 1.20 2005/03/07 15:34:37 Oleksiy Balabay Exp $"; static const char* bmainwindowstate_h_id_id=bmainwindowstate_h_id;
/** \file bmainwindowstate.h
  \brief  BMainWindowState  BattleFleet

     BMainWindowState  BattleFleet
*/
#include <qpixmap.h>

#include "battleobserver.h"

class QAction;
class QActionGroup;
class QToolBar;
class BMainWindow;
class BUiState;
class BPropertyView;
class BGroupView;
class BattleObject;
struct BUiSelectionInfo;
class BEventsView;
class BAIs;
class QComboBox;
class QLabel;
//class BLog;

class BMainWindowState: public BattleObserver {
  Q_OBJECT
public:
  BMainWindowState(Battle* battle, BMainWindow* mainWindow, const char* name);
  virtual ~BMainWindowState();
  virtual void init();
  virtual void update();
  virtual bool wantToQuitBattle();
//public slots:
//  void slotLog(const QString& message);
protected:
  virtual void initBase();
  virtual void clear();
  virtual void loadSettings();
  virtual void saveSettings();
  virtual QString stateName() const = 0;
  void openScenario(int pg);
  virtual QString defaultSettingsString() const = 0;
protected:
  BMainWindow* _mainWindow;
protected:
  QPixmap imageSave;
  QPixmap imageNew;
  QPixmap imageOpen;
// file data
protected:
  virtual void initFileGroup();
protected:
  QAction* fileNewAction;
  QAction* fileOpenAction;
  QAction* fileExitAction;
  QToolBar* fileToolBar;
protected slots:
  virtual void fileNew();
  virtual void fileOpen();
  virtual void fileExit();
// view toolbar
protected:
  virtual void initViewGroup();
protected:
  QAction* viewZoomInAction;
  QAction* viewZoomOutAction;
  QAction* viewFullViewAction;
  QAction* viewModeSelectAction;
  QAction* viewModeScaleAction;
  QAction* viewModeConnectAction;
  QActionGroup* viewModeActionGroup;
  QToolBar *viewToolBar;
protected slots:
  virtual void viewZoomIn();
  virtual void viewZoomOut();
  virtual void viewFullView();
  virtual void viewMode(QAction*);
// central data
protected:
  virtual void initCentralGroup();
protected:
  BattleObserver* _central;
protected slots:
  void slotMouseOver(const int canvasX, const int canvasY, 
    const QString& message);
// help data
protected:
  virtual void initHelpGroup();
protected:
  QAction* helpContentsAction;
  QAction* helpIndexAction;
  QAction* helpAboutAction;
  QToolBar *helpToolBar;
protected slots:
  virtual void helpIndex();
  virtual void helpContents();
  virtual void helpAbout();
// ships data
protected:
  virtual void initShipsGroup();
protected:
  QAction* shipsGroupAction;
  QAction* shipsUngroupAction;
  QToolBar *shipsToolBar;
protected slots:
  virtual void shipsGroup();
  virtual void shipsUngroup();
protected: 
  // init property view
  virtual void initPropertyView();
  BPropertyView* _propertyView;
protected slots:
  void slotSelectionChanged(const BUiSelectionInfo& selectionInfo);
  // init ships view
  virtual void initGroupView();
protected:
  BGroupView* _groupView;

// log data
//protected:
//  virtual void initLogGroup();
//protected:
//  BLog* _log;
// tools data
protected:
  virtual void initToolsGroup();
protected:
  QAction* toolsOptionsAction;
//  QAction* toolsEventsAction;
protected slots:
  virtual void toolsOptions();
//  virtual void toolsEvents();
// ui state
protected:
  BUiState* _uiState;
// ai 
  QToolBar *aiToolBar;
  QAction* aiShowCsAction;
  QAction* aiHideCsAction;
  QActionGroup* aiCsActionGroup;
  QAction* aiAutoControlAction;
  QAction* aiManualControlAction;
  QActionGroup* aiControlActionGroup;
  BAIs* _ais;
  virtual void initAIs(bool initControl = true);
  virtual void initAIsControl();
protected slots:
  virtual void aiConnections(QAction*);
  virtual void aiControls(QAction*);
protected: 
  // init events view
  virtual void initEventsView();
  BEventsView* _eventsView;
// battle toolbar
protected:
  virtual void initBattleGroup();
protected:
  QComboBox* speedCB;
  QAction* battlePlayAction;
  QAction* battlePauseAction;
  QToolBar *battleToolBar;
  QActionGroup* battleSpeedActionGroup;
  QAction* battleRealtimeAction;
  QAction* battleSlowAction;
  QAction* battleNormalAction;
  QAction* battleFastAction;
  QAction* battleUltraFastAction;

protected slots:
  virtual void battlePlay();
  virtual void battlePause();
  virtual void battleSpeedChanged(int);
  virtual void battleSpeedChanged(QAction*);
// status bar
protected:
  virtual void initStatusBar();
protected:
  QLabel* _timeLabel;
protected slots:
  virtual void showBallonHelp();
};

#endif // BMainWindowState_H
