/******************************************************************************/
/* BMainWindowStateEdit -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bmainwindowstateedit_cpp_id[]="@(#) $Id: bmainwindowstateedit.cpp,v 1.21 2005/04/03 14:21:40 Oleksiy Balabay Exp $"; static const char* bmainwindowstateedit_cpp_id_id=bmainwindowstateedit_cpp_id;

#include <qaction.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <algorithm>
#include <qprogressdialog.h>

#include "bapplication.h"
#include "bmainwindowstateedit.h"
#include "bshipslistview.h"
#include "bmainwindow.h"
#include "battle.h"
#include "bships.h"
#include "bpropertyview.h"
#include "bgroupview.h"
#include "buistate.h"
#include "bscenariosave.h"
#include "bscenarioheader.h"
#include "battleedit.h"
#include "bscenarioheaderedit.h"
#include "bprogram.h"
#include "bdomutils.h"
#include "blog.h"
#include "bmainwindowtexts.h"

/** \file bmainwindowstateedit.cpp
  \brief  BMainWindowStateEdit  BattleFleet (impl.)

     BMainWindowStateEdit  BattleFleet
*/

/** \class BMainWindowStateEdit
  \brief 


*/
/** \brief 
*/
BMainWindowStateEdit::BMainWindowStateEdit(Battle* battle, 
  BMainWindow* mainWindow, const char* name):BMainWindowState(battle, 
  mainWindow, name){
  _mainWindow->setCaption(tr("%1. Editor Mode - BattleFleet").arg(
    battle->scenarioHeader()->name()));
}

QString BMainWindowStateEdit::defaultSettingsString() const {
  //    ,     
  QString s = 
"\n"
"\n"
"\n"+fileToolBar->caption()+","+
viewToolBar->caption()+","+
shipsToolBar->caption()+","+
aiToolBar->caption()+","+
helpToolBar->caption()+",\n"+
"["+fileToolBar->caption()+",0,1,-1,-1,1]["
+viewToolBar->caption()+",0,0,-1,-1,1]["
+shipsToolBar->caption()+",0,0,-1,-1,1]["
+aiToolBar->caption()+",0,0,-1,-1,1]["
+helpToolBar->caption()+",0,0,-1,-1,1]\n"
+_mainWindow->logDockWindow()->caption()+",\n"
"["+_mainWindow->logDockWindow()->caption()+",0,1,520,-1,1]\n"
+_groupView->caption()+","+
_propertyView->caption()+",\n"
"["+_groupView->caption()+",96,1,174,-1,1]["+
_propertyView->caption()+",0,0,174,-1,1]\n" +
_shipsDbView->caption()+",\n"
"["+_shipsDbView->caption()+",-3,1,184,-1,1]\n";
  return s;
}

/** \brief 
*/
BMainWindowStateEdit::~BMainWindowStateEdit(){
  saveSettings();
  if (_central) {
    delete _central;
  }
  clear();
}

QString BMainWindowStateEdit::stateName() const {
  return "Edit";
}

void BMainWindowStateEdit::initBase() {
  initFileGroup();
//  bApp->checkProgressDialog(81);
  initViewGroup();
//  bApp->checkProgressDialog(82);
  initToolsGroup();
//  bApp->checkProgressDialog(83);
  initShipsGroup();
//  bApp->checkProgressDialog(84);
  initHelpGroup();
  bApp->checkProgressDialog(85);
  initShipsDbView();
//  bApp->checkProgressDialog(86);
  initPropertyView();
//  bApp->checkProgressDialog(87);
  initGroupView();
  initStatusBar();
}

void BMainWindowStateEdit::init() {
  BMainWindowState::init();
  qApp->processEvents();
  initCentralGroup();
//  bApp->checkProgressDialog(95, 
//    QProgressDialog::tr("Loading control settings..."));
  initAIs(false);
  loadSettings();
  slotBattleInfoChanged(bApp->battle()->battleInfo());
  _central->viewWidget()->setFocus();
}

void BMainWindowStateEdit::initShipsDbView() {
  _shipsDbView = new BShipsDbView(_mainWindow, "shipsdbview");
  _mainWindow->addDockWindow(_shipsDbView, DockLeft, true);
  _mainWindow->addDockWindow(_shipsDbView, DockRight, true);
  _mainWindow->setDockEnabled(_shipsDbView, DockTop, false);
  _mainWindow->setDockEnabled(_shipsDbView, DockBottom, false);
  _shipsDbView->hide();
}

void BMainWindowStateEdit::slotBattleInfoChanged(const BattleInfo& info) {
  _timeLabel->setText(info.modelTime.toString());
  _shipsDbView->updateView(bApp->battle()->ships()->unUsedShips());
  _propertyView->updateView();
  _groupView->updateView(info);
  update();
}

void BMainWindowStateEdit::initCentralGroup() {
  BMainWindowState::initCentralGroup();
}

void BMainWindowStateEdit::initFileGroup() {
  fileNewAction = new QAction( this, "fileNewAction" );
  fileNewAction->setIconSet( QIconSet( imageNew ) );
  fileOpenAction = new QAction( this, "fileOpenAction" );
  fileOpenAction->setIconSet( QIconSet( imageOpen ) );
  fileSaveAction = new QAction( this, "fileSaveAction" );
  fileSaveAction->setIconSet( QIconSet( imageSave ) );
  fileSaveAsAction = new QAction( this, "fileSaveAsAction" );
  filePropertiesAction = new QAction( this, "fileProperties" );
  fileLoadShipsAction = new QAction( this, "fileLoadShips" );
  fileExitAction = new QAction( this, "fileExitAction" );

  fileToolBar = new QToolBar( qApp->translate("BMainWindowState", fileToolBar_N), 
    _mainWindow, DockTop ); 
  fileToolBar->setCloseMode(QDockWindow::Undocked);

  fileNewAction->addTo( fileToolBar );
  fileOpenAction->addTo( fileToolBar );
  fileSaveAction->addTo( fileToolBar );

  fileToolBar->hide();

  QPopupMenu* fileMenu = new QPopupMenu( _mainWindow );
  fileNewAction->addTo( fileMenu );
  fileOpenAction->addTo( fileMenu );
  fileSaveAction->addTo( fileMenu );
  fileSaveAsAction->addTo( fileMenu );
  fileMenu->insertSeparator();
  filePropertiesAction->addTo(fileMenu);
  fileMenu->insertSeparator();
  fileLoadShipsAction->addTo( fileMenu );
  fileMenu->insertSeparator();
  fileExitAction->addTo( fileMenu );
  _mainWindow->menuBar()->insertItem( qApp->translate("BMainWindowState", fileMenu_N ), fileMenu, -1, 0 );

  // signals and slots connections
  connect( fileNewAction, SIGNAL( activated() ), this, SLOT( fileNew() ) );
  connect( fileOpenAction, SIGNAL( activated() ), this, SLOT( fileOpen() ) );
  connect( fileSaveAction, SIGNAL( activated() ), this, SLOT( fileSave() ) );
  connect( fileSaveAsAction, SIGNAL( activated() ), this, SLOT( fileSaveAs() ) );
  connect( filePropertiesAction, SIGNAL( activated() ), this, SLOT( fileProperties() ) );
  connect( fileLoadShipsAction, SIGNAL( activated() ), 
    this, SLOT( fileLoadShips() ) );
  connect( fileExitAction, SIGNAL( activated() ), this, SLOT( fileExit() ) );
  fileNewAction->setText( qApp->translate("BMainWindowState", fileNewAction_TT ) );
  fileNewAction->setMenuText( qApp->translate("BMainWindowState", fileNewAction_N) );
  fileNewAction->setAccel( CTRL+Key_N );
  fileNewAction->setStatusTip( qApp->translate("BMainWindowState", fileNewAction_ST ) );
  fileNewAction->setWhatsThis( qApp->translate("BMainWindowState", fileNewAction_WT ) );

  fileOpenAction->setText( qApp->translate("BMainWindowState",fileOpenAction_TT) );
  fileOpenAction->setMenuText( qApp->translate("BMainWindowState", fileOpenAction_N) );
  fileOpenAction->setAccel( CTRL+Key_O );
  fileOpenAction->setStatusTip( qApp->translate("BMainWindowState", fileOpenAction_ST ) );
  fileOpenAction->setWhatsThis( qApp->translate("BMainWindowState", fileOpenAction_WT ) );

  fileSaveAction->setText( qApp->translate("BMainWindowState", fileSaveAction_TT) );
  fileSaveAction->setMenuText( qApp->translate("BMainWindowState", fileSaveAction_N) );
  fileSaveAction->setAccel( CTRL+Key_S );
  fileSaveAction->setStatusTip( qApp->translate("BMainWindowState", fileSaveAction_ST ) );
  fileSaveAction->setWhatsThis( qApp->translate("BMainWindowState", fileSaveAction_WT ) );

  fileSaveAsAction->setText( qApp->translate("BMainWindowState", fileSaveAsAction_TT) );
  fileSaveAsAction->setMenuText( qApp->translate("BMainWindowState",fileSaveAsAction_N) );
  fileSaveAsAction->setAccel( QString::null );
  fileSaveAsAction->setStatusTip( qApp->translate("BMainWindowState", fileSaveAsAction_ST ) );
  fileSaveAsAction->setWhatsThis( qApp->translate("BMainWindowState", fileSaveAsAction_WT ) );

  filePropertiesAction->setText( qApp->translate("BMainWindowState", filePropertiesAction_TT) );
  filePropertiesAction->setAccel( QString::null );
  filePropertiesAction->setStatusTip( qApp->translate("BMainWindowState", filePropertiesAction_ST ) );
  filePropertiesAction->setWhatsThis( qApp->translate("BMainWindowState", filePropertiesAction_WT ) );

  fileLoadShipsAction->setText( qApp->translate("BMainWindowState", fileLoadShipsAction_TT) );
  fileLoadShipsAction->setMenuText( qApp->translate("BMainWindowState",fileLoadShipsAction_N) );
  fileLoadShipsAction->setAccel( QString::null );
  fileLoadShipsAction->setStatusTip( qApp->translate("BMainWindowState", fileLoadShipsAction_ST ) );
  fileLoadShipsAction->setWhatsThis( qApp->translate("BMainWindowState", fileLoadShipsAction_WT ) );

  fileExitAction->setText( qApp->translate("BMainWindowState",fileExitAction_TT) );
  fileExitAction->setMenuText( qApp->translate("BMainWindowState",fileExitAction_N) );
  fileExitAction->setAccel( QString::null );
  fileExitAction->setStatusTip( qApp->translate("BMainWindowState", fileExitAction_ST ) );
  fileExitAction->setWhatsThis( qApp->translate("BMainWindowState", fileExitAction_WT ) );

  connect(_mainWindow, SIGNAL(newScenario()), this, SLOT(fileOpen()));
}

void BMainWindowStateEdit::fileSave() {
  _battle->save();
}

void BMainWindowStateEdit::fileLoadShips() {
  QString fn = QFileDialog::getOpenFileName(qApp->applicationDirPath()+
    "/common", tr("Ships (*.xml);;All (*)"), _mainWindow,
    "load ships", tr("Load additional ships database"));
  if (!fn.isEmpty()) {
    // 
    try {
      bApp->battle()->ships()->addShipsFromFile(fn);
    }
    catch (BConfigException& e) {
      _mainWindow->slotLog(e.message());
      return;
    }
    catch (BException& e) {
      _mainWindow->slotLog(e.message());
      return;
    }
    catch (...) {
      _mainWindow->slotLog(tr("Error loading file '%1'").arg(fn));
      return;
    }
    //     
    _shipsDbView->addShipsFromFile(fn);
    ((BattleEdit*)_battle)->addShipsDbFile(fn);
  }
}

void BMainWindowStateEdit::fileSaveAs() {
  BScenarioSave ss(bApp->scenariosDir(), 
    _battle->scenarioHeader()->fileName(), _mainWindow, "savescenariownd");
  ss.exec();
  if (ss.result() == QDialog::Accepted) {
    if (ss.fileName() == _battle->scenarioHeader()->fileName()) {
      _battle->save();
    }
    else {
      _battle->saveAs(ss.fileName());
    }
  }
//  
}

//void BMainWindowStateEdit::shipsGroup() {
//  ;
//}

//void BMainWindowStateEdit::shipsUngroup() {
//  ;
//}

void BMainWindowStateEdit::initShipsGroup() {
  shipsGroupAction = new QAction( this, "shipsGroupAction" );
  shipsGroupAction->setIconSet( 
    QIconSet( QPixmap::fromMimeSource("group.png") ) );
  shipsUngroupAction = new QAction( this, "shipsUngroupAction" );
  shipsUngroupAction->setIconSet( 
    QIconSet( QPixmap::fromMimeSource("ungroup.png") ) );
  shipsGroupAction->setText( qApp->translate("BMainWindowState", shipsGroupAction_TT) );
  shipsGroupAction->setAccel( CTRL+Key_G );
  shipsUngroupAction->setText( qApp->translate("BMainWindowState", shipsUngroupAction_TT) );
  shipsUngroupAction->setAccel( CTRL+Key_U );
  shipsGroupAction->setStatusTip( qApp->translate("BMainWindowState", shipsGroupAction_ST ) );
  shipsGroupAction->setWhatsThis( qApp->translate("BMainWindowState", shipsGroupAction_WT ) );
  shipsUngroupAction->setStatusTip( qApp->translate("BMainWindowState", shipsUngroupAction_ST ) );
  shipsUngroupAction->setWhatsThis( qApp->translate("BMainWindowState", shipsUngroupAction_WT ) );

  shipsOrderAction = new QAction( this, "shipsOrderAction" );
  shipsOrderAction->setIconSet( 
    QIconSet( QPixmap::fromMimeSource("order.png") ) );
  shipsOrderAction->setText( qApp->translate("BMainWindowState", shipsOrderAction_TT) );
  shipsOrderAction->setStatusTip( qApp->translate("BMainWindowState", shipsOrderAction_ST ) );
  shipsOrderAction->setWhatsThis( qApp->translate("BMainWindowState", shipsOrderAction_WT ) );

  shipsToolBar = new QToolBar( qApp->translate("BMainWindowState",shipsToolBar_N), _mainWindow, DockTop ); 
  shipsToolBar->setCloseMode(QDockWindow::Undocked);

  shipsGroupAction->addTo( shipsToolBar );
  shipsUngroupAction->addTo( shipsToolBar );
  shipsOrderAction->addTo( shipsToolBar );

  shipsToolBar->hide();

  QPopupMenu* shipsMenu = new QPopupMenu( _mainWindow );
  shipsGroupAction->addTo( shipsMenu );
  shipsUngroupAction->addTo( shipsMenu );
  shipsOrderAction->addTo( shipsMenu );
  _mainWindow->menuBar()->insertItem( qApp->translate("BMainWindowState", shipsMenu_N), shipsMenu, -1, 2 );

  // signals and slots connections
  connect( shipsGroupAction, SIGNAL( activated() ), this, SLOT( shipsGroup() ) );
  connect( shipsUngroupAction, SIGNAL( activated() ), 
    this, SLOT( shipsUngroup() ) );
  connect( shipsOrderAction, SIGNAL( activated() ), 
    this, SLOT( shipsOrder() ) );
}

void BMainWindowStateEdit::shipsOrder() {
  _uiState->orderSelection();
}

bool BMainWindowStateEdit::wantToQuitBattle() {
  if (!_battle)
    return true;
  bool isRunning = (_battle->battleInfo().state == Running);
  if (isRunning) {
    _battle->slotPause();
    update();
  }
  if (!_battle->isModified()) {
    return true;
  }
  int res = QMessageBox::question(_mainWindow, 
    tr("Save current battle"),
    tr("Do you want to save changes?"), 
    QMessageBox::Yes|QMessageBox::Default, QMessageBox::No,
    QMessageBox::Cancel|QMessageBox::Escape);

  if (QMessageBox::No == res) {
    return true;
  }
  else if (QMessageBox::Yes == res) {
    fileSave();
    return true;
  }
  else {
    if (isRunning) {
      _battle->slotPause();
      update();
    }
    return false;
  }
}

void BMainWindowStateEdit::update() {
  shipsUngroupAction->setEnabled(_uiState->canUngroupSelection());
  shipsOrderAction->setEnabled(_uiState->canOrderSelection());
}

class reset_t0: public std::unary_function<BattleObject*, void> {
public:
  reset_t0(const QDateTime& newT0):_t0(newT0) {
  }
  void operator()(BattleObject* bo) {
    const BattleObject* uus = bApp->battle()->ships()->unUsedShips();
    if (uus->find(bo->name()))
      return;
    QDomDocument doc;
    QDomElement el = bo->state(doc);
    //   t0
    QDomElement programT0Element;
    bool programT0Found = false;
    QDomNode n = el.firstChild();
    while( !n.isNull() ) {
      QDomElement e = n.toElement(); 
      n = n.nextSibling();
      if( !e.isNull() ) { 
        if (e.tagName() == BProgram::PROGRAM_T0) {
          programT0Found = true;
          programT0Element = e;
        }
      }
    }
    if (programT0Found) {
      QDomElement e1 = BDomUtils::dateTimeToXml(_t0, BProgram::PROGRAM_T0, doc);
      el.replaceChild(e1, programT0Element);
      bo->setState(el);
    }
  }
private:
  QDateTime _t0;
};

void BMainWindowStateEdit::fileProperties() {
  BScenarioHeaderEdit e(_battle->scenarioHeader(), _mainWindow, "header_edit");
  e.exec();
  if (e.result() == QDialog::Accepted) {
    BScenarioHeader* h = const_cast<BScenarioHeader*>(_battle->scenarioHeader());
    QDateTime dt = h->beginDate();
    h->setBeginDate(e.scenarioStartDate());
    h->setEndDate(e.scenarioStartDate());
    h->setName(e.scenarioName());
    h->setDescription(e.scenarioDescription());
    h->setSideName(1, e.scenarioSide1Name());
    h->setSideName(2, e.scenarioSide2Name());
    if (dt != e.scenarioStartDate()) {
      BattleObjects bos = _battle->ships()->objs(BattleObject::AllSelectionType);
      std::for_each(bos.begin(), bos.end(), reset_t0(e.scenarioStartDate()));
      dynamic_cast<BattleEdit*>(_battle)->setModelTime(e.scenarioStartDate());
    }
    _battle->setModified(true);
    _propertyView->updateView();
    _mainWindow->setCaption(tr("%1. Editor Mode - BattleFleet").arg(
      _battle->scenarioHeader()->name()));
  }
}