/******************************************************************************/
/* BNewScenario -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bnewscenario_cpp_id[]="@(#) $Id: bnewscenario.cpp,v 1.5 2004/10/13 05:33:51 Oleksiy Balabay Exp $"; static const char* bnewscenario_cpp_id_id=bnewscenario_cpp_id;

#include <qdatetimeedit.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qdir.h>
#include <qtextedit.h>
#include <qmessagebox.h>

#include "bnewscenario.h"
#include "bscenariotable.h"
#include "bscenarioheader.h"
#include "bdomutils.h"
#include "bapplication.h"

/** \file bnewscenario.cpp
  \brief  BNewScenario  BattleFleet (impl.)

     BNewScenario  BattleFleet
*/

/** \class BNewScenario
  \brief 


*/
/** \brief 
*/
BNewScenario::BNewScenario(QWidget* parent, const char* name, 
  WFlags fl):BNewScenarioBase(parent, name, fl) {
  _beginDate->setDateTime(QDateTime::currentDateTime());
  _name->setText(EMPTY_SCENARIO);
  const BSideHeader::CountryFlags& cfs = BSideHeader::flags();
  for (BSideHeader::CountryFlags::const_iterator it = cfs.begin();
  it != cfs.end(); it++) {
    _side1->insertItem(*(it->second.second), it->second.first);
    _side2->insertItem(*(it->second.second), it->second.first);
  }
  _side1->setMinimumHeight(cfs.begin()->second.second->height()+8);
  _side2->setMinimumHeight(cfs.begin()->second.second->height()+8);
}

/** \brief 
*/
BNewScenario::~BNewScenario(){

}

bool BNewScenario::slotOk(const QString& description) {
  bool save = checkValidity();
  if (save) {
    QFileInfo fi(bApp->scenariosDir()+"/"+ _fileName->text()+".xml");
    if (fi.exists()) {
      int res = QMessageBox::question((QWidget*)parent(), 
        tr("File exists"),
        tr("File \n'%1'\nalready exists. Do you want to overwrite it?").arg(
        fi.absFilePath()), 
        QMessageBox::Yes|QMessageBox::Default, 
        QMessageBox::No|QMessageBox::Escape);

      if (QMessageBox::No == res) {
        return false;
      }
    }
    saveDefaultScenario(description);
  }
  return save;
}

void BNewScenario::saveDefaultScenario(const QString& description) {
  //   
  QDomDocument doc;
  QDomElement date = 
    BDomUtils::dateTimeToXml(_beginDate->dateTime(), "BeginDate", doc);

  QString defaultHeaders = QString(
    "<?xml version='1.0' encoding='UTF-8'?>\n"
    "<Scenario>\n"
    "  <Name>"+_name->text()+ "</Name>\n"
    "  <Side1 Name='"+_side1->currentText()+"' />\n"
    "  <Side2 Name='"+_side2->currentText()+"' />\n"
    "  "+shipDbFiles()+"\n"
    "  <MapFileName>"+bApp->scenariosDir()+"/"+_fileName->text()+"/map.xml</MapFileName>\n"
//    "  <ShipsOnMapFileName>data/%6/ships_on_map.xml</ShipsOnMapFileName>\n"
    "  <EventsFileName>"+bApp->scenariosDir()+"/"+_fileName->text()+"/events.xml</EventsFileName>\n"
    "  <AIFileName>"+bApp->scenariosDir()+"/"+_fileName->text()+"/ai.xml</AIFileName>\n"
    "  <Description>"+description+"</Description>\n"
    "  "+BDomUtils::elementToString(date)+"/>\n"
    "</Scenario>\n"
    );
  
  QString defaultMap = QString(
    "<?xml version='1.0' encoding='UTF-8'?>\n"
    "<Map>\n"
    "  <Geometry>\n"
    "    <BottomLeft X='E 0D 0M 0S' Y='N 60D 0M 0S' />\n"
    "    <TopRight X='E 0D 30M 0S' Y='N 60D 15M 0S' />\n"
    "  </Geometry>\n"
    "</Map>\n");
 
  QString defaultAI = QString(
    "<?xml version='1.0' encoding='UTF-8'?>\n"
    "<AIs/>\n");

  QDomElement dateE = 
    BDomUtils::dateTimeToXml(_beginDate->dateTime(), "Date", doc);
  QString defaultEvents = QString(
    "<?xml version='1.0' encoding='UTF-8' ?>\n"
    "<Events>\n"
    "  "+BDomUtils::elementToString(dateE)+"\n"
    "</Events>\n");

  saveFile(bApp->scenariosDir()+"/"+ _fileName->text()+".xml", defaultHeaders);
  QDir d(bApp->scenariosDir());
  d.mkdir(_fileName->text());
  saveFile(bApp->scenariosDir()+"/"+ _fileName->text()+"/map.xml", defaultMap);
  saveFile(bApp->scenariosDir()+"/"+ _fileName->text()+"/events.xml", defaultEvents);
  saveFile(bApp->scenariosDir()+"/"+ _fileName->text()+"/ai.xml", defaultAI);
}

QString BNewScenario::shipDbFiles() {
  return "  <ShipsFileName>common/russia.xml</ShipsFileName>\n"
         "  <ShipsFileName>common/japan.xml</ShipsFileName>";
}

void BNewScenario::saveFile(const QString& fileName, const QString& data) {
  QFile f(fileName);
  if (!f.open(IO_WriteOnly)) {
    throw BOutputException(__FILE__, __LINE__, fileName);
  }
  QTextStream t(&f);
  t.setEncoding(QTextStream::UnicodeUTF8);
  t << data;
  f.close();
}

QString BNewScenario::scenarioFileName() const {
  return bApp->scenariosDir()+"/"+_fileName->text()+".xml";
}

QDateTime BNewScenario::scenarioStartDate() const {
  return _beginDate->dateTime();
}

QString BNewScenario::scenarioName() const {
  return _name->text();
}

QString BNewScenario::scenarioSide1Name() const {
  return _side1->currentText();
}

QString BNewScenario::scenarioSide2Name() const {
  return _side2->currentText();
}

void BNewScenario::setScenarioHeader(const BScenarioHeader* h) {
  _beginDate->setDateTime(h->beginDate());
  setSide(_side1, h->sideHeader(1)->name(), h);
  setSide(_side2, h->sideHeader(2)->name(), h);
  _name->setText(h->name());
  QFileInfo fi(h->fileName());
  _fileName->setText(fi.baseName(false));
  _fileName->setEnabled(false);
}

void BNewScenario::setSide(QComboBox* c, const QString& name, 
  const BScenarioHeader* h) {
  for(int i = 0; i < c->count(); i++) {
    if (c->text(i).upper() == name.upper()) {
      c->setCurrentItem(i);
      return;
    }
  }
  //   Unknown
  c->changeItem(h->sideHeader(1)->flag(), name, c->currentItem());
  c->setCurrentItem(c->currentItem());
}

bool BNewScenario::checkValidity() {
  if (_fileName->text().isEmpty()) {
    QMessageBox::information((QWidget*)parent(), tr("Enter file name"),
      tr("File name cannot be empty. Please, enter file name"), 
      QMessageBox::Ok|QMessageBox::Default);
    _fileName->setFocus();
    return false;
  }
  if (_side1->currentText() == _side2->currentText()) {
    QMessageBox::information((QWidget*)parent(), tr("Enter side name"),
      tr("Side names should be different"), 
      QMessageBox::Ok|QMessageBox::Default);
    _side1->setFocus();
    return false;
  }
  if (_side1->currentText().isEmpty()) {
    QMessageBox::information((QWidget*)parent(), tr("Enter side name"),
      tr("Side name cannot be empty"), 
      QMessageBox::Ok|QMessageBox::Default);
    _side1->setFocus();
    return false;
  }
  if (_side2->currentText().isEmpty()) {
    QMessageBox::information((QWidget*)parent(), tr("Enter side name"),
      tr("Side name cannot be empty"), 
      QMessageBox::Ok|QMessageBox::Default);
    _side2->setFocus();
    return false;
  }
  return true;
}
