/******************************************************************************/
/* BObserverCentral -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bobservercentral_cpp_id[]="@(#) $Id: bobservercentral.cpp,v 1.14 2005/03/07 15:34:38 Oleksiy Balabay Exp $"; static const char* bobservercentral_cpp_id_id=bobservercentral_cpp_id;


/** \file bobservercentral.cpp
  \brief  BObserverCentral  BattleFleet (impl.)

     BObserverCentral  BattleFleet
*/
#include <qwidget.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qcursor.h>

#include "bobservercentral.h"
#include "bscrollbar.h"
#include "bcentral.h"
#include "bapplication.h"
#include "boceditstate.h"
#include "bcanvasship.h"
#include "buistate.h"
#include "battleobject.h"
#include "bai.h"
#include "bcanvasconnection.h"
#include "bcustomeventconsts.h"

/** \class BObserverCentral
  \brief 


*/
/** \brief 
*/
BObserverCentral::BObserverCentral(BUiState* uiState, Battle* battle,
  QObject* parent, const char* name)
  :BattleObserver(battle, parent, name), _uiState(uiState) {
  _currentState = new BOCEditStateIdle(this);
}

/** \brief 
*/
BObserverCentral::~BObserverCentral(){
  delete _currentState;
  delete _viewWidget;
}

void BObserverCentral::init() {
  QWidget* c = new QWidget( 0, "qt_central_widget" );
  QGridLayout* BMainWindowBaseLayout = new QGridLayout( c, 
    1, 1, 1, 0, "BMainWindowBaseLayout"); 

  _vScrollBar = new BScrollBar( c, "_vScrollBar" );
  _vScrollBar->setFrameShape( BScrollBar::NoFrame );
  _vScrollBar->setFrameShadow( BScrollBar::Plain );
  _vScrollBar->setProperty( "orient", "Vertical" );

  BMainWindowBaseLayout->addWidget( _vScrollBar, 0, 1 );

  _hScrollBar = new BScrollBar( c, "_hScrollBar" );

  BMainWindowBaseLayout->addWidget( _hScrollBar, 1, 0 );

  _central = createCentral( c, "_central" );

  BMainWindowBaseLayout->addWidget( _central, 0, 0 );
  _viewWidget = c;
  _viewWidget->setFocusPolicy(QWidget::StrongFocus);
  _viewWidget->installEventFilter(this);
  connect( _central, SIGNAL( visiblePart(Orientation,int,int) ), _vScrollBar, SLOT( setVisiblePartSlot(Orientation,int,int) ) );
  connect( _central, SIGNAL( visiblePart(Orientation,int,int) ), _hScrollBar, SLOT( setVisiblePartSlot(Orientation,int,int) ) );
  connect( _hScrollBar, SIGNAL( contentsMoved(Orientation,int) ), _central, SLOT( contentsMovedSlot(Orientation,int) ) );
  connect( _vScrollBar, SIGNAL( contentsMoved(Orientation,int) ), _central, SLOT( contentsMovedSlot(Orientation,int) ) );

  //    Pursuier - Evader
  connect( _uiState, SIGNAL( showChasingConnections(bool) ), 
    _central, SLOT( slotShowChasingConnections(bool) ) );
  connect( _uiState->ais(), 
    SIGNAL( chasingConnectionChanged(BAIChasing*, const bool*, const bool*) ), 
    _central, SLOT( slotChasingConnectionChanged(BAIChasing*, const bool*, const bool*)));
  //         
  if (_uiState->editorMode() == BUiState::ScaleMode) {
//    _viewWidget->setCursor(CrossCursor);
    _central->setCursor(CrossCursor);
  }
  else {
    _central->setCursor(ArrowCursor);
  }
  _central->battleChanged(_battle);
}

BattleObject* BObserverCentral::activeObject() const {
  return _uiState->activeObject();
}

const BattleObjects& BObserverCentral::selectedObjects() const {
  return _uiState->selectedObjects();
}

BCentral* BObserverCentral::createCentral(QWidget* parent, const char* name) {
  return new BCentral(this, parent, name);
}

void BObserverCentral::slotBattleInfoChanged(const BattleInfo& info) {
  _central->slotBattleInfoChanged(info);
}

void BObserverCentral::slotAboutToQuitBattle() {
  ;
}

void BObserverCentral::customEvent(QCustomEvent* e) {
  if (e->type() == BZoomEventNumber) {
    BZoomEvent* ze = (BZoomEvent*) e;
    if (ze->zoom() == BZoomEvent::ZoomIn) {
      _central->zoomIn();
    }
    else if (ze->zoom() == BZoomEvent::FullView) {
      _central->fullView();
    }
    else if (ze->zoom() == BZoomEvent::ZoomOut) {
      _central->zoomOut();
    }
  }
}

BCanvasShip* BObserverCentral::objAtPoint(const QPoint& p) {
  QCanvasItemList list = _central->canvas()->collisions(p);
  if (list.isEmpty()) 
    return 0;
  else {
    //      ?
    QCanvasItemList::Iterator it = list.begin();
    for(; it != list.end(); ++it) {
      BCanvasShip* shp = dynamic_cast<BCanvasShip*>(*it);
      if (shp) {
        return shp;
      }
    }
    return 0;
  }
}

BCanvasConnection* BObserverCentral::connectionAtPoint(const QPoint& p) {
  QCanvasItemList list = _central->canvas()->collisions(p);
  if (list.isEmpty()) 
    return 0;
  else {
    //      ?
    QCanvasItemList::Iterator it = list.begin();
    for(; it != list.end(); ++it) {
      BCanvasConnection* c = dynamic_cast<BCanvasConnection*>(*it);
      if (c) {
        return c;
      }
    }
    return 0;
  }
}


BOCEditState* BObserverCentral::currentState(QMouseEvent* e) {
  if ( (e->type() == QEvent::MouseButtonPress)&& 
    (e->button() == QMouseEvent::LeftButton)) {
    BCanvasShip* o = objAtPoint(e->pos());
    BCanvasConnection* c = connectionAtPoint(e->pos());
    if (( 0 == o ) && (0 == c)) {
      //     canvas 
      //   
      delete _currentState;
      if (_uiState->editorMode() == BUiState::SelectMode) {
        _currentState = new BOCEditStateSelect(this);
      }
      else if (_uiState->editorMode() == BUiState::ScaleMode) {
        _currentState = new BOCEditStateScale(this);
      }
      else {
        _currentState = new BOCEditStateIdle(this);
      }
    }
    else { // - 
      if (0 != c) {
        if (dynamic_cast<BOCEditStateIdle*>(_currentState) == 0) {
          delete _currentState;
          _currentState = new BOCEditStateIdle(this);
        }
      }
      if (0 != o) {
        if (_uiState->editorMode() == BUiState::ConnectMode) {
          delete _currentState;
          _currentState = new BOCEditStateConnect(this, o,
            _central->viewportToContents(e->pos()));
        }
        else if (dynamic_cast<BOCEditStateIdle*>(_currentState) == 0) {
          delete _currentState;
          _currentState = new BOCEditStateIdle(this);
        }
      }
    }
  }
  else if ((e->type() == QEvent::MouseButtonRelease)&& 
    (e->button() == QMouseEvent::LeftButton)) {

  }
  else {

  }
  return _currentState;
}

void BObserverCentral::slotSelectionChanged(const BUiSelectionInfo& selectionInfo) {
  BattleObject* newO = selectionInfo.activeObject;
  _central->setActiveObject(newO);
  _central->setSelectedObjects(*selectionInfo.selectedObjects);
}

void BObserverCentral::removeSelectedConnections() {
  std::vector<BAI*> r = _central->selectedConnections();
  for(std::vector<BAI*>::iterator it = r.begin(); it != r.end(); it++) {
    _uiState->ais()->erase(*it);  
  }
}

void BObserverCentral::removeConnection(BCanvasConnection* con) {
  _uiState->ais()->erase(_central->connection(con));
}
