/******************************************************************************/
/* BObserverCentralGame -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bobservercentralgame_cpp_id[]="@(#) $Id: bobservercentralgame.cpp,v 1.9 2005/01/24 03:50:43 Oleksiy Balabay Exp $"; static const char* bobservercentralgame_cpp_id_id=bobservercentralgame_cpp_id;

#include <qdatetime.h>

#include "bobservercentralgame.h"
#include "bcentralgame.h"
#include "bprogramfactory.h"
#include "beventprogram.h"
#include "battle.h"
#include "battleobject.h"
#include "bunitsconsts.h"
#include "buistate.h"
#include "bai.h"

/** \file bobservercentralgame.cpp
  \brief  BObserverCentralGame  BattleFleet (impl.)

     BObserverCentralGame  BattleFleet
*/

/** \class BObserverCentralGame
  \brief 


*/
/** \brief 
*/

BObserverCentralGame::BObserverCentralGame(BUiState* uiState, Battle* battle,
  QObject* parent, const char* name)
  :BObserverCentral(uiState, battle, parent, name) {
  _turnLeft = _turnRight = _accelerate = _break = false;
}

/** \brief 
*/
BObserverCentralGame::~BObserverCentralGame(){
}

BCentral* BObserverCentralGame::createCentral(QWidget* parent, const char* name) {
  return new BCentralGame(this, parent, name);
}

void BObserverCentralGame::init() {
  BObserverCentral::init();
//  _central->installEventFilter(this);
  _commands.insert( Qt::Key_Up, Accelerate );
  _commands.insert( Qt::Key_Left, TurnLeft );
  _commands.insert( Qt::Key_Right, TurnRight );
  _commands.insert( Qt::Key_Down, Brake );
  _commands.insert( Qt::Key_Delete, Delete );
}

bool BObserverCentralGame::eventFilter(QObject* o, QEvent* e) {
  if ((e->type() != QEvent::KeyPress)&&(e->type() != QEvent::KeyRelease)) {
    return false;
  }
  QKeyEvent*event = (QKeyEvent*)e;
  if (e->type() == QEvent::KeyPress) {
    if ( event->isAutoRepeat() || !_commands.contains( event->key() ) ) {
      return false;
    }
    BCommand b = _commands[ event->key() ];
    switch ( b ) {
    case TurnLeft:
      turnLeft( true );
      break;
    case TurnRight:
      turnRight( true );
      break;
    case Accelerate:
      accelerate( true );
      break;
    case Brake:
      brake( true );
      break;
    case Delete:
      removeSelectedConnections();
      break;
    default:
      return false;
    }
    return true;
  }
  else if (e->type() == QEvent::KeyRelease) {
    if ( event->isAutoRepeat() || !_commands.contains( event->key() ) ) {
      return false;
    }
    BCommand b = _commands[ event->key() ];
    switch ( b ) {
    case TurnLeft:
      turnLeft( false );
      break;
    case TurnRight:
      turnRight( false );
      break;
    case Accelerate:
      accelerate( false );
      break;
    case Brake:
      brake( false );
      break;
    default:
      return false;
    }
    return true;
  }
  else {
    return false;
  }
}

void BObserverCentralGame::turnLeft(const bool r) {
  BattleObject* o = activeObject();
  if (0 == o)
    return;
  _turnLeft = r;
  processCommand(o);
}

void BObserverCentralGame::turnRight(const bool r) {
  BattleObject* o = activeObject();
  if (0 == o)
    return;
  _turnRight = r;
  processCommand(o);
}

void BObserverCentralGame::accelerate(const bool r) {
  BattleObject* o = activeObject();
  if (0 == o)
    return;
  _accelerate = r;
  processCommand(o);
}

void BObserverCentralGame::brake(const bool r) {
  BattleObject* o = activeObject();
  if (0 == o)
    return;
  _break = r;
  processCommand(o);
}

void BObserverCentralGame::processCommand(BattleObject* activeObject) {
  QDateTime dt = _battle->battleInfo().modelTime;
  dt.setTime(QTime(dt.time().hour(), dt.time().minute(), dt.time().second()));
  BKinematicData kd = activeObject->kinematicData(dt);
  double angle = 0;
  double newSpeed = kd.v(false);

  if (_turnLeft && _accelerate) {
     angle = -2*PI;
     newSpeed = activeObject->maxSpeed();
  }
  else if (_turnLeft && _break) {
    angle = -2*PI;
    newSpeed = -activeObject->maxSpeed();
  }
  else if (_turnRight && _accelerate) {
    angle = 2*PI;
    newSpeed = activeObject->maxSpeed();
  }
  else if (_turnRight && _break) {
    angle = 2*PI;
    newSpeed = -activeObject->maxSpeed();
  }
  else if (_turnLeft) {
    angle = -2*PI;
  }
  else if (_turnRight) {
    angle = 2*PI;
  }
  else if (_accelerate) {
    newSpeed = activeObject->maxSpeed();
  }
  else if (_break) {
    newSpeed = -activeObject->maxSpeed();
  }
  _uiState->ais()->setControlMode(BAI::ControlModeManual, activeObject);
  postEvent(
    new BEventProgram(BProgramFactory::createTurnAccelerateProgram(
    _battle, &angle, &newSpeed), activeObject->name(), dt),
  true, false);
}
