/******************************************************************************/
/* BOCEditState -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char boceditstate_cpp_id[]="@(#) $Id: boceditstate.cpp,v 1.19 2005/03/07 15:34:38 Oleksiy Balabay Exp $"; static const char* boceditstate_cpp_id_id=boceditstate_cpp_id;

#include <qcanvas.h>
#include <qcolor.h>
#include <qcursor.h>

#include "boceditstate.h"
#include "bobservercentral.h"
#include "bcentral.h"
#include "bseamap.h"
#include "bcanvasship.h"
#include "bship.h"
#include "bprogramfactory.h"
#include "battle.h"
#include "beventprogram.h"
#include "buiutils.h"
#include "bapplication.h"
#include "buistate.h"
#include "bai.h"
#include "bcanvasconnection.h"

const int SELECTION_LAYER = 3000;

/** \file boceditstate.cpp
  \brief  BOCEditState  BattleFleet (impl.)

     BOCEditState  BattleFleet
*/

/** \class BOCEditState
  \brief 


*/
BOCEditStateRect::BOCEditStateRect(BObserverCentral* observer,
  const QPen& linePen):BOCEditState(observer) {
//  qDebug("BOCEditStateRect state created");
  //  
  for (int i = 0; i < 4; i++) {
    _selectionLines[i] = new QCanvasLine(_observer->_central->canvas());
    _selectionLines[i]->setPen(linePen);
    _selectionLines[i]->setZ(SELECTION_LAYER);
  }
}

BOCEditStateRect::~BOCEditStateRect() {
//  qDebug("BOCEditStateRect state deleted");
  for (int j = 0; j < 4; j++) {
      delete _selectionLines[j];
  }
  if (_selectionLines[0])
    _observer->_central->canvas()->update();
}

void BOCEditStateRect::contentsMousePressEvent(QMouseEvent* e) {
  if (e->button() != QMouseEvent::LeftButton)
    return;
//  qDebug("BOCEditStateScale::contentsMousePressEvent");
  _mouseStart = e->pos();
  _mouseEnd = e->pos();
  _selectionLines[0]->setPoints(_mouseStart.x(), _mouseStart.y(),
    _mouseEnd.x(), _mouseStart.y());
  _selectionLines[1]->setPoints(_mouseEnd.x(), _mouseStart.y(),
    _mouseEnd.x(), _mouseEnd.y());
  _selectionLines[2]->setPoints(_mouseEnd.x(), _mouseEnd.y(),
    _mouseStart.x(), _mouseEnd.y());
  _selectionLines[3]->setPoints(_mouseStart.x(), _mouseEnd.y(),
    _mouseStart.x(), _mouseStart.y());
  for (int j = 0; j < 4; j++) {
    _selectionLines[j]->show();
  }
  _observer->_central->canvas()->update();
}

void BOCEditStateRect::contentsMouseReleaseEvent(QMouseEvent* e) {
  if (e->button() != QMouseEvent::LeftButton)
    return;
//  qDebug("BOCEditStateScale::contentsMouseReleaseEvent");
  _mouseEnd = e->pos();
  for (int j = 0; j < 4; j++) {
    delete _selectionLines[j];
    _selectionLines[j] = 0;
  }
  _observer->_central->canvas()->update();
  QRect r = QRect(_mouseStart, _mouseEnd).normalize();
  releaseAction(r);
}

void BOCEditStateRect::releaseAction(const QRect& /* r */) {
  _observer->_currentState = new BOCEditStateIdle(_observer);
  delete this;
}

void BOCEditStateScale::releaseAction(const QRect& r) {
  if (r.width() > 4 && r.height() > 4) {
    BMapPoint bl = 
      _observer->_central->coordTransformer()->pointToMap(r.bottomLeft());
    BMapPoint tr = 
      _observer->_central->coordTransformer()->pointToMap(r.topRight());
    _observer->_central->showNewCanvas(
      BCoordinates(bl.first, bl.second, tr.first - bl.first, 
      tr.second - bl.second));
  }
  else {
    _observer->_uiState->setSelectedObjects(BattleObjects());
    _observer->_central->deselectAllConnections();
  }
  _observer->_currentState = new BOCEditStateIdle(_observer);
  delete this;
}

void BOCEditStateRect::contentsMouseMoveEvent(QMouseEvent* e) {
  _mouseEnd = _observer->_central->viewportToContents(e->pos());
  _selectionLines[0]->setPoints(_mouseStart.x(), _mouseStart.y(),
    _mouseEnd.x(), _mouseStart.y());
  _selectionLines[1]->setPoints(_mouseEnd.x(), _mouseStart.y(),
    _mouseEnd.x(), _mouseEnd.y());
  _selectionLines[2]->setPoints(_mouseEnd.x(), _mouseEnd.y(),
    _mouseStart.x(), _mouseEnd.y());
  _selectionLines[3]->setPoints(_mouseStart.x(), _mouseEnd.y(),
    _mouseStart.x(), _mouseStart.y());
  _observer->_central->canvas()->update();
  BOCEditState::contentsMouseMoveEvent(e);
}

BOCEditStateSelect::BOCEditStateSelect(BObserverCentral* observer)
:BOCEditStateRect(observer, QPen(Qt::green, 1, Qt::DotLine)) {
}

BOCEditStateSelect::~BOCEditStateSelect() {
}

BOCEditStateScale::BOCEditStateScale(BObserverCentral* observer)
  :BOCEditStateRect(observer, QPen(Qt::black, 1, Qt::DashLine)) {
}

BOCEditStateScale::~BOCEditStateScale() {
}

void BOCEditStateSelect::releaseAction(const QRect& r) {
  QCanvasItemList list = _observer->_central->canvas()->collisions(r);
  BattleObjects bos;
  typedef std::set<BCanvasConnection*> BCanvasConnections;
  BCanvasConnections ccs;
  QCanvasItemList::Iterator it = list.begin();
  for(; it != list.end(); ++it) {
    BCanvasShip* shp = dynamic_cast<BCanvasShip*>(*it);
    if (shp) {
      bos.insert(shp->ship());
    }
    BCanvasConnection* c = dynamic_cast<BCanvasConnection*>(*it);
    if (c) {
      ccs.insert(c);
    }
  }
  if (ccs.size() > 0) {
    for (BCanvasConnections::iterator it = ccs.begin(); it != ccs.end(); it++) {
      (*it)->setSelected(true);
    }
  }
  else if (bos.size() == 0) {
    _observer->_central->deselectAllConnections();
  }
  _observer->_uiState->setSelectedObjects(bos);
  _observer->_currentState = new BOCEditStateIdle(_observer);
  delete this;
}

void BOCEditStateIdle::contentsMousePressEvent(QMouseEvent*  e ) {
  if (e->button() == QMouseEvent::LeftButton)
    _keyPressed = true;
}

void BOCEditStateIdle::processShip(BCanvasShip* shp, QMouseEvent* e) {
 //    ,   .
  //      _uiState
  if (shp->ship() == _observer->_uiState->activeObject()) {
    _observer->_uiState->setActiveObject(0);
  }
  else {
    //      ,    ,
    //    
    if (_observer->_uiState->selectedObjects().find(shp->ship()) 
      != _observer->_uiState->selectedObjects().end()) {
      _observer->_uiState->setActiveObject(shp->ship());
    }
    else {
      //   Shift     
      //   
      if ( (e->state() & Qt::ShiftButton) != 0) {
        _observer->_uiState->setActiveObject(shp->ship());  
      }
      //   Ctrl     
      else if ((e->state() & Qt::ControlButton) != 0) {
        BattleObjects bos;
        bos.insert(shp->ship());
        _observer->_uiState->addSelectedObjects(bos);  
      }
      else {
        //       
        _observer->_uiState->setActiveObjectAsSelection(shp->ship());
      }
    }
  }
}
void BOCEditStateIdle::processConnection(BCanvasConnection* c, QMouseEvent* /*e*/) {
  if (c == 0) {
    _observer->_central->deselectAllConnections();
  }
  if (c->isSelected())
    c->setSelected(false);
  else
    c->setSelected(true);
  _observer->_central->canvas()->update();
}

void BOCEditStateIdle::contentsMouseReleaseEvent(QMouseEvent* e) {

  if ( (!_keyPressed) && (e->button() == Qt::RightButton)) {
    BCanvasConnection* c = _observer->connectionAtPoint(e->pos());
    if (c) {
      c->popup(QCursor::pos());
    }
    return;
  }
  if (!_keyPressed)
    return;
  //   
  BCanvasShip* shp = _observer->objAtPoint(e->pos());
  BCanvasConnection* c = _observer->connectionAtPoint(e->pos());
  if (e->button() == Qt::LeftButton) {
    if (shp) {
      processShip(shp, e); 
    }
    if (c) {
      processConnection(c, e);
    }
    else {
      _observer->_central->deselectAllConnections();
    }
  }
  _keyPressed = false;
//  _observer->_central->canvas()->update();
}

BOCEditStateMove::BOCEditStateMove(BObserverCentral* observer, 
  BCanvasShip* movedObject):BOCEditState(observer), _movedObject(movedObject) {
//  qDebug("Move state created");
  _dx = 0;
  _dy = 0;
  _initialized = false;
}

BOCEditStateMove::~BOCEditStateMove() {
//  qDebug("Move state deleted");
}

void BOCEditStateMove::contentsMousePressEvent(QMouseEvent* /* e */) {
}

void BOCEditStateMove::contentsMouseReleaseEvent(QMouseEvent* e) {
  if (e->button() != QMouseEvent::LeftButton)
    return;
//  qDebug("BOCEditStateMove::contentsMouseReleaseEvent");
  Battle* b = _observer->_battle;
  QDateTime dt = b->battleInfo().modelTime;
  BattleObject* bo = _movedObject->ship();

  QPoint point = QPoint(e->pos().x()+_dx, e->pos().y() + _dy);
  BMapPoint pnt = _observer->_central->coordTransformer()->pointToMap(point);
//  QPoint debugPnt = _observer->_central->coordTransformer()->pointFromMap(pnt);
  BProgram* p = BProgramFactory::createStandByProgram(
    pnt.first, pnt.second, b);
  BEvent* evp = new BEventProgram(p, bo->name(), dt);
  _observer->postEvent(evp);
  _observer->_currentState = new BOCEditStateIdle(_observer);
  delete this;
}

void BOCEditStateMove::contentsMouseMoveEvent(QMouseEvent* e) {
  if (!_initialized) {
//    qDebug("BOCEditStateMove::contentsMousePressEvent");
    if (!_movedObject->isActive())
      _observer->_uiState->setActiveObject(_movedObject->ship());
    _dx = int(_movedObject->x() - e->pos().x());
    _dy = int(_movedObject->y() - e->pos().y());
    _initialized = true;
  }
  _movedObject->moveBy(e->pos().x() - _movedObject->x() + _dx,
    e->pos().y() - _movedObject->y() + _dy);
  _observer->_central->updateChasingConnections();
  _observer->_central->canvas()->update();
  BOCEditState::contentsMouseMoveEvent(e);
}

void BOCEditState::contentsMouseMoveEvent(QMouseEvent* e) {
  //     
  QPoint pos = e->pos();
  BMapPoint mp = _observer->_central->coordTransformer()->pointToMap(pos);

  QString shipName = "";
  QCanvasItemList list = 
    _observer->_central->canvas()->collisions(e->pos());
  if ( !list.isEmpty() ) {
    QCanvasItemList::Iterator it = list.begin();
    for(; it != list.end(); ++it) {
      BCanvasShip* shp = dynamic_cast<BCanvasShip*>(*it);
      if (shp) {
        shipName = shp->ship()->name();
        break;
      }
    }
  }
  if (BUiUtils* _uiUtils = bApp->uiUtils()) {
    QString message = qApp->translate("BOCEditState",
      "X=%1(%3), Y=%2(%4) %5").arg(
      pos.x()).arg(pos.y()).arg(
      _uiUtils->toString(mp.first, coordinateValue, BSeaMap::longitude)).arg(
      _uiUtils->toString(mp.second, coordinateValue, BSeaMap::latitude)).arg(
      shipName);
    _observer->emitMouseOver(pos.x(),pos.y(),message);
  }
  if (shipName.isEmpty()) {
    if (_observer->_uiState->editorMode() == BUiState::ScaleMode) {
      if (_observer->_central->cursor().shape() != Qt::CrossCursor) {
        _observer->_central->setCursor(Qt::CrossCursor);
      }
    }
    else {
      if (_observer->_central->cursor().shape() != Qt::ArrowCursor) {
        _observer->_central->setCursor(Qt::ArrowCursor);
      }
    }
  }
  else {
    //  
    if (_observer->_central->cursor().shape() != Qt::PointingHandCursor) {
      _observer->_central->setCursor(Qt::PointingHandCursor);
    }
  }
}

BOCEditStateConnect::BOCEditStateConnect(BObserverCentral* observer, 
  BCanvasShip* connectObject, const QPoint& mouseStart):BOCEditState(observer), 
  _connectObject(connectObject), 
  _mouseStart(QPoint((int)connectObject->x(), (int)connectObject->y())), 
  _mouseEnd(mouseStart) {
//  qDebug("Connect state created");
  _connectLine = new QCanvasLine(_observer->_central->canvas());
  _connectLine->setPen(QPen(Qt::black, 1, Qt::DotLine));
  _connectLine->setZ(SELECTION_LAYER);
  _connectLine->setPoints(_mouseStart.x(), _mouseStart.y(),
    _mouseEnd.x(), _mouseEnd.y());
  _connectLine->show();
  _observer->_central->canvas()->update();
}

BOCEditStateConnect::~BOCEditStateConnect() {
//  qDebug("Connect state deleted");
  delete _connectLine;
}

void BOCEditStateConnect::contentsMousePressEvent(QMouseEvent* /* e */) {
}

void BOCEditStateConnect::contentsMouseReleaseEvent(QMouseEvent* e) {
  if (e->button() != QMouseEvent::LeftButton)
    return;
//  qDebug("BOCEditStateConnect::contentsMouseReleaseEvent");
  BCanvasShip* shp = _observer->objAtPoint(e->pos());
  _connectLine->hide();
  _observer->_central->canvas()->update();
  if ((shp == 0)||(shp == _connectObject)) {
    _observer->_currentState = new BOCEditStateIdle(_observer);
    delete this;
    return;
  }
  else {
    // TODO: send command to AI
    _observer->_uiState->ais()->addChasing(_connectObject->ship(),shp->ship() );
    _observer->_currentState = new BOCEditStateIdle(_observer);
    delete this;
    return;
  }
}

void BOCEditStateConnect::contentsMouseMoveEvent(QMouseEvent* e) {
  _mouseEnd = _observer->_central->viewportToContents(e->pos());
  _connectLine->setPoints(_mouseStart.x(), _mouseStart.y(),
    _mouseEnd.x(), _mouseEnd.y());
  _observer->_central->canvas()->update();
  BOCEditState::contentsMouseMoveEvent(e);
}

