/******************************************************************************/
/* BOCEditState -   BattleFleet              */
/******************************************************************************/

#ifndef BOCEditState_H
#define BOCEditState_H
static const char boceditstate_h_id[]="@(#) $Id: boceditstate.h,v 1.6 2005/02/20 14:45:00 Oleksiy Balabay Exp $"; static const char* boceditstate_h_id_id=boceditstate_h_id;
/** \file boceditstate.h
  \brief  BOCEditState  BattleFleet

     BOCEditState  BattleFleet
*/
#include "bobservercentral.h"

class QCanvasLines;
class BCanvasShip;

class BOCEditState {
public:
  BOCEditState(BObserverCentral* observer):_observer(observer) {};
  virtual ~BOCEditState() {};
  virtual void contentsMousePressEvent(QMouseEvent* e) = 0;
  virtual void contentsMouseReleaseEvent(QMouseEvent* e) = 0;
  virtual void contentsMouseMoveEvent(QMouseEvent* e);
protected:
  BObserverCentral* _observer;
};

class BOCEditStateIdle: public BOCEditState {
public:
  BOCEditStateIdle(BObserverCentral* observer)
    :BOCEditState(observer), _keyPressed(false)
  { /* qDebug("Idle state created"); */ };
  virtual ~BOCEditStateIdle() { /*qDebug("Idle state deleted"); */};
  virtual void contentsMousePressEvent(QMouseEvent* e);
  virtual void contentsMouseReleaseEvent(QMouseEvent* e);
protected:
  void processShip(BCanvasShip* shp, QMouseEvent* e);
  void processConnection(BCanvasConnection* c, QMouseEvent* e);
private:
  bool _keyPressed;
};

class BOCEditStateRect: public BOCEditState {
public:
  BOCEditStateRect(BObserverCentral* observer, const QPen& linePen);
  virtual ~BOCEditStateRect();
  virtual void contentsMousePressEvent(QMouseEvent* e);
  virtual void contentsMouseReleaseEvent(QMouseEvent* e);
  virtual void contentsMouseMoveEvent(QMouseEvent* e);
protected:
  virtual void releaseAction(const QRect& r);
private:
  QCanvasLine* _selectionLines[4];
  QPoint _mouseStart;
  QPoint _mouseEnd;
};

class BOCEditStateScale: public BOCEditStateRect {
public:
  BOCEditStateScale(BObserverCentral* observer);
  virtual ~BOCEditStateScale();
protected:
  virtual void releaseAction(const QRect& r);
};

class BOCEditStateSelect: public BOCEditStateRect {
public:
  BOCEditStateSelect(BObserverCentral* observer);
  virtual ~BOCEditStateSelect();
protected:
  virtual void releaseAction(const QRect& r);
};

class BOCEditStateMove: public BOCEditState {
public:
  BOCEditStateMove(BObserverCentral* observer, BCanvasShip* movedObject);
  virtual ~BOCEditStateMove();
  virtual void contentsMousePressEvent(QMouseEvent* e);
  virtual void contentsMouseReleaseEvent(QMouseEvent* e);
  virtual void contentsMouseMoveEvent(QMouseEvent* e);
private:
  BCanvasShip* _movedObject;
  int _dx, _dy;
  int _x0, _y0;
  bool _initialized;
};

class BOCEditStateConnect: public BOCEditState {
public:
  BOCEditStateConnect(BObserverCentral* observer, BCanvasShip* connectObject,
    const QPoint& mouseStart);
  virtual ~BOCEditStateConnect();
  virtual void contentsMousePressEvent(QMouseEvent* e);
  virtual void contentsMouseReleaseEvent(QMouseEvent* e);
  virtual void contentsMouseMoveEvent(QMouseEvent* e);
private:
  BCanvasShip* _connectObject;
  QPoint _mouseStart;
  QPoint _mouseEnd;
  QCanvasLine* _connectLine;
};
#endif // BOCEditState_H
