/******************************************************************************/
/* BOptionsDialog -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char boptionsdialog_cpp_id[]="@(#) $Id: boptionsdialog.cpp,v 1.8 2005/04/09 16:14:26 Oleksiy Balabay Exp $"; static const char* boptionsdialog_cpp_id_id=boptionsdialog_cpp_id;

#include <qcombobox.h>
#include <qsettings.h>
#include <qcheckbox.h>
#include <qtabwidget.h>
#include <qpushbutton.h>

#include "boptionsdialog.h"
#include "bapplication.h"
#include "buiutils.h"
#include "bexception.h"
#include "ballonhelp.h"

/** \file boptionsdialog.cpp
  \brief  BOptionsDialog  BattleFleet (impl.)

     BOptionsDialog  BattleFleet
*/

/** \class BOptionsDialog
  \brief 


*/

const char* BOptionsDialog::_unitStrings[] = {
  QT_TRANSLATE_NOOP("BOptionsDialog", "mm"),
  QT_TRANSLATE_NOOP("BOptionsDialog", "cm"),
  QT_TRANSLATE_NOOP("BOptionsDialog", "m"),
  QT_TRANSLATE_NOOP("BOptionsDialog", "km"),
  QT_TRANSLATE_NOOP("BOptionsDialog", "cable"),
  QT_TRANSLATE_NOOP("BOptionsDialog", "mile"),
  QT_TRANSLATE_NOOP("BOptionsDialog", "degree"),
  QT_TRANSLATE_NOOP("BOptionsDialog", "knots"),
  QT_TRANSLATE_NOOP("BOptionsDialog", "km per hour"),
  QT_TRANSLATE_NOOP("BOptionsDialog", "inch"),
};

/** \brief 
*/
BOptionsDialog::BOptionsDialog(QWidget* parent, const char* name)
  :BOptionsDialogBase(parent, name, true, 0) {
//  bApp->addWidgetHelp(this, 
//    bApp->applicationDirPath()+"/"+"doc/ar01s03.html#edit-scenario-properties");
  _buttonHelp->hide();
  readOptions();
  _tabWidget->removePage(_units);
}

/** \brief 
*/
BOptionsDialog::~BOptionsDialog(){
//  bApp->removeWidgetHelp(this);
}

void BOptionsDialog::readOptions(){
  readUnitsPage();
  readMiscPage();
}

void BOptionsDialog::writeOptions(){
  writeUnitsPage();
  writeMiscPage();
}

void BOptionsDialog::readUnitsPage(){
  BPresentationUnits pu = BUiUtils::readConfig(bApp->settings());
  setShipSizeBox(pu);
  setDistanceBox(pu);
  setSpeedBox(pu);
  setArmorBox(pu);
  setCaliberBox(pu);
}

void BOptionsDialog::writeUnitsPage(){
  QSettings* settings = bApp->settings();
  settings->beginGroup(S_OPTIONS);
    settings->beginGroup(S_OPTIONS_UNITS);
      settings->writeEntry(S_OPTIONS_UNITS_SHIP_SIZE,
        unitFromString(_shipSizeBox->currentText()));
      settings->writeEntry(S_OPTIONS_UNITS_DISTANCE,
        unitFromString(_distanceBox->currentText()));
      settings->writeEntry(S_OPTIONS_UNITS_SPEED,
        unitFromString(_speedBox->currentText()));
      settings->writeEntry(S_OPTIONS_UNITS_ARMOR,
        unitFromString(_armorBox->currentText()));
      settings->writeEntry(S_OPTIONS_UNITS_CALIBER,
        unitFromString(_caliberBox->currentText()));
    settings->endGroup();
  settings->endGroup();
}

void BOptionsDialog::readMiscPage() {
  BallonHelp* bh = bApp->ballonHelp();
  _showBallonHelp->setChecked(bh->isAutoShow());
}

void BOptionsDialog::writeMiscPage() {
  BallonHelp* bh = bApp->ballonHelp();
  bh->setAutoShow(_showBallonHelp->isChecked());
}

void BOptionsDialog::setShipSizeBox(const BPresentationUnits& pu) {
  QComboBox* box = _shipSizeBox;
  box->insertItem(tr(_unitStrings[mUnit]));
  box->setCurrentText(tr(_unitStrings[pu.shipSize]));
}

void BOptionsDialog::setDistanceBox(const BPresentationUnits& pu) {
  QComboBox* box = _distanceBox;
  box->insertItem(tr(_unitStrings[mUnit]));
  box->insertItem(tr(_unitStrings[kmUnit]));
  box->insertItem(tr(_unitStrings[cableUnit]));
  box->insertItem(tr(_unitStrings[mileUnit]));
  box->setCurrentText(tr(_unitStrings[pu.distance]));
}

void BOptionsDialog::setSpeedBox(const BPresentationUnits& pu) {
  QComboBox* box = _speedBox;
  box->insertItem(tr(_unitStrings[knotUnit]));
  box->insertItem(tr(_unitStrings[kmperhUnit]));
  box->setCurrentText(tr(_unitStrings[pu.speed]));
}

void BOptionsDialog::setArmorBox(const BPresentationUnits& pu) {
  QComboBox* box = _armorBox;
  box->insertItem(tr(_unitStrings[mmUnit]));
  box->setCurrentText(tr(_unitStrings[pu.armor]));
}

void BOptionsDialog::setCaliberBox(const BPresentationUnits& pu) {
  QComboBox* box = _caliberBox;
  box->insertItem(tr(_unitStrings[inchUnit]));
  box->insertItem(tr(_unitStrings[mmUnit]));
  box->insertItem(tr(_unitStrings[cmUnit]));
  box->setCurrentText(tr(_unitStrings[pu.caliber]));
}

void BOptionsDialog::accept() {
  writeOptions();
  BOptionsDialogBase::accept();
}

BPresentationUnits BOptionsDialog::presentationsUnits() const {
  return BUiUtils::readConfig(bApp->settings());
}

BUnits BOptionsDialog::unitFromString(const QString& unit) {
  for (int i = 0; i < (int)(sizeof(_unitStrings)/sizeof(const char*)); i++) {
    if (unit == tr(_unitStrings[i])) {
      return static_cast<BUnits>(i);
    }
  }
  throw BInternalException(__FILE__, __LINE__,
    tr("Bad string '%1' requested to convert to units").arg(unit));
}

void BOptionsDialog::slotResetHelp() {
  BallonHelp* bh = bApp->ballonHelp();
  bh->reset();
}

void BOptionsDialog::slotHelp() {
  bApp->showHelp(this);
}
