/******************************************************************************/
/* BOrderAlgorithms -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char borderalgorithms_cpp_id[]="@(#) $Id: borderalgorithms.cpp,v 1.8 2005/02/15 16:43:35 Oleksiy Balabay Exp $"; static const char* borderalgorithms_cpp_id_id=borderalgorithms_cpp_id;

#include "borderalgorithms.h"
#include "bunitsconsts.h"
#include "bprogram.h"
#include "beventprogram.h"
#include "bapplication.h"
#include "battle.h"
#include "bprogramfactory.h"
#include "baichasingtracks.h"

/** \file borderalgorithms.cpp
  \brief  BOrderAlgorithms  BattleFleet (impl.)

     BOrderAlgorithms  BattleFleet
*/

/** \class BOrderAlgorithms
  \brief 


*/
bool operator<(const BObjectData& od1, const BObjectData& od2) {
  return od1.weight < od2.weight;
}

bool operator<(const BObjectDataPair& od1, const BObjectDataPair& od2) {
  return od1.second < od2.second;
}

BShipsOrderWeight::BShipsOrderWeight(BShipsOrder& shipsOrder, 
  const QDateTime& dt, const BattleObject* baseObject, 
  const double longitudeCoef): _shipsOrder(shipsOrder),
  _dt(dt), _longitudeCoef(longitudeCoef) {
  BKinematicData kd1 = baseObject->kinematicData(dt);
  _xf = kd1.x();
  _yf = kd1.y();
  _cosFi = cos(PI/2 - kd1.course());
  _sinFi = sin(PI/2 - kd1.course());
}

BShipsOrderWeight::result_type 
BShipsOrderWeight::operator()(argument_type f) {
  BKinematicData kd2 = f->kinematicData(_dt);
  double xs = kd2.x();
  double ys = kd2.y();
  BObjectData objData;
  objData.weight = (xs - _xf)*_longitudeCoef*_cosFi + (ys - _yf)*_sinFi;
  _shipsOrder.push_back(BObjectDataPair(f, objData));
}

BShipsOrderExec::result_type BShipsOrderExec::operator()(argument_type f) {
  QDateTime dt1 = _dt.addSecs(int(f.second.dt*3600));
  BProgram* p = _program->clone();
  BEventProgram* e = new BEventProgram(p, f.first->name(), dt1);
  e->setEventFlags(p->programFlags());
  bApp->battle()->postEvent(e);
}

BShipsOrderNormalize::result_type 
BShipsOrderNormalize::operator()(argument_type f) {
  f.second.weight = BUnitsConverter::degreesToMiles(f.second.weight - _base);
  BKinematicData kd1 = f.first->kinematicData(_dt);
  f.second.dt = fabs(f.second.weight/kd1.v(false));
//  qDebug("BShipsOrderNormalize %s %f", (const char*)f.first->name().local8Bit(), 
//    f.second.weight);
}

BShipsOrderSet::BShipsOrderSet(const BattleObject* baseObject, const QDateTime& dt,
  const double baseLatitude, const double intervalMiles):_dt(dt),
  _deltaX(BUnitsConverter::degreesFromMiles(intervalMiles,
  baseLatitude)), _deltaY(BUnitsConverter::degreesFromMiles(intervalMiles)),
  _useIntervalMiles(intervalMiles != 0), _baseLatitude(baseLatitude) {
  BKinematicData kd1 = baseObject->kinematicData(dt);
  _xf = kd1.x();
  _yf = kd1.y();
  _curX = _xf;
  _curY = _yf;
  _speed = kd1.v(false);
  _course = kd1.course();
  _cosFi = cos(PI/2 - _course);
  _sinFi = sin(PI/2 - _course);
  _curLength = baseObject->length();
//  _n = 1;
}

BShipsOrderSet::result_type 
BShipsOrderSet::operator()(argument_type f) {
  if (!_useIntervalMiles) {
    double intervalMiles = BAIChasingTracks::timeInterval(f.first)*_speed;
    if (intervalMiles < 3*f.first->length()/2+_curLength/2)
      intervalMiles = 3*f.first->length()/2 + _curLength/2;
    _curLength = f.first->length();
    _deltaX = BUnitsConverter::degreesFromMiles(intervalMiles,
      _baseLatitude);
    _deltaY = BUnitsConverter::degreesFromMiles(intervalMiles);
  }
  _curX = _curX - _deltaX*_cosFi;
  _curY = _curY - _deltaY*_sinFi;

  BProgram* p = BProgramFactory::createStandByProgram(
    _curX, _curY, _course, _speed, bApp->battle());
  BEventProgram* e = new BEventProgram(p, f.first->name(), _dt);
  bApp->battle()->postEvent(e);
//  qDebug("%d %s %f", _n, (const char*)f.first->name().local8Bit(), 
//    f.second.weight);
//  _n++;
}

BShipsOrderIsLineAhead::BShipsOrderIsLineAhead(const BattleObject* baseObject, const QDateTime& dt,
  const double lk, bool& lineAhead): 
  _baseObject(baseObject), _dt(dt), _lineAhead(lineAhead), _lk(lk),
  _dCourse(3.0/3600), _dWidth(1), _dLength(20) {
  BKinematicData baseKd = _baseObject->kinematicData(dt);
  _baseX = BUnitsConverter::degreesToMiles(baseKd.x()*_lk);
  _baseY = BUnitsConverter::degreesToMiles(baseKd.y());
  _baseC = baseKd.course();
  //    
  //  .   A (_baseObject)
  QWMatrix m1(1, 0, 0, 1, _baseX, _baseY);
  //  x   
  QWMatrix m2(cos(_baseC - Pi/2), -sin(_baseC - Pi/2), 
              sin(_baseC - Pi/2), cos(_baseC - Pi/2), 0, 0);
  _mInv = (m2*m1).invert();
  _lineAhead = true;
  double debugX, debugY;
  _mInv.map(_baseX, _baseY, &debugX, &debugY);
}

BShipsOrderIsLineAhead::result_type 
BShipsOrderIsLineAhead::operator()(argument_type f) {
  if (!_lineAhead)
    return;
  if (f == _baseObject)
    return;
  BKinematicData kd = f->kinematicData(_dt);
  double d1 = fabs(_baseC - kd.course());
  if (d1 > Pi) {
    d1 = fabs(2*Pi - d1);
  }
  if (d1 > _dCourse) {
    _lineAhead = false;
    return;
  }
  double x1 = BUnitsConverter::degreesToMiles(kd.x()*_lk);
  double y1 = BUnitsConverter::degreesToMiles(kd.y());
  _mInv.map(x1, y1, &x1, &y1);
  if (fabs(y1/f->width()) > _dWidth) {
    _lineAhead = false;
    return;
  }
  // TODO:   
}
