/******************************************************************************/
/* BOrderAlgorithms -   BattleFleet              */
/******************************************************************************/

#ifndef BOrderAlgorithms_H
#define BOrderAlgorithms_H
static const char borderalgorithms_h_id[]="@(#) $Id: borderalgorithms.h,v 1.6 2005/02/15 16:43:35 Oleksiy Balabay Exp $"; static const char* borderalgorithms_h_id_id=borderalgorithms_h_id;
/** \file borderalgorithms.h
  \brief  BOrderAlgorithms  BattleFleet

     BOrderAlgorithms  BattleFleet
*/
#include <qdatetime.h>
#include <qwmatrix.h>

#include <vector>
#include <algorithm>

#include "battleobject.h"

class BProgram;

#include <functional>

typedef struct BObjectData {
  double weight;
  double dt;
} BObjectData;

//extern bool operator<(const BObjectData& od1, const BObjectData& od2);

typedef std::pair<BattleObject*, BObjectData> BObjectDataPair;
//extern bool operator<(const BObjectDataPair& od1, const BObjectDataPair& od2);

typedef std::vector<BObjectDataPair> BShipsOrder;
inline bool more_obj_data(const BObjectDataPair& od1, const BObjectDataPair& od2) {
  return od1.second.weight > od2.second.weight;
}

class BShipsOrderWeight: std::unary_function<BattleObject*, void> {
public:
  BShipsOrderWeight(BShipsOrder& shipsOrder, const QDateTime& dt,
    const BattleObject* baseObject, const double longitudeCoef);
  result_type operator()(argument_type f);
private:
  BShipsOrder& _shipsOrder;
  QDateTime _dt;
  double _longitudeCoef;
  double _xf;
  double _yf;
  double _cosFi;
  double _sinFi;
};

class BShipsOrderNormalize: std::unary_function<BObjectDataPair&, void> {
public:
  BShipsOrderNormalize(const double base, const QDateTime& dt)
    :_base(base), _dt(dt) {
  }
  result_type operator()(argument_type f);
private:
  double _base;
  QDateTime _dt;
};

class BShipsOrderExec: std::unary_function<BObjectDataPair&, void> {
public:
  BShipsOrderExec(const BProgram* program, const QDateTime& dt): 
    _program(program), _dt(dt) {
  }
  result_type operator()(argument_type f);
private:
  const BProgram* _program;
  QDateTime _dt;
};

class BShipsOrderSet: std::unary_function<BObjectDataPair&, void> {
public:
  BShipsOrderSet(const BattleObject* baseObject, const QDateTime& dt,
  const double baseLatitude, const double intervalMiles);
  result_type operator()(argument_type f);
private:
  QDateTime _dt;
  double _deltaX, _deltaY;
  double _xf;
  double _yf;
  double _speed;
  double _course;
  double _cosFi;
  double _sinFi;
  int _n;
  bool _useIntervalMiles;
  double _baseLatitude;
  double _curX, _curY;
  double _curLength;
};

class BShipsOrderIsLineAhead: std::unary_function<const BattleObject*, void> {
public:
  BShipsOrderIsLineAhead(const BattleObject* baseObject, const QDateTime& dt,
    const double lk, bool & lineAhead);
  result_type operator()(argument_type f);
  bool isLineAhead() const {
    return _lineAhead;
  }
private:
  const BattleObject* _baseObject;
  QDateTime _dt;
  bool & _lineAhead;
  double _baseX, _baseY, _baseC;
  double _lk;
private:
  const double _dCourse;
  const double _dWidth;
  const double _dLength;
  QWMatrix _mInv;
};

#endif // BOrderAlgorithms_H
