/******************************************************************************/
/* BProgram -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bprogram_cpp_id[]="@(#) $Id: bprogram.cpp,v 1.16 2005/01/09 18:03:05 Oleksiy Balabay Exp $"; static const char* bprogram_cpp_id_id=bprogram_cpp_id;

#include "bprogram.h"
#include "bexception.h"
#include "bseamap.h"
#include "bdomutils.h"
#include "bunitsconsts.h"
#include "battleobject.h"
#include "battle.h"

/** \file bprogram.cpp
  \brief  BProgram  BattleFleet (impl.)

     BProgram  BattleFleet
*/

/** \class BProgram
  \brief 


*/
const QString BProgram::PROGRAM = "Program";
const QString BProgram::PROGRAM_TYPE = "Type";
const QString BProgram::PROGRAM_X = "X";
const QString BProgram::PROGRAM_Y = "Y";
const QString BProgram::PROGRAM_SPEED = "Speed";
const QString BProgram::PROGRAM_COURSE = "Course";
const QString BProgram::PROGRAM_T0 = "T0";

/** \brief 
*/
BProgram::BProgram():_battle(0), 
  _course0(0), _course0Initialized(false), 
  _speed0(0), _speed0Initialized(false),
  _x0(0), _x0Initialized(false),
  _y0(0), _y0Initialized(false) {
}

/** \brief 
*/
BProgram::~BProgram(){
}

void BProgram::checkT(const QDateTime& t) const {
  if (t < t0()) {
    throw BTimeException(__FILE__, __LINE__,
      t, qApp->translate("BProgram", "Time must be after %1").arg(
      t0().toString()));
  }
}

/**
  : x, y - , course -   N, speed - knots
*/
void BProgram::parseValues(const double *x, const double *y, 
  const double *course, const double *speed) {
  if (x) _x0 = *x;
  _x0Initialized = (x!=0);

  if (y) _y0 = *y;
  _y0Initialized = (y!=0);

  if (course) _course0 = *course;
  _course0Initialized = (course!=0);

  if (speed) _speed0 = *speed;
  _speed0Initialized = (speed!=0);
}

void BProgram::parseValues(const QDomElement& element) {
  if (element.hasAttribute(PROGRAM_X)) {
    _x0 = BSeaMap::degreesFromString(element.attribute(PROGRAM_X),
      BSeaMap::longitude);
    _x0Initialized = true;
  }
  else {
    _x0Initialized = false;
  }
  if (element.hasAttribute(PROGRAM_Y)) {
    _y0 = BSeaMap::degreesFromString(element.attribute(PROGRAM_Y),
    BSeaMap::latitude);
    _y0Initialized = true;
  }
  else {
    _y0Initialized = false;
  }
  if (element.hasAttribute(PROGRAM_COURSE)) {
    _course0 = BUnitsConverter::angleFromDegrees(
      BDomUtils::doubleAttribute(element, PROGRAM_COURSE));
    _course0Initialized = true;
  }
  else {
    _course0Initialized = false;
  }
  if (element.hasAttribute(PROGRAM_SPEED)) {
    _speed0 = BDomUtils::doubleAttribute(element, PROGRAM_SPEED);
    _speed0Initialized = true;
  }
  else {
    _speed0Initialized = false;
  }
}

double BProgram::hours(const QDateTime& t0, const QDateTime& t) {
  double hs = 0;
  if (t.date() == t0.date()) {
    int msecs = t0.time().msecsTo(t.time());
    hs = msecs/(1000.0*3600);
  }
  else {
    // TODO: Process millececonds correctly
    hs = t0.secsTo(t)/3600.0;
  }
  return hs;
}

QDateTime BProgram::addHours(const QDateTime& t0, const double hours) {
  int secs = hours * 3600;
  double fr, v;
  v = modf((hours*3600 - secs)*1000, &fr);
  int msecs = (hours*3600 - secs)*1000;
  if (fr >=0.5)
    msecs++;
  QDateTime dt = t0.addSecs(secs);
  QTime t = dt.time();
//  QString debugS = t.toString("hh:mm:zzz");
//  int debugM = t.msec
  t = t.addMSecs(msecs);
  if (t < dt.time())
    dt = dt.addDays(1);
  dt.setTime(t);
//  int debugMSecs = dt.time().msecsTo(t;
    // TODO: Process millececonds correctly
  return dt;
}

void BProgram::initProgramBase(const BProgramData& programData) {
  if ((!_x0Initialized) || (!_y0Initialized) || (!_speed0Initialized) ||
    (!_course0Initialized)) {
    BKinematicData kd = programData.kinematicData();
    if (!_x0Initialized) {
      _x0 = kd.x();
      _x0Initialized = true;
    }
    if (!_y0Initialized) {
      _y0 = kd.y();
      _y0Initialized = true;
    }
    if (!_speed0Initialized) {
      _speed0 = kd.v(false);
      _speed0Initialized = true;
    }
    if (!_course0Initialized) {
      _course0 = kd.course();
      _course0Initialized = true;
    }
  }
}

QDateTime BProgram::t0() const {
  if (!_t0.isValid()) {
    throw BInternalException(__FILE__, __LINE__,
      qApp->translate("BProgram", "Call to t0() when _t0 is invalid"));
  }
  return _t0;
}

//void BProgram::add(BProgram* program, const int secs) {
//  throw BInternalException(__FILE__, __LINE__,
//    qApp->translate("BProgram", "Function BProgram::add call"));
//}

//void BProgram::remove(BProgram* program) {
//  throw BInternalException(__FILE__, __LINE__,
//    qApp->translate("BProgram", "Function BProgram::remove call"));
//}

void BProgram::initProgram(const BProgramData& programData, const QDateTime& t0) {
  setT0(t0);
  initProgramCustom(programData);
}

void BProgram::parse( const QDomElement& element, const Battle* battle) 
  {
  _battle = battle;
  parseCustom(element, battle);
}

void BProgram::setT0(const QDateTime& t0) {
  _t0 = t0;
}

QDomElement BProgram::toXml(QDomDocument& doc) const {
//  if (!_t0.isValid()) {
//    throw BInternalException(__FILE__, __LINE__,
//      "Call BProgram::toXml() before setT0() call");
//  }
  QDomElement e = doc.createElement(PROGRAM);
  if (_x0Initialized) {
    e.setAttribute(PROGRAM_X, BSeaMap::degreesToString(
      _x0, BSeaMap::longitude));
  }
  if (_y0Initialized) {
    e.setAttribute(PROGRAM_Y, BSeaMap::degreesToString(
      _y0, BSeaMap::latitude));
  }
  if (_speed0Initialized) {
    e.setAttribute(PROGRAM_SPEED, _speed0);
  }
  if (_course0Initialized) {
    e.setAttribute(PROGRAM_COURSE, BUnitsConverter::angleToDegrees(
      _course0));
  }
  return e;
}

/**      ,   
.

        
*/
void BProgram::setProgramFlags(const unsigned int flags) {
  _flags = flags;
}

unsigned int BProgram::programFlags() const {
  return _flags;
}
