/******************************************************************************/
/* BProgram -   BattleFleet              */
/******************************************************************************/

#ifndef BProgram_H
#define BProgram_H
static const char bprogram_h_id[]="@(#) $Id: bprogram.h,v 1.16 2005/01/15 16:44:51 Oleksiy Balabay Exp $"; static const char* bprogram_h_id_id=bprogram_h_id;
/** \file bprogram.h
  \brief  BProgram  BattleFleet

     BProgram  BattleFleet
*/
#include <map>

#include <qdom.h>
#include <qdatetime.h>
#include <qstring.h>

#include "bkinematicdata.h"
#include "bevent.h"

class Battle;
class BProgram;
class BProgramData;
class BException;

const QString BASE_LATITUDE = "BaseLatitude";

class BProgram {
public:
  BProgram();
  virtual ~BProgram();
  virtual BKinematicData kinematicData(const QDateTime& t) 
    const = 0;
  virtual void initProgram(const BProgramData& programData, const QDateTime& t0);
  virtual void initProgramCustom(const BProgramData& programData) = 0;
  virtual void parse( const QDomElement& element,
    const Battle* battle); 
  virtual void parseCustom( const QDomElement& element,
    const Battle* battle) = 0; 
  virtual QDateTime t0() const;
//  virtual void add(BProgram* program, const int secs);
//  virtual void remove(BProgram* program);
  virtual BProgram* clone() const = 0;
  virtual QDomElement toXml(QDomDocument& doc) const;
  void setProgramFlags(const unsigned int flags);
  unsigned int programFlags() const;
  virtual BProgram* restOfProgram(const QDateTime& dt) const = 0;
public:
  static const QString PROGRAM;
  static const QString PROGRAM_TYPE;
  static const QString PROGRAM_X;
  static const QString PROGRAM_Y;
  static const QString PROGRAM_SPEED;
  static const QString PROGRAM_COURSE;
  static const QString PROGRAM_T0;
  static double hours(const QDateTime& t0, const QDateTime& t);
  static QDateTime addHours(const QDateTime& t0, const double hours);
protected:
  void setT0(const QDateTime& t0);
  void checkT(const QDateTime& t) const;
  void parseValues(const QDomElement& element);
  void parseValues(const double *x = 0, const double *y = 0, 
    const double *course = 0, const double *speed = 0);
  void initProgramBase(const BProgramData& programData);
protected:
  QDateTime _t0;

  bool _x0Initialized;
  bool _y0Initialized;
  bool _speed0Initialized;
  bool _course0Initialized;

  double _x0;
  double _y0;
  double _speed0;
  double _course0;
  const Battle* _battle;
private:
  unsigned int _flags;
};

#endif // BProgram_H
