/******************************************************************************/
/* BProgramFactory -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bprogramfactory_cpp_id[]="@(#) $Id: bprogramfactory.cpp,v 1.15 2005/01/12 03:57:23 Oleksiy Balabay Exp $"; static const char* bprogramfactory_cpp_id_id=bprogramfactory_cpp_id;

#include <qdom.h>

#include "bprogramfactory.h"
#include "bprogram.h"
#include "bexception.h"
#include "bdomutils.h"
#include "bseamap.h"
#include "battle.h"
#include "bunitsconsts.h"

#include "bprogramturnaccelerate.h"
#include "bprogramturninsuccession.h"
#include "bprogramturnsimultaneous.h"

/** \file bprogramfactory.cpp
  \brief  BProgramFactory  BattleFleet (impl.)

     BProgramFactory  BattleFleet
*/

/** \class BProgramFactory
  \brief 
*/

BProgram* BProgramFactory::create(const QDomElement& e,  
  const Battle* battle) {
  if (e.tagName() != BProgram::PROGRAM) {
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate("BProgramFactory",
      "Element is not '%1' but '%2'").arg(BProgram::PROGRAM).arg(
      e.tagName()));
  }
  BProgram * prg = 0;
  QString programType = BDomUtils::stringAttribute(e, BProgram::PROGRAM_TYPE);
  if (programType.lower() == BProgramTurnAccelerate::PROGRAM_TYPE_TURN_ACCELERATE.lower()) {
    prg = new BProgramTurnAccelerate();
    prg->parse(e, battle);
    return prg;
  }
  else if (programType.lower() == 
    BProgramTurnInSuccession::PROGRAM_TYPE_TURN_IN_SUCCESSION.lower()) {
    prg = new BProgramTurnInSuccession();
    prg->parse(e, battle);
    return prg;
  }
  else if (programType.lower() == 
    BProgramTurnSimultaneous::PROGRAM_TYPE_TURN_SIMULTANEOUS.lower()) {
    prg = new BProgramTurnSimultaneous();
    prg->parse(e, battle);
    return prg;
  }
  else
    throw BConfigException(__FILE__, __LINE__,
    qApp->translate(BProgram::PROGRAM, 
      "Invalid %1: '%2'").arg(BProgram::PROGRAM_TYPE).arg(programType));
  // keep compiler happy
  return 0;
}

BProgram* BProgramFactory::createTurnAccelerateProgram(
  const Battle* battle,
  const double* turnAngleRadians, 
  const double* newSpeedMiles,  
  const double *fabsA,
  const double *turnRadius) {
  return createTAProgram(BProgramTurnSimultaneous::PROGRAM_TYPE_TURN_ACCELERATE,
  battle, turnAngleRadians, newSpeedMiles, fabsA, turnRadius);
}

BProgram* BProgramFactory::createTurnInSuccessionProgram(
  const Battle* battle,
  const double* turnAngleRadians, 
  const double* newSpeedMiles,  
  const double *fabsA,
  const double *turnRadius) {
  return createTAProgram(BProgramTurnInSuccession::PROGRAM_TYPE_TURN_IN_SUCCESSION,
  battle, turnAngleRadians, newSpeedMiles, fabsA, turnRadius);
}

BProgram* BProgramFactory::createTurnSimultaneousProgram(
  const Battle* battle,
  const double* turnAngleRadians, 
  const double* newSpeedMiles,  
  const double *fabsA,
  const double *turnRadius) {
  return createTAProgram(BProgramTurnSimultaneous::PROGRAM_TYPE_TURN_SIMULTANEOUS,
  battle, turnAngleRadians, newSpeedMiles, fabsA, turnRadius);
}

QDomElement BProgramFactory::tAPtoXml(QDomDocument& doc, const QString& type, 
    const double* turnAngleRadians, 
    const double* newSpeedMiles,  
    const double* fabsA,
    const double* turnRadius) {
  QDomElement e = doc.createElement(BProgram::PROGRAM);
  e.setAttribute(BProgram::PROGRAM_TYPE, type);
  if (turnAngleRadians) {
    e.setAttribute(BProgramTurnAccelerate::PROGRAM_TURN_ANGLE,  QString::number(
      BUnitsConverter::angleToDegrees(*turnAngleRadians), 'e', 12));
  }
  if (newSpeedMiles) {
    e.setAttribute(BProgramTurnAccelerate::PROGRAM_NEW_SPEED, QString::number(
      *newSpeedMiles, 'e', 12));
  }
  if (fabsA) {
    e.setAttribute(BProgramTurnAccelerate::PROGRAM_FABS_A,
      QString::number(*fabsA, 'e', 12));
  }
  if (turnRadius) {
    e.setAttribute(BProgramTurnAccelerate::PROGRAM_TURN_RADIUS,
      QString::number(*turnRadius, 'e', 12));
  }
  return e;
}

BProgram* BProgramFactory::createTAProgram(const QString& type, 
    const Battle* battle, 
    const double* turnAngleRadians, 
    const double* newSpeedMiles,  
    const double* fabsA,
    const double* turnRadius) {
  QDomDocument doc;
  return create(tAPtoXml(doc, type, 
    turnAngleRadians, newSpeedMiles, fabsA, turnRadius), battle);
}

BProgram* BProgramFactory::createStandByProgram(const double x, 
  const double y, const Battle* battle) {
  BProgramTurnAccelerate* prg = new BProgramTurnAccelerate();
  prg->parseCustom(battle, &x, &y);
  return prg;
/*  QString programXml = 
    QString("<%1 %2='%3' %4='%5' %6='%7' />").arg(BProgram::PROGRAM).arg(
    BProgram::PROGRAM_TYPE).arg(
    BProgramTurnAccelerate::PROGRAM_TYPE_TURN_ACCELERATE).arg(
    BProgram::PROGRAM_X).arg(
    BSeaMap::degreesToString(x, BSeaMap::longitude)).arg(
    BProgram::PROGRAM_Y).arg(
    BSeaMap::degreesToString(y, BSeaMap::latitude));
  QDomDocument doc;
  doc.setContent(programXml);
  return create(doc.documentElement(), battle);  
*/
}

BProgram* BProgramFactory::createStandByProgram(const double x, 
  const double y, const double course,
  const double speed, const Battle* battle) {
  BProgramTurnAccelerate* prg = new BProgramTurnAccelerate();
  prg->parseCustom(battle, &x, &y, &course, &speed);
  return prg;
/*  QString speedCourse = QString(" %1='%2' %3='%4' ").arg(
    BProgram::PROGRAM_COURSE).arg(
    BUnitsConverter::angleToDegrees(course), 0, 'e', 12).arg(
    BProgram::PROGRAM_SPEED).arg(speed);
  QString programXml = 
    QString("<%1 %2='%3' %4='%5' %6='%7' %8/>").arg(BProgram::PROGRAM).arg(
    BProgram::PROGRAM_TYPE).arg(
    BProgramTurnAccelerate::PROGRAM_TYPE_TURN_ACCELERATE).arg(
    BProgram::PROGRAM_X).arg(
    BSeaMap::degreesToString(x, BSeaMap::longitude)).arg(
    BProgram::PROGRAM_Y).arg(
    BSeaMap::degreesToString(y, BSeaMap::latitude)).arg(
    speedCourse);
  QDomDocument doc;
  doc.setContent(programXml);
  return create(doc.documentElement(), battle);  
*/
}
