/******************************************************************************/
/* BPTAStrategyBiDir -     (impl.)            */
/******************************************************************************/
static const char bptastrategybidir_cpp_id[]="@(#) $Id: bptastrategybidir.cpp,v 1.5 2005/01/15 16:44:51 Oleksiy Balabay Exp $"; static const char* bptastrategybidir_cpp_id_id=bptastrategybidir_cpp_id;

#include "bptastrategybidir.h"
#include "bunitsconsts.h"
#include "battleobject.h"
#include "battle.h"
#include "bseamap.h"
#include "bdomutils.h"
#include "bmathturn.h"
#include "bprogramfactory.h"
#include "bship.h"

/** \file bptastrategybidir.cpp
  \brief  BPTAStrategyBiDir   (impl.)

     BPTAStrategyBiDir  
*/

/** \class BPTAStrategyBiDir
  \brief 


*/
const double MIN_SPEED_LIMIT = 0.01; // knots

/** \brief 
*/
BPTAStrategyBiDir::BPTAStrategyBiDir(BProgramTurnAccelerate* program):
  BPTAStrategy(program),
  _turnR(0), _turnRMiles(0), _turnRInitialized(false),
  _newSpeed(0), _newSpeedInitialized(false), 
  _fabsA(1), _fabsAInitialized(false),
  _turnAngle(0), _acceleration(0) {

}

/** \brief 
*/
BPTAStrategyBiDir::~BPTAStrategyBiDir(){

}

BKinematicData BPTAStrategyBiDir::kinematicData(
  const QDateTime& t) const {
  _program->checkT(t);
  BTurnAccelerateInput in;
  in.t = _program->hours(_program->_t0, t);
  in.v_0 = _program->_speed0;
  in.v_k = _newSpeed;
  in.angle = _turnAngle;
  in.a_max = _acceleration;
  in.k_v = _k;
  in.r_c = _turnRMiles;
  BPoint p = turnAccelerate(in);
  p.x = BUnitsConverter::degreesFromMiles(p.x);
  p.y = BUnitsConverter::degreesFromMiles(p.y);
  double curX, curY;
  _mTurn.map(p.x, p.y, &curX, &curY);

  return BKinematicData(
    curX,
    curY,
    p.c + (_program->_course0 ),
    p.v,
    p.vcomp,
    p.a,
    p.r
  );
}


void BPTAStrategyBiDir::initProgramTurn(const BProgramData& programData) {
  double turnRadius = programData.availableTurnRadius();
  if (!_turnRInitialized) {
    _turnRMiles = turnRadius;
    _turnRInitialized = true;
  }
  else if (turnRadius > _turnRMiles) {
    _turnRMiles = turnRadius;
  }
  _turnR = BUnitsConverter::degreesFromMiles(_turnRMiles);
  _k = programData.lossOfSpeedCoef();
 // _k = 1 + programData.availableTurnRadius()*(_k-1)/_turnRMiles;
}

void BPTAStrategyBiDir::initProgramAccelerate(
  const BProgramData& programData) {
  if (!_newSpeedInitialized) {
    _newSpeed = _program->_speed0;
    _newSpeedInitialized = true;
  }
  else {
    if (fabs(_newSpeed) > fabs(programData.availableSpeed())) {
      int sgn = (_newSpeed>0)?1:-1;
      _newSpeed = sgn*fabs(programData.availableSpeed());
    }
 }
  if (fabs(_newSpeed) < MIN_SPEED_LIMIT) {
    _newSpeed = 0;
  }
  if (!_fabsAInitialized) {
    _fabsA = programData.availableAcceleration();
    _fabsAInitialized = true;
  }
  _acceleration = (_fabsA <= programData.availableAcceleration())?_fabsA:
    programData.availableAcceleration();
  //      
  if ((_acceleration == 0) && (_newSpeed != _program->_speed0)) {
    _acceleration = programData.availableAcceleration();
  }

}

void BPTAStrategyBiDir::initProgramCustom(const BProgramData& programData) {
  _program->initProgramBase(programData);
  initProgramAccelerate(programData);
  initProgramTurn(programData);
  //     

  //    
  //  . .    
  QWMatrix m1(1, 0, 0, 1, _program->_x0, _program->_y0);
  //   
  QWMatrix m11(1/BUnitsConverter::longitudeCoef(_baseLatitude), 0, 0, 1, 0, 0);
  //  x   
  QWMatrix m2(cos(_program->_course0 - PI/2), -sin(_program->_course0 - PI/2), 
              sin(_program->_course0 - PI/2), cos(_program->_course0 - PI/2), 0, 0);
  _mTurn = m2*m11*m1;
//  debugDump();
}

/**
  turnAngle - 

*/
void BPTAStrategyBiDir::parseCustom(const Battle* battle,
  const double *x, const double *y, const double *course,
  const double *speed, const double *turnAngle,
  const double *turnRMiles, const double *newSpeed,
  const double *fabsA) {

  _program->_battle = battle;
  _baseLatitude = battle->seaMap()->originalCoordinates().y0();

  _program->parseValues(x, y, course, speed);

  if (turnAngle) 
    _turnAngle = *turnAngle;
  else
    _turnAngle = 0;

  if (_turnRMiles) _turnRMiles = *turnRMiles;
  _turnRInitialized = (turnRMiles!=0);

  if (newSpeed) _newSpeed = *newSpeed;
  _newSpeedInitialized = (newSpeed!=0);

  if (fabsA) {
    _fabsA = *fabsA;
    _fabsAInitialized = true;
  }
  else {
    _fabsAInitialized = false;
  }
}

void BPTAStrategyBiDir::parseCustom( const QDomElement& element, 
  const Battle* battle) { 
  _baseLatitude = _program->_battle->seaMap()->originalCoordinates().y0();
  _program->parseValues(element);
  
  if (element.hasAttribute(_program->PROGRAM_TURN_ANGLE)) {
    _turnAngle = BUnitsConverter::angleFromDegrees(
        BDomUtils::doubleAttribute(element, _program->PROGRAM_TURN_ANGLE));
  }
  else {
    _turnAngle = 0;
  }
  if (element.hasAttribute(_program->PROGRAM_TURN_RADIUS)) {
    _turnRMiles = BDomUtils::doubleAttribute(element, 
      _program->PROGRAM_TURN_RADIUS);
    _turnRInitialized = true;
  }
  else {
    _turnRInitialized = false;
  }

  if (element.hasAttribute(_program->PROGRAM_NEW_SPEED)) {
    _newSpeed = BDomUtils::doubleAttribute(element, _program->PROGRAM_NEW_SPEED);
    _newSpeedInitialized = true;
  }
  else {
    _newSpeedInitialized = false;
  }

  if (element.hasAttribute(_program->PROGRAM_FABS_A)) {
    _fabsA = BDomUtils::doubleAttribute(element, _program->PROGRAM_FABS_A);
    _fabsAInitialized = true;
  }
}

BPTAStrategy* BPTAStrategyBiDir::clone(BProgramTurnAccelerate* newProgram) const {
  BPTAStrategyBiDir* ps = new BPTAStrategyBiDir(*this);
  ps->_program = newProgram;
  return ps;
}

QDomElement BPTAStrategyBiDir::toXml(QDomDocument& doc, QDomElement& e) const
  {
  return BProgramFactory::tAPtoXml(doc, _program->PROGRAM_TYPE_TURN_ACCELERATE,
    &_turnAngle, &_newSpeed, &_fabsA, &_turnRMiles);
}

/*void BPTAStrategyBiDir::debugDump() {
  QString s;
  s = s + "_x0="+QString::number(_program->_x0, 'e', 12) + "\n";
  s = s + "_y0="+QString::number(_program->_y0, 'e', 12) + "\n";
  s = s + "_speed0="+QString::number(_program->_speed0, 'e', 12) + "\n";
  s = s + "_course0="+QString::number(_program->_course0, 'e', 12) + "\n";
  s = s + "_t0="+_program->_t0.toString("yyyyMMdd hh:mm:ss:zzz") + "\n";
  s = s + "_turnAngle="+QString::number(_turnAngle, 'e', 12) + "\n";
  s = s + "_turnR="+QString::number(_turnR, 'e', 12) + "\n";
  s = s + "_turnRMiles="+QString::number(_turnRMiles, 'e', 12) + "\n";
  s = s + "_fabsA="+QString::number(_fabsA, 'e', 12) + "\n";
  s = s + "_acceleration="+QString::number(_acceleration, 'e', 12) + "\n";
  s = s + "_newSpeed="+QString::number(_newSpeed, 'e', 12) + "\n";
  s = s + "_k="+QString::number(_k, 'e', 12) + "\n";
  BShip::writeToDebugFile(s);
}*/

void BPTAStrategyBiDir::info(double* r, double* a, 
  double* v, double* angle) const {
  if ((!_turnRInitialized) || (!_newSpeedInitialized) || 
    (!_fabsAInitialized)) {
    throw BInternalException(__FILE__, __LINE__, "Call to info() in not "
      "initialized program");
  }
  *r = fabs(_turnRMiles);
  *a = fabs(_acceleration);
  *v = _newSpeed;
  *angle = _turnAngle;
}
