/******************************************************************************/
/* BPVState -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bpvstate_cpp_id[]="@(#) $Id: bpvstate.cpp,v 1.13 2005/03/07 15:34:38 Oleksiy Balabay Exp $"; static const char* bpvstate_cpp_id_id=bpvstate_cpp_id;

#include "bpvstate.h"
#include "bship.h"
#include "bshipgroup.h"
#include "battle.h"
#include "bapplication.h"
#include "bseamap.h"
#include "bprogramfactory.h"
#include "beventprogram.h"
#include "bscenarioheader.h"
#include "beventseamap.h"
#include "beventgroup.h"
#include "beventstate.h"
#include "bmainwindowtexts.h"
#include "bgroupview.h"
#include "bcontrolmoveview.h"
#include "battleobserver.h"
#include "bships.h"
#include "bunitsconsts.h"

/** \file bpvstate.cpp
  \brief  BPVState  BattleFleet (impl.)

     BPVState  BattleFleet
*/
const char* B_SHIP_SIDE = QT_TRANSLATE_NOOP("BPVStateShip",
    "Active ship or ships group side<br><br>"
    "One side ships are <b><font color=red>red</font></b>. "
    "Opponent's ships are <b><font color=black>black</font></b>. "
    "Neutral ships are <b><font color=blue>blue</font></b>. "
    "To set active ship side select it from the list");
const char* B_SHIP_TYPE = QT_TRANSLATE_NOOP("BPVStateShip",
    "Active ship type<br><br>"
    "This info is stored in the ships configuration file");
const char* B_SHIP_SIZE = QT_TRANSLATE_NOOP("BPVStateShip",
    "Active ship relative size in terms of <b>BattleFleet</b> rule "
    "description "
    "<br><br>"
    "This info is stored in the ships configuration file");
const char* B_SHIP_LENGTH = QT_TRANSLATE_NOOP("BPVStateShip",
    "Active ship length (m) "
    "<br><br>"
    "This info is stored in the ships configuration file");
const char* B_SHIP_WIDTH = QT_TRANSLATE_NOOP("BPVStateShip",
    "Active ship width (m) "
    "<br><br>"
    "This info is stored in the ships configuration file");
const char* B_SHIP_MAX_SPEED = QT_TRANSLATE_NOOP("BPVStateShip",
    "Active ship max speed in knots "
    "<br><br>"
    "This info is stored in the ships configuration file");
const char* B_SHIP_TURN_R = QT_TRANSLATE_NOOP("BPVStateShip",
    "Active ship minimal turn radius (m) "
    "<br><br>"
    "This info is stored in the ships configuration file");
const char* B_SHIP_LOSC = QT_TRANSLATE_NOOP("BPVStateShip",
    "Coefficient of speed degradation in turn "
    "<br><br>"
    "This info is calculated");
/** \class BPVState
  \brief 


*/
void BPVStateShip::updateView(BShip* o) {
  _o = o;
  _objProps = objectProperties(o);
  if (_objProps.get() != 0)
    _view->setupProperties(*_objProps);
  else
    _view->clear();
}

void BPVStateShip::updateView() {
  _objProps = objectProperties(_o);
  if (_objProps.get() != 0)
    _view->updateProperties(*_objProps);
  else
    _view->clear();
}

BProperty* BPVStateShip::propertyName(BShip* o) {
  BProperty* p = new BProperty(BProperty::StringType, "N", 
    qApp->translate("BPVStateShip", "Name"), o->name());
  p->setWhatsThisText(qApp->translate("BPVStateShip",
    "Active ship name<br><br>Active ship is highlighted with green. "
    "To change active ship select another one in the map or in <b>%1</b> window "
    "using mouse").arg(qApp->translate("BGroupView", B_GROUP_VIEW)));
  return p;
}
  
BProperty* BPVStateShip::propertyX(BShip* o, const BProgramData& pd, const BKinematicData& kd) {
  BProperty* p = new BProperty(BProperty::LongitudeType, "X", 
    qApp->translate("BPVStateShip","Longitude"), kd.x());
  p->setWhatsThisText(qApp->translate("BPVStateShip",
    "Active ship longitude at this moment<br><br>"
    "To control active ship in <b>Game</b> mode use arrow keys or <b>%1</b> window. "
    "To position active ship in <b>Edit</b> mode drag it using mouse or "
    "enter new coordinate here").arg(
    qApp->translate("BControlMove", B_CONTROL_MOVE_VIEW)));
  return p;
}

BProperty* BPVStateShip::propertyY(BShip* o, const BProgramData& pd, const BKinematicData& kd) {
  BProperty* p = new BProperty(BProperty::LatitudeType, "Y", 
    qApp->translate("BPVStateShip","Latitude"), kd.y());
  p->setWhatsThisText(qApp->translate("BPVStateShip",
    "Active ship latitude at this moment<br><br>"
    "To control active ship in <b>Game</b> mode use arrow keys or <b>%1</b> window. "
    "To position active ship in <b>Edit</b> mode drag it using mouse or "
    "enter new coordinate here").arg(
    qApp->translate("BControlMove", B_CONTROL_MOVE_VIEW)));
  return p;
}

BProperty* BPVStateShip::propertyCourse(BShip* o, const BProgramData& pd, const BKinematicData& kd) {
  BProperty* p = new BProperty(BProperty::CourseType, "C", 
    qApp->translate("BPVStateShip","Course"), kd.course());
  p->setWhatsThisText(qApp->translate("BPVStateShip",
    "Active ship course at this moment<br><br>"
    "To control active ship in <b>Game</b> mode use arrow keys or <b>%1</b> window. "
    "To set active ship course in <b>Edit</b> mode "
    "enter new value here").arg(
    qApp->translate("BControlMove", B_CONTROL_MOVE_VIEW)));
  return p;
}

BProperty* BPVStateShip::propertySpeed(BShip* o, const BProgramData& pd, const BKinematicData& kd) {
  QValueList<QVariant> vl;
  vl.append(QVariant(-pd.availableSpeed()));
  vl.append(QVariant(pd.availableSpeed()));
  vl.append(QVariant(3));
  BProperty* p = new BProperty(BProperty::DoubleType, "V", 
    qApp->translate("BPVStateShip","Speed"), kd.v(false), vl);
  p->setWhatsThisText(qApp->translate("BPVStateShip",
    "Active ship speed at this moment in knots<br><br>"
    "To control active ship in <b>Game</b> mode use arrow keys or <b>%1</b> window. "
    "To set active ship speed in <b>Edit</b> mode "
    "enter new value here").arg(
    qApp->translate("BControlMove", B_CONTROL_MOVE_VIEW)));
  return p;
}

BProperty* BPVStateShip::propertySide(BShip* o) {
  int curItem = (_o->side() == NO_SIDE)?0:_o->side();
  BProperty* p = new BProperty(BProperty::StringListType, "SIDE", 
    qApp->translate("BPVStateShip","Side"), QVariant(_sides), QVariant(curItem));
  p->setWhatsThisText(qApp->translate("BPVStateShip", B_SHIP_SIDE));
  return p;
}

BProperty* BPVStateShip::propertyLossOfSpeedCoef(BShip* o) {
  BProperty* p = new BProperty(BProperty::DoubleType, "LOSC", 
    qApp->translate("BPVStateShip","Turn speed coef."), 
    o->lossOfSpeedCoef(o->size()));
  p->setWhatsThisText(qApp->translate("BPVStateShip", B_SHIP_LOSC));
  return p;
}

std::auto_ptr<BProperties> BPVStateShip::objectProperties(BShip* o) {
  if (0 == o) 
    return std::auto_ptr<BProperties>(0);
  BProperties* props = new BProperties();
  BProgramData pd = o->programData(bApp->battle()->battleInfo().modelTime);
  BKinematicData kd = pd.kinematicData();
  props->addProperty(propertyName(o));
  props->addProperty(propertyX(o, pd, kd));
  props->addProperty(propertyY(o, pd, kd));
  props->addProperty(propertyCourse(o, pd, kd));
  props->addProperty(propertySpeed(o, pd, kd));
  BProperty* p = propertySide(o);
  p->setEditable(true);
  props->addProperty(p);
  props->addProperty(propertyType(o));
  props->addProperty(propertySize(o));
  props->addProperty(propertyLength(o));
  props->addProperty(propertyWidth(o));
  props->addProperty(propertyMaxSpeed(o));
  props->addProperty(propertyTurnR(o));
  props->addProperty(propertyLossOfSpeedCoef(o));
  return std::auto_ptr<BProperties>(props);
}

BPVStateShip::BPVStateShip(PropertyList* view, 
  BattleObserver* observer):BPVState<BShip>(view, observer) {
  _sides.append(qApp->translate("BPVStateShip","Neutral"));
  _sides.append(bApp->battle()->scenarioHeader()->sideHeader(1)->name());
  _sides.append(bApp->battle()->scenarioHeader()->sideHeader(2)->name());
}

void BPVStateShip::valueChanged(const BProperty& property) {
  BMapPoint pnt;
  double course = 0;
  double speed = 0;
  QDateTime dt = bApp->battle()->battleInfo().modelTime;
  BKinematicData kd = _o->kinematicData(dt);
  pnt.first = kd.x();
  pnt.second = kd.y();
  course = kd.course();
  speed = kd.v(false);
  if (property.key() == "X") {
    pnt.first = property.value().toDouble();
  }
  else if (property.key() == "Y") {
    pnt.second = property.value().toDouble();
  }
  else if (property.key() == "C") {
    course = property.value().toDouble();
  }
  else if (property.key() == "V") {
    speed = property.value().toDouble();
  }
  else if (property.key() == "SIDE") {
    int sdi;
    QString sd = property.value().toString();
    if (sd == _sides[0]) {
      sdi =NO_SIDE;
    }
    else if (sd == _sides[1]) {
      sdi = 1;
    }
    else if (sd == _sides[2]) {
      sdi = 2;
    }
    BEvent* evs = new BEventSide(_o->name(), sdi, dt);
    _observer->postEvent(evs);
    return;
  }
  BProgram* p = BProgramFactory::createStandByProgram(
    pnt.first, pnt.second, course, speed, bApp->battle());
  BEvent* evp = new BEventProgram(p, _o->name(), dt);
  _observer->postEvent(evp);
}

void BPVStateSeaMap::updateView(BSeaMap* o) {
  _o = o;
  _objProps = objectProperties(o);
  if (_objProps.get() != 0)
    _view->setupProperties(*_objProps);
  else
    _view->clear();
}

std::auto_ptr<BProperties> BPVStateSeaMap::objectProperties(BSeaMap* o) {
  if (0 == o) 
    return std::auto_ptr<BProperties>(0);
  BProperties* props = new BProperties();
  //    
  props->addProperty(propertyName(o));
  props->addProperty(propertyX0(o));
  props->addProperty(propertyY0(o));
  props->addProperty(propertyX1(o));
  props->addProperty(propertyY1(o));
  props->addProperty(propertyLK(o));
  return std::auto_ptr<BProperties>(props);
}

BProperty* BPVStateSeaMap::propertyName(BSeaMap* /* o */) {
  BProperty* p = new BProperty(BProperty::StringType, "N", 
    qApp->translate("BPVStateSeaMap", "Name"), 
    bApp->battle()->scenarioHeader()->name());
  p->setWhatsThisText(qApp->translate("BPVStateSeaMap",
    "Current scenario name<br>It can be changed using <b>%1/%2</b> menu in "
    "<b>Edit</b> mode").arg(qApp->translate("BMainWindowState", fileToolBar_N)).arg(
    qApp->translate("BMainWindowState", filePropertiesAction_TT)));
  // fileToolBar_N    , .
  return p;
}

BProperty* BPVStateSeaMap::propertyX0(BSeaMap* o) {
  BCoordinates c = o->originalCoordinates();
  BProperty* p = new BProperty(BProperty::LongitudeType, "X0", 
    qApp->translate("BPVStateSeaMap","Bottom Left Longitude"), c.x0());
  p->setWhatsThisText(qApp->translate("BPVStateSeaMap",
    "Bottom left map point longitude<br>It can be changed in <b>Edit</b> mode"));
  return p;
}

BProperty* BPVStateSeaMap::propertyY0(BSeaMap* o) {
  BCoordinates c = o->originalCoordinates();
  BProperty* p = new BProperty(BProperty::LatitudeType, "Y0", 
    qApp->translate("BPVStateSeaMap","Bottom Left Latitude"), c.y0());
  p->setWhatsThisText(qApp->translate("BPVStateSeaMap",
    "Bottom left map point latitude<br>It can be changed in <b>Edit</b> mode"));
  return p;
}

BProperty* BPVStateSeaMap::propertyX1(BSeaMap* o) {
  BCoordinates c = o->originalCoordinates();
  BProperty* p = new BProperty(BProperty::LongitudeType, "X1", 
    qApp->translate("BPVStateSeaMap","Top Right Longitude"), c.x0()+c.w());
  p->setWhatsThisText(qApp->translate("BPVStateSeaMap",
    "Top right map point longitude<br>It can be changed in <b>Edit</b> mode"));
  return p;
}

BProperty* BPVStateSeaMap::propertyY1(BSeaMap* o) {
  BCoordinates c = o->originalCoordinates();
  BProperty* p = new BProperty(BProperty::LatitudeType, "Y1", 
    qApp->translate("BPVStateSeaMap","Top Right Latitude"), c.y0()+c.h());
  p->setWhatsThisText(qApp->translate("BPVStateSeaMap",
    "Top right map point latitude<br>It can be changed in <b>Edit</b> mode"));
  return p;
}

BProperty* BPVStateSeaMap::propertyLK(BSeaMap* o) {
  BCoordinates c = o->originalCoordinates();
  BProperty* p = new BProperty(BProperty::DoubleType, "LK", 
    qApp->translate("BPVStateSeaMap","Miles in 1' longitude"), 
    BUnitsConverter::longitudeCoef(c.y0()));
  p->setWhatsThisText(qApp->translate("BPVStateSeaMap",
    "Miles in 1' longitude in bottom left map point"));
  return p;
}


BPVStateSeaMap::BPVStateSeaMap(PropertyList* view, 
  BattleObserver* observer):BPVState<BSeaMap>(view, observer) {
}

void BPVStateSeaMap::valueChanged(const BProperty& property) {
  BCoordinates c = bApp->battle()->seaMap()->originalCoordinates();
  double x0 = c.x0();
  double y0 = c.y0();
  double x1 = c.x0() + c.w();
  double y1 = c.y0() + c.h();
  if (property.key() == "X0") {
    x0 = property.value().toDouble();
  }
  else if (property.key() == "Y0") {
    y0 = property.value().toDouble();
  }
  else if (property.key() == "X1") {
    x1 = property.value().toDouble();
  }
  else if (property.key() == "Y1") {
    y1 = property.value().toDouble();
  }
  BCoordinates c1(x0, y0, x1-x0, y1-y0);
  if (c != c1) {
    BEventSeaMap* e = 
      new BEventSeaMap(bApp->battle()->battleInfo().modelTime, c1);
    _observer->postEvent(e);
  }
}

void BPVStateSeaMap::updateView() {
  _objProps = objectProperties(_o);
  if (_objProps.get() != 0)
    _view->updateProperties(*_objProps);
  else
    _view->clear();
}

void BPVStateShipGroup::updateView(BShipGroup* o) {
  _o = o;
  _objProps = objectProperties(o);
  if (_objProps.get() != 0)
    _view->setupProperties(*_objProps);
  else
    _view->clear();
}

void BPVStateShipGroup::updateView() {
  _objProps = objectProperties(_o);
  if (_objProps.get() != 0)
    _view->updateProperties(*_objProps);
  else
    _view->clear();
}

BProperty* BPVStateShipGroup::propertyName(BShipGroup* o) {
  BProperty* p = new BProperty(BProperty::StringType, "N", 
    qApp->translate("BPVStateShipGroup", "Name"), o->name());
  return p;
}

BProperty* BPVStateShipGroup::propertySide(BShipGroup* o) {
  int curItem = (_o->side() == NO_SIDE)?0:_o->side();
  BProperty* p = new BProperty(BProperty::StringListType, "SIDE", 
    qApp->translate("BPVStateShipGroup","Side"), QVariant(_sides), 
    QVariant(curItem));
  p->setWhatsThisText(qApp->translate("BPVStateShip", B_SHIP_SIDE));
  return p;
}

std::auto_ptr<BProperties> BPVStateShipGroup::objectProperties(BShipGroup* o) {
  if (0 == o) 
    return std::auto_ptr<BProperties>(0);
  BProperties* props = new BProperties();
  BProperty* p = propertyName(o);
  p->setEditable(true);
  props->addProperty(p);
  p = propertySide(o);
  p->setEditable(true);
  props->addProperty(p);
  return std::auto_ptr<BProperties>(props);
}

BPVStateShipGroup::BPVStateShipGroup(PropertyList* view, 
  BattleObserver* observer):BPVState<BShipGroup>(view, observer) {
  _sides.append(qApp->translate("BPVStateShipGroup","Neutral"));
  _sides.append(bApp->battle()->scenarioHeader()->sideHeader(1)->name());
  _sides.append(bApp->battle()->scenarioHeader()->sideHeader(2)->name());
}

void BPVStateShipGroup::valueChanged(const BProperty& property) {
  QDateTime dt = bApp->battle()->battleInfo().modelTime;
  BMapPoint pnt;
  QString name = _o->name();
  if (property.key() == "SIDE") {
    int sdi;
    QString sd = property.value().toString();
    if (sd == _sides[0]) {
      sdi =NO_SIDE;
    }
    else if (sd == _sides[1]) {
      sdi = 1;
    }
    else if (sd == _sides[2]) {
      sdi = 2;
    }
    BEvent* evs = new BEventSide(_o->name(), sdi, dt);
    _observer->postEvent(evs);
    return;
  }
  else if (property.key() == "N") {
    QString newName = property.value().toString();
    Battle* b = bApp->battle();
    if (name == newName)
      return;
    if ((newName.isEmpty()) || (b->ships()->find(newName))) {
      bApp->log(qApp->translate("BPVStateShipGroup", 
        "New group name '%1' already used or not correct. The old "
        "group name will be restored: '%2'").arg(newName).arg(name));
      newName = name;
    }
    BEvent* e = new BEventGroupRename(name, newName, bApp->battle(), dt);
    _observer->postEvent(e);
  }
}

std::auto_ptr<BProperties> BPVStateSeaMapEdit::objectProperties(BSeaMap* o) {
  if (0 == o) 
    return std::auto_ptr<BProperties>(0);
  BProperties* props = new BProperties();
  BCoordinates c = _o->originalCoordinates();
  props->addProperty(propertyName(o));

  BProperty* p = propertyX0(o);
  p->setEditable(true);
  props->addProperty(p);

  p = propertyY0(o);
  p->setEditable(true);
  props->addProperty(p);

  p = propertyX1(o);
  p->setEditable(true);
  props->addProperty(p);

  p = propertyY1(o);
  p->setEditable(true);
  props->addProperty(p);

  p = propertyLK(o);
  props->addProperty(p);
  return std::auto_ptr<BProperties>(props);
}

std::auto_ptr<BProperties> BPVStateShipEdit::objectProperties(BShip* o) {
  if (0 == o) 
    return std::auto_ptr<BProperties>(0);
  BProperties* props = new BProperties();
  BProgramData pd = o->programData(bApp->battle()->battleInfo().modelTime);
  BKinematicData kd = pd.kinematicData();
  props->addProperty(propertyName(o));

  BProperty* p = propertyX(o, pd, kd);
  p->setEditable(true);
  props->addProperty(p);

  p = propertyY(o, pd, kd);
  p->setEditable(true);
  props->addProperty(p);

  p = propertyCourse(o, pd, kd);
  p->setEditable(true);
  props->addProperty(p);

  p = propertySpeed(o, pd, kd);
  p->setEditable(true);
  props->addProperty(p);

  p = propertySide(o);
  p->setEditable(true);
  props->addProperty(p);

  props->addProperty(propertyType(o));
  props->addProperty(propertySize(o));
  props->addProperty(propertyLength(o));
  props->addProperty(propertyWidth(o));
  props->addProperty(propertyMaxSpeed(o));
  props->addProperty(propertyTurnR(o));
  props->addProperty(propertyLossOfSpeedCoef(o));
  return std::auto_ptr<BProperties>(props);
}

std::auto_ptr<BProperties> BPVStateSeaMapPlay::objectProperties(BSeaMap* o) {
  if (0 == o) 
    return std::auto_ptr<BProperties>(0);
  std::auto_ptr<BProperties> op = BPVStateSeaMap::objectProperties(o);
  op->setAllEditable(false);
  return op;
}

std::auto_ptr<BProperties> BPVStateShipPlay::objectProperties(BShip* o) {
  if (0 == o) 
    return std::auto_ptr<BProperties>(0);
  std::auto_ptr<BProperties> op = BPVStateShip::objectProperties(o);
  op->setAllEditable(false);
  return op;
}

std::auto_ptr<BProperties> BPVStateShipGroupPlay::objectProperties(
  BShipGroup* o) {
  if (0 == o) 
    return std::auto_ptr<BProperties>(0);
  std::auto_ptr<BProperties> op = BPVStateShipGroup::objectProperties(o);
  op->setAllEditable(false);
  return op;
}

BProperty* BPVStateShip::propertyType(BShip* o) {
  BProperty* p = new BProperty(BProperty::StringType, "TYPE", 
    qApp->translate("BPVStateShip","Vessel type"), 
    QVariant(o->vesselTypeToStringHRF(o->vesselType())));
  p->setWhatsThisText(qApp->translate("BPVStateShip", B_SHIP_TYPE));
  return p;
}

BProperty* BPVStateShip::propertySize(BShip* o) {
  BProperty* p = new BProperty(BProperty::IntType, "SIZE", 
    qApp->translate("BPVStateShip","Vessel size"), 
    QVariant(o->size()));
  p->setWhatsThisText(qApp->translate("BPVStateShip", B_SHIP_SIZE));
  return p;
}

BProperty* BPVStateShip::propertyLength(BShip* o) {
  BProperty* p = new BProperty(BProperty::DoubleType, "LENGTH", 
    qApp->translate("BPVStateShip","Length (m)"), 
    QVariant(BUnitsConverter::mFromMiles(o->length())));
  p->setWhatsThisText(qApp->translate("BPVStateShip", B_SHIP_LENGTH));
  return p;
}

BProperty* BPVStateShip::propertyWidth(BShip* o) {
  BProperty* p = new BProperty(BProperty::DoubleType, "WIDTH", 
    qApp->translate("BPVStateShip","Width (m)"), 
    QVariant(BUnitsConverter::mFromMiles(o->width())));
  p->setWhatsThisText(qApp->translate("BPVStateShip", B_SHIP_WIDTH));
  return p;
}

BProperty* BPVStateShip::propertyMaxSpeed(BShip* o) {
  BProperty* p = new BProperty(BProperty::DoubleType, "MAXSPEED", 
    qApp->translate("BPVStateShip","Max. speed (knots)"), 
    QVariant(o->maxSpeed()));
  p->setWhatsThisText(qApp->translate("BPVStateShip", B_SHIP_MAX_SPEED));
  return p;
}

BProperty* BPVStateShip::propertyTurnR(BShip* o) {
  BProperty* p = new BProperty(BProperty::DoubleType, "TURNR", 
    qApp->translate("BPVStateShip","Turn radius (m)"), 
    QVariant(ceil(BUnitsConverter::mFromMiles(o->minTurnRadius()))));
  p->setWhatsThisText(qApp->translate("BPVStateShip", B_SHIP_MAX_SPEED));
  return p;
}
