/******************************************************************************/
/* BPVState -   BattleFleet              */
/******************************************************************************/

#ifndef BPVState_H
#define BPVState_H
static const char bpvstate_h_id[]="@(#) $Id: bpvstate.h,v 1.7 2005/03/07 15:34:38 Oleksiy Balabay Exp $"; static const char* bpvstate_h_id_id=bpvstate_h_id;
/** \file bpvstate.h
  \brief  BPVState  BattleFleet

     BPVState  BattleFleet
*/

#include <memory>

#include <qstringlist.h>

#include "propertylist/bproperty.h"
#include "propertylist/propertyeditor.h"
#include "bship.h"
#include "bseamap.h"

class BattleObserver;
class BFactory;
class BFactoryEdit;
class BFactoryPlay;

template<class T>
class BPVState {
public:
  BPVState(PropertyList* view, 
  BattleObserver* observer):_view(view), _objProps(0), _observer(observer) {
  }
  virtual ~BPVState() {};
  virtual void updateView(T* /* o */) {
    if (_view->childCount() != 0) {
      _view->clear();
    }
  }
  virtual void updateView() {
    updateView(_o);
  }
  virtual void valueChanged(const BProperty& /* property */) {} ;
protected:
  virtual std::auto_ptr<BProperties> objectProperties(T* /* o */) { 
	  return std::auto_ptr<BProperties>(0); 
  };
protected:
  std::auto_ptr<BProperties> _objProps;
  T* _o;
  PropertyList* _view;
  BattleObserver* _observer;
};

typedef BPVState<void> BPVStateIdle;

class BPVStateShip : public BPVState<BShip> {
  friend class BFactory;
protected:
  BPVStateShip(PropertyList* view, 
  BattleObserver* observer);
public:
  virtual ~BPVStateShip() {};
  virtual void updateView(BShip* o);
  virtual void updateView();
  virtual void valueChanged(const BProperty& property);
protected:
  virtual std::auto_ptr<BProperties> objectProperties(BShip* o);
  BProperty* propertyName(BShip* o);
  BProperty* propertyX(BShip* o, const BProgramData& pd, const BKinematicData& kd);
  BProperty* propertyY(BShip* o, const BProgramData& pd, const BKinematicData& kd);
  BProperty* propertyCourse(BShip* o, const BProgramData& pd, const BKinematicData& kd);
  BProperty* propertySpeed(BShip* o, const BProgramData& pd, const BKinematicData& kd);
  BProperty* propertySide(BShip* o);
  BProperty* propertyType(BShip* o);
  BProperty* propertySize(BShip* o);
  BProperty* propertyLength(BShip* o);
  BProperty* propertyWidth(BShip* o);
  BProperty* propertyMaxSpeed(BShip* o);
  BProperty* propertyTurnR(BShip* o);
  BProperty* propertyLossOfSpeedCoef(BShip* o);
protected:
  QStringList _sides;
};

class BPVStateShipGroup : public BPVState<BShipGroup> {
  friend class BFactory;
protected:
  BPVStateShipGroup(PropertyList* view, 
  BattleObserver* observer);
public:
  virtual ~BPVStateShipGroup() {};
  virtual void updateView(BShipGroup* o);
  virtual void updateView();
  virtual void valueChanged(const BProperty& property);
protected:
  virtual std::auto_ptr<BProperties> objectProperties(BShipGroup* o);
  BProperty* propertyName(BShipGroup* o);
  BProperty* propertySide(BShipGroup* o);
private:
  QStringList _sides;
};

class BPVStateSeaMap : public BPVState<BSeaMap> {
  friend class BFactory;
protected:
  BPVStateSeaMap(PropertyList* view, 
  BattleObserver* observer);
public:
  virtual ~BPVStateSeaMap() {};
  virtual void updateView(BSeaMap* o);
  virtual void updateView();
  virtual void valueChanged(const BProperty& property);
protected:
  virtual std::auto_ptr<BProperties> objectProperties(BSeaMap* o);
  BProperty* propertyName(BSeaMap* o);
  BProperty* propertyX0(BSeaMap* o);
  BProperty* propertyX1(BSeaMap* o);
  BProperty* propertyY0(BSeaMap* o);
  BProperty* propertyY1(BSeaMap* o);
  BProperty* propertyLK(BSeaMap* o);

};

class BPVStateSeaMapEdit : public BPVStateSeaMap {
  friend class BFactoryEdit;
protected:
  BPVStateSeaMapEdit(PropertyList* view, 
  BattleObserver* observer):BPVStateSeaMap(view, observer) {};
public:
  virtual ~BPVStateSeaMapEdit() {};
protected:
  virtual std::auto_ptr<BProperties> objectProperties(BSeaMap* o);
};

class BPVStateShipEdit : public BPVStateShip {
  friend class BFactoryEdit;
protected:
  BPVStateShipEdit(PropertyList* view, 
  BattleObserver* observer):BPVStateShip(view, observer) {};
public:
  virtual ~BPVStateShipEdit() {};
protected:
  virtual std::auto_ptr<BProperties> objectProperties(BShip* o);
};

class BPVStateShipGroupEdit : public BPVStateShipGroup {
  friend class BFactoryEdit;
  BPVStateShipGroupEdit(PropertyList* view, 
  BattleObserver* observer):BPVStateShipGroup(view, observer) {};
protected:
public:
  virtual ~BPVStateShipGroupEdit() {};
};

class BPVStateSeaMapPlay : public BPVStateSeaMap {
  friend class BFactoryPlay;
protected:
  BPVStateSeaMapPlay(PropertyList* view, 
  BattleObserver* observer):BPVStateSeaMap(view, observer) {};
public:
  virtual ~BPVStateSeaMapPlay() {};
protected:
  virtual std::auto_ptr<BProperties> objectProperties(BSeaMap* o);
};

class BPVStateShipPlay : public BPVStateShip {
  friend class BFactoryPlay;
protected:
  BPVStateShipPlay(PropertyList* view, 
  BattleObserver* observer):BPVStateShip(view, observer) {};
public:
  virtual ~BPVStateShipPlay() {};
protected:
  virtual std::auto_ptr<BProperties> objectProperties(BShip* o);
};

class BPVStateShipGroupPlay : public BPVStateShipGroup {
  friend class BFactoryPlay;
protected:
  BPVStateShipGroupPlay(PropertyList* view, 
  BattleObserver* observer):BPVStateShipGroup(view, observer) {};
public:
  virtual ~BPVStateShipGroupPlay() {};
protected:
  virtual std::auto_ptr<BProperties> objectProperties(BShipGroup* o);
};


#endif // BPVState_H
