/******************************************************************************/
/* BScales -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bscales_cpp_id[]="@(#) $Id: bscales.cpp,v 1.5 2004/08/27 15:54:02 Oleksiy Balabay Exp $"; static const char* bscales_cpp_id_id=bscales_cpp_id;

#include <qfont.h>
#include <qfontmetrics.h>
#include <qpainter.h>

#include "bscales.h"
const int SCALE_LAYER = 2000;

/** \file bscales.cpp
  \brief  BScales  BattleFleet (impl.)

     BScales  BattleFleet
*/

const int BScaleType::intervalsInSeconds[] = {
  10,     20,     30, 
  1*60,   2*60,   5*60,   10*60,  20*60,  30*60,
  1*60*60,2*60*60,5*60*60
};

const int BScaleType::MAX_LINES = 10;
const int BScaleType::MIN_LINES = 1;
const double BScaleType::TEXT_PART_IN_INTERVAL = 2.0/3.0;
const int BScaleType::FONT_SIZE = 8;
const QString BScaleType::FORMAT_DMS = QString("%1")+QChar(176)+"%2\'%3\"";
const QString BScaleType::FORMAT_DM = QString("%1")+QChar(176)+"%2\'";
const QString BScaleType::FORMAT_D = QString("%1")+QChar(176);

BScaleType::BScaleType(const double map0, const double map1,
  const int canvas0, const int canvas1, 
  const BSeaMap::BCoordinatesType type) 
  :_map0(map0), _map1(map1), 
  _canvas0(canvas0), _canvas1(canvas1), _type(type),
  _bestInterval(-1), _count(-1), _seconds(-1) {
  if (_map0 > _map1) {
    throw BInternalException(__FILE__, __LINE__,
      qApp->translate("BScaleType", "Coordinate map0 (%1) "
      "should be lower then map1 (%2)").arg(
      _map0).arg(_map1));
  }
  selectBestInterval();
}

BScaleType::~BScaleType() {
  ;
}

int BScaleType::count() const {
  return _count;
}

QString BScaleType::label(const int index) {
  if ((index < 0) || (index >= count())) {
    throw BRangeException(__FILE__, __LINE__, 0, count() - 1, index);
  }
  int coord = _label0 + index*intervalsInSeconds[_bestInterval];
  int deg = abs(coord)/(60*60);
  int min = (abs(coord) - deg*60*60)/60;
  int secs = (abs(coord) - deg*60*60)%60;
  if (intervalsInSeconds[_bestInterval] < 60) {
    return FORMAT_DMS.arg(deg).arg(min).arg(secs);
  }
  else if (intervalsInSeconds[_bestInterval] < 60*60) {
    return FORMAT_DM.arg(deg).arg(min);
  }
  else {
    return FORMAT_D.arg(deg);
  }
}

double BScaleType::labelCoordinate(const int index) {
  if ((index < 0) || (index >= count())) {
    throw BRangeException(__FILE__, __LINE__, 0, count() - 1, index);
  }
  double coord = BSeaMap::degreesFromSeconds(_label0 + 
    index*intervalsInSeconds[_bestInterval]);
  return coord;
}

void BScaleType::selectBestInterval() {
  _seconds = BSeaMap::degreesToSeconds(_map1 - _map0);
  int maxOnBestI = -1;
  int bestI = -1;
  for (int i = 0; i < sizeof(intervalsInSeconds)/sizeof(int); i++) {
    int c = countIntervals(i);
    if ( c > maxOnBestI) {
      bestI = i;
      maxOnBestI = c;
    }
  }
  _bestInterval = bestI;
  _count = maxOnBestI;
  if (_count == -1) {
    return;
  }
  _count++;
  //    
  int map0 = BSeaMap::degreesToSeconds(_map0);
  int intervalSize = intervalsInSeconds[bestI];
  while ((map0%intervalSize) != 0) {
    map0++;
  }
  _label0 = map0;
}

int BScaleType::countIntervals(const int interval) const {
  int cnt = _seconds/intervalsInSeconds[interval];
  if (cnt > MAX_LINES) {
    return -1;
  }
  if (cnt < MIN_LINES) {
    return -1;
  }
  int canvasInterval = abs(_canvas1 - _canvas0)/cnt;
  //   
  //  
  QString text;
  if (intervalsInSeconds[interval] < 60) {
    text = FORMAT_DMS.arg(180).arg(59).arg(59);
  }
  else if (intervalsInSeconds[interval] < 60*60) {
    text = FORMAT_DM.arg(180).arg(59);
  }
  else {
    text = FORMAT_D.arg(180);
  }
  QFont f;
  f.setPointSize(FONT_SIZE);
  int textWidth = QFontMetrics(f).width(text);
  if (textWidth > canvasInterval*TEXT_PART_IN_INTERVAL) {
    return -1;
  }
  return cnt;
}

class BRotatedText: public QCanvasPolygonalItem {
public:
//  enum BAngles {
//    clockwise90,
//    clockwise180,
//    clockwise270
//  };
  BRotatedText(const QString& s, const int angle, const QFont& f, 
    QCanvas* canvas):QCanvasPolygonalItem(canvas), _s(s), _angle(angle), 
    _f(f) {
    QFontMetrics fm(f);
    _rect = fm.boundingRect(s);
  }
  virtual ~BRotatedText() {
    hide();
  }
  virtual QPointArray areaPoints() const {
    QWMatrix m;
    m.translate(x(),y());
    m.rotate(_angle);
    return m.mapToPolygon(_rect);
  };
protected:
  virtual void drawShape(QPainter& p) {
    p.save();
    p.translate(x(), y());
    p.rotate(_angle);
    p.drawText(QPoint(0,0), _s);
    p.restore();
  };
private:
  const QString _s;
  const int _angle;
  const QFont _f;
  QRect _rect;
};

/** \class BScales
  \brief 


*/
/** \brief 
*/
BScales::BScales(const BCoordTransformer* coordTransformer, QCanvas* canvas)
:_coordTransformer(coordTransformer), _canvas(canvas), _xScale(0), _yScale(0) {
  const BCoordinates coMap = _coordTransformer->currentCoordinates();
  const BCoordinates coCanvas = _coordTransformer->currentCanvasCoordinates();
  _xScale = new BScaleType(coMap.x0(), coMap.x0() + coMap.w(),
    coCanvas.x0(), coCanvas.x0() + coCanvas.w(), BSeaMap::longitude);
  _yScale = new BScaleType(coMap.y0(), coMap.y0() + coMap.h(),
    coCanvas.y0(), coCanvas.y0() + coCanvas.h(), BSeaMap::latitude);
}

/** \brief 
*/
BScales::~BScales(){
  clearScaleTypes();
}

void BScales::clearScaleTypes(){
  if (_xScale) {
    delete _xScale;
    _xScale = 0;
  }
  if (_yScale) {
    delete _yScale;
    _yScale = 0;
  }
}

void BScales::drawScale(){
  QCanvasText* firstX = 0;
  if (_xScale->count() != -1) {
    for (int i = 0; i < _xScale->count(); i++) {
      QCanvasText* t = new QCanvasText(_canvas);
      if (0 == i)
        firstX = t;
      t->setText(_xScale->label(i));
      QPoint startPoint = _coordTransformer->pointFromMap(BMapPoint(
        _xScale->labelCoordinate(i), 
        _coordTransformer->currentCoordinates().y0()));
//      t->setColor(QColor("gray75"));
      t->setTextFlags(Qt::AlignHCenter);
      QFont f;
      f.setPointSize(BScaleType::FONT_SIZE);
      t->setFont(f);
      t->setX(startPoint.x());
      t->setY(startPoint.y() - t->boundingRect().height()-1), 
      t->setZ(SCALE_LAYER+1);
      if (_canvas->rect().contains(t->boundingRect()))
        t->show();
    }
  }
  if (_yScale->count() != -1) {
    for (int i = 0; i < _yScale->count(); i++) {
      QFont f;
      f.setPointSize(BScaleType::FONT_SIZE);
      BRotatedText* t = new BRotatedText(_yScale->label(i), -90, QFont(), _canvas);
      QPoint startPoint = _coordTransformer->pointFromMap(BMapPoint(
        _coordTransformer->currentCoordinates().x0(), _yScale->labelCoordinate(i)));
//      t->setPen(QPen(QColor("gray75")));
      t->setX(startPoint.x()+t->boundingRect().width());
      t->setY(startPoint.y()+t->boundingRect().height()/2), 
      t->setZ(SCALE_LAYER+1);
      if ((firstX == 0) ||(!t->collidesWith(firstX))) {
        if (_canvas->rect().contains(t->boundingRect()))
          t->show();
      }
/*      QCanvasText* t = new QCanvasText(_canvas);
      t->setText(_yScale->label(i));
      QPoint startPoint = _seaMap->pointFromMap(BMapPoint(
        _seaMap->currentCoordinates()->x0(), _yScale->labelCoordinate(i)));
      t->setX(startPoint.x());
      t->setY(startPoint.y()), 
      t->setColor(QColor("gray75"));
      t->setTextFlags(Qt::AlignHCenter);
      QFont f;
      f.setPointSize(BScaleType::FONT_SIZE);
      t->setFont(f);
      t->setZ(0);
      t->show();
*/
    }
  }
}

void BScales::drawLines(){
  const BCoordinates co = _coordTransformer->currentCoordinates();
  if (_xScale->count() != -1) {
    for (int i = 0; i < _xScale->count(); i++) {
      QCanvasLine* l = new QCanvasLine(_canvas);
      QPoint startPoint = _coordTransformer->pointFromMap(BMapPoint(
        _xScale->labelCoordinate(i), co.y0()));
      QPoint endPoint = _coordTransformer->pointFromMap(BMapPoint(
        _xScale->labelCoordinate(i), co.y0() + co.h()));
      l->setPoints(startPoint.x(), startPoint.y(), 
        endPoint.x(), endPoint.y());
      l->setPen(QPen(QColor("gray75"), 1, Qt::SolidLine));
      l->setZ(SCALE_LAYER);
      l->show();
    }
  }
  if (_yScale->count() != -1) {
    for (int i = 0; i < _yScale->count(); i++) {
      QCanvasLine* l = new QCanvasLine(_canvas);
      QPoint startPoint = _coordTransformer->pointFromMap(BMapPoint(
        co.x0(), _yScale->labelCoordinate(i)));
      QPoint endPoint = _coordTransformer->pointFromMap(BMapPoint(
        co.x0() + co.w(),_yScale->labelCoordinate(i) ));
      l->setPoints(startPoint.x(), startPoint.y(), 
        endPoint.x(), endPoint.y());
      l->setPen(QPen(QColor("gray75"), 1, Qt::SolidLine));
      l->setZ(SCALE_LAYER);
      l->show();
    }
  }
}
