/******************************************************************************/
/* BScenarioHeader -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bscenarioheader_cpp_id[]="@(#) $Id: bscenarioheader.cpp,v 1.18 2005/01/21 16:18:00 Oleksiy Balabay Exp $"; static const char* bscenarioheader_cpp_id_id=bscenarioheader_cpp_id;

#include <qfile.h>
#include <qdir.h>
#include <qapplication.h>

#include "bscenarioheader.h"
#include "bexception.h"
#include "bdomutils.h"
#include "bapplication.h"

#include "xpms/unknown.xpm"
#include "xpms/russia.xpm"
#include "xpms/japan.xpm"

const QString SCN_SCENARIO = "Scenario";
const QString SCN_NAME = "Name";
const QString SCN_SIDE1 = "Side1";
const QString SCN_SIDE2 = "Side2";
const QString SCN_SIDE_NAME = "Name";
const QString SCN_BEGIN_DATE = "BeginDate";
const QString SCN_END_DATE = "EndDate";
const QString SCN_SHIPS_FILE_NAME = "ShipsFileName";
const QString SCN_MAP_FILE_NAME = "MapFileName";
const QString SCN_EVENTS_FILE_NAME = "EventsFileName";
const QString SCN_AI_FILE_NAME = "AIFileName";
const QString SCN_DESCRIPTION = "Description";

/** \file bscenarioheader.cpp
  \brief  BScenarioHeader  BattleFleet (impl.)

     BScenarioHeader  BattleFleet
*/

int BSideHeader::_count = 0;
BSideHeader::CountryFlags BSideHeader::_flags;

/** \class BSideHeader
  \brief    ,   

       name(),  - flag().  
    :
  <ol>
    <li></li>
    <li></li>
    <li></li>
  </ol>

         
   shipsFileNames()

    flags()     
*/

/** \brief 

    Dom- \a element.  
  BScenarioHeader.   \a SCN_SIDE_NAME   
   BScenarioHeader.
*/

BSideHeader::BSideHeader(const QDomElement& element) {
  if (_count == 0) {
    initFlags();
  }
  _name = BDomUtils::stringAttribute(element, SCN_SIDE_NAME);
  QDomNode n = element.firstChild();
  _count++;
}

BSideHeader::BSideHeader() {
  if (_count == 0) {
    initFlags();
  }
  _name = qApp->translate("BSideHeader", "Unknown", "country");
  _count++;
}

BSideHeader::~BSideHeader() {
  _count--;
  if (_count == 0)
    freeFlags();
}

void BSideHeader::initFlags() {
  _flags[UnknownCountry]=CountryFlag(
    qApp->translate("Countries", "Unknown"), new QPixmap(unknown));
  _flags[JapanCountry]=CountryFlag(
    qApp->translate("Countries", "Japan"), new QPixmap(japan));
  _flags[RussiaCountry]=CountryFlag(
    qApp->translate("Countries", "Russia"), new QPixmap(russia));
}

void BSideHeader::freeFlags() {
  for(CountryFlags::iterator it = _flags.begin(); it != _flags.end(); it++) {
    delete it->second.second;
  }
  _flags.clear();
}

int BSideHeader::stringToCountry() const {
  QString nameUpper = name().upper();
  for (CountryFlags::const_iterator it = _flags.begin(); 
  it != _flags.end(); it++) {
    if (it->second.first.upper() == nameUpper) {
      return it->first;
    }
  }
  return UnknownCountry;
}

QString BSideHeader::name() const {
  return _name;
}

QPixmap BSideHeader::flag() const {
  if (_flags.find(stringToCountry()) != _flags.end()) {
    return *(_flags[stringToCountry()].second);
  }
  else {
    return *(_flags[UnknownCountry].second);
  }
}

const BSideHeader::CountryFlags& BSideHeader::flags() {
  if (_count == 0) {
    initFlags();
  }
  return _flags;
}

void BSideHeader::setName(const QString& name) {
  _name = name;
}

/** \class BScenarioHeader
  \brief   

          
   name(). 
  
    :
  <ul>
    <li>.      
       mapFileName()
    </li>
    <li> .   beginDate()
    </li>
    <li> .     2 . 
       sideHeader().    
     .   1  2.
    </li>
  </ul>

     :
  <BR> <\a SCN_SCENARIO > 
  <BR>&nbsp;&nbsp;<\a SCN_NAME >   </\a SCN_NAME >
  <BR>&nbsp;&nbsp;<\a SCN_SIDE1 \a SCN_SIDE_NAME ='  '/>
  <BR>&nbsp;&nbsp;<\a SCN_SIDE2  \a SCN_SIDE_NAME ='  '/>
  <BR>&nbsp;&nbsp;<\a SCN_SHIPS_FILE_NAME >   </\a SCN_SHIPS_FILE_NAME >
  <BR>&nbsp;&nbsp;&nbsp;&nbsp;...
  <BR>&nbsp;&nbsp;<\a SCN_MAP_FILE_NAME >  </\a SCN_MAP_FILE_NAME >
  <BR>&nbsp;&nbsp;<\a SCN_BEGIN_DATE \a DATE_YEAR ='' \a DATE_MONTH ='' 
  \a DATE_DAY ='' \a DATE_HOURS ='' \a DATE_MINUTES ='' 
  \a DATE_SECONDS=''/>
  <BR></\a SCN_SCENARIO > 

     :
<PRE>
<Scenario>
  <Name>test</Name>
  <Side1 Name="Russia" />
  <Side2 Name="Japan" />
  <ShipsFileName>common/russia.xml</ShipsFileName>
  <ShipsFileName>common/japan.xml</ShipsFileName>
  <MapFileName>data/test/map.xml</MapFileName>
  <EventsFileName>data/test/events.xml</EventsFileName>
  <BeginDate Minutes="20" Day="14" Hours="13" Seconds="0" Year="1905" Month="5" />
</Scenario>
</PRE>

           ,  
       \a BattleFleet
*/
const QString BScenarioHeader::DEF_MAP_FILE_NAME = "map.xml";
const QString BScenarioHeader::DEF_SHIPS_ON_MAP_FILE_NAME = "ships_on_map.xml";
const QString BScenarioHeader::DEF_EVENTS_FILE_NAME = "events.xml";
const QString BScenarioHeader::DEF_AI_FILE_NAME = "ai.xml";

/** \brief 

     \a fileName  ,    
    \a BattleFleet.

  \exception BException      . 
  BException::message()   .   
     
*/
BScenarioHeader::BScenarioHeader(const QString& fileName)
  :BXmlConfig(fileName), _sideHeader1(0), _sideHeader2(0) {
  try {
    parseDocument(*_doc);
    delete _doc;
    _doc = 0;
  }
  catch(BConfigException& e) {
    clearSideHeaders();
    throw addFileName(e);
  }
  catch(BException&) {
    clearSideHeaders();
    throw;
  }
  catch(...) {
    clearSideHeaders();
    throw BInternalException(__FILE__, __LINE__,
      qApp->translate("BXmlConfig",
      "Unknown error loading file '%1'").arg(this->fileName()));
  }
}

/** \brief 
*/
BScenarioHeader::~BScenarioHeader(){
  clearSideHeaders();
}

void BScenarioHeader::clearSideHeaders() {
  if (_sideHeader1) {
    delete _sideHeader1;
    _sideHeader1 = 0;
  }
  if (_sideHeader2) {
    delete _sideHeader2;
    _sideHeader2 = 0;
  }
}

void BScenarioHeader::parseDocument(const QDomDocument& doc) {
  bool nameFound = false;
  bool beginDateFound = false;
  bool side1Found = false;
  bool side2Found = false;
  bool mapFileNameFound = false;
  bool eventsFileNameFound = false;
  bool shipsFileNameFound = false;
//  bool aiFileNameFound = false;
  QDomElement docElem = doc.documentElement();
  if (docElem.tagName() != SCN_SCENARIO) {
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate("BXmlConfig",
      "File '%1' is not '%2' file but '%3'").arg(fileName()).arg(
      SCN_SCENARIO).arg(docElem.tagName()));
  }
  QDomNode n = docElem.firstChild();
  while( !n.isNull() ) {
    QDomElement e = n.toElement(); // try to convert the node to an element.
    n = n.nextSibling();
    if( !e.isNull() ) { // the node was really an element.
      if (e.tagName() == SCN_NAME) {
        _name = e.text();
        nameFound = true;
      }
      else if (e.tagName() == SCN_BEGIN_DATE) {
        parseBeginDate(e);
        beginDateFound = true;
      }
      else if (e.tagName() == SCN_END_DATE) {
        parseEndDate(e);
      }
      else if (e.tagName() == SCN_SIDE1) {
        _sideHeader1 = new BSideHeader(e);
        side1Found = true;
      }
      else if (e.tagName() == SCN_SIDE2) {
        _sideHeader2 = new BSideHeader(e);
        side2Found = true;
      }
      else if (e.tagName() == SCN_MAP_FILE_NAME) {
        _mapFileName = bApp->extendFileName(e.text());
        mapFileNameFound = true;
      }
      else if (e.tagName() == SCN_EVENTS_FILE_NAME) {
        _eventsFileName = bApp->extendFileName(e.text());
        eventsFileNameFound = true;
      }
      else if (e.tagName() == SCN_SHIPS_FILE_NAME) {
        _shipsFileNames.append(bApp->extendFileName(e.text()));
        shipsFileNameFound = true;
      }
      else if (e.tagName() == SCN_AI_FILE_NAME) {
        _aiFileName = bApp->extendFileName(e.text());
//        aiFileNameFound = true;
      }
      else if (e.tagName() == SCN_DESCRIPTION) {
        _description = e.text();
      }
      else {
        qWarning("Unsupported tag '%s' in file '%s'", 
          (const char*)e.tagName().local8Bit(),
          (const char*)fileName().local8Bit());
      }
    }
  }
  if (!nameFound) {
    BDomUtils::elementDoesNotExist(SCN_NAME);
  }
  if (!beginDateFound) {
    BDomUtils::elementDoesNotExist(SCN_BEGIN_DATE);
  }  
  if (!side1Found) {
    BDomUtils::elementDoesNotExist(SCN_SIDE1);
  }
  if (!side2Found) {
    BDomUtils::elementDoesNotExist(SCN_SIDE2);
  }
  if (!mapFileNameFound) {
    BDomUtils::elementDoesNotExist(SCN_MAP_FILE_NAME);
  }
  if (!eventsFileNameFound) {
    BDomUtils::elementDoesNotExist(SCN_EVENTS_FILE_NAME);
  }
  if (!shipsFileNameFound) {
    BDomUtils::elementDoesNotExist(SCN_SHIPS_FILE_NAME);
  }
//  if (!aiFileNameFound) {
//    BDomUtils::elementDoesNotExist(SCN_AI_FILE_NAME);
//  }
}

void BScenarioHeader::parseBeginDate(const QDomElement& e) {
  _beginDate = BDomUtils::dateTimeFromXml(e);
  if (!_endDate.isValid()) {
    _endDate = _beginDate;
  }
}

void BScenarioHeader::parseEndDate(const QDomElement& e) {
  _endDate = BDomUtils::dateTimeFromXml(e);
}

QString BScenarioHeader::name() const {
  return _name;
}

QDateTime BScenarioHeader::beginDate() const {
  return _beginDate;
}

QDateTime BScenarioHeader::endDate() const {
  return _endDate;
}

QString BScenarioHeader::mapFileName() const {
  return bApp->extendFileName(_mapFileName);
}

QString BScenarioHeader::eventsFileName() const {
  return bApp->extendFileName(_eventsFileName);
}

/// side == 1  2.  
const BSideHeader* BScenarioHeader::sideHeader(int side) const 
{
  if (1 == side)  
    return _sideHeader1;
  else if (2 == side)
    return _sideHeader2;
  else {
    throw BRangeException(__FILE__, __LINE__, 1, 2, side);
  }
}

QStringList BScenarioHeader::shipsFileNames() const {
  return _shipsFileNames;
}

QDomDocument BScenarioHeader::toXml() const {
  QDomDocument doc;
  QDomProcessingInstruction pi = doc.createProcessingInstruction(
    "xml", QString("version='1.0' encoding='UTF-8'"));
  doc.insertBefore(pi, doc.documentElement());
  doc.appendChild(doc.createElement(SCN_SCENARIO));

  {
    QDomElement nameEl = doc.createElement(SCN_NAME);
    nameEl.appendChild(doc.createTextNode(name()));
    doc.documentElement().appendChild(nameEl);
  }
  
  {
    QDomElement side1El = doc.createElement(SCN_SIDE1);
    side1El.setAttribute(SCN_SIDE_NAME, sideHeader(1)->name());
    doc.documentElement().appendChild(side1El);
  }

  {
    QDomElement side2El = doc.createElement(SCN_SIDE2);
    side2El.setAttribute(SCN_SIDE_NAME, sideHeader(2)->name());
    doc.documentElement().appendChild(side2El);
  }

  {
    QStringList sfn = shipsFileNames();
    for(QStringList::iterator it = sfn.begin(); it != sfn.end(); it++) {
      QDomElement sfnEl = doc.createElement(SCN_SHIPS_FILE_NAME);
      sfnEl.appendChild(doc.createTextNode(bApp->compressFileName(*it)));
      doc.documentElement().appendChild(sfnEl);
    }
  }

  {
    QDomElement nameEl = doc.createElement(SCN_MAP_FILE_NAME);
    nameEl.appendChild(doc.createTextNode(
      bApp->compressFileName(mapFileName())));
    doc.documentElement().appendChild(nameEl);
  }

  {
    QDomElement nameEl = doc.createElement(SCN_EVENTS_FILE_NAME);
    nameEl.appendChild(doc.createTextNode(
      bApp->compressFileName(eventsFileName())));
    doc.documentElement().appendChild(nameEl);
  }
  
  {
    QDomElement nameEl = doc.createElement(SCN_AI_FILE_NAME);
    nameEl.appendChild(doc.createTextNode(
      bApp->compressFileName(aiFileName())));
    doc.documentElement().appendChild(nameEl);
  }

  {
    QDomElement nameEl = doc.createElement(SCN_DESCRIPTION);
    nameEl.appendChild(doc.createTextNode(description()));
    doc.documentElement().appendChild(nameEl);
  }

  doc.documentElement().appendChild(BDomUtils::dateTimeToXml(beginDate(),
    SCN_BEGIN_DATE, doc));

  doc.documentElement().appendChild(BDomUtils::dateTimeToXml(endDate(),
    SCN_END_DATE, doc));

  return doc;
}

/** \class BScenarioHeaders
  \brief  ,     \a dirName

   \a dirName -  ,     
    BattleFleet.

   -        ,
       ,      .

  \exception BException   \a dirName  
*/
BScenarioHeaders::BScenarioHeaders(const QString& dirName)
:_dirName(bApp->extendFileName(dirName)) {
  parseDir();
}

BScenarioHeaders::~BScenarioHeaders() {
  deleteItems();
}

void BScenarioHeaders::parseDir() {
  QDir d(_dirName, "*.xml", QDir::Name|QDir::IgnoreCase, QDir::Files);
  if (!d.exists()) {
    throw BFileException(BConfigException(__FILE__, __LINE__,
      qApp->translate("BScenarioHeaders",
      "Directory does not exist")),_dirName);
  }
  QStringList l = d.entryList();
  for (QStringList::iterator it = l.begin(); it != l.end(); it++) {
    try {
      BScenarioHeader* h = new BScenarioHeader(d.absFilePath(*it));
      insert(h);
    }
    catch (BException& e) {
      bApp->log(e.message());
      bApp->log(qApp->translate("BScenarioHeaders", 
        "Scenario file %1 skipped").arg(d.absFilePath(*it)));
      continue;
    }
    catch (...) {
      bApp->log(qApp->translate("BScenarioHeaders", 
        "Error parsing scenario file %1").arg(d.absFilePath(*it)));
      bApp->log(qApp->translate("BScenarioHeaders", 
        "Scenario file %1 skipped").arg(d.absFilePath(*it)));
      continue;
    }
  }
}

void BScenarioHeaders::deleteItems() {
  for (iterator it = begin(); it != end(); it++) {
    delete *it;
  }
  clear();
}

void BScenarioHeader::setName(const QString& name) {
  _name = name;
}

void BScenarioHeader::setMapFileName(const QString& fileName) {
  _mapFileName = fileName;
}

void BScenarioHeader::setEventsFileName(const QString& fileName) {
  _eventsFileName = fileName;
}

void BScenarioHeader::setBeginDate(const QDateTime& beginDate) {
  _beginDate = beginDate;
  if (!_endDate.isValid())
    _endDate = beginDate;
}

void BScenarioHeader::setEndDate(const QDateTime& endDate) {
  _endDate = endDate;
}

void BScenarioHeader::addShipsFileName(const QString& fileName) {
  QString exFileName = bApp->extendFileName(fileName);
  if (_shipsFileNames.find(exFileName) == _shipsFileNames.end()) {
    _shipsFileNames.append(exFileName);
  }
}
void BScenarioHeader::removeShipsFileName(const QString& fileName) {
  QString exFileName = bApp->extendFileName(fileName);
  QStringList::iterator it = _shipsFileNames.find(exFileName);
  if (it != _shipsFileNames.end()) {
    _shipsFileNames.remove(it);
  }
}

QString BScenarioHeader::aiFileName() const {
  return bApp->extendFileName(_aiFileName);
}

void BScenarioHeader::setAIFileName(const QString& fileName) {
  _aiFileName = fileName;
}

QString BScenarioHeader::description() const {
  return _description;
}

void BScenarioHeader::setDescription(const QString& d) {
  _description = d;
}

void BScenarioHeader::setSideName(int side, const QString& name) {
  BSideHeader* sh = const_cast<BSideHeader*>(sideHeader(side));
  sh->setName(name);
}
