/******************************************************************************/
/* BScenarioHeader -   BattleFleet              */
/******************************************************************************/

#ifndef BScenarioHeader_H
#define BScenarioHeader_H
static const char bscenarioheader_h_id[]="@(#) $Id: bscenarioheader.h,v 1.16 2005/01/21 16:25:03 Oleksiy Balabay Exp $"; static const char* bscenarioheader_h_id_id=bscenarioheader_h_id;
/** \file bscenarioheader.h
  \brief  BScenarioHeader  BattleFleet

     BScenarioHeader  BattleFleet
*/

#include <qstring.h>
#include <qstringlist.h>
#include <qdom.h>
#include <qpixmap.h>
#include <qdatetime.h>
#ifdef Q_WS_WIN
#pragma warning(disable: 4786)
#endif

#include <set>

#include "bxmlconfig.h"

class BException;
class QFile;

class BSideHeader {
private:
  friend class BScenarioHeader;
//  typedef std::map<int, QPixmap*> BFlags;
  BSideHeader(const QDomElement& element);
  enum BCountries {
    UnknownCountry = 0,
    JapanCountry = 1,
    RussiaCountry = 2,
    CountCountries
  };
public:
  typedef std::pair<QString, QPixmap*> CountryFlag;
  typedef std::map<int, CountryFlag> CountryFlags;
public:
  BSideHeader();
  ~BSideHeader();
  QString name() const;
  QPixmap flag() const;
  static const CountryFlags& flags();
  void setName(const QString& name);
private:
  static void initFlags();
  void freeFlags();
  int stringToCountry() const;
private:
  QString _name;
  static CountryFlags _flags;
  static int _count;
};

class BScenarioHeader: public BXmlConfig {
public:
  BScenarioHeader(const QString& fileName);
  virtual ~BScenarioHeader();
  QString name() const;
  QString mapFileName() const;
  QString eventsFileName() const;
  QString aiFileName() const;
  QDateTime beginDate() const;
  QDateTime endDate() const;
  QStringList shipsFileNames() const;
  QString description() const;
  void setName(const QString& name);
  void setMapFileName(const QString& fileName);
  void setAIFileName(const QString& fileName);
  void setShipsOnMapFileName(const QString& fileName);
  void setEventsFileName(const QString& fileName);
  void setBeginDate(const QDateTime& beginDate);
  void setEndDate(const QDateTime& endDate);
  void setDescription(const QString& d);
  void addShipsFileName(const QString& fileName);
  void removeShipsFileName(const QString& fileName);
  void setSideName(int side, const QString& name);

  /// side == 1  side == 2
  const BSideHeader* sideHeader(int side) const;
  static const QString DEF_MAP_FILE_NAME;
  static const QString DEF_SHIPS_ON_MAP_FILE_NAME;
  static const QString DEF_EVENTS_FILE_NAME;
  static const QString DEF_AI_FILE_NAME;
protected:
  virtual QDomDocument toXml() const;
private:
  void parseDocument(const QDomDocument& doc);
  void parseBeginDate(const QDomElement& e);
  void parseEndDate(const QDomElement& e);
  void clearSideHeaders();
private:
  QString _name;
  QString _mapFileName;
  QString _eventsFileName;
  QString _aiFileName;
  QDateTime _beginDate;
  QDateTime _endDate;
  QString _description;
  BSideHeader* _sideHeader1;
  BSideHeader* _sideHeader2;
  QStringList _shipsFileNames;
};

class BScenarioHeaders : public std::set<BScenarioHeader*> {
public:
  BScenarioHeaders(const QString& dirName);
  ~BScenarioHeaders();
private:
  void parseDir();
  void deleteItems();
private:
  QString _dirName;
};

#endif // BScenarioHeader_H
