/******************************************************************************/
/* BScenarioSelection -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bscenarioselection_cpp_id[]="@(#) $Id: bscenarioselection.cpp,v 1.13 2005/04/09 16:14:26 Oleksiy Balabay Exp $"; static const char* bscenarioselection_cpp_id_id=bscenarioselection_cpp_id;

#include <qlayout.h>
#include <qtabwidget.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qbuttongroup.h>
#include <qwidgetstack.h>
#include <qsettings.h>
#include <qtextedit.h>
#include <qslider.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qtimer.h>

#include "bscenarioselection.h"
#include "bscenariotable.h"
#include "bapplication.h"
#include "bexception.h"
#include "bnewscenario.h"
#include "bdateedit.h"
#include "ballonhelp.h"

static const QString S_SCENARIO_SELECTION = "ScenarioSelection";
static const QString S_SCENARIO_SELECTION_MODE = "Mode";

const char* BALLON_HELP_MODES_TEXT = QT_TRANSLATE_NOOP("BScenarioSelection", 
  "You are welcome to <b>Battlefleet</b> world!<br><br>"
  "<b>Battlefleet</b> is the program for naval conflicts "
  "modelling<br><br>"
  "The program can be used in 3 different modes:<br>"
  "- Mode to create new battles "
  "or to modify initial conditions of existing ones. "
  "Select button <b>%1</b><br>"
  "- Mode to control the battle. Select button <b>%2</b><br>"
  "- Mode to review the battle as film on VCR. Select button <b>%3</b>");

/** \file bscenarioselection.cpp
  \brief  BScenarioSelection  BattleFleet (impl.)

     BScenarioSelection  BattleFleet
*/

/** \class BScenarioSelection
  \brief 


*/
/** \brief 
*/
BScenarioSelection::BScenarioSelection(QWidget* parent, BPages page)
:BScenarioSelectionBase(parent, "bscenarioselection", true, 0),
_initialized(false) {
  bApp->addWidgetHelp(this, 
    bApp->applicationDirPath()+"/"+"doc/ar01s02.html");
//  _buttonHelp->hide();
  initPlayPage(page);
  initEditPage(page);
  initGamePage(page);

  //     
  QSettings* s = bApp->settings();
  s->beginGroup(S_SCENARIO_SELECTION);
  BAppModes as = BAppModes(s->readNumEntry(S_SCENARIO_SELECTION_MODE, GameMode));
  s->endGroup();
  if (as == EditMode) {
    //  ""
    buttonGroup1->setButton(buttonGroup1->id(_buttonPlay));
    buttonGroup1->setButton(buttonGroup1->id(_buttonEdit));
  }
  else if (as == GameMode) {
    buttonGroup1->setButton(buttonGroup1->id(_buttonGame));
  }
  else if (as == PlayMode) {
    buttonGroup1->setButton(buttonGroup1->id(_buttonPlay));
  }
  else {
    buttonGroup1->setButton(buttonGroup1->id(_buttonGame));
  }
  _initialized = true;
  QTimer::singleShot(1000, this, SLOT(showBallonHelp()));
//  widgetStack1->raiseWidget(1);
}

void BScenarioSelection::showBallonHelp() {
  if (bApp->ballonHelp()->shouldBeShown(BallonHelp::BALLON_HELP_MODES)) {
    bApp->ballonHelp()->setShown(BallonHelp::BALLON_HELP_MODES);
    QWhatsThis::display(tr(BALLON_HELP_MODES_TEXT, "BALLON_HELP_MODES_TEXT").arg(
      _buttonEdit->text()).arg(
      _buttonGame->text()).arg(
      _buttonPlay->text()), QPoint(x()+ width()/2, y()+ height()/3));
  }
}

void BScenarioSelection::initPlayPage(BPages /* page */) {
  _playOpenTable = 
    new BScenarioTable(bApp->scenarioSaversDir(), _playOpenPage, "openscenarios");
   QLayout* layout = new QVBoxLayout(_playOpenPage);
  layout->add(_playOpenTable);
  _playDateEdit = new BDateEdit(_playOpenPage);
  layout->add(_playDateEdit);
  _playOpenTable->show();
  connect(_playOpenTable, SIGNAL(accepted()), this, SLOT(accept()));
  connect(_playOpenTable, SIGNAL(currentChanged(int, int)), 
    this, SLOT(slotTableCurrentChanged(int, int)));
  connect(_playWidget, SIGNAL(currentChanged(QWidget*)), 
    this, SLOT(slotCurrentChanged(QWidget*)));
  if (_playOpenTable->numRows() == 0) {
    _playWidget->setTabEnabled(_playOpenPage, false);
  }
  else {
    _playOpenTable->selectRow(0);
  }
//  QToolTip::add(_playOpenPage, tr("List of all recorded scenarios"));
  QWhatsThis::add(_playOpenPage, tr("List of all recorded scenarios. "
    "You can review each of these scenarios as film in VCR, with control "
    "over speed and scale. Each scenario has the begin date and end date. "
    "It is possible to select starting date in this range using "
    "slider"));
}
/**   New -    .   Open - 
   BApplication::scenariosDir()
*/
void BScenarioSelection::initEditPage(BPages page) {
  _editNewTable = new BNewScenario(_editNewPage, "editnewscenarios");
  QLayout* layout = new QHBoxLayout(_editNewPage);
  layout->add(_editNewTable);
//  _editNewTable->selectRow(0);
  _editNewTable->show();
//  connect(_editNewTable, SIGNAL(accepted()), this, SLOT(accept()));
//  connect(_editNewTable, SIGNAL(currentChanged(int, int)), 
//    this, SLOT(slotTableCurrentChanged(int, int)));
  _editOpenTable = 
    new BScenarioTable(bApp->scenariosDir(), _editOpenPage, "openscenarios");
  layout = new QHBoxLayout(_editOpenPage);
  layout->add(_editOpenTable);
  _editOpenTable->selectRow(0);
  _editOpenTable->show();
  connect(_editOpenTable, SIGNAL(accepted()), this, SLOT(accept()));
  connect(_editOpenTable, SIGNAL(currentChanged(int, int)), 
    this, SLOT(slotTableCurrentChanged(int, int)));
  _editWidget->setCurrentPage(page);
  if (_editOpenTable->numRows() == 0) {
    _editWidget->setTabEnabled(_editOpenPage, false);
  }
  connect(_editWidget, SIGNAL(currentChanged(QWidget*)), 
    this, SLOT(slotCurrentChanged(QWidget*)));
//  QToolTip::add(_editNewPage, tr("Properties of new scenario"));
  QWhatsThis::add(_editNewPage, tr("Properties of new scenario. "
    "These properties should be set if you want to create new battle scenario"));
//  QToolTip::add(_editOpenPage, tr("List of all scenarios available to edit"));
  QWhatsThis::add(_editOpenPage, tr("List of all scenarios available to edit. "
    "You can modify the initial conditions (map size, list of ships, "
    "ships positions and grouping) for any of these scenarios using special "
    "editor"));
}

void BScenarioSelection::initGamePage(BPages page) {
  _gameNewTable = 
    new BScenarioTable(bApp->scenariosDir(), _gameNewPage, "gamenewscenarios");
  QLayout* layout = new QHBoxLayout(_gameNewPage);
  layout->add(_gameNewTable);
  connect(_gameNewTable, SIGNAL(accepted()), this, SLOT(accept()));
  connect(_gameNewTable, SIGNAL(currentChanged(int, int)), 
    this, SLOT(slotTableCurrentChanged(int, int)));
  _gameNewTable->selectRow(0);
  _gameNewTable->show();
  _gameOpenTable = 
    new BScenarioTable(bApp->scenarioSaversDir(), _gameOpenPage, "gameopenscenarios");
  layout = new QVBoxLayout(_gameOpenPage);
  layout->add(_gameOpenTable);
  _gameDateEdit = new BDateEdit(_gameOpenPage);
  layout->add(_gameDateEdit);
  _gameOpenTable->selectRow(0);
  _gameOpenTable->show();
  connect(_gameOpenTable, SIGNAL(accepted()), this, SLOT(accept()));
  connect(_gameOpenTable, SIGNAL(currentChanged(int, int)), 
    this, SLOT(slotTableCurrentChanged(int, int)));
  _gameWidget->setCurrentPage(page);
  if (_gameNewTable->numRows() == 0) {
    _gameWidget->setTabEnabled(_gameNewPage, false);
  }
  if (_gameOpenTable->numRows() == 0) {
    _gameWidget->setTabEnabled(_gameOpenPage, false);
  }
  connect(_gameWidget, SIGNAL(currentChanged(QWidget*)), 
    this, SLOT(slotCurrentChanged(QWidget*)));
//  QToolTip::add(_gameNewPage, tr("List of all scenarios available to play"));
  QWhatsThis::add(_gameNewPage, tr("List of all scenarios available to play. "
    "You can start new battle selecting any scenario in this list" ));
//  QToolTip::add(_gameOpenPage, tr("List of all saved battles"));
  QWhatsThis::add(_gameOpenPage, tr("List of all saved battles. "
    "You can continue to play each of these scenarios. "
    "Each scenario has the begin date and end date (when it was stopped). "
    "It is possible to select starting date in this range using "
    "slider"));
}

/** \brief 
*/
BScenarioSelection::~BScenarioSelection(){
  bApp->removeWidgetHelp(this);
  QSettings* s = bApp->settings();
  s->beginGroup(S_SCENARIO_SELECTION);
  s->writeEntry(S_SCENARIO_SELECTION_MODE, applicationMode());
  s->endGroup();
  bApp->removeWidgetHelp(this);
}

BAppModes BScenarioSelection::applicationMode() const {
  int btn = buttonGroup1->selectedId();
  if (btn == 0)
    return EditMode;
  else if (btn == 1)
    return GameMode;
  else if (btn == 2)
    return PlayMode;
  else 
    return UndefinedMode;
}


QString BScenarioSelection::scenarioFileName() const {
  if ((applicationMode() == EditMode)&&(_editWidget->currentPageIndex()==0)) {
    return _editNewTable->scenarioFileName();
  }
  else {
    BScenarioTable* t = currentTable();
    return currentTable()->currentFileName();
  }
}

void BScenarioSelection::buttonToggled(bool on) {
  if (on) {
    QPushButton* b = (QPushButton*)sender();
    if (b == _buttonEdit) {
      widgetStack1->raiseWidget(1);
      _editWidget->currentPage()->setFocus();
      _buttonOk->setEnabled(true);
    }
    else if (b == _buttonGame) {
      widgetStack1->raiseWidget(2);
      if ((_gameNewTable->numRows() == 0) && 
        (_gameOpenTable->numRows() == 0)) {
        _buttonOk->setEnabled(false);
      }
      else {
        _gameWidget->currentPage()->setFocus();
        _buttonOk->setEnabled(true);
      }
    }
    else if (b == _buttonPlay) {
      widgetStack1->raiseWidget(0);
      if ( /* (_newTable->numRows() == 0) && */ 
        (_playOpenTable->numRows() == 0)) {
        _buttonOk->setEnabled(false);
      }
      else {
        _buttonOk->setEnabled(true);
      }
    }
    updateDescription();
    updateDateEdit();
  }
}

BScenarioTable* BScenarioSelection::currentTable() const {
  if (applicationMode() == EditMode) {
    switch (_editWidget->currentPageIndex()) {
    case 0: 
      {
        return 0;//_editNewTable;
      }
    case 1:
      {
        return _editOpenTable;
      }
    default:
      {
        throw BInternalException(__FILE__, __LINE__,
          tr("Page does not exists: %1").arg(_editWidget->currentPageIndex()));
      }
    }
  }
  else if (applicationMode() == PlayMode) {
    return _playOpenTable;
  }
  else if (applicationMode() == GameMode) {
    switch (_gameWidget->currentPageIndex()) {
    case 0: 
      {
        return _gameNewTable;
      }
    case 1:
      {
        return _gameOpenTable;
      }
    default:
      {
        throw BInternalException(__FILE__, __LINE__,
          tr("Page does not exists: %1").arg(_gameWidget->currentPageIndex()));
      }
    }
  }
  else {
        throw BInternalException(__FILE__, __LINE__,
          tr("Unknown mode '%1'").arg(applicationMode()));
  }
  // keep compiler happy
  return 0;
}

void BScenarioSelection::slotTableCurrentChanged(int row, int col) {
  updateDescription();
  updateDateEdit();
}

void BScenarioSelection::slotCurrentChanged(QWidget* w) {
  updateDescription();
  updateDateEdit();
}

void BScenarioSelection::updateDescription() {
  if (!_initialized)
    return;
  if ((applicationMode() == EditMode)&&(_editWidget->currentPageIndex()==0)) {
    _descriptionText->setReadOnly(false);
    _descriptionText->selectAll(true);
  }
  else {
    _descriptionText->selectAll(false);
    _descriptionText->setReadOnly(true);
    BScenarioTable* t = currentTable();
    if (t)
      _descriptionText->setText(t->currentDescription());
  }
}

void BScenarioSelection::slotBeforeOk() {
  if ((applicationMode() == EditMode)&&(_editWidget->currentPageIndex()==0)) {
    if (_editNewTable->slotOk(_descriptionText->text())) {
      _exit = false;
      accept();
    }
  }
  else {
    _exit = false;
    accept();
  }
}

void BScenarioSelection::slotBeforeCancel() {
  _exit = false;
  reject();
}

void BScenarioSelection::slotBeforeExit() {
  _exit = true;
  reject();
}

void BScenarioSelection::updateDateEdit() {
  if (applicationMode() == PlayMode) {
    if (_playOpenTable->numRows() != 0) {
      QDateTime s = _playDateEdit->selectedDate();
      QDateTime b = _playOpenTable->currentBeginDate();
      QDateTime e = _playOpenTable->currentEndDate();
      if ((b>s)||(s>e))
        s=b;
      _playDateEdit->setInterval(b, e, s);
    }
  }
  else if ((applicationMode() == GameMode)&&(currentTable() == _gameOpenTable)) {
    if (_gameOpenTable->numRows() != 0) {
      QDateTime s = _gameDateEdit->selectedDate();
      QDateTime b = _gameOpenTable->currentBeginDate();
      QDateTime e = _gameOpenTable->currentEndDate();
      if ((b>s)||(s>e))
        s=b;
      _gameDateEdit->setInterval(b, e, s);
    }
  }
}

QDateTime BScenarioSelection::scenarioStartDate() const {
  if (applicationMode() == PlayMode) {
    return _playDateEdit->selectedDate();
  }
  else if ((applicationMode() == GameMode)&&(currentTable() == _gameOpenTable)) {
    return _gameDateEdit->selectedDate();
  }
  else if ((applicationMode() == EditMode)&&(_editWidget->currentPageIndex()==0)) {
    //  
    return _editNewTable->scenarioStartDate();
  }
  else {
    return currentTable()->currentBeginDate();
  }
}

void BScenarioSelection::slotHelp() {
  bApp->showHelp(this);
}
