/******************************************************************************/
/* BScenarioTable -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bscenariotable_cpp_id[]="@(#) $Id: bscenariotable.cpp,v 1.8 2004/10/10 10:18:21 Oleksiy Balabay Exp $"; static const char* bscenariotable_cpp_id_id=bscenariotable_cpp_id;

#include <qfileinfo.h>

#include "bscenariotable.h"
#include "bscenarioheader.h"
#include "bexception.h"

/** \file bscenariotable.cpp
  \brief  BScenarioTable  BattleFleet (impl.)

     BScenarioTable  BattleFleet
*/
const int ROW_DELTA = 8;
const char * EMPTY_SCENARIO = QT_TRANSLATE_NOOP("BEditScenarioTable", 
  "Empty scenario");
const char * UNKNOWN_SIDE1 = QT_TRANSLATE_NOOP("BEditScenarioTable", 
  "Unknown 1");
const char * UNKNOWN_SIDE2 = QT_TRANSLATE_NOOP("BEditScenarioTable", 
  "Unknown 2");
//const QString BScenarioTableEdit::NEW_SCENARIO_FILE_NAME = " ";
/** \class BScenarioTable
  \brief 


*/
/** \brief 

    \a dirName -  ,     
*/
BScenarioTable::BScenarioTable(const QString& dirName, QWidget* parent, 
  const char* name) :QTable(parent, name) {
  _baseDir = dirName;
  try {
    setNumCols(CountColumns);
    hideColumn(DescriptionColumn);
    horizontalHeader()->setLabel(SideOneColumn, tr("Side 1", "header label"));
    horizontalHeader()->setLabel(SideTwoColumn, tr("Side 2", "header label"));
    horizontalHeader()->setLabel(NameColumn, tr("Battle", "header label"));
    horizontalHeader()->setLabel(FileNameColumn, tr("File", "header label"));
    horizontalHeader()->setLabel(BeginDateColumn, tr("Begin", "header label"));
    setSelectionMode(SingleRow);
    setFocusStyle(FollowStyle);
    connect(this, SIGNAL(doubleClicked(int, int, int, const QPoint&)),
      this, SLOT(slotDoubleClicked(int, int, int, const QPoint&)));
    connect(this, SIGNAL(currentChanged(int, int)),
      this, SLOT(slotCurrentChanged(int, int)));
    if (!dirName.isEmpty()) {
      BScenarioHeaders headers(dirName);
      setNumRows(headers.size());
      QFileInfo fi;
      //    horizontalHeader()->setStretchEnabled(true, NameColumn);
      typedef BScenarioHeaders::const_iterator CIT;
      int i = 0;
      for (CIT it = headers.begin(); it != headers.end(); it++, i++) {
        const BSideHeader* sideHeader1 = (*it)->sideHeader(1);
        const BSideHeader* sideHeader2 = (*it)->sideHeader(2);
        QTableItem* tableItem = new QTableItem(this, QTableItem::Never, 
          sideHeader1->name(), sideHeader1->flag());
        setItem(i, SideOneColumn, tableItem);
        tableItem = new QTableItem(this, QTableItem::Never, 
          sideHeader2->name(), sideHeader2->flag());
        setItem(i, SideTwoColumn, tableItem);
        tableItem = new QTableItem(this, QTableItem::Never, (*it)->name());
        setItem(i, NameColumn, tableItem);
        fi.setFile((*it)->fileName());
        setText(i, FileNameColumn, fi.fileName());
        tableItem = new QTableItem(this, QTableItem::Never, 
          (*it)->beginDate().toString(Qt::ISODate));
        setItem(i, BeginDateColumn, tableItem);
        tableItem = new QTableItem(this, QTableItem::Never, 
          (*it)->endDate().toString(Qt::ISODate));
        setItem(i, EndDateColumn, tableItem);
        tableItem = new QTableItem(this, QTableItem::Never, (*it)->description());
        setItem(i, DescriptionColumn, tableItem);
        setRowHeight(i, rowHeight(i)+ROW_DELTA);
      }
    }
    adjustColumn(SideOneColumn);
    adjustColumn(SideTwoColumn);
    adjustColumn(NameColumn);
//    hideColumn(FileNameColumn);      
    hideColumn(BeginDateColumn);
    hideColumn(EndDateColumn);
    verticalHeader()->setResizeEnabled(false);
    setLeftMargin(0);
    sortColumn(FileNameColumn, true, true);
    setColumnStretchable(FileNameColumn, true);
  }
  catch (BException& e) {
    qWarning((const char*)e.message().local8Bit());
    return;
  }
  catch (...) {
    qWarning("Error loading scenarios from dir %s", 
      (const char*)dirName.local8Bit());
    return;
  }
}

/** \brief 
*/
BScenarioTable::~BScenarioTable(){

}

QString BScenarioTable::currentFileName() const {
  Q_ASSERT(currentRow() < numRows());
  Q_ASSERT(currentRow() >= 0);
  QString fn = text(currentRow(), FileNameColumn);
//  if (fn == BScenarioTableEdit::NEW_SCENARIO_FILE_NAME) {
//    return fn;
//  }
  return _baseDir+"/"+fn; 
}

QString BScenarioTable::currentDescription() const {
  return text(currentRow(), DescriptionColumn);
}

void BScenarioTable::slotDoubleClicked(int row, int column, int /* button */, 
  const QPoint& /* mousePos */) {
  if ((row != -1) && (column != -1))
    emit accepted();
}

void BScenarioTable::slotCurrentChanged(int /* row */, int /* column */) {
  emit fileChanged(currentFileName());
}

void BScenarioTable::selectScenario(const QString& fileName) {
  QFileInfo fi(fileName);
  QString n = fi.fileName();
  for (int i = 0; i < numRows(); i++) {
    if (item(i, FileNameColumn)->text() == n) {
      setCurrentCell(i, FileNameColumn);
      break;
    }
  }
}

QDateTime BScenarioTable::currentBeginDate() const {
  return QDateTime::fromString(text(currentRow(), BeginDateColumn), Qt::ISODate);
}

QDateTime BScenarioTable::currentEndDate() const {
  return QDateTime::fromString(text(currentRow(), EndDateColumn), Qt::ISODate);
}

/*BScenarioTableEdit::BScenarioTableEdit(QWidget* parent, 
  const char* name) :BScenarioTable("", parent, name) {
  setNumRows(numRows()+1);
  int i = numRows()-1;
  BSideHeader sideHeader;
  QTableItem* tableItem = new QTableItem(this, QTableItem::Never, 
    UNKNOWN_SIDE1, sideHeader.flag());
  setItem(i, SideOneColumn, tableItem);
  tableItem = new QTableItem(this, QTableItem::Never, 
    UNKNOWN_SIDE2, sideHeader.flag());
  setItem(i, SideTwoColumn, tableItem);
  tableItem = new QTableItem(this, QTableItem::Never, EMPTY_SCENARIO);
  setItem(i, NameColumn, tableItem);
  setText(i, FileNameColumn, NEW_SCENARIO_FILE_NAME);
  tableItem = new QTableItem(this, QTableItem::Never, 
    QDateTime::currentDateTime().toString());
  setItem(i, BeginDateColumn, tableItem);
  setRowHeight(i, rowHeight(i)+ROW_DELTA);
  adjustColumn(SideOneColumn);
  adjustColumn(SideTwoColumn);
  adjustColumn(NameColumn);
  hideColumn(FileNameColumn);      
  adjustColumn(BeginDateColumn);
}

BScenarioTableEdit::~BScenarioTableEdit() {
}

*/
