/******************************************************************************/
/* BSeaMap -   BattleFleet              */
/******************************************************************************/

#ifndef BSeaMap_H
#define BSeaMap_H
static const char bseamap_h_id[]="@(#) $Id: bseamap.h,v 1.14 2005/02/15 16:43:35 Oleksiy Balabay Exp $"; static const char* bseamap_h_id_id=bseamap_h_id;
/** \file bseamap.h
  \brief  BSeaMap  BattleFleet

     BSeaMap  BattleFleet
*/
#include <qcanvas.h>
#include <qwmatrix.h>

#include <list>
#include <memory>

//#include "bexception.h"
#include "bxmlconfig.h"

class QFile;
class BSeaMap;
class BException;
class BInternalException;

class BCoordinates {
public:
  BCoordinates(double x0, double y0, double w, double h);
//  BCoordinates(double x0, double y0, double x1, double y1, bool /* unused */)
//    :_x0(x0), _y0(y0), _w(x1 - x0), _h(y1 - y0) {
//    // Temporary Implementation
//  }

  ~BCoordinates() {
  }
  double x0() const {
    return _x0;
  }
  double y0() const {
    return _y0;
  }
  double w() const {
    return _w;
  }
  double h() const {
    return _h;
  }
  bool operator==(const BCoordinates& c) {
    return ( (x0() == c.x0()) && 
      (y0() == c.y0()) && 
      (w() == c.w()) && 
      (h() == c.h()) );
  }
  bool operator!=(const BCoordinates& c) {
    return !(*this == c);
  }
private:
  double _x0, _y0, _w, _h;
};

class BSeaMap;
typedef std::pair<double, double> BMapPoint;
typedef std::list<BMapPoint> BMapPoints;


class BCoordTransformer {
  friend class BSeaMap;
  BCoordTransformer(const BCoordinates& origCo,
    const BCoordinates& mapCo,
    const BCoordinates& canvasCo,
    const QSize& canvasSize);
public:
  const BCoordinates originalCoordinates() const;
  const BCoordinates currentCoordinates() const;
  const BCoordinates currentCanvasCoordinates() const;
  QPoint pointFromMap(const BMapPoint& point) const;
  BMapPoint pointToMap(const QPoint& point) const;
  int sizeFromMap(const double miles) const;
  double sizeToMap(const int pixels) const;
  BCoordinates adjustMapCoordinates(const BCoordinates& mapCo,
    const BCoordinates& canvasCo) const;
  QPointArray shape(const BMapPoints& mapPoints) const;
  QSize canvasSize() const;
private:
  double minRegion(const BCoordinates& canvasCo) const;
private:
  BCoordinates _co;
  ///    
  BCoordinates _mapCo;
  ///  ,    
  BCoordinates _canvasCo;
  QWMatrix _originalMatrix;
  QWMatrix _currentMatrix;
  QSize _canvasSize;
};

class BMapItem {
public:
  BMapItem(const QDomElement& e, const BSeaMap* seaMap);
  virtual ~BMapItem();
//  virtual QCanvasItem* createCanvasItem(QCanvas* canvas) = 0;
  static BMapPoints mapPointsFromXml(const QDomElement& e);
  virtual QString name() const;
  virtual void drawBackground(const BCoordTransformer& coordTransformer,
    QPainter& p) = 0;
protected:
  virtual void setName(const QString& name);
  const BSeaMap* seaMap() const;
private:
  QString _name;
  const BSeaMap* _seaMap;
};

class BMapIsland : public BMapItem {
private:
  enum BIslandType {
    PolygonType,
    SplineType
  };
public:
  BMapIsland(const QDomElement& e, const BSeaMap* seaMap);
  virtual ~BMapIsland();
//  virtual QCanvasItem* createCanvasItem(QCanvas* canvas);
  virtual void drawBackground(const BCoordTransformer& coordTransformer,
    QPainter& p);
private:
  static BIslandType islandTypeFromString(const QString& islandType);
private:
  BMapPoints _points;
  BIslandType _type;
  QColor _color;
};

class BSeaMap : public BXmlConfig {
private:
  typedef std::list<BMapItem*> BMapItems;
  static const double MIN_REGION;
public:
  typedef BMapItems::iterator iterator;
  typedef BMapItems::const_iterator const_iterator;
  enum BCoordinatesType {
    longitude,
    latitude,
  };
  BSeaMap(const QString& fileName);
  ~BSeaMap();
//  void setRegion(const BCoordinates& mapCo, 
//    const BCoordinates& canvasCo, const QSize& canvasSize);
  static double degreesFromString(const QString& degrees, 
    const BCoordinatesType type);
  static QString degreesToString(const double degrees, 
    const BCoordinatesType type, const bool hrf = false);
  static int degreesToSeconds(const double degrees);
  static double degreesFromSeconds(const int seconds);
  static int secondsPart(const double degrees);
  static int minutesPart(const double degrees);
  static int degreesPart(const double degrees);
  static double degrees(const int degrees, const int minutes, 
    const int seconds);
  iterator begin();
  iterator end();
  const_iterator begin() const;
  const_iterator end() const;
  QPixmap background(const BCoordTransformer& coordTransformer) const;
  BCoordTransformer* createTransformer(const BCoordinates& mapCo, 
    const BCoordinates& canvasCo, const QSize& canvasSize);
  const BCoordinates originalCoordinates() const;
  void setOriginalCoordinates(const BCoordinates& original);
  double mileage(const double x1, const double y1, 
    const double x2, const double y2); 
  double mileage2(const double x1, const double y1, 
    const double x2, const double y2); 
protected:
  virtual QDomDocument toXml() const;
private:
  void parseDocument(const QDomDocument& doc);
  void clearMap();
  void parseGeometry(const QDomElement& e);
private:
  BMapItems _mapItems;
  std::auto_ptr<BCoordTransformer> _coordTransformer;
};

#endif // BSeaMap_H
