/******************************************************************************/
/* BShip -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bship_cpp_id[]="@(#) $Id: bship.cpp,v 1.32 2005/03/07 15:34:39 Oleksiy Balabay Exp $"; static const char* bship_cpp_id_id=bship_cpp_id;

#include <qfile.h>
#include <qimage.h>
#include <qtextstream.h>

#include "bship.h"
#include "bexception.h"
#include "bdomutils.h"
#include "bapplication.h"
#include "bprogram.h"
#include "bprogramfactory.h"
#include "bunitsconsts.h"
#include "battle.h"
#include "bscenarioheader.h"
#include "beventgroup.h"
#include "bseamap.h"

/** \file bship.cpp
  \brief  BShip  BattleFleet (impl.)

     BShip  BattleFleet
*/

//const QString SHP_SHIPS = "Ships";
const QString BShip::SHP_SHIP = "Ship";
const QString BShip::SHP_SHIP_NAME = "Name";

const QString SHP_SHIP_SIZE = "Size";
const QString SHP_SHIP_WIDTH = "Width";
const QString SHP_SHIP_LENGTH = "Length";
const QString SHP_SHIP_TURN_RADIUS = "TurnRadius";
const QString SHP_SHIP_SPEED = "Speed";
const QString SHP_SHIP_AVAILABLE_TURN_RADIUS = "AvailableTurnRadius";
const QString SHP_SHIP_AVAILABLE_SPEED = "AvailableSpeed";
const QString SHP_SHIP_AVAILABLE_ACCELERATION = "AvailableAcceleration";

/// Width in meters 
const double SHP_DEFAULT_WIDTH = 20;
/// Length in meters
const double SHP_DEFAULT_LENGTH = 100;

/** \class BShip
  \brief 

        Composite.

         
  \a element,    parse().

    \a element :

  <\a SHP_SHIP \a SHP_SHIP_NAME=" " ... />

    \a element   .

  ,   \a SHP_SHIP_NAME,  
    .   ,    
       .    
       . ,  ""  2-  
    , "Orel"  "Orel(H)". 

      ,   
          
    .
*/

/** \brief 

       \a element    
       .

*/
BShip::BShip(BattleObject* parent):BattleObject(parent){
/*  if (!debugFile) {
    debugFile = new QFile("C:\\bfdbg.txt");
    if (!debugFile->open(IO_WriteOnly)) {
      throw BInternalException(__FILE__, __LINE__, "debugFile");
    }
    debugStream = new QTextStream(debugFile);
    debugStream->setEncoding(QTextStream::Locale);
  }
*/
}

/** \brief 
*/
BShip::~BShip(){
  clear();
//  debugFile->flush();
}

void BShip::clear() {
  if (_program)
    delete _program;
}

void BShip::parse(const QDomElement& element) {
  if (element.tagName() != SHP_SHIP) {
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate("BShip",
      "Element '%1' expected, '%2' found").arg(
      SHP_SHIP).arg(element.tagName()));
  }
  parseAttributes(element);
  parseElements(element);
}

void BShip::parseAttributes(const QDomElement& element) {
  //  
  _name = BDomUtils::stringAttribute(element, SHP_SHIP_NAME);
  //  
  _vesselType = vesselTypeFromString(element.attribute(VT_VESSEL_TYPE));
  //  (  BattleFleet).   ,  - 
  //    
  if (element.hasAttribute(SHP_SHIP_SIZE)) {
    _size = BDomUtils::intAttribute(element, SHP_SHIP_SIZE);
  }
  else {
    _size = defaultSize(_vesselType);
  }
  // TODO:       
  // .   ,   
  if (element.hasAttribute(SHP_SHIP_WIDTH)) {
    _width = BDomUtils::doubleAttribute(element, SHP_SHIP_WIDTH);
  }
  else {
    _width = SHP_DEFAULT_WIDTH;
  }
  _width = BUnitsConverter::mToMiles(_width);

  // .   ,   
  if (element.hasAttribute(SHP_SHIP_LENGTH)) {
    _length = BDomUtils::doubleAttribute(element, SHP_SHIP_LENGTH);
  }
  else {
    _length = SHP_DEFAULT_LENGTH;
  }
  _length = BUnitsConverter::mToMiles(_length);

  //  
  if (element.hasAttribute(SHP_SHIP_TURN_RADIUS)) {
    _minTurnRadius = BDomUtils::doubleAttribute(element, SHP_SHIP_TURN_RADIUS);
  }
  else {
    _minTurnRadius = defaultTurnRadius(_size);
  }
//  _minTurnRadius = BUnitsConverter::mToMiles(_minTurnRadius);
  //  
  _maxSpeed = BDomUtils::doubleAttribute(element, SHP_SHIP_SPEED);
  //  :
  //      3     1/3 
  //        3   2/3 
  if (DestroyerType == _vesselType) {
    _maxAcceleration = (_maxSpeed*2.0/3.0)/(3.0/60.0);
  }
  else {
    _maxAcceleration = (_maxSpeed*1.0/3.0)/(3.0/60.0);
  }
}

void BShip::parseElements(const QDomElement& /* element */) {
  return;
}

BKinematicData BShip::kinematicData(const QDateTime& t) const {
  if (_program == 0) {
    return BKinematicData(0, 0, 0, 0, 0, 0, 0);
  }
  return _program->kinematicData(t);
}

BProgramData BShip::programData(const QDateTime& t) const {
  if (_program == 0) {
    return BProgramData(kinematicData(t), availableTurnRadius(t), 
      availableSpeed(t), availableAcceleration(t), lossOfSpeedCoef(size()));
  }
  return BProgramData(kinematicData(t), availableTurnRadius(t),
    availableSpeed(t), availableAcceleration(t), lossOfSpeedCoef(size()));
}

void BShip::setProgram(BProgram* program, const QDateTime& t)  
  {
//  writeToDebugFile("t="+t.toString("yyyyMMdd hh:mm:ss:zzz"));
  if (_program) {
    if (_program->t0() > t) {
      throw BProgramTimeException(__FILE__, __LINE__,
        program, _program->t0(), t, this);
    }
  }
  setProgramInternal(program, programData(t), t);
}

void BShip::setProgramInternal(BProgram* program, 
  const BProgramData& pd, const QDateTime& t) {
//  BProgramData pd = programData(t);
/*  QString debugPd;
  debugPd = debugPd + "pd.aa= "+ QString::number(pd.availableAcceleration())+"\n";
  debugPd = debugPd + "pd.as= "+ QString::number(pd.availableSpeed())+"\n";
  debugPd = debugPd + "pd.at= "+ pd.availableTurnRadius()+"\n";
  debugPd = debugPd + "pd.kd.x= "+ pd.kinematicData().x()+"\n";
  debugPd = debugPd + "pd.kd.y= "+ pd.kinematicData().y()+"\n";
  debugPd = debugPd + "pd.kd.course= "+ pd.kinematicData().course()+"\n";
  debugPd = debugPd + "pd.kd.v= "+ pd.kinematicData().v(true)+"\n";
  debugPd = debugPd + "pd.kd.v1= "+ pd.kinematicData().v(false)+"\n";
  debugPd = debugPd + "pd.kd.r= "+ pd.kinematicData().r()+"\n";
  writeToDebugFile(debugPd);*/
//  qDebug("new prg time: %s, cur time: %s", 
//    (const char*)t.toString().local8Bit(),
//    (const char*)bApp->battle()->battleInfo().modelTime.toString().local8Bit());
  program->initProgram(pd, t);
  if (_program) {
    delete _program;
  }
  _program = program;
/*  BKinematicData debugKd = kinematicData(_program->t0());
  QString debugStr = QString("x=%1\ny=%2\nv=%3\nv1=%4\nc=%5\nr=%6\tn0=%7").arg(
    debugKd.x(), 0, 'e', 12).arg(
    debugKd.y(), 0, 'e', 12).arg(
    debugKd.v(true), 0, 'e', 12).arg(
    debugKd.v(false), 0, 'e', 12).arg(
    debugKd.course(), 0, 'e', 12).arg(
    debugKd.r(), 0, 'e', 12).arg(
    _program->t0().toString("yyyyMMdd hh:mm:ss.zzz"));
  writeToDebugFile(debugStr); */
//  QDomDocument doc;
//  qDebug("%s - %s", (const char*)name().local8Bit(),
//    (const char*)BDomUtils::elementToString(_program->toXml(doc)).local8Bit());
  insertTrack(t,kinematicData(t));
}

QImage BShip::image() const {
  switch (_vesselType) {
  case BattleshipType:
  /*    png    
       
  */
  case CoastDefenseBattleshipType:
    {
      return QImage::fromMimeSource(
        (side()==1)?"battleship1.bmp":
        (side()==2)?"battleship2.bmp":"battleship0.bmp");
    }
  case ArmoredCruiserType:
    {
      return QImage::fromMimeSource(
        (side()==1)?"armoredcruiser1.bmp":
        (side()==2)?"armoredcruiser2.bmp":"armoredcruiser0.bmp");
    }
  case ProtectedCruiserType:
  case UnprotectedCruiserType:
    {
      return QImage::fromMimeSource(
        (side()==1)?"cruiser1.bmp":
        (side()==2)?"cruiser2.bmp":"cruiser0.bmp");
    }
  case DestroyerType:
    {
      return QImage::fromMimeSource(
        (side()==1)?"destroyer1.bmp":
        (side()==2)?"destroyer2.bmp":"destroyer0.bmp");
    }
  case TorpedoBoatType:
    {
      return QImage::fromMimeSource(
        (side()==1)?"torpedoboat1.bmp":
        (side()==2)?"torpedoboat2.bmp":"torpedoboat0.bmp");
    }
  default:
    {
      return QImage::fromMimeSource(
        (side()==1)?"ship1.bmp":
        (side()==2)?"ship2.bmp":"ship0.bmp");
    }
  }
}

QDomElement BShip::toXml(QDomDocument& doc, bool /* elementOnly */) const {
  QDomElement e = doc.createElement(SHP_SHIP);
  e.setAttribute(SHP_SHIP_NAME, name());
  return e;
}

QDomElement BShip::state(QDomDocument& doc) const {
  QDomElement e = BattleObject::state(doc);
  if (!_program) {
    throw BInternalException(__FILE__, __LINE__,
      "Call to state() without program");
  }
  QDomElement p = _program->toXml(doc);
//  qDebug("Program: %s", (const char*)BDomUtils::elementToString(p));
  BKinematicData kd = kinematicData(_program->t0());
  p.setAttribute(BProgram::PROGRAM_X, 
    BSeaMap::degreesToString(kd.x(), BSeaMap::longitude));
  p.setAttribute(BProgram::PROGRAM_Y, 
    BSeaMap::degreesToString(kd.y(), BSeaMap::latitude));
  p.setAttribute(BProgram::PROGRAM_COURSE, 
    BUnitsConverter::angleToDegrees(kd.course()));
  p.setAttribute(BProgram::PROGRAM_SPEED, kd.v(false));
  e.appendChild(p);
  e.appendChild(BDomUtils::dateTimeToXml(_program->t0(), 
    BProgram::PROGRAM_T0, doc));
  if (side() == 1 || side() == 2) {
    e.setAttribute(BEventGroupCreate::SOM_SIDE, 
      bApp->battle()->scenarioHeader()->sideHeader(side())->name());
  }
  return e;
}

void BShip::setState(QDomElement& element) {
  bool programFound = false;
  bool programT0Found = false;
  _objectTracks.clear();
  QDomElement programElement;
  QDomElement programT0Element;
  QDomNode n = element.firstChild();
  while( !n.isNull() ) {
    QDomElement e = n.toElement(); 
    n = n.nextSibling();
    if( !e.isNull() ) { 
//      try {
        if (e.tagName() == BProgram::PROGRAM) {
          programFound = true;
          programElement = e;
        }
        else if (e.tagName() == BProgram::PROGRAM_T0) {
          programT0Found = true;
          programT0Element = e;
        }
        else {
          qWarning("Unsupported element '%s' in '%s'",
            (const char*)e.tagName().local8Bit(),
            (const char*)BDomUtils::elementToString(e).local8Bit());
        }
      }
//      catch (BConfigException& ex) {
//        throw BConfigException(ex.sourceCodeFile(),
//          ex.sourceCodeLine(), ex.message()+"\n"+qApp->translate("BShip",
//          "Error in element %1").arg(BDomUtils::elementToString(e)));
//      }
//      catch (BException&) {
//        throw;
//      }
//      catch (...) {
//        throw BInternalException(__FILE__, __LINE__,
//          qApp->translate("BShip",
//          "Unknown error parsing element %1").arg(
//          BDomUtils::elementToString(e)));
//      }
//    }
  }
  if (!programFound) {
    BDomUtils::elementDoesNotExist(BProgram::PROGRAM);
  }
  if (!programT0Found) {
    BDomUtils::elementDoesNotExist(BProgram::PROGRAM_T0);
  }
  QDateTime t0 = BDomUtils::dateTimeFromXml(programT0Element);
  BProgram* p = BProgramFactory::create(programElement,
    bApp->battle());
  p->setProgramFlags(BEvent::PlayerEventFlag);
//  setProgram(p, t0);
  //    
  if (_program) {
    setProgramInternal(p, programData(_program->t0()), t0);
  }
  else {
    BProgramData pd = programData(t0);
    setProgramInternal(p, pd, t0);
  }
  if (element.hasAttribute(BEventGroupCreate::SOM_SIDE)) {
    QString s1 = 
      bApp->battle()->scenarioHeader()->sideHeader(1)->name().upper();
    QString s2 = 
      bApp->battle()->scenarioHeader()->sideHeader(2)->name().upper();
    QString s = element.attribute(BEventGroupCreate::SOM_SIDE).upper();
    if (s1 == s) {
      setSide(1);
    }
    else if (s2 == s) {
      setSide(2);
    }
    else {
      throw BConfigException(__FILE__, __LINE__,
        qApp->translate("Ship",
        "Bad side: '%1'. Should be '%2' or '%3'").arg(s).arg(s1).arg(s2));
    }
  }
  else {
    setSide(NO_SIDE);
  }
}

//QFile* BShip::debugFile = 0;
//QTextStream* BShip::debugStream = 0;

//void BShip::writeToDebugFile(const QString& s) {
//  *debugStream << s <<"\n"; 
//}

void BShip::insertTrack(const QDateTime& t,const BKinematicData& kD) {
  static int idOriginator = 0;
  while (_objectTracks.size() >= OBJECT_TRACKS_MAX) {
    _objectTracks.pop_back();
  };
  BObjectTrackPoint tp;
  tp.kd = kD;
  QDomDocument doc;
  tp.program = _program->toXml(doc);
  tp.id = idOriginator++;
//  qDebug("%d - %d", tp.id, idOriginator);
  _objectTracks.push_front(BObjectTrack(t, tp));
  // debug order test
  QDateTime debug = _objectTracks.front().first;
  for (BObjectTracks::const_iterator it = _objectTracks.begin();
  it != _objectTracks.end(); it++) {
    if (debug < (*it).first) {
      throw BInternalException(__FILE__, __LINE__, QString("%1 < %2").arg(
        debug.toString()).arg((*it).first.toString()));
    }
  }
}
