/******************************************************************************/
/* BShip -   BattleFleet              */
/******************************************************************************/

#ifndef BShip_H
#define BShip_H
static const char bship_h_id[]="@(#) $Id: bship.h,v 1.21 2005/01/28 17:06:08 Oleksiy Balabay Exp $"; static const char* bship_h_id_id=bship_h_id;
/** \file bship.h
  \brief  BShip  BattleFleet

     BShip  BattleFleet
*/
#include <qimage.h>

#include "battleobject.h"

class QFile;
class QTextStream;
class BProgram;
class BException;

class BShip: public BattleObject {
private:
  static BVesselType vesselTypeFromString(const QString& vesselType);
  static QString vesselTypeToString(const BVesselType vesselType);
  static const QString VT_VESSEL_TYPE;
public:
  static QString vesselTypeToStringHRF(const BVesselType vesselType);
public:
  enum BStability {
    NoneStability = 0,
    PoorStability = 1,
    GoodStability = 2,
    GreatStability = 3,
  };
private:
  static BStability stabilityFromString(const QString& stability);
  static QString stabilityToString(const BStability stability);

public:
  enum BSeawayRating {
    NoneSeawayRating = 0,
    PoorSeawayRating = 1,
    GoodSeawayRating = 2,
    GreatSeawayRating = 3,
  };
private:
  static BSeawayRating seawayRatingFromString(const QString& seawayRating);
  static QString seawayRatingToString(const BSeawayRating seawayRating);

public:
  enum BFloodDefense {
    NoneFloodDefense = 0,
    PoorFloodDefense = 1,
    GoodFloodDefense = 2,
    GreatFloodDefense = 3,
  };
private:
  static BFloodDefense floodDefenseFromString(const QString& floodDefense);
  static QString floodDefenseToString(const BFloodDefense floodDefense);

public:
  enum BArmPositions {
    AArmPosition,
    BArmPosition,
    CArmPosition,
    DArmPosition,
    EArmPosition,
    MArmPosition,
    OArmPosition,
    PArmPosition,
    RArmPosition,
    SArmPosition,
    UArmPosition,
    VArmPosition,
    WArmPosition,
    XArmPosition,
    YArmPosition,
    ZArmPosition,
  };
private:
  static BArmPositions armPositionFromString(const QString& armPosition);
  static QString armPositionToString(const BArmPositions armPosition);

public:
  enum BTorpedoPositions {
    SternTorpedoPosition,
    StemTorpedoPosition,
  };
private:
  static BTorpedoPositions 
    torpedoPositionFromString(const QString& torpedoPosition);
  static QString 
    torpedoPositionToString(const BTorpedoPositions torpedoPosition);

public:
  enum BArmPatterns {
    ArmPattern0,
    ArmPattern1,
    ArmPattern2,
  };
private:
  static BArmPatterns armPatternFromString(const QString& armPattern);
  static QString armPatternToString(const BArmPatterns armPattern);

public:
  BShip(BattleObject* parent);
  virtual ~BShip();
  virtual BKinematicData kinematicData(const QDateTime& t) const;
  virtual BProgramData programData(const QDateTime& t) const;
  virtual void setProgram(BProgram* program, const QDateTime& t);
  virtual QImage image() const;
  virtual void parse(const QDomElement& element);
  virtual QDomElement toXml(QDomDocument& doc, bool elementOnly = false) const;
  virtual QDomElement state(QDomDocument& doc) const;
  virtual void setState(QDomElement& element);
  static const QString SHP_SHIP;
  static const QString SHP_SHIP_NAME;
private:
  void parseAttributes(const QDomElement& element);
  void parseElements(const QDomElement& element);
  static int defaultSize(const BVesselType type);
  static double defaultTurnRadius(const int size);
  void clear();
  void setProgramInternal(BProgram* program, const BProgramData& pd, const QDateTime& t);
  void insertTrack(const QDateTime& t,const BKinematicData& kD);
public:
//  static QFile* debugFile;
//  static QTextStream* debugStream;
//  static void writeToDebugFile(const QString& s);
};



#endif // BShip_H
