/******************************************************************************/
/* BShipGroup -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bshipgroup_cpp_id[]="@(#) $Id: bshipgroup.cpp,v 1.16 2005/01/24 03:50:43 Oleksiy Balabay Exp $"; static const char* bshipgroup_cpp_id_id=bshipgroup_cpp_id;

#include <qdom.h>
#include <qobjectlist.h>
#include <algorithm>

#include "bshipgroup.h"
#include "bship.h"
#include "bshipgroupstate.h"
#include "bshipgroupstated.h"
#include "bprogramturninsuccession.h"
#include "bprogramturnsimultaneous.h"
#include "bexception.h"
#include "bdomutils.h"
#include "bapplication.h"
#include "battle.h"
#include "borderalgorithms.h"

const QString BShipGroup::SHP_SHIP_GROUP = "Ships";
const QString BShipGroup::SHP_SHIP_GROUP_NAME = "Name";

/** \file bshipgroup.cpp
  \brief  BShipGroup  BattleFleet (impl.)

     BShipGroup  BattleFleet
*/

/** \class BShipGroup
  \brief    Composite.

    ,     ,
       flagShip().

     BShipGroupState   State. 
     setProgram().

         
   parse().     :

  <BR><\a SHP_SHIP_GROUP \a SHP_SHIP_GROUP_NAME=" ">
  <BR>&nbsp;&nbsp;<\a SHP_SHIP_GROUP \a SHP_SHIP_GROUP_NAME=" ">
  <BR>&nbsp;&nbsp;&nbsp;&nbsp;...
  <BR>&nbsp;&nbsp;</ \a SHP_SHIP_GROUP>
  <BR>&nbsp;&nbsp;...
  <BR>&nbsp;&nbsp;<\a BShip::SHP_SHIP \a BShip::SHP_SHIP_NAME="" .../>
  <BR>&nbsp;&nbsp;...
  <BR></\a SHP_SHIP_GROUP>

        .  
     BShip.

*/
/** \brief 

    .       
  insertChild(),      - parse() 
*/
BShipGroup::BShipGroup(BattleObject* parent, const QString name )
  :BattleObject(parent){
  _name = name;
  _state = new BShipGroupStateG(this);
  setFlagShip(_state->selectFlagShip());
}

/** \brief 
*/
BShipGroup::~BShipGroup(){
  if (_state) {
    delete _state;
  }
  for (BattleObjects::iterator it = _children.begin(); 
    it != _children.end(); it++) {
//    BattleObject* debug = *it;
    delete (*it);
  }
  delete _program;
}

BKinematicData BShipGroup::kinematicData(const QDateTime& t) const {
  return _state->kinematicData(t);
}

BProgramData BShipGroup::programData(const QDateTime& t) const {
  return _state->programData(t);
}

void BShipGroup::setProgram(BProgram* program, const QDateTime& t)
  {
  BProgramTurnInSuccession* p = dynamic_cast<BProgramTurnInSuccession*>(program);
  if (p != 0) {
    BShipGroupStateD* gs = dynamic_cast<BShipGroupStateD*>(_state);
    if (gs == 0) {
      //  BShipGroupStateD   _state
      BShipGroupStateD* sgs = new BShipGroupStateD(this, t);
      delete _state;
      _state = sgs;
    }
  }
  else {
    BProgramTurnSimultaneous* p = dynamic_cast<BProgramTurnSimultaneous*>(program);
    if (p != 0) {
      BShipGroupStateG* gs = dynamic_cast<BShipGroupStateG*>(_state);
      if (gs == 0) {
        delete _state;
        _state = new BShipGroupStateG(this);
      }
    }
  }
  _state->setProgram(program, t);
  //  ,  BEvent::toXml()  
  if (_program)
    delete _program;
  _program = program;
}

void BShipGroup::parse(const QDomElement& element) {
  if (element.tagName() != SHP_SHIP_GROUP) {
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate("BXmlConfig",
      "Element is not '%1' but '%2'").arg(
      SHP_SHIP_GROUP).arg(element.tagName()));
  }
  parseAttributes(element);
  QDomNode n = element.firstChild();
  while( !n.isNull() ) {
    QDomElement e = n.toElement(); // try to convert the node to an element.
    n = n.nextSibling();
    if( !e.isNull() ) { // the node was really an element.
      if (e.tagName() == BShip::SHP_SHIP) {
        BShip* ship = new BShip(this);
        ship->parse(e);
        insertChild(ship);
      }
      else if (e.tagName() == SHP_SHIP_GROUP) {
        BShipGroup* shipGroup = new BShipGroup(this);
        shipGroup->parse(e);
        insertChild(shipGroup);
      }
      else {
        qWarning("Unsupported tag '%s' in '%s'", 
          (const char*)e.tagName().local8Bit(),
          (const char*)BDomUtils::elementToString(e));
      }
    }
  }
  setFlagShip(_state->selectFlagShip());
}

void BShipGroup::parseAttributes(const QDomElement& element) 
  {
  if (_name.isEmpty()) {
    _name = BDomUtils::stringAttribute(element, SHP_SHIP_GROUP_NAME);
  }
  else {
    if (element.hasAttribute(SHP_SHIP_GROUP_NAME)) {
      _name = BDomUtils::stringAttribute(element, SHP_SHIP_GROUP_NAME);
    }
  }
}

BattleObjects BShipGroup::objs(BSelectionTypes selType) const {
  if (selType == TreeSelectionType) {
    return _children;
  }
  else if (selType == LeafsOnlySelectionType) {
    BattleObjects bo;
    BattleObjects::const_iterator it = _children.begin();
    while ( it != _children.end()) {
      BattleObjects bos = (*it)->objs(LeafsOnlySelectionType);
      if (bos.size() > 0) {
        for(BattleObjects::iterator it1= bos.begin(); it1 != bos.end(); it1++) {
          bo.insert(*it1);
        }
      }
      else {
        if (!((*it)->isGroup()))
          bo.insert(*it);
      }
      ++it;
    }
    return bo;
  }
  else { // All
    BattleObjects bo;
    BattleObjects::const_iterator it = _children.begin();
    while ( it != _children.end()) {
      BattleObjects bos = (*it)->objs(AllSelectionType);
      for(BattleObjects::iterator it1= bos.begin(); it1 != bos.end(); it1++) {
        bo.insert(*it1);
      }
      ++it;
    }
    bo.insert(const_cast<BShipGroup*>(this));
    return bo;
  }
}

QImage BShipGroup::image() const {
  if (_flagShip != 0)
    return _flagShip->image();
  else
    return QImage();
}

void BShipGroup::setFlagShip(BattleObject* flagShip) {
  if (flagShip != 0) {
//    if (flagShip->parent() != this) {
//      throw BInternalException(__FILE__, __LINE__,
//        QString("Attempt to set flagship not from group: %1").arg(
//        flagShip->name()));
//    }
  }
  BattleObject::setFlagShip(flagShip);
  if (flagShip != 0) {
    _length = flagShip->length();
    _width = flagShip->width();
    _maxAcceleration = flagShip->maxAcceleration();
    _maxSpeed = flagShip->maxSpeed();
    _minTurnRadius = flagShip->minTurnRadius();
    _vesselType = flagShip->vesselType();
    _size = flagShip->size();
  }
  else {
    _length = 0;
    _width = 0;
    _maxAcceleration = 0;
    _maxSpeed = 0;
    _minTurnRadius = 0;
    _vesselType = UndefinedType;
    _size = 0;
  }
}

void BShipGroup::removeChild(BattleObject* obj) {
  _children.erase(obj);
  obj->setParent(0);
  if (obj == _flagShip) {
    setFlagShip(_state->selectFlagShip());
  }
}

/**
     ,     
*/
void BShipGroup::insertChild(BattleObject* obj) {
  if (obj->parent()) {
    obj->parent()->removeChild(obj);
  }
  _children.insert(obj);
  obj->setParent(this);
  if (NO_SIDE != side())
    obj->setSide(side());
  setFlagShip(_state->selectFlagShip());
}

void BShipGroup::setSide(const int side) {
  for(BattleObjects::iterator it=_children.begin(); it!=_children.end(); it++){
    (*it)->setSide(side);
  }
  BattleObject::setSide(side);
}

bool BShipGroup::isGroup() const {
  return true;
}

BattleObject* BShipGroup::find(const QString& objectName) const {
  if (objectName == name()) 
    return const_cast<BShipGroup*>(this);
  else {
    BattleObjects bo = objs();
    for (BattleObjects::iterator it = bo.begin(); it != bo.end(); it++) {
//      BattleObject* debug = (*it);
      BattleObject* o = (*it)->find(objectName);
      if (o != 0) {
        return o;
      }
    }
    return 0;
  }
}

QDomElement BShipGroup::toXml(QDomDocument& doc, bool elementOnly) const {
  QDomElement e = doc.createElement(SHP_SHIP_GROUP);
  e.setAttribute(SHP_SHIP_GROUP_NAME, name());
  if (!elementOnly) {
    BattleObjects bo = objs();
    for(BattleObjects::iterator it = bo.begin(); it != bo.end(); it++) {
      e.appendChild((*it)->toXml(doc));
    }
  }
  return e;
}

void BShipGroup::setParentControl(bool isOn) {
  BattleObject::setParentControl(isOn);
  if (isOn) {
    BShipGroupStateD* gs = dynamic_cast<BShipGroupStateD*>(_state);
    if (gs == 0) {
      //  BShipGroupStateD   _state
      BShipGroupStateD* sgs = new BShipGroupStateD(this, 
        bApp->battle()->battleInfo().modelTime);
      delete _state;
      _state = sgs;
    }
  }
  else {
    BShipGroupStateG* gs = dynamic_cast<BShipGroupStateG*>(_state);
    if (gs == 0) {
      delete _state;
      _state = new BShipGroupStateG(this);
    }
  }
}

double BShipGroup::availableSpeed(const QDateTime& t) const {
  double minSpeed = 0;
  if (_children.size() > 0) {
    minSpeed = (*_children.begin())->availableSpeed(t);
  }
  for(BattleObjects::const_iterator it = _children.begin(); 
  it != _children.end(); it++) {
    if (minSpeed > (*it)->availableSpeed(t)) {
      minSpeed = (*it)->availableSpeed(t);
    }
  }
  return minSpeed;
}

double BShipGroup::availableTurnRadius(const QDateTime& t) const {
  double minR = 0;
  if (_children.size() > 0) {
    minR = (*_children.begin())->availableTurnRadius(t);
  }
  for(BattleObjects::const_iterator it = _children.begin(); 
  it != _children.end(); it++) {
    if (minR < (*it)->availableTurnRadius(t)) {
      minR = (*it)->availableTurnRadius(t);
    }
  }
  return minR;
}

double BShipGroup::availableAcceleration(const QDateTime&  t ) const {
  double minA = 0;
  if (_children.size() > 0) {
    minA = (*_children.begin())->availableAcceleration(t);
  }
  for(BattleObjects::const_iterator it = _children.begin(); 
  it != _children.end(); it++) {
    if (minA > (*it)->availableAcceleration(t)) {
      minA = (*it)->availableAcceleration(t);
    }
  }
  return minA;
}

bool BShipGroup::isTurnInSuccessionEnabled(BattleObject* o, 
  const QDateTime& t0, double lk) {
  if (!o->isGroup()) {
    return false;
  }
  if (o->objs().size() == 0) {
    return false;
  }
  if ((o->flags() & IsLineAhead) == IsLineAhead) {
    return true;
  }
  BattleObjects bos = o->objs(LeafsOnlySelectionType);
  BShipsOrder so;
  BShipGroupStateD::orderObjectsInGroup(bos, so, t0, lk);
  bool lineA;
  BShipsOrderIsLineAhead ila((*so.begin()).first, t0, lk, lineA);
  std::for_each(bos.begin(), bos.end(), ila);
  return ila.isLineAhead();
}
