/******************************************************************************/
/* BShipGroupState -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bshipgroupstate_cpp_id[]="@(#) $Id: bshipgroupstate.cpp,v 1.13 2005/01/24 03:50:43 Oleksiy Balabay Exp $"; static const char* bshipgroupstate_cpp_id_id=bshipgroupstate_cpp_id;

#include <qapplication.h>

#include "bshipgroupstate.h"
#include "bshipgroup.h"
#include "bexception.h"
#include "bship.h"
#include "bprogram.h"
#include "bunitsconsts.h"
#include "bapplication.h"
#include "battle.h"
#include "bseamap.h"

/** \file bshipgroupstate.cpp
  \brief  BShipGroupState  BattleFleet (impl.)

     BShipGroupState  BattleFleet
*/

/** \class BShipGroupState
  \brief   State  BShipGroup.

     setProgram()
*/
/** \brief 
*/
BShipGroupState::BShipGroupState(BShipGroup* shipGroup)
  :_shipGroup(shipGroup) {

}

/** \brief 
*/
BShipGroupState::~BShipGroupState(){

}

/** \class BShipGroupStateG
 \brief     " "
*/
BShipGroupStateG::BShipGroupStateG(BShipGroup* shipGroup)
  :BShipGroupState(shipGroup) {
  _shipGroup->setFlags(0, BattleObject::IsLineAhead);
  BattleObjects bos = _shipGroup->objs();
  for (BattleObjects::iterator it = bos.begin(); it != bos.end(); it++) {
    (*it)->setParentControl(false);
  }
}

BShipGroupStateG::~BShipGroupStateG(){

}

BShip* BShipGroupStateG::selectFlagShip() {
  BattleObjects bo = _shipGroup->objs(BattleObject::LeafsOnlySelectionType);
  if (bo.size() == 0) {
    return 0;
  }
  BattleObject* bobj = *bo.begin();
  BShip* shp = dynamic_cast<BShip*>(bobj);
  if (!shp) {
    throw BInternalException(__FILE__, __LINE__,
      qApp->translate("BShipGroupStateG", 
      "Leaf object '%1' is not a ship").arg( (*(bo.begin()))->name()));
  }
  return shp;
}

void BShipGroupStateG::setProgram(BProgram* program, const QDateTime& t) {
  try {
    BattleObjects bo = _shipGroup->objs();
    for (BattleObjects::iterator it = bo.begin(); it != bo.end(); it++) {
      BProgram* p = program->clone();
      p->setProgramFlags(p->programFlags() | BEvent::SystemEventFlag);
      (*it)->setProgram(p, t);
    }
  //    BShipGroup::setProgram   ~BShipGroup
//    delete program;
  }
  catch (...) {
  //    BShipGroup::setProgram   ~BShipGroup
//    delete program;
    throw;
  }
}

BKinematicData BShipGroupStateG::kinematicData(const QDateTime& t) const {
  _longitudeCoef = BUnitsConverter::longitudeCoef(
    bApp->battle()->seaMap()->originalCoordinates().y0());
  orderObjectsInGroup(t);
  if (_shipsOrder.size() > 0) {
    return _shipsOrder.begin()->first->kinematicData(t);
  }
  else
    return BKinematicData(0, 0, 0, 0, 0, 0, 0);
}

BProgramData BShipGroupStateG::programData(const QDateTime& t) const {
  _longitudeCoef = BUnitsConverter::longitudeCoef(
    bApp->battle()->seaMap()->originalCoordinates().y0());
  orderObjectsInGroup(t);
  if (_shipsOrder.size() > 0) {
    return _shipsOrder.begin()->first->programData(t);
  }
  else
    return BProgramData( kinematicData(t), 0, 0, 0, 1);
}

void BShipGroupStateG::orderObjectsInGroup(const QDateTime& t0) const {
  _shipsOrder.clear();
  BattleObjects bo = _shipGroup->objs();
  if (bo.size() == 0) {
    //    
    return;
    //throw BInternalException(__FILE__, __LINE__,
    //  QString("Command to order objects in empty group %1").arg(
    //  _shipGroup->name()));
  }
  BattleObject* o = *bo.begin();
  std::for_each(bo.begin(), bo.end(), BShipsOrderWeight(_shipsOrder,
    t0, o, _longitudeCoef));
  std::stable_sort(_shipsOrder.begin(), _shipsOrder.end(), 
    std::ptr_fun(more_obj_data));
  double base = (*_shipsOrder.begin()).second.weight;
  std::for_each(_shipsOrder.begin(), _shipsOrder.end(), 
    BShipsOrderNormalize(base, t0));
}
