/******************************************************************************/
/* BShipGroupStateD -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bshipgroupstated_cpp_id[]="@(#) $Id: bshipgroupstated.cpp,v 1.12 2005/01/24 03:50:44 Oleksiy Balabay Exp $"; static const char* bshipgroupstated_cpp_id_id=bshipgroupstated_cpp_id;

#include <algorithm>
#include <functional>

#include "bshipgroup.h"
#include "bexception.h"
#include "bship.h"
#include "bprogram.h"
#include "bshipgroupstated.h"
#include "bunitsconsts.h"
#include "bapplication.h"
#include "battle.h"
#include "bseamap.h"
#include "beventprogram.h"

/** \file bshipgroupstated.cpp
  \brief  BShipGroupStateD  BattleFleet (impl.)

     BShipGroupStateD  BattleFleet
*/

/** \class BShipGroupStateD
 \brief     ""
*/

/** \brief 
*/
BShipGroupStateD::BShipGroupStateD(BShipGroup* shipGroup, const QDateTime& t0)
  :BShipGroupState(shipGroup){
  _shipGroup->setFlags(BattleObject::IsLineAhead, BattleObject::IsLineAhead);
  BattleObjects bos = _shipGroup->objs();
  for (BattleObjects::iterator it = bos.begin(); it != bos.end(); it++) {
    (*it)->setParentControl(true);
  }
  _longitudeCoef = BUnitsConverter::longitudeCoef(
    bApp->battle()->seaMap()->originalCoordinates().y0());
  orderObjectsInGroup(t0);
}

/** \brief 
*/
BShipGroupStateD::~BShipGroupStateD(){

}

BShip* BShipGroupStateD::selectFlagShip() {
  BattleObjects bo = _shipGroup->objs(BattleObject::LeafsOnlySelectionType);
  if (bo.size() == 0) {
    return 0;
  }
  BattleObject* bobj = *bo.begin();
  BShip* shp = dynamic_cast<BShip*>(bobj);
  if (!shp) {
    throw BInternalException(__FILE__, __LINE__,
      qApp->translate("BShipGroupStateG", 
      "Leaf object '%1' is not a ship").arg( (*(bo.begin()))->name()));
  }
  return shp;
}

void BShipGroupStateD::setProgram(BProgram* program, const QDateTime& t) {
  program->setProgramFlags(program->programFlags() | BEvent::SystemEventFlag);
  std::for_each(_shipsOrder.begin(), _shipsOrder.end(), 
    BShipsOrderExec(program, t));
  //    BShipGroup::setProgram   ~BShipGroup
//  delete program;
}

void BShipGroupStateD::orderObjectsInGroup(const QDateTime& t0) {
  BattleObjects objs = _shipGroup->objs();
  orderObjectsInGroup(objs, _shipsOrder, t0, _longitudeCoef);
}

BKinematicData BShipGroupStateD::kinematicData(const QDateTime& t) const {
  if (_shipsOrder.size() > 0) {
    return _shipsOrder.begin()->first->kinematicData(t);
  }
  else
    return BKinematicData(0, 0, 0, 0, 0, 0, 0);
}

BProgramData BShipGroupStateD::programData(const QDateTime& t) const {
  if (_shipsOrder.size() > 0) {
    return _shipsOrder.begin()->first->programData(t);
  }
  else
    return BProgramData( kinematicData(t), 0, 0, 0, 1);
}

void BShipGroupStateD::orderObjectsInGroup(const BattleObjects& objs,
  BShipsOrder& shipsOrder, const QDateTime& t0, double longitudeCoef) {
  if (objs.size() == 0) {
    throw BInternalException(__FILE__, __LINE__,
      "Command to order empty objects list");
  }
  BattleObject* o = *objs.begin();

  std::for_each(objs.begin(), objs.end(), BShipsOrderWeight(shipsOrder,
    t0, o, longitudeCoef));
  std::stable_sort(shipsOrder.begin(), shipsOrder.end(), 
    std::ptr_fun(more_obj_data));
  double base = (*shipsOrder.begin()).second.weight;
  std::for_each(shipsOrder.begin(), shipsOrder.end(), 
    BShipsOrderNormalize(base, t0));
}
