/******************************************************************************/
/* BShips -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bships_cpp_id[]="@(#) $Id: bships.cpp,v 1.12 2005/01/05 11:47:27 Oleksiy Balabay Exp $"; static const char* bships_cpp_id_id=bships_cpp_id;

#include <qfile.h>
#include <qdom.h>

#include "bships.h"
#include "bapplication.h"
#include "battle.h"
#include "bdomutils.h"
#include "bprogram.h"
#include "bevent.h"
#include "beventstate.h"
#include "beventgroup.h"
#include "bseamap.h"
#include "bunitsconsts.h"
#include "beventsparser.h"

/** \file bships.cpp
  \brief  BShips  BattleFleet (impl.)

     BShips  BattleFleet
*/

/** \class BShips
  \brief 


*/
/** \brief 
*/
BShips::BShips():BShipGroup(0, "AllShips"){
  _initShipsGroup = new BShipGroup(0, "ShipsDb");
  insertChild(_initShipsGroup);
}

/** \class BShips
  \brief   

          ().

           ,
    addShipsFromFile().    
         BShipGroup.

    addShipsFromFile()      
    unUsedShips().    
       ,  setUnused(). 
        unUsedShips(), 
   setUnused(),      initAfterParse().

  TODO:   initAfterParse()  removeChild()  insertChild().

     unUsedShips()       
    removeUnusedShips().

   initAfterParse()     (, 
    ).     
  , begin()  end(). 

         ship().   
   O(log(N)).        
  .

*/

/** \brief      

  \exception BException         .   
      .     
    .      <b>
  </b>.
*/
void BShips::addShipsFromFile(const QString& fileName) {
  _fileName = bApp->extendFileName(fileName);
  BShipGroup* gr = 0;
  try {
    QFile f(_fileName);
    if (!f.exists()) {
      throw BConfigException(__FILE__, __LINE__,
        qApp->translate("BShips",
        "File '%1' does not exists").arg(_fileName));
    }
    QDomDocument doc;
    loadDocument(doc, f);
    gr = new BShipGroup(0, "");
    gr->parse(doc.documentElement());
    checkUnique(gr);
    _initShipsGroup->insertChild(gr);
  }
  catch(BConfigException& e) {
    if (gr) {
      delete gr;
    }
//    clearShipsMap();
    throw addFileName(e);
  }
  catch (BException&) {
    if (gr) {
      delete gr;
    }
//    clearShipsMap();
    throw;
  }
  catch(...) {
    if (gr) {
      delete gr;
    }
//    clearShipsMap();
    throw BInternalException(__FILE__, __LINE__,
      qApp->translate("BShips",
      "Error loading file '%1'").arg(_fileName));
  }
}

/**   .    BShip,  
  ship(),  .
*/
BShips::~BShips() {
  clearShipsMap();
}

void BShips::clearShipsMap() {
  _shipsMap.clear();
}


void BShips::loadDocument(QDomDocument& doc, QFile& file) {
  QString errorMessage;
  int errorLine;
  int errorColumn;
  if (!doc.setContent(&file, &errorMessage, &errorLine, &errorColumn)) {
    throw BFileException(BConfigException(__FILE__, __LINE__,
      qApp->translate("BXmlConfig", "Error parsing file: '%1'").arg(
      errorMessage)), file.name(), errorLine, errorColumn);
 }
}

BShips::iterator BShips::begin() {
  return _shipsMap.begin();
}

BShips::const_iterator BShips::begin() const {
  return _shipsMap.begin();
}

BShips::iterator BShips::end() {
  return _shipsMap.end();
}

BShips::const_iterator BShips::end() const {
  return _shipsMap.end();
}

/**        \a name ,  0, 
   .
 
  \param useLocale   
*/
BShip* BShips::ship(const QString& name) {
  iterator it = _shipsMap.find(name);
  if (it == _shipsMap.end())
    return 0;
  else
    return it.data();
}

///        
BFileException BShips::addFileName(const BConfigException& e) {
  return BFileException(e, _fileName);
}

void BShips::initAfterParse() {
  _shipsMap.clear();
  BattleObjects bo = objs(BattleObject::LeafsOnlySelectionType);
  BattleObjects::iterator it(bo.begin());
  for (it = bo.begin(); it != bo.end(); it++) {
    BShip* obj = dynamic_cast<BShip*>(*it);
    if (!obj) {
      throw BInternalException(__FILE__, __LINE__,
        QString("Object '%1' is not ship").arg(obj->name()));
    }
    if (_initShipsGroup != 0) {
      if (_initShipsGroup->find(obj->name())) {
        continue;
      }
    }
    if (_shipsMap.contains(obj->name())) {
      throw BConfigException(__FILE__, __LINE__,
        QString("Ship %1 duplicated").arg(obj->name()));
    }
    _shipsMap.insert(obj->name(), obj);
  }
}

void BShips::removeUnusedShips() {
  removeChild(_initShipsGroup);
  delete _initShipsGroup;
  _initShipsGroup = 0;
  initAfterParse();
}

void BShips::save(const QString& fileName) {
  QFile f(fileName);
  if (!f.open(IO_WriteOnly)) {
    throw BOutputException(__FILE__, __LINE__, fileName);
  }
  QTextStream t(&f);
  t.setEncoding(QTextStream::UnicodeUTF8);
  t << toXml().toString(2);
  f.close();
}

QDomDocument BShips::toXml() const {
  QDomDocument doc;
  QDomProcessingInstruction pi = doc.createProcessingInstruction(
    "xml", QString("version='1.0' encoding='UTF-8'"));
  doc.insertBefore(pi, doc.documentElement());
  doc.appendChild(doc.createElement(BEventsParser::SOM_EVENTS));
  QDateTime dt = bApp->battle()->battleInfo().modelTime;
  QDomElement date = 
    BDomUtils::dateTimeToXml(dt, BEventsParser::SOM_DATE, doc);
  doc.documentElement().appendChild(date);
  //  
  QDomElement allShips = doc.createElement(BEventsParser::SOM_OBJECT);
  allShips.setAttribute(BEventsParser::SOM_NAME, name());
  date.appendChild(allShips);
  appendObjects(doc, allShips);
  //  
  // TODO:    
  BattleObjects bos = objs(BattleObject::LeafsOnlySelectionType);
  for (BattleObjects::const_iterator oit = bos.begin(); 
  oit != bos.end(); oit++) {
    if ((unUsedShips() == 0) || (unUsedShips()->find((*oit)->name()) == 0)) {
      QDomElement sh = doc.createElement(BEventsParser::SOM_OBJECT);
      sh.setAttribute(BEventsParser::SOM_NAME, (*oit)->name());
      QDomElement ev = doc.createElement(BEvent::BE_EVENT);
      ev.setAttribute(BEvent::EVENT_FLAGS, BEvent::EVENT_FLAG_PLAYER);
      ev.setAttribute(BEvent::BE_EVENT_TYPE, 
        BEventState::BE_EVENT_TYPE_STATE);
      sh.appendChild(ev);
      ev.appendChild((*oit)->state(doc));
      date.appendChild(sh);
    }
  }
  return doc;
}

void BShips::appendObjects(QDomDocument& doc, QDomElement& element) const {
  BattleObjects bo = objs();
  for (BattleObjects::iterator it1 = bo.begin(); it1 != bo.end(); it1++) {
    if ((unUsedShips() == 0) || ((*it1)->name() != unUsedShips()->name())) {
      QDomElement evShips = doc.createElement(BEvent::BE_EVENT);
      evShips.setAttribute(BEvent::EVENT_FLAGS, BEvent::EVENT_FLAG_PLAYER);
      evShips.setAttribute(BEvent::BE_EVENT_TYPE, 
        BEventGroupCreate::BE_EVENT_TYPE_GROUP_CREATE);
      evShips.appendChild((*it1)->toXml(doc));
      if (evShips.hasChildNodes())
        element.appendChild(evShips);
    }
  }
}

const BShipGroup* BShips::unUsedShips() const {
  return _initShipsGroup;
}

void BShips::setUnused(BattleObject* bo) {
  if (_initShipsGroup == 0) {
    //       ,   
    if (bo->name() == name()) {
      throw BInternalException(__FILE__, __LINE__,
      qApp->translate("BEventGroupRemove", 
      "Requested to remove top level group"));
    }
    BattleObjects bos = bo->objs(BattleObject::LeafsOnlySelectionType);
    if (bos.size() != 0) {
      throw BInternalException(__FILE__, __LINE__,
        QString("Attempt to remove object '%1' when unUsedShips is 0").arg(
        bo->name()));
    }
    bo->parent()->removeChild(bo);
    delete bo;
  }
  else {
    _initShipsGroup->insertChild(bo);
  }
  initAfterParse();
}

void BShips::checkUnique(BShipGroup* group) {
  BattleObjects bos = group->objs(AllSelectionType);
  for(BattleObjects::const_iterator it = bos.begin(); it != bos.end(); it++) {
    if (this->find((*it)->name()) != 0) {
      throw BConfigException(__FILE__, __LINE__, qApp->translate("BShips",
        "Object '%1' already exists").arg((*it)->name()));
    }
  }
}
