/******************************************************************************/
/* BShipsListView -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bshipslistview_cpp_id[]="@(#) $Id: bshipslistview.cpp,v 1.7 2005/01/16 15:33:40 Oleksiy Balabay Exp $"; static const char* bshipslistview_cpp_id_id=bshipslistview_cpp_id;

#include <qdom.h>
#include <qfile.h>
#include <qdragobject.h>
#include <qiconset.h>
#include <qwhatsthis.h>

#include "bshipslistview.h"
#include "battleobject.h"
#include "bapplication.h"
#include "battle.h"
#include "bscenarioheader.h"
#include "bshipgroup.h"
#include "xpms/xpms.h"
#include "bships.h"
#include "bgroupview.h"
#include "bmainwindowtexts.h"

const char* B_SHIPS_DB_VIEW_CAPTION = 
  QT_TRANSLATE_NOOP("BShipsDbView", "Ships Db");

/** \file bshipslistview.cpp
  \brief  BShipsListView  BattleFleet (impl.)

     BShipsListView  BattleFleet
*/

/** \class BShipsListView
  \brief 


*/
/** \brief 
*/
BShipsListView::BShipsListView(QWidget *parent, const char *name)
  :QListView( parent, name ) {
  addColumn(tr("Name"));
  setColumnWidthMode(0, Maximum);
  folderClosed = new QPixmap( folder_closed_xpm );
  folderOpen = new QPixmap( folder_open_xpm );
  setRootIsDecorated(true);
  setSelectionMode(Extended);
}

/** \brief 
*/
BShipsListView::~BShipsListView(){
  delete folderClosed;
  delete folderOpen;
}

void BShipsListView::showBattleObject(const BattleObject* battleObject) {
  BattleObjects bo = battleObject->objs();
  for (BattleObjects::iterator it = bo.begin(); it != bo.end(); it++) {
    QListViewItem* item = new QListViewItem(this, (*it)->name());
    item->setSelectable(false);
    parseBattleObject(item, *it);
  }
}

void BShipsListView::parseBattleObject(QListViewItem* parent,
  const BattleObject* battleObject) {
  BattleObjects bo = battleObject->objs();
  if (!battleObject->isGroup()) {
    parent->setDragEnabled(true);
    QIconSet is(QPixmap(battleObject->image().smoothScale(16,16, 
      QImage::ScaleMin)));
    parent->setPixmap(0, is.pixmap(QIconSet::Small, QIconSet::Normal));
//    parent->setPixmap(0, *shipNotUsed);
    return;
  }
  parent->setSelectable(false);
  parent->setExpandable(true);
  parent->setPixmap(0, *folderClosed);
  for (BattleObjects::iterator it = bo.begin(); it != bo.end(); it++) {
    QListViewItem* item = new QListViewItem(parent, (*it)->name());
    parseBattleObject(item, *it);
  }
}

void BShipsListView::setOpen(QListViewItem* item, bool open) {
  if ( open )
  	item->setPixmap(0, *folderOpen);
//  	item->setup();
//    item->widthChanged( 0 );
//    item->invalidateHeight();
//    item->repaint();
  else
	  item->setPixmap( 0, *folderClosed );
  QListView::setOpen(item, open);
}

QDragObject* BShipsListView::dragObject() {
  //   
  QListViewItemIterator it(this, QListViewItemIterator::Selected);
  QString s;
  while (it.current()) {
    s+=(*it)->text(0)+"\t";
    it++;
  }
  QListViewItem* itm = currentItem();
  QDragObject* d = new QTextDrag(s, this);
  d->setPixmap(*itm->pixmap(0));
  return d;
}

void BShipsListView::updateView(const BattleObject* unUsedShips) {
  if (unUsedShips == 0)
    return;
  QListViewItemIterator it(this, QListViewItemIterator::NotExpandable);
  while ( it.current()) {
    QString name = it.current()->text(0);
    BattleObject* bo = bApp->battle()->ships()->find(name);
    if (unUsedShips->find(name) != 0) {
      it.current()->setEnabled(true);
      if (!(it.current()->dragEnabled())) {
        it.current()->setDragEnabled(true);
        it.current()->setSelectable(true);
        QIconSet is(QPixmap(bo->image().smoothScale(16,16, QImage::ScaleMin)));
        it.current()->setPixmap(0, 
          is.pixmap(QIconSet::Small, QIconSet::Normal));
      }
    }
    else {
      it.current()->setEnabled(false);
      if (this->currentItem() == it.current()) {
        this->setCurrentItem(this->firstChild());
      }
      if (it.current()->dragEnabled()) {
        it.current()->setDragEnabled(false);
        it.current()->setSelectable(false);
        if (it.current()->isSelected()) {
          it.current()->setSelected(false);
        }
//        it.current()->setPixmap(0, *shipUsed);
        QIconSet is(QPixmap(bo->image().smoothScale(16,16, QImage::ScaleMin)));
        it.current()->setPixmap(0, 
          is.pixmap(QIconSet::Small, QIconSet::Disabled));
      }
    }
    ++it;
  }
}

void BShipsListView::addBattleObject(const BattleObject* battleObject) {
  QListViewItem* item = new QListViewItem(this, battleObject->name());
  parseBattleObject(item, battleObject);
}


BShipsDbView::BShipsDbView(QWidget* parent, const char* name):QDockWindow(parent, name) {
  QWhatsThis::add(this, tr(
    "<b>%4</b><br><br>"
    "To add ship to the scenario select it in this window and drag into map "
    "or into <b>%1</b> window<br><br>"
    "Additional ships database files can be added to this list using "
    "<b>%2/%3</b> menu. How to create additional ships files see online "
    "documentation").arg(qApp->translate("BGroupView", B_GROUP_VIEW)).arg(
    qApp->translate("BMainWindowState", fileToolBar_N)).arg(
    qApp->translate("BMainWindowState", filePropertiesAction_TT)).arg(
    tr(B_SHIPS_DB_VIEW_CAPTION)));
  _view = new BShipsListView(this, "shipslistview");
  setWidget(_view);
  setCloseMode(Always);
  setResizeEnabled(true);
  setMovingEnabled(true);
  setCaption(tr(B_SHIPS_DB_VIEW_CAPTION));
  init();
  _view->show();
}

BShipsDbView::~BShipsDbView() {
}

void BShipsDbView::init() {
  QStringList l = bApp->battle()->scenarioHeader()->shipsFileNames();
  BShipGroup sg(0, "Ships Database");
  for(QStringList::iterator it = l.begin(); it != l.end(); it++) {
    BShipGroup* shipGroup = loadShipGroupFromFile(*it);
    sg.insertChild(shipGroup);
  }
  _view->showBattleObject(&sg);
}

void BShipsDbView::updateView(const BattleObject* unUsedShips) {
  _view->updateView(unUsedShips);
}

void BShipsDbView::addShipsFromFile(const QString& fileName) {
  BShipGroup* shipGroup = loadShipGroupFromFile(fileName);
  _view->addBattleObject(shipGroup);
}

BShipGroup* BShipsDbView::loadShipGroupFromFile(const QString& fileName) {
  BShipGroup* shipGroup = 0;
  try {
    shipGroup = new BShipGroup(0, "");
    QFile f;
    f.setName(bApp->extendFileName(fileName));
    if (!f.exists()) {
      throw BConfigException(__FILE__, __LINE__,
        qApp->translate("BShipsDbView", "File does not exists"));
    }
    QDomDocument doc;
    QString errorMessage;
    int errorLine;
    int errorColumn;
    if (!doc.setContent(&f, &errorMessage, &errorLine, &errorColumn)) {
      throw BFileException(BConfigException(__FILE__, __LINE__,
        qApp->translate("BShipsDbView","Error parsing file: '%1'").arg(
        errorMessage)), f.name(), errorLine, errorColumn);
    }
    shipGroup->parse(doc.documentElement());
  }
  catch (...) {
    delete shipGroup;
    throw;
  }
  return shipGroup;
}
