/******************************************************************************/
/* BShip -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bshipstatic_cpp_id[]="@(#) $Id: bshipstatic.cpp,v 1.5 2005/01/28 17:06:08 Oleksiy Balabay Exp $"; static const char* bshipstatic_cpp_id_id=bshipstatic_cpp_id;

#include <qfile.h>
#include <qimage.h>

#include "bship.h"
#include "bexception.h"
#include "bdomutils.h"
#include "bapplication.h"
#include "bprogram.h"
#include "bunitsconsts.h"

/** \file bshipstatic.cpp
  \brief  BShip  BattleFleet (impl.)

       BShip  BattleFleet
*/

const QString SHP_SHIP_ARMOR = "Armor";
const QString ARMOR_SHORT = "Short";
const QString ARMOR_LONG = "Long";

const QString SHP_SHIP_STABILITY = "Stability";
const QString STABILITY_NONE = "none";
const QString STABILITY_POOR = "poor";
const QString STABILITY_GOOD = "good";
const QString STABILITY_GREAT = "great";

const QString SHP_SHIP_SEAWAY_RATING = "SeawayRating";
const QString SEAWAY_RATING_NONE = "none";
const QString SEAWAY_RATING_POOR = "poor";
const QString SEAWAY_RATING_GOOD = "good";
const QString SEAWAY_RATING_GREAT = "great";

const QString SHP_SHIP_FLOOD_DEFENSE = "FloodDefense";
const QString FLOOD_DEFENSE_NONE = "none";
const QString FLOOD_DEFENSE_POOR = "poor";
const QString FLOOD_DEFENSE_GOOD = "good";
const QString FLOOD_DEFENSE_GREAT = "great";

const QString SHP_ARMAMENT = "Armament";

const QString ARM_GUN_TYPE = "GunType";
const QString ARM_GUN_TYPE_TURRET = "Turret";
const QString ARM_GUN_TYPE_CANNON = "Cannon";
const QString ARM_GUN_TYPE_TORPEDO = "Torpedo";

///  BattleFleet::Armament::Quantity-TYPE
const QString ARM_TYPE = "Type"; 
const QString ARM_QUANTITY = "Quantity"; 
const QString ARM_SIZE = "Size"; 
const QString ARM_ROF = "ROF"; 
const QString ARM_PENETRATION = "Penetration"; 
const QString ARM_PENETRATION_SHORT = "Short"; 
const QString ARM_PENETRATION_MIDDLE = "Middle"; 
const QString ARM_PENETRATION_LONG = "Long"; 
const QString ARM_POSITION = "Position"; 

const QString ARM_POSITION_A = "A"; 
const QString ARM_POSITION_B = "B"; 
const QString ARM_POSITION_C = "C"; 
const QString ARM_POSITION_D = "D"; 
const QString ARM_POSITION_E = "E"; 
const QString ARM_POSITION_M = "M"; 
const QString ARM_POSITION_O = "O"; 
const QString ARM_POSITION_P = "P"; 
const QString ARM_POSITION_R = "R"; 
const QString ARM_POSITION_S = "S"; 
const QString ARM_POSITION_U = "U"; 
const QString ARM_POSITION_V = "V"; 
const QString ARM_POSITION_W = "W"; 
const QString ARM_POSITION_X = "X"; 
const QString ARM_POSITION_Y = "Y"; 
const QString ARM_POSITION_Z = "Z";

///   
const QString ARM_TORPEDO_POSITION_STEM = "stem";
///  
const QString ARM_TORPEDO_POSITION_STERN = "stern";

const QString ARM_PATTERN = "Pattern"; 
const QString ARM_PATTERN_0 = "0"; 
const QString ARM_PATTERN_1 = "1"; 
const QString ARM_PATTERN_2 = "2"; 

const QString BShip::VT_VESSEL_TYPE = "VesselType";
const QString VT_BATTLESHIP = "Battleship";
const QString VT_COAST_DEFENSE_BATTLESHIP = "CoastDefenseBattleship";
const QString VT_ARMORED_CRUISER = "ArmoredCruiser";
const QString VT_PROTECTED_CRUISER = "ProtectedCruiser";
const QString VT_UNPROTECTED_CRUISER = "UnprotectedCruiser";
const QString VT_GUN_BOAT = "GunBoat";
const QString VT_DESTROYER = "Destroyer";
const QString VT_TOPEDO_BOAT = "TorpedoBoat";

BShip::BStability BShip::stabilityFromString(const QString& stability) 
  {
  if (stability.lower() == STABILITY_NONE)
    return NoneStability;
  else if (stability.lower() == STABILITY_POOR)
    return PoorStability;
  else if (stability.lower() == STABILITY_GOOD)
    return GoodStability;
  else if (stability.lower() == STABILITY_GREAT)
    return GreatStability;
  else
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate(SHP_SHIP_STABILITY, 
      "Invalid %1: '%2'").arg(SHP_SHIP_STABILITY).arg(stability));
  // keep compiler happy
  return NoneStability;
}

QString BShip::stabilityToString(const BStability stability) {
  switch (stability) {
  case NoneStability:
    return STABILITY_NONE;
  case PoorStability:
    return STABILITY_POOR;
  case GoodStability:
    return STABILITY_GOOD;
  case GreatStability:
    return STABILITY_GREAT;
  default:
    qFatal("Unknown %s: %d", (const char*)SHP_SHIP_STABILITY.local8Bit(),
      stability);
    return STABILITY_NONE;
  }
}

BShip::BSeawayRating BShip::seawayRatingFromString(const QString& seawayRating) 
  {
  if (seawayRating.lower() == SEAWAY_RATING_NONE)
    return NoneSeawayRating;
  else if (seawayRating.lower() == SEAWAY_RATING_POOR)
    return PoorSeawayRating;
  else if (seawayRating.lower() == SEAWAY_RATING_GOOD)
    return GoodSeawayRating;
  else if (seawayRating.lower() == SEAWAY_RATING_GREAT)
    return GreatSeawayRating;
  else
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate(SHP_SHIP_SEAWAY_RATING, 
      "Invalid %1: '%2'").arg(SHP_SHIP_SEAWAY_RATING).arg(seawayRating));
  // keep compiler happy
  return NoneSeawayRating;
}

QString BShip::seawayRatingToString(const BSeawayRating seawayRating) {
  switch (seawayRating) {
  case NoneSeawayRating:
    return SEAWAY_RATING_NONE;
  case PoorSeawayRating:
    return SEAWAY_RATING_POOR;
  case GoodSeawayRating:
    return SEAWAY_RATING_GOOD;
  case GreatSeawayRating:
    return SEAWAY_RATING_GREAT;
  default:
    qFatal("Unknown %s: %d", (const char*)SHP_SHIP_SEAWAY_RATING.local8Bit(),
      seawayRating);
    return SEAWAY_RATING_NONE;
  }
}

BShip::BFloodDefense BShip::floodDefenseFromString(const QString& floodDefense) 
  {
  if (floodDefense.lower() == FLOOD_DEFENSE_NONE)
    return NoneFloodDefense;
  else if (floodDefense.lower() == FLOOD_DEFENSE_POOR)
    return PoorFloodDefense;
  else if (floodDefense.lower() == FLOOD_DEFENSE_GOOD)
    return GoodFloodDefense;
  else if (floodDefense.lower() == FLOOD_DEFENSE_GREAT)
    return GreatFloodDefense;
  else
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate(SHP_SHIP_FLOOD_DEFENSE, 
      "Invalid %1: '%2'").arg(SHP_SHIP_FLOOD_DEFENSE).arg(floodDefense));
  // keep compiler happy
  return NoneFloodDefense;
}

QString BShip::floodDefenseToString(const BFloodDefense floodDefense) {
  switch (floodDefense) {
  case NoneFloodDefense:
    return FLOOD_DEFENSE_NONE;
  case PoorFloodDefense:
    return FLOOD_DEFENSE_POOR;
  case GoodFloodDefense:
    return FLOOD_DEFENSE_GOOD;
  case GreatFloodDefense:
    return FLOOD_DEFENSE_GREAT;
  default:
    qFatal("Unknown %s: %d", (const char*)SHP_SHIP_FLOOD_DEFENSE.local8Bit(),
      floodDefense);
    return FLOOD_DEFENSE_NONE;
  }
}

BShip::BArmPositions BShip::armPositionFromString(const QString& armPosition) 
 {
  if (ARM_POSITION_A == armPosition.upper())
    return AArmPosition;
  else if (ARM_POSITION_B == armPosition.upper())
    return BArmPosition;
  else if (ARM_POSITION_C == armPosition.upper())
    return CArmPosition;
  else if (ARM_POSITION_D == armPosition.upper())
    return DArmPosition;
  else if (ARM_POSITION_E == armPosition.upper())
    return EArmPosition;
  else if (ARM_POSITION_M == armPosition.upper())
    return MArmPosition;
  else if (ARM_POSITION_O == armPosition.upper())
    return OArmPosition;
  else if (ARM_POSITION_P == armPosition.upper())
    return PArmPosition;
  else if (ARM_POSITION_R == armPosition.upper())
    return RArmPosition;
  else if (ARM_POSITION_S == armPosition.upper())
    return SArmPosition;
  else if (ARM_POSITION_U == armPosition.upper())
    return UArmPosition;
  else if (ARM_POSITION_V == armPosition.upper())
    return VArmPosition;
  else if (ARM_POSITION_W == armPosition.upper())
    return WArmPosition;
  else if (ARM_POSITION_X == armPosition.upper())
    return XArmPosition;
  else if (ARM_POSITION_Y == armPosition.upper())
    return YArmPosition;
  else if (ARM_POSITION_Z == armPosition.upper())
    return ZArmPosition;
  else
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate(ARM_POSITION, 
      "Invalid %1: '%2'").arg(ARM_POSITION).arg(armPosition));
  return AArmPosition;
}

QString BShip::armPositionToString(const BArmPositions armPosition) {
  switch (armPosition) {
  case AArmPosition:
    return ARM_POSITION_A;
  case BArmPosition:
    return ARM_POSITION_B;
  case CArmPosition:
    return ARM_POSITION_C;
  case DArmPosition:
    return ARM_POSITION_D;
  case EArmPosition:
    return ARM_POSITION_E;
  case MArmPosition:
    return ARM_POSITION_M;
  case OArmPosition:
    return ARM_POSITION_O;
  case PArmPosition:
    return ARM_POSITION_P;
  case RArmPosition:
    return ARM_POSITION_R;
  case SArmPosition:
    return ARM_POSITION_S;
  case UArmPosition:
    return ARM_POSITION_U;
  case VArmPosition:
    return ARM_POSITION_V;
  case WArmPosition:
    return ARM_POSITION_W;
  case XArmPosition:
    return ARM_POSITION_X;
  case YArmPosition:
    return ARM_POSITION_Y;
  case ZArmPosition:
    return ARM_POSITION_Z;
  default:
    qFatal("Unknown %s: %d", (const char*)ARM_POSITION.local8Bit(),
      armPosition);
    return ARM_POSITION_Z;
  }
}

BShip::BTorpedoPositions BShip::torpedoPositionFromString(
  const QString& torpedoPosition) {
  if (ARM_TORPEDO_POSITION_STEM == torpedoPosition.lower()) {
    return StemTorpedoPosition;
  }
  else if (ARM_TORPEDO_POSITION_STERN == torpedoPosition.lower()) {
    return SternTorpedoPosition;
  }
  else
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate("TorpedoPositions", 
      "Invalid torpedo position: '%2'").arg(torpedoPosition));
  return SternTorpedoPosition;
}

QString BShip::torpedoPositionToString(const BTorpedoPositions torpedoPosition){
  switch (torpedoPosition) {
  case SternTorpedoPosition:
    return ARM_TORPEDO_POSITION_STERN;
  case StemTorpedoPosition:
    return ARM_TORPEDO_POSITION_STEM;
  default:
    qFatal("Unknown torpedo position: %d", torpedoPosition);
    return ARM_TORPEDO_POSITION_STEM;
  }
}

BShip::BArmPatterns BShip::armPatternFromString(const QString& armPattern) 
  {
  if (ARM_PATTERN_0 == armPattern.lower()) {
    return ArmPattern0;
  }
  else if (ARM_PATTERN_1 == armPattern.lower()) {
    return ArmPattern1;
  }
  else if (ARM_PATTERN_2 == armPattern.lower()) {
    return ArmPattern2;
  }
  else
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate(ARM_PATTERN, 
      "Invalid %1: '%2'").arg(ARM_PATTERN).arg(armPattern));
  return ArmPattern0;
}

QString BShip::armPatternToString(const BArmPatterns armPattern) {
  switch (armPattern) {
  case ArmPattern0:
    return ARM_PATTERN_0;
  case ArmPattern1:
    return ARM_PATTERN_1;
  case ArmPattern2:
    return ARM_PATTERN_2;
  default:
    qFatal("Unknown %s: %d", (const char*)ARM_PATTERN.local8Bit(),armPattern);
    return ARM_PATTERN_0;
  }
}

int BShip::defaultSize(const BShip::BVesselType type) {
  switch (type) {
  case BattleshipType:
    return 50;
  case CoastDefenseBattleshipType:
    return 15;
  case ArmoredCruiserType:
    return 35;
  case ProtectedCruiserType:
    return 15;
  case UnprotectedCruiserType:
    return 5;
  case GunBoatType:
    return 5;
  case DestroyerType:
    return 1;
  case TorpedoBoatType:
    return 1;
  default:
    qFatal("Unknown %s: %d", (const char*)VT_VESSEL_TYPE.local8Bit(),
      type);
    return 1;
  }
}

double BShip::defaultTurnRadius(const int size) {
  if (size <= 5) {
    return 21*1000.0/(85*MILE);
  }
  else if (size <= 35) {
    return 32*1000.0/(85*MILE);
  }
  else if (size <= 65) {
    return 43*1000.0/(85*MILE);
  }
  else {
    return 55*1000.0/(85*MILE);
  }
}

BShip::BVesselType BShip::vesselTypeFromString(const QString& vesselType) 
  {
  if (vesselType.lower() == VT_BATTLESHIP.lower())
    return BattleshipType;
  else if (vesselType.lower() == VT_COAST_DEFENSE_BATTLESHIP.lower())
    return CoastDefenseBattleshipType;
  else if (vesselType.lower() == VT_ARMORED_CRUISER.lower())
    return ArmoredCruiserType;
  else if (vesselType.lower() == VT_PROTECTED_CRUISER.lower())
    return ProtectedCruiserType;
  else if (vesselType.lower() == VT_UNPROTECTED_CRUISER.lower())
    return UnprotectedCruiserType;
  else if (vesselType.lower() == VT_GUN_BOAT.lower())
    return GunBoatType;
  else if (vesselType.lower() == VT_DESTROYER.lower())
    return DestroyerType;
  else if (vesselType.lower() == VT_TOPEDO_BOAT.lower())
    return TorpedoBoatType;
  else
    throw BConfigException(__FILE__, __LINE__,
      qApp->translate(VT_VESSEL_TYPE, 
      "Invalid %1: '%2'").arg(VT_VESSEL_TYPE).arg(vesselType));
  // keep compiler happy
  return BattleshipType;
}

QString BShip::vesselTypeToString(const BVesselType vesselType) {
  switch (vesselType) {
  case BattleshipType:
    return VT_BATTLESHIP;
  case CoastDefenseBattleshipType:
    return VT_COAST_DEFENSE_BATTLESHIP;
  case ArmoredCruiserType:
    return VT_ARMORED_CRUISER;
  case ProtectedCruiserType:
    return VT_PROTECTED_CRUISER;
  case UnprotectedCruiserType:
    return VT_UNPROTECTED_CRUISER;
  case GunBoatType:
    return VT_GUN_BOAT;
  case DestroyerType:
    return VT_DESTROYER;
  case TorpedoBoatType:
    return VT_TOPEDO_BOAT;
  default:
    qFatal("Unknown %s: %d", (const char*)VT_VESSEL_TYPE.local8Bit(),
      vesselType);
    return VT_BATTLESHIP;
  }
}

/// Human Readable Form
QString BShip::vesselTypeToStringHRF(const BVesselType vesselType) {
  switch (vesselType) {
  case BattleshipType:
    return qApp->translate("BShip", "Battleship");
  case CoastDefenseBattleshipType:
    return qApp->translate("BShip", "Coast Defense Battleship");
  case ArmoredCruiserType:
    return qApp->translate("BShip", "Armored Cruiser");
  case ProtectedCruiserType:
    return qApp->translate("BShip", "Protected Cruiser");
  case UnprotectedCruiserType:
    return qApp->translate("BShip", "Unprotected Cruiser");
  case GunBoatType:
    return qApp->translate("BShip", "Gun Boat");;
  case DestroyerType:
    return qApp->translate("BShip", "Destroyer");
  case TorpedoBoatType:
    return qApp->translate("BShip", "Torpedo Boat");
  default:
    qFatal("Unknown %s: %d", (const char*)VT_VESSEL_TYPE.local8Bit(),
      vesselType);
    return qApp->translate("BShip", "Battleship");
  }
}
