/******************************************************************************/
/* BUiState -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char buistate_cpp_id[]="@(#) $Id: buistate.cpp,v 1.15 2005/02/20 14:45:01 Oleksiy Balabay Exp $"; static const char* buistate_cpp_id_id=buistate_cpp_id;

#pragma warning(disable: 4786)
#include <algorithm>

#include "buistate.h"
#include "battleobject.h"
#include "bapplication.h"
#include "battle.h"
#include "bships.h"
#include "beventgroup.h"
#include "borderalgorithms.h"
#include "bseamap.h"
#include "bunitsconsts.h"
#include "battleobserver.h"
#include "beventclear.h"
#include "bai.h"
#include "bprogramfactory.h"
#include "beventprogram.h"

const double SHIPS_INTERVAL_MILES = 0.2;
/** \file buistate.cpp
  \brief  BUiState  BattleFleet (impl.)

     BUiState  BattleFleet
*/

/** \class BUiState
  \brief 


*/
/** \brief 
*/
BUiState::BUiState(BattleObserver* observer, QObject* parent, const char* name)
  :QObject(parent, name), _activeObject(0), _mode(ScaleMode),
  _groupSuffix(0), _observer(observer), _ais(0){
}

/** \brief 
*/
BUiState::~BUiState(){

}

BattleObject* BUiState::activeObject() const {
  return _activeObject;
}

void BUiState::setActiveObject(BattleObject* object) {
  if (setActiveObjectInternal(object)) {
    emit selectionChanged(BUiSelectionInfo(activeObject(), 
      &(selectedObjects())));
  }
  Q_ASSERT( ((_activeObject == 0) && (_selectedObjects.size() == 0)) ||
    ((_activeObject != 0) && (_selectedObjects.size() != 0)));
}

void BUiState::setActiveObjectAsSelection(BattleObject* object) {
  bool res1 = setSelectedObjectsInternal(BattleObjects(), false);
  bool res2 = setActiveObjectInternal(object);
  if (res1 || res2) {
    emit selectionChanged(BUiSelectionInfo(activeObject(), 
      &(selectedObjects())));
  }
  Q_ASSERT( ((_activeObject == 0) && (_selectedObjects.size() == 0)) ||
    ((_activeObject != 0) && (_selectedObjects.size() != 0)));
}

///  \a true   
bool BUiState::setActiveObjectInternal(BattleObject* object) {
  if (_activeObject == object) 
    return false;

  BattleObject* old = _activeObject;
  if (object != 0) {
    _activeObject = object;
    BattleObjects objs = object->objs(BattleObject::AllSelectionType);
    addSelectedObjectsInternal(objs, false);
  }
  else {
    removeFromSelectionInternal(old, false);
    if (_selectedObjects.size() == 0)
      _activeObject = 0;
    else 
      _activeObject = *_selectedObjects.begin();
  }
  return true;
}

void BUiState::setEditorMode(BUiState::BEditorMode mode) {
  _mode = mode;
//  emit showChasingConnections(mode == ConnectMode);
}

BUiState::BEditorMode BUiState::editorMode() const {
  return _mode;
}

const BattleObjects& BUiState::selectedObjects() const {
  return _selectedObjects;
}

void BUiState::setSelectedObjects(const BattleObjects& selObjs, bool exactly) {
  if (setSelectedObjectsInternal(selObjs, exactly)) {
    emit selectionChanged(BUiSelectionInfo(activeObject(), 
      &(selectedObjects())));
  }
  Q_ASSERT( ((_activeObject == 0) && (_selectedObjects.size() == 0)) ||
    ((_activeObject != 0) && (_selectedObjects.size() != 0)));
}

bool BUiState::setSelectedObjectsInternal(const BattleObjects& selObjs, 
  bool exactly) {
  if ((selObjs.size() == 0) && (_selectedObjects.size() == 0)) {
    return false;
  }
  if (selObjs.size() == 0) {
    _selectedObjects.clear();
    setActiveObjectInternal(0);
    return true;
  }
  bool eq1 = std::equal(_selectedObjects.begin(), _selectedObjects.end(),  
        selObjs.begin());
  if (eq1) {
    bool eq2 = std::equal(selObjs.begin(), selObjs.end(),  
        _selectedObjects.begin());
    if (eq2)
      return false;
  }
  _selectedObjects.clear();
  _activeObject = 0;
  addSelectedObjectsInternal(selObjs, exactly);
  return true;
}


void BUiState::addSelectedObjects(const BattleObjects& selObjs, bool exactly) {
  if (addSelectedObjectsInternal(selObjs, exactly)) {
    emit selectionChanged(BUiSelectionInfo(activeObject(), 
      &(selectedObjects())));
  }
  Q_ASSERT( ((_activeObject == 0) && (_selectedObjects.size() == 0)) ||
    ((_activeObject != 0) && (_selectedObjects.size() != 0)));
}

bool BUiState::addSelectedObjectsInternal(const BattleObjects& selObjs,
  bool exactly) {
  if (selObjs.size() == 0) {
    return false;
  }
//  if (std::includes(_selectedObjects.begin(), _selectedObjects.end(),
//    selObjs.begin(), selObjs.end())) {
//    return false;
//  }
  for(BattleObjects::const_iterator it = selObjs.begin(); 
  it != selObjs.end(); it++) {
    if (!exactly) {
      BattleObjects bos = (*it)->objs(BattleObject::AllSelectionType);
      for(BattleObjects::const_iterator it1 = bos.begin(); 
      it1 != bos.end(); it1++) {
        _selectedObjects.insert(*it1);
      }
    }
    else {
      _selectedObjects.insert(*it);
    }
  }
  if (activeObject() == 0) {
    setActiveObjectInternal(*(selObjs.begin()));
  }
  return true;
}


/**   ,       
.    - ,    , 
     

       

      
*/
void BUiState::slotGroup() {
  QString name = newGroupName();
  QDateTime dt = bApp->battle()->battleInfo().modelTime;
//  for (BattleObjects::iterator it = _selectedObjects.begin();
//  it != _selectedObjects.end(); it++) {
//    BEvent* e = new BEventClear((*it)->name(), dt);
//    _observer->postEvent(e);
//  }
  BattleObjects os = _selectedObjects;
  BattleObjects::iterator eit = std::remove_if(os.begin(), os.end(),
    is_parent_in(_selectedObjects, bApp->battle()->ships(), true));
  os.erase(eit, os.end());

  BEvent* event = new BEventGroupCreate(bApp->battle()->ships()->name(),
    bApp->battle(), name, NO_SIDE, BattleObjects(), dt);
  _observer->postEvent(event);
  for(BattleObjects::iterator it = os.begin(); it != os.end(); it++) {
     BEvent* event = 
       new BEventGroupAdd((*it)->name(), name, bApp->battle(), dt);
     _observer->postEvent(event, false, false);
  }
}

void BUiState::slotUngroup() {
  QString name = bApp->battle()->ships()->name();
  QDateTime dt = bApp->battle()->battleInfo().modelTime;
  for(BattleObjects::const_iterator it = _selectedObjects.begin(); 
  it != _selectedObjects.end(); it++) {
    BattleObject* o = *it;
    if (o->isGroup()) {
      //   
      BattleObjects oos = o->objs(BattleObject::TreeSelectionType);
      for(BattleObjects::const_iterator it1 = oos.begin(); 
      it1 != oos.end(); it1++) {
        BEvent* event = new BEventGroupAdd((*it1)->name(),
          name, bApp->battle(), dt);
        _observer->postEvent(event, false, false);
      }
    }
    else if (o->parent()->name() != name) {
      BEvent* event = new BEventGroupAdd(o->name(),
        name, bApp->battle(), dt);
      _observer->postEvent(event, false, false);
    }
  }
}

QString BUiState::newGroupName() const {
  QString name = tr("Group%1").arg(++_groupSuffix);
  BShipGroup* ships = bApp->battle()->ships();
  while (ships->find(name)) {
    name = tr("Group%1").arg(++_groupSuffix);
  }
  return name;
}

void BUiState::removeObject(const QString& name) {
  BShips* ships = bApp->battle()->ships();
  QDateTime dt = bApp->battle()->battleInfo().modelTime;
  BattleObject* bo = ships->find(name);
  if (bo == 0) {
    throw BInternalException(__FILE__, __LINE__,
      tr("Requested to remove object '%1' does not exist").arg(name));
  }
//  if (ships->unUsedShips() == 0) {
//    throw BInternalException(__FILE__, __LINE__,
//      tr("Requested to remove object '%1' "
//      "when unUsedShips group is deleted").arg(name));
//  }
//  if (ships->unUsedShips()->find(name)) {
//    qDebug("request to delete object already deleted");
//    return;
//  }
  removeFromSelection(bo);
  BEvent* event = new BEventGroupRemove(name, bApp->battle(), dt);
  _observer->postEvent(event, true);
}

void BUiState::removeSelection() {
  QDateTime dt = bApp->battle()->battleInfo().modelTime;
  BattleObjects bos = _selectedObjects;
  setSelectedObjects(BattleObjects());
  for(BattleObjects::const_iterator it = bos.begin(); it != bos.end(); it++) {
    BEvent* event = new BEventGroupRemove((*it)->name(), bApp->battle(), dt);
    _observer->postEvent(event, true);
  }
}

void BUiState::moveObject(BattleObject* o, BattleObject* parent) {
  const BShipGroup* uus = bApp->battle()->ships()->unUsedShips();
  bool newObject = false;
  if (uus) {
    if (uus->find(o->name()))
      newObject = true;
  }
  QDateTime dt = bApp->battle()->battleInfo().modelTime;
  BEvent* event = new BEventGroupAdd(o->name(),
    parent->name(), bApp->battle(), dt);
  _observer->postEvent(event, true);
  if (newObject) {
    BSeaMap* sm = bApp->battle()->seaMap();
    BMapPoint pnt;
    pnt.first = sm->originalCoordinates().x0()+sm->originalCoordinates().w()/2;
    pnt.second= sm->originalCoordinates().y0()+sm->originalCoordinates().h()/2;
    BProgram* p = BProgramFactory::createStandByProgram(
      pnt.first, pnt.second, bApp->battle());
    BEvent* evp = new BEventProgram(p, o->name());
    _observer->postEvent(evp);
  }
}

/*template<class P, class R, class O>
class pair_second: public std::unary_function<P, R> {
public:
  R operator()(P p) {
    return O.operator()(p.second);
  }
};
*/

class add_ai_chasing: public std::unary_function<
  std::pair<BattleObject*, BObjectData>, void> {
public:
  add_ai_chasing(BAIs* ais):_prev(0), _ais(ais){
  }
  void operator()(std::pair<BattleObject*, BObjectData> p) {
    if (_prev == 0) {
      _prev = p.first;
      return;
    }
    _ais->addChasing(_prev, p.first);
    _prev = p.first;
  }
private:
  BAIs* _ais;
  BattleObject* _prev;
};

bool BUiState::canOrderSelection() const {
  BattleObject* ao = activeObject();
  if (ao == 0) {
    return false;
  }
  if (ao->isGroup()) {
    return false;
  }
  if (selectedObjects().size() == 0) {
    return false;
  }
  if ((selectedObjects().size() == 1) && (*(selectedObjects().begin()) == ao)) {
    return false;
  }
  
  BattleObjects bos;
  //    ,   
  for (BattleObjects::const_iterator it = selectedObjects().begin();
  it != selectedObjects().end(); it++) {
    BattleObjects os = (*it)->objs(BattleObject::LeafsOnlySelectionType);
    for(BattleObjects::iterator it1 = os.begin(); it1 != os.end(); it1++) {
      bos.insert(*it1);
      if (bos.size() > 1)
        return true;
    }
  }
  return false;
}

void BUiState::orderSelection() {
  //    - 
  if (!canOrderSelection())
    return;
  BattleObject* ao = activeObject();
  QDateTime dt = bApp->battle()->battleInfo().modelTime;
  //    
  BattleObjects bos;
  for (BattleObjects::const_iterator it = selectedObjects().begin();
  it != selectedObjects().end(); it++) {
    if ((*it)->isGroup()) {
      BattleObjects os = (*it)->objs(BattleObject::LeafsOnlySelectionType);
      for(BattleObjects::const_iterator it1=os.begin(); it1!=os.end(); it1++) {
        bos.insert(*it1);
      }
    }
    else {
      bos.insert(*it);
    }
  }
  bos.erase(ao);
  //   
  BShipsOrder shipsOrder;
  double baseLatitude = bApp->battle()->seaMap()->originalCoordinates().y0();
  double longitudeCoef = BUnitsConverter::longitudeCoef(baseLatitude);
  std::for_each(bos.begin(), bos.end(), BShipsOrderWeight(shipsOrder,
    dt, ao, longitudeCoef));
  std::for_each(shipsOrder.begin(), shipsOrder.end(), 
    BShipsOrderNormalize(0, dt));
  std::stable_sort(shipsOrder.begin(), shipsOrder.end(), 
    std::ptr_fun(more_obj_data));
  //    
  std::for_each(shipsOrder.begin(), shipsOrder.end(), 
    BShipsOrderSet(ao, dt, baseLatitude, 0 /*, SHIPS_INTERVAL_MILES*/));
  //      
  std::for_each(shipsOrder.rbegin(), shipsOrder.rend(), 
    add_ai_chasing(ais()));
  ais()->addChasing( shipsOrder.begin()->first, ao);
}

void BUiState::removeFromSelection(BattleObject* object, bool exactly) {
  if (removeFromSelectionInternal(object, exactly)) {
    emit selectionChanged(BUiSelectionInfo(activeObject(), 
      &(selectedObjects())));
  }
  Q_ASSERT( ((_activeObject == 0) && (_selectedObjects.size() == 0)) ||
    ((_activeObject != 0) && (_selectedObjects.size() != 0)));
}

bool BUiState::removeSelectedObjectsInternal(const BattleObjects& objs) {
//  if (objs.size() == 0)
//    return false;
//  std::for_each(objs.begin(), objs.end(), 
//    std::bind1st(std::ptr_fun(this->*removeFromSelectionInternal), this));
  bool rez = false;
  for(BattleObjects::const_iterator it = objs.begin(); it != objs.end(); it++) {
    if (removeFromSelectionInternal(*it, true)) 
      rez = true;
  }
  return rez;
}
bool BUiState::removeFromSelectionInternal(BattleObject* object, bool exactly) {
  if (object == 0)
    return false;
  BattleObjects::const_iterator cit = _selectedObjects.find(object);
  if ( cit == _selectedObjects.end()) {
    return false;
  }
  if (exactly) {
    _selectedObjects.erase(object);
    if (object == _activeObject)
      setActiveObjectInternal(0);
  }
  else {
    BattleObjects bos = object->objs(BattleObject::AllSelectionType);
    for(BattleObjects::const_iterator it1 = bos.begin(); 
    it1 != bos.end(); it1++) {
      _selectedObjects.erase(*it1);
    }
    if (bos.find(_activeObject) != bos.end()) {
      setActiveObjectInternal(0);
    }
  }
  return true;
}

void BUiState::changeSelection(const BattleObjects& add, 
  const BattleObjects& remove) {
  bool rez1 = addSelectedObjectsInternal(add, true);
  bool rez2 = removeSelectedObjectsInternal(remove);
  if (add.size() == 1) {
    setActiveObjectInternal(*add.begin());
  }
  if ( rez1 || rez2) {
    emit selectionChanged(BUiSelectionInfo(activeObject(), 
      &(selectedObjects())));
  }
}

bool BUiState::canUngroupSelection() const {
  if (_selectedObjects.size() == 0) 
    return false;
  for(BattleObjects::const_iterator it = _selectedObjects.begin();
  it != _selectedObjects.end(); it++) {
    BattleObject* o = *it;
    if (o->isGroup()) {
      if (o->objs(BattleObject::TreeSelectionType).size() != 0) {
        return true;
      }
    }
    else {
      if (o->parent() != bApp->battle()->ships())
        return true;
    }
  }
  return false;
}

void BUiState::customEvent(QCustomEvent* e) {
  if (e->type() == BSelectionEventNumber) {
    BSelectionEvent* se = (BSelectionEvent*)e;
    changeSelection(se->newSelected(), se->unSelected());
  }
  else if (e->type() == BRegroupEventNumber) {
    BRegroupEvent* re = (BRegroupEvent*)e;
    BattleObject* t = bApp->battle()->ships()->find(re->target());
//    if (t == 0) {
//      //       
//      if (re->target() == bApp->battle()->ships()->name()) {
//        t = bApp->battle()->ships();
//      }
//    }
    if (!t->isGroup())
      t = t->parent();
    for(BattleObjects::const_iterator it = re->battleObjects().begin(); 
    it != re->battleObjects().end(); it++) {
        moveObject(*it, t);
    }
  }
}
