/******************************************************************************/
/* BUiState -   BattleFleet              */
/******************************************************************************/

#ifndef BUiState_H
#define BUiState_H
static const char buistate_h_id[]="@(#) $Id: buistate.h,v 1.12 2005/01/16 15:33:40 Oleksiy Balabay Exp $"; static const char* buistate_h_id_id=buistate_h_id;
/** \file buistate.h
  \brief  BUiState  BattleFleet

     BUiState  BattleFleet
*/
#include <qobject.h>

#include "battleobject.h"

extern const int BSelectionEventNumber;
extern const int BRegroupEventNumber;
class BattleObserver;
class BAIs;

struct BUiSelectionInfo {
  BUiSelectionInfo(BattleObject* aO, const BattleObjects* sO)
    :activeObject(aO),selectedObjects(sO){
  }
  BattleObject* activeObject;
  const BattleObjects* selectedObjects;
};

class is_parent_in: public std::unary_function<BattleObject*, bool> {
public:
  is_parent_in(const BattleObjects& selectedObjects, const BattleObject* root, 
    bool recursive):_os(selectedObjects), _root(root), _recursive(recursive) {
  }
  bool operator()(BattleObject* o) const {
    if (!_recursive) {
      return _os.find(o->parent()) != _os.end();
    }
    if (o->parent() == _root)
      return false;
    else if (_os.find(o->parent()) != _os.end())
      return true;
    else
      return operator()(o->parent());
  }
private:
  const BattleObjects& _os;
  const BattleObject* _root;
  bool _recursive;
};

class BSelectionEvent: public QCustomEvent {
public:
  BSelectionEvent(const BattleObjects& newSelected, 
    const BattleObjects& unSelected)
    :QCustomEvent(BSelectionEventNumber), _newSelected(newSelected),
    _unSelected(unSelected) {
  }
  const BattleObjects& newSelected() const {
    return _newSelected;
  }
  const BattleObjects& unSelected() const {
    return _unSelected;
  }
private:
  BattleObjects _newSelected;
  BattleObjects _unSelected;
};

class BRegroupEvent: public QCustomEvent {
public:
  BRegroupEvent(const BattleObjects& bos, const QString& target)
    :QCustomEvent(BRegroupEventNumber), _bos(bos), _target(target) {
  }
  const BattleObjects& battleObjects() const {
    return _bos;
  }
  const QString& target() const {
    return _target;
  }
private:
  BattleObjects _bos;
  QString _target;
};

class BUiState: public QObject {
  Q_OBJECT
public:
  enum BEditorMode {
    SelectMode,
    ScaleMode,
    ConnectMode
  };
public:
  BUiState(BattleObserver* observer, QObject* parent, const char* name);
  virtual ~BUiState();
  BattleObject* activeObject() const;
  void setActiveObject(BattleObject* object);
  void setActiveObjectAsSelection(BattleObject* object);
  const BattleObjects& selectedObjects() const;
  void setSelectedObjects(const BattleObjects& selObjs, bool exactly = false);
  void addSelectedObjects(const BattleObjects& selObjs, bool exactly = false);
  void changeSelection(const BattleObjects& add, const BattleObjects& remove);
  void setEditorMode(BEditorMode mode);
  BEditorMode editorMode() const;
  void removeObject(const QString& name);
  void removeSelection();
  void removeFromSelection(BattleObject* object, bool exactly = false);
  void moveObject(BattleObject* o, BattleObject* parent);
  void orderSelection();
  bool canOrderSelection() const;
  BAIs* ais() {
    return _ais;
  }
  void setAIs(BAIs* ais) {
    _ais = ais;
  }
  void setShowChasingConnections(bool visible) {
    emit showChasingConnections(visible);
  }
  bool canUngroupSelection() const;
public slots:
  void slotGroup();
  void slotUngroup();
signals:
  void selectionChanged(const BUiSelectionInfo& selectionInfo);
  void showChasingConnections(bool visible);
protected:
  virtual void customEvent(QCustomEvent* e);
private:
  QString newGroupName() const;
  bool setActiveObjectInternal(BattleObject* object);
  bool addSelectedObjectsInternal(const BattleObjects& selObjs, bool exactly);
  bool removeFromSelectionInternal(BattleObject* object, bool exactly);
  bool removeSelectedObjectsInternal(const BattleObjects& objs);
  bool setSelectedObjectsInternal(const BattleObjects& selObjs, bool exactly);
private:
  BEditorMode _mode;
  mutable int _groupSuffix;
  BattleObject* _activeObject;
  BattleObjects _selectedObjects;
  BattleObserver* _observer;
  BAIs* _ais;
};

#endif // BUiState_H
