/******************************************************************************/
/* BUnitsConsts -                                 */
/******************************************************************************/

#ifndef BUnitsConsts_H
#define BUnitsConsts_H
static const char bunitsconsts_h_id[]="@(#) $Id: bunitsconsts.h,v 1.5 2005/01/21 16:25:03 Oleksiy Balabay Exp $"; static const char* bunitsconsts_h_id_id=bunitsconsts_h_id;
/** \file bbunitsconsts.h
  \brief  BUnitsConsts  BattleFleet

         
     .
*/
#include <qstring.h>

#include <math.h>

enum BUnits {
  mmUnit = 0,
  cmUnit,
  mUnit,
  kmUnit,
  cableUnit,
  mileUnit,
  degreeUnit,
  knotUnit,
  kmperhUnit,
  inchUnit,
  _countUnits,
};

typedef struct BPresentationUnits {
  BUnits shipSize;
  BUnits distance;
  BUnits speed;
  BUnits armor;
  BUnits caliber;
} BPresentationUnits;

enum BPresentationValues {
  shipSizeValue = 0,
  distanceValue,
  timeValue,
  coordinateValue,
  speedValue,
  depthValue,
  numberValue,
  caliberValue,
  courseValue
};

///   "  "  . .., , 1985
const double EARTH_RADIUS_M = 12756000.0/2;
///   . -. .  ., .1 , , 1985.
const double MILE = 1852.0;

const double PI = 3.1415926535;

const QString S_OPTIONS = "Options";
const QString S_OPTIONS_UNITS = "Units";
const QString S_OPTIONS_UNITS_SHIP_SIZE = "ShipSize";
const QString S_OPTIONS_UNITS_DISTANCE = "Distance";
const QString S_OPTIONS_UNITS_SPEED = "Speed";
const QString S_OPTIONS_UNITS_ARMOR = "Armor";
const QString S_OPTIONS_UNITS_CALIBER = "Caliber";

class BUnitsConverter {
public:
  static double mToMiles(const double m) {
    return m/MILE;
  }
  static double mFromMiles(const double miles) {
    return miles*MILE;
  }
  static double degreesFromMiles(const double miles, 
    const double baseLatitude = 0) {
    return miles/(60.0*longitudeCoef(baseLatitude));
  }
  static double degreesToMiles(const double degrees, 
    const double baseLatitude = 0) {
    return longitudeCoef(baseLatitude)*degrees*60.0;
  }
  static double longitudeCoef(const double latitude) {
    double angleInRad = latitude*2*PI/360.0;
    return 2*PI*EARTH_RADIUS_M*cos(angleInRad)/(360*60*MILE);
  }
  static double angleToDegrees(const double angle) {
    return angle*180.0/PI;
  }
  static double angleFromDegrees(const double degrees) {
    return degrees*PI/180.0;
  }
  static void angle(const double angleDegrees, int* degrees,
    int* minutes, int* seconds, double* secondsDouble, int* sign) {
    int d; int m; int s; double sd; int si;
    si = (angleDegrees > 0)?1:((angleDegrees < 0)?-1:0);
    double fd = fabs(angleDegrees);
    d = static_cast<int>(floor(fd));
    int m_s = static_cast<int>((fd - d)*60*60);
    m = m_s/60;
    sd = (fd - d - m/60.0)*3600;//m_s%60;
    double p, f;
    f = modf(sd, &p);
    if (f<0.5)
      s = static_cast<int>(p);
    else {
      s = static_cast<int>(p) + 1;
      //     double 
      if (s == 60) {
        //     60,   60
        if (f > 0.99999) {
          s = 0;
          sd = 0;
          m = m + 1;
          if (m == 60) {
            m = 0;
            d = d + 1;
          }
        }
        else {
          s = 59;
        }
      }
    }
    if (degrees) *degrees = d;
    if (minutes) *minutes = m;
    if (seconds) *seconds = s;
    if (secondsDouble) *secondsDouble = sd;
    if (sign) *sign = si;
  }
};

#endif
