/******************************************************************************/
/* BProperty -                 */
/******************************************************************************/

#ifndef BProperty_H
#define BProperty_H
static const char bproperty_h_id[]="@(#) $Id: bproperty.h,v 1.6 2005/01/05 11:47:27 Oleksiy Balabay Exp $"; static const char* bproperty_h_id_id=bproperty_h_id;
/** \file bproperty.h
  \brief  BProperty  

     BProperty  
*/
#include <qstring.h>
#include <qvariant.h>
#include <vector>

class BException;

class BProperty {
public:
  enum BPropertyTypes {
	  IntType,
	  UIntType,
	  DoubleType,
    StringType,
	  CourseType,
    LongitudeType,
    LatitudeType,
    StringListType
  };
  BProperty(BPropertyTypes type, const QString& key, const QString& name, 
	  const QVariant& value, const QVariant& options = QVariant());
  QString name() const { return _name; }
  QString key() const { return _key; }
  QVariant value() const { return _value; } 
  QVariant options() const { return _options; } 
  void setValue(const QVariant& v) { _value = v; }
  void setOptions(const QVariant& o) { _options = o; }
  BPropertyTypes type() const { return _type; }
  virtual ~BProperty();
  bool isEditable() const {
    return _editable;
  }
  void setEditable(bool editable) {
    _editable = editable;
  }
  void setWhatsThisText(const QString& whatsThisTxt) {
    _whatsThisText = whatsThisTxt;
  }
  const QString& whatsThisText() const {
    return _whatsThisText;
  }
private:
  BPropertyTypes _type;
  QString _key;
  QString _name;
  QVariant _value;
  QVariant _options;
  bool _editable;
  QString _whatsThisText;
};

class BProperties {
  typedef std::vector<BProperty*> BProps;
public:
  typedef BProps::iterator iterator;
  typedef BProps::const_iterator const_iterator;
public:
  BProperties();
  virtual ~BProperties();
  void clear();
  BProperty& property(const QString& key) const;
  void addProperty(BProperty* property);
  iterator begin() { return _props.begin(); }
  iterator end() { return _props.end(); }
  const_iterator begin() const { return _props.begin(); }
  const_iterator end() const { return _props.end(); }
  int size() const { return _props.size(); }
  void setAllEditable(bool editable) {
    for(BProperties::iterator it = begin(); it != end();
    it++) {
      (*it)->setEditable(editable);
    }
  }
private:
  BProps _props;
};

#endif // BProperty_H
