/******************************************************************************/
/* BPropertyItems -   BattleFleet  (impl.)            */
/******************************************************************************/
static const char bpropertyitems_cpp_id[]="@(#) $Id: bpropertyitems.cpp,v 1.7 2005/02/15 16:43:35 Oleksiy Balabay Exp $"; static const char* bpropertyitems_cpp_id_id=bpropertyitems_cpp_id;

#include <qvariant.h> 
#include <qhbox.h>
#include <qlineedit.h>

#include "bpropertyitems.h"
#include "bunitsconsts.h"
#include "buiutils.h"
#include "bapplication.h"

/** \file bpropertyitems.cpp
*/

BPropertyCourseItem::BPropertyCourseItem( PropertyList *l, PropertyItem *after, PropertyItem *prop, const QString &propName )
    : PropertyItem( l, after, prop, propName )
{
    box = new QHBox( listview->viewport() );
    box->hide();
    lined = new QLineEdit( box );
//    button = new QPushButton( "...", box );
//    setupStyle( button );
//    button->setFixedWidth( 20 );
    box->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
    box->setLineWidth( 2 );
    lined->setFrame( FALSE );
    lined->setReadOnly( TRUE );
    box->setFocusProxy( lined );
    box->installEventFilter( listview );
    lined->installEventFilter( listview );
//    button->installEventFilter( listview );
//    connect( button, SIGNAL( clicked() ),
//	     this, SLOT( getFont() ) );
}

void BPropertyCourseItem::createChildren()
{
    PropertyItem *i = this;
    int bl = 0;
    int ul = 359;
    i = new PropertyIntItem( listview, i, this, tr( "Degrees" ), FALSE,
      &bl, &ul);
    i->setEditable(isEditable());
    i->setWhatsThisText(whatsThisText());
    addChild( i );
    bl = 0;
    ul = 59;
    i = new PropertyIntItem( listview, i, this, tr( "Minutes" ), FALSE, 
      &bl, &ul );
    i->setEditable(isEditable());
    i->setWhatsThisText(whatsThisText());
    addChild( i );
    i = new PropertyIntItem( listview, i, this, tr( "Seconds" ), FALSE, 
      &bl, &ul );
    i->setEditable(isEditable());
    i->setWhatsThisText(whatsThisText());
    addChild( i );
}

void BPropertyCourseItem::initChildren()
{
    PropertyItem *item = 0;
    int d, m, s, si;
    BUnitsConverter::angle(BUnitsConverter::angleToDegrees(val.toDouble()),
      &d, &m, &s, 0, &si);
    for ( int i = 0; i < childCount(); ++i ) {
	    item = PropertyItem::child( i );
	    if ( item->name() == tr( "Degrees" ) )
	      item->setValue( d*si );
	    else if ( item->name() == tr( "Minutes" ) )
	      item->setValue( m );
	    else if ( item->name() == tr( "Seconds" ) )
	      item->setValue( s );
    }
}

BPropertyCourseItem::~BPropertyCourseItem()
{
    delete (QHBox*)box;
}

void BPropertyCourseItem::showEditor()
{
  if (!isEditable())
    return;
  PropertyItem::showEditor();
  placeEditor( box );
  if ( !box->isVisible() || !lined->hasFocus() ) {
	  box->show();
	  setFocus( lined );
  }
}

void BPropertyCourseItem::hideEditor()
{
  if (!isEditable())
    return;
  PropertyItem::hideEditor();
  box->hide();
}

void BPropertyCourseItem::setValue( const QVariant &v )
{
  double nv = v.toDouble();
  while (nv < 0)
    nv = 2*PI + nv;
  if (nv > 2*PI) {
    int n = nv/(2*PI);
    nv = nv - n*2*PI;
  }
  QVariant v1 = v;
  if (nv != v.toDouble())
    v1 = QVariant(nv);

  if ( value() == v1 )
    return;
  QString s = bApp->uiUtils()->toString(v1.toDouble(), courseValue, 0);
  if (nv == 0) {
    s += " " +tr("N");
  }
  else if (nv < PI/2) {
    s += " " + tr("NE");
  }
  else if (nv == PI/2) {
    s += " " + tr("E");
  }
  else if (nv < PI) {
    s += " " + tr("SE");
  }
  else if (nv == PI) {
    s += " " + tr("S");
  }
  else if (nv < 3*PI/2) {
    s += " " + tr("SW");
  }
  else if (nv == 3*PI/2) {
    s += " " + tr("W");
  }
  else {
    s += " " + tr("NW");
  }
  setText(1, s);
  lined->setText( s  );
  PropertyItem::setValue( v1 );
  initChildren();
}

bool BPropertyCourseItem::hasSubItems() const
{
    return TRUE;
}

void BPropertyCourseItem::childValueChanged( PropertyItem *child )
{
  int d, m, s, si;
  BUnitsConverter::angle(BUnitsConverter::angleToDegrees(val.toDouble()),
    &d, &m, &s, 0, &si);
  d = d*si;
  if ( child->name() == tr( "Degrees" ) )
	  d = child->value().toInt();
  else if ( child->name() == tr( "Minutes" ) )
	  m = child->value().toInt();
  else if ( child->name() == tr( "Seconds" ) )
	  s = child->value().toInt();
  if (d < 0) {
    m = m*(-1);
    s = s*(-1);
  }
  setValue( BUnitsConverter::angleFromDegrees(d+m/60.0+s/(60.0*60)));
  notifyValueChange();
}

BPropertyCoordinateItem::BPropertyCoordinateItem( 
  PropertyList *l, PropertyItem *after, PropertyItem *prop, 
  const QString &propName, BSeaMap::BCoordinatesType type )
  : PropertyItem( l, after, prop, propName ), _type(type)
{
  box = new QHBox( listview->viewport() );
  box->hide();
  lined = new QLineEdit( box );
  box->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
  box->setLineWidth( 2 );
  lined->setFrame( FALSE );
  lined->setReadOnly( TRUE );
  box->setFocusProxy( lined );
  box->installEventFilter( listview );
  lined->installEventFilter( listview );
}

void BPropertyCoordinateItem::createChildren()
{
  PropertyItem *i = this;
  int dll = (_type == BSeaMap::longitude)?0:0;
  int dhl = (_type == BSeaMap::longitude)?179:89;
 
  i = new PropertyListItem(listview, i, this, tr( "Hemisphere" ), false);
  ((PropertyListItem *)i)->addItem(
    (_type == BSeaMap::longitude)?tr("E"):tr("N"));
  ((PropertyListItem *)i)->addItem(
    (_type == BSeaMap::longitude)?tr("W"):tr("S"));
  i->setEditable(isEditable());
  i->setWhatsThisText(whatsThisText());
  addChild( i );

  i = new PropertyIntItem( listview, i, this, tr( "Degrees" ), TRUE, &dll, &dhl);
  i->setEditable(isEditable());
  i->setWhatsThisText(whatsThisText());
  addChild( i );
  int bl = 0;
  int ul = 59;
  i = new PropertyIntItem( listview, i, this, tr( "Minutes" ), FALSE, 
    &bl, &ul );
  i->setEditable(isEditable());
  i->setWhatsThisText(whatsThisText());
  addChild( i );
  i = new PropertyIntItem( listview, i, this, tr( "Seconds" ), FALSE, 
    &bl, &ul );
  i->setEditable(isEditable());
  i->setWhatsThisText(whatsThisText());
  addChild( i );
}

void BPropertyCoordinateItem::initChildren()
{
  PropertyItem *item = 0;
  int d, m, s, si;
  BUnitsConverter::angle(val.toDouble(), &d, &m, &s, 0, &si);
  for ( int i = 0; i < childCount(); ++i ) {
	  item = PropertyItem::child( i );
    if ( item->name() == tr( "Hemisphere" ) ) {
      ((PropertyListItem*)item)->setCurrentItem(
        (val.toDouble() < 0)?1:0);
      item->setText(1, ((PropertyListItem*)item)->currentItem());
    }
	  if ( item->name() == tr( "Degrees" ) )
	    item->setValue( d );
	  else if ( item->name() == tr( "Minutes" ) )
	    item->setValue( m );
	  else if ( item->name() == tr( "Seconds" ) )
	    item->setValue( s );
  }
}

BPropertyCoordinateItem::~BPropertyCoordinateItem()
{
  delete (QHBox*)box;
}

void BPropertyCoordinateItem::showEditor()
{
  if (!isEditable())
    return;
  PropertyItem::showEditor();
  placeEditor( box );
  if ( !box->isVisible() || !lined->hasFocus() ) {
	  box->show();
	  setFocus( lined );
  }
}

void BPropertyCoordinateItem::hideEditor()
{
  if (!isEditable())
    return;
  PropertyItem::hideEditor();
  box->hide();
}

void BPropertyCoordinateItem::setValue( const QVariant &v )
{
    if ( value() == v )
    	return;
    //   
    int d, m, s, si;
    BUnitsConverter::angle(v.toDouble(),
      &d, &m, &s, 0, &si);
    d = d*si;
    if (si < 0) {
      m = m*(-1);
      s = s*(-1);
    }
    double v1 = d+m/60.0+s/(60.0*60);
    QString st = bApp->uiUtils()->toString(v1, coordinateValue, _type);
    setText(1, st);
    lined->setText( st );
    PropertyItem::setValue( v );
    initChildren();
//    notifyValueChange();
}

bool BPropertyCoordinateItem::hasSubItems() const
{
    return TRUE;
}

void BPropertyCoordinateItem::childValueChanged( PropertyItem *child )
{
  int d, m, s, hs;
  BUnitsConverter::angle(val.toDouble(),
    &d, &m, &s, 0, &hs);
  if (hs == 0)
    hs = 1;
//  d = d*si;
  if ( child->name() == tr( "Degrees" ) )
	  d = child->value().toInt();
  else if ( child->name() == tr( "Minutes" ) )
	  m = child->value().toInt();
  else if ( child->name() == tr( "Seconds" ) )
	  s = child->value().toInt();
  else if ( child->name() == tr( "Hemisphere" ) ) {
    hs = (((PropertyListItem*)child)->currentIntItem() == 0)?1:-1;
  }
  d*=hs;
  m*=hs;
  s*=hs;
  setValue(d+m/60.0+s/(60.0*60));
  notifyValueChange();
}
