/******************************************************************************/
/* BCFinder -     (impl.)            */
/******************************************************************************/
static const char bcfinder_cpp_id[]="@(#) $Id: bcfinder.cpp,v 1.5 2005/03/07 15:34:40 Oleksiy Balabay Exp $"; static const char* bcfinder_cpp_id_id=bcfinder_cpp_id;

#include <math.h>

#include "bcfinder.h"

/** \file bcfinder.cpp
  \brief  BCFinder   (impl.)

     BCFinder  
*/

/** \class BFinder
  \brief      ""

   .   _finder.jpg.     
    ,       
.

    :   B   ,    
 B   vk   c ( ).  
 B,     .  : 
 B   r (    ), 
 maxV,   minV,   aMax,
  aMin.

         :
  -      r   
  -   ,    
  -    < pi/2

     .      3 . 
   -     B
   -        
   -    ,  O2  
  

        ,  x   
 .

     O1, O2   .  
  .

      ,   ,
    .

      .

   ,     .
*/

/** \brief 
*/
BCFinder::BCFinder(){

}

/** \brief 
*/
BCFinder::~BCFinder(){

}

BFinder::PathFinderOutput 
BCFinder::findPath(const BCFinder::PathFinderInput& pInput) {
  _input = pInput;
  PathFinderOutput pfout;
  pfout.countElements = 0;
  transformToAO();
  if (!check1())
    return pfout;
  transformToBC();
  calcAC();
  calcO1();
  calcO2();
  if (!check2())
    return pfout;
  calcAngle1();
  calcAngle2();
  calcLength();

  PathProgramDatas v;
  SpeedProgramIn spin;
  spin.v0 = _input.b.v;
  spin.vk = _input.a.v;
  spin.T = _input.t;
  spin.l = _l1+_l2+_l3;
  spin.aMax = _input.pR.maxA;
  spin.aMin = _input.pR.minA;
  spin.vMax = _input.pR.maxV;
  spin.vMin = _input.pR.minV;
  spin.r = _input.pR.minR;
  bool speedRez = speedProgram(spin, v);
  if (!speedRez)
    return pfout;
  PathProgramDatas v1 = addTurnPrograms(v);
  pfout.countElements = v1.size();
  for(int i = 0; i <= static_cast<int>(v1.size()); i++) {
    pfout.prgs[i] = v1[i];
  }
  return pfout;
}

BMATH_EXPORT BFinder::PathFinderOutput findCPath(
  const BFinder::PathFinderInput& pInput) {
  BCFinder pf;
  return pf.findPath(pInput);
}

void BCFinder::transformToAO() {
  _inputAO = _input;
  //    
  //  .   A
  QWMatrix m1(1, 0, 0, 1, _input.a.x, _input.a.y);
  //  x   
  QWMatrix m2(cos(_input.a.c - Pi/2), -sin(_input.a.c - Pi/2), 
              sin(_input.a.c - Pi/2), cos(_input.a.c - Pi/2), 0, 0);
  _mAO = m2*m1;
  _inputAO.a.c = 0;
  _inputAO.a.x = 0;
  _inputAO.a.y = 0;
  QWMatrix inv = _mAO.invert();

  inv.map(_input.b.x, _input.b.y, &(_inputAO.b.x), &(_inputAO.b.y));
  _inputAO.b.c = _input.a.c - _input.b.c; 
}

bool BCFinder::check1() const {
/*  QWMatrix inv = _mAO.invert();
  double x, y;
  inv.map(_inputAO.b.x, _inputAO.b.y, &x, &y);
  if (fabs(y)>_input.pR.minR)
    return false;
*/
  if (fabs(_inputAO.b.y) > _input.pR.minR)
    return false;
  return true;
}

void BCFinder::transformToBC() {
  _inputBC = _inputAO;
  double x,y;
  {
    //    
    //  .   B
    QWMatrix m1(1, 0, 0, 1, _inputAO.b.x, _inputAO.b.y);
    //  x   B
    QWMatrix m2(cos(_inputAO.b.c), sin(_inputAO.b.c), 
                -sin(_inputAO.b.c), cos(_inputAO.b.c), 0, 0);
    QWMatrix mB = m2*m1;
    mB.map(0,(_inputAO.b.y<0)?(_inputAO.pR.minR):(-_inputAO.pR.minR),  &x, &y);
  }
  //    
  //  .    B
  _mBC = QWMatrix(1, 0, 0, 1, x, y);
  QWMatrix inv = _mBC.invert();
  inv.map(_inputAO.a.x, _inputAO.a.y, &(_inputBC.a.x), &(_inputBC.a.y));
  inv.map(_inputAO.b.x, _inputAO.b.y, &(_inputBC.b.x), &(_inputBC.b.y));
}

void BCFinder::calcAC() {
  double yAO = _inputBC.a.y;
//  double xAO = _inputBC.a.x;
  double r = _inputBC.pR.minR;
  _yAC = (yAO > 0)?(r+yAO):(-r+yAO);
  _xAC = sqrt(4*r*r - (_yAC*_yAC));
}

void BCFinder::calcO1() {
  _xO1 = _xAC/2;
  _yO1 = _yAC/2;
}

void BCFinder::calcO2() {
  _xO2 = _xAC;
  _yO2 = _inputBC.a.y;
}

bool BCFinder::check2() const {
  if (_xO1 < _inputBC.b.x) 
    return false;
  if (fabs(_yO1) > fabs(_inputBC.b.y)) 
    return false;
  if (_xO2 > _inputBC.a.x) 
    return false;
  if (_inputBC.b.x < 0) 
    return false;
  return true;
}

void BCFinder::calcAngle1() {
  double r = _inputBC.pR.minR;
  double cosBO = _inputBC.b.x/r;
  double sinBO = _inputBC.b.y/r;
  double angleB = angle(sinBO, cosBO);
  double cosO1 = _xO1/r;
  double sinO1 = _yO1/r;
  double angleO1 = angle(sinO1, cosO1);
  _angle1 = turnAngle(angleB, angleO1, _inputBC.a.y < 0, 0);
}

void BCFinder::calcAngle2() {
  double r = _inputBC.pR.minR;
  double cosO1 = (_xO1 - _xAC)/r;
  double sinO1 = (_yO1 - _yAC)/r;
  double angleO1 = angle(sinO1, cosO1);
  double cosO2 = (_xO2 - _xAC)/r;
  double sinO2 = (_yO2 - _yAC)/r;
  double angleO2 = angle(sinO2, cosO2);
  _angle2 = turnAngle(angleO1, angleO2, _inputBC.a.y >= 0, 0);
}

void BCFinder::calcLength() {
  double r = _inputBC.pR.minR;
  _l1 = _angle1*r/_inputBC.pR.lk;
  _l2 = _angle2*r/_inputBC.pR.lk;
  _l3 = _inputBC.a.x - _xO2;
}

BCFinder::PathProgramDatas BCFinder::addTurnPrograms(
  const BCFinder::PathProgramDatas& vv) {
  PathProgramDatas vt;
  //      _l1 (   )
  //    _l2
  //    _l3
  double angle1 = _angle1;
  //    1
  if (_inputBC.a.y < 0)
    angle1 = -angle1;
  double angle2 = _angle2;
  if (_inputBC.a.y >= 0)
    angle2 = -angle2;
  int t2Start = vv.size();
  //    -   2
  std::vector<PathProgramData> vv1;
  if (angle2 != 0) {
    double tA = t_s(_l1, _input.b.v, _input.t, vv);
    for(int i = 0; i < static_cast<int>(vv.size()); i++) {
      double t1 = _input.t;
      if (i != static_cast<int>(vv.size())-1) {
        t1 = vv[i+1].t;
      }
      if ((vv[i].t <= tA)&&(t1>tA)) {
        t2Start = i + 1;
        PathProgramData ppd = vv[i];
        double startV = _input.b.v;
        if ( i != 0) {
          startV = vv[i-1].v;
        }
        ppd.v = startV + ppd.a*(tA - ppd.t);
        vv1.push_back(ppd);
        ppd.v = vv[i].v;
        ppd.t = tA;
        vv1.push_back(ppd);
      }
      else {
        vv1.push_back(vv[i]);
      }
    }
  }
  else {
    vv1 = vv;
  }
  double curS, curT, curAngle1, curAngle2;
  curS = 0; curT = 0; curAngle1 = 0; curAngle2 = 0;
  double T = 0;
  double V = 0;
  for(int i = 0; i <static_cast<int>(vv1.size()); i++) {
    if ( i == static_cast<int>(vv1.size())) {
      T = _input.t;
    }
    else {
      T = vv1[i+1].t;
    }
    if (i == 0) {
      V = _input.b.v;
    }
    else {
      V = vv1[i-1].v;
    }
    double dt = T - vv1[i].t;
    double ds = V*dt + vv1[i].a*dt*dt/2;
    curS += ds;
    PathProgramData pd = vv1[i];
    if (curS < _l1) {
      double da = angle1*ds/_l1;
      if (fabs(curAngle1 + da) > fabs(angle1)) {
        da = angle1 - curAngle1;
      }
      curAngle1 += da;
      pd.angle = da;
    }
    else {
      if (curAngle1 != angle1) {
        pd.angle = angle1  -  curAngle1;
        curAngle1 = angle1;
      }
      else {
        pd.angle = 0;
      }
    }
    if ((_l2 != 0) && (t2Start <= i)&& (curS < _l1+_l2)) {
      double da = angle2*ds/_l2;
      if (fabs(curAngle2 + da) > fabs(angle2)) {
        da = angle2 - curAngle2;
      }
      curAngle2 += da;
      pd.angle = da;

 //     curAngle2 = angle2*(1- (curS - (_l1) )/_l2);
 //     pd.angle = curAngle2;
    }
    else {
      if ( ((curAngle2 != 0) ) && (curAngle2 != angle2)) {
        pd.angle = angle2 - curAngle2;
        curAngle2 = angle2;
      }
      else if ( (curAngle2 == 0) && (t2Start == i) && (curAngle2 != angle2)) {
        pd.angle = angle2 - curAngle2;
        curAngle2 = angle2;
      }
    }
    vt.push_back(pd);
  }
  return vt;
}
