/******************************************************************************/
/* BCollisionEvader -   BMath  (impl.)            */
/******************************************************************************/
static const char bcollisionevader_cpp_id[]="@(#) $Id: bcollisionevader.cpp,v 1.1 2005/01/15 16:44:52 Oleksiy Balabay Exp $"; static const char* bcollisionevader_cpp_id_id=bcollisionevader_cpp_id;

#include <math.h>

#include "bcollisionevader.h"

/** \file bcollisionevader.cpp
  \brief  BCollisionEvader  BMath (impl.)

     BCollisionEvader  BMath
*/

BMATH_EXPORT BFinder::PathFinderOutput evadeCollision(
  const BFinder::PathFinderInput& in, 
  const BCollisionEvader::CollisionInput& colIn) {
  BCollisionEvader ce;
  return ce.evadeCollision(in, colIn);
}
/** \class BCollisionEvader
  \brief 


*/
/** \brief 
*/
BCollisionEvader::BCollisionEvader(){

}

/** \brief 
*/
BCollisionEvader::~BCollisionEvader(){

}

BFinder::PathFinderOutput BCollisionEvader::evadeCollision(
  const BFinder::PathFinderInput& pInput, 
  const BCollisionEvader::CollisionInput& colIn) {
  _input = pInput;
  _colIn = colIn;
  transformToAO();
  setAngles();
  setCourse();
  setSide();
  calcPrograms();
  return _output;
} 


void BCollisionEvader::transformToAO() {
  _inputAO = _input;
  //    
  //  .   A
  QWMatrix m1(1, 0, 0, 1, _input.a.x, _input.a.y);
  //  x   
  QWMatrix m2(cos(_input.a.c - Pi/2), -sin(_input.a.c - Pi/2), 
              sin(_input.a.c - Pi/2), cos(_input.a.c - Pi/2), 0, 0);
  _mAO = m2*m1;
  _inputAO.a.c = 0;
  _inputAO.a.x = 0;
  _inputAO.a.y = 0;
  QWMatrix inv = _mAO.invert();

  inv.map(_input.b.x, _input.b.y, &(_inputAO.b.x), &(_inputAO.b.y));
  _inputAO.b.c = _input.a.c - _input.b.c; 
}

void BCollisionEvader::setAngles() {
  double lA2 = _colIn.lengthA/2;
  double wA2 = _colIn.widthA/2;
  double r = sqrt(lA2*lA2 + wA2*wA2);
  lA2/=r;
  wA2/=r;
  _angleFront = angle(wA2, lA2);
  _angleLeft = angle(wA2, -lA2);
  _angleRear = angle(-wA2, -lA2);
  _angleRight = angle(-wA2, lA2);
}

void BCollisionEvader::setCourse() {
  double angleB = _inputAO.b.c;
  while (angleB < 0) {
    angleB+=2*Pi;
  }
  while (angleB >=2*Pi) {
    angleB-=2*Pi;
  }
  if (angleB <= _angleFront)
    _c = FrontCourse;
  else if (angleB <= _angleLeft)
    _c = LeftCourse;
  else if (angleB <= _angleRear)
    _c = RearCourse;
  else if (angleB <= _angleRight)
    _c = RightCourse;
  else
    _c = FrontCourse;
}

void BCollisionEvader::setSide() {
  double xB = _inputAO.b.x;
  double yB = _inputAO.b.y;
  double r = sqrt(xB*xB+yB*yB);
  xB/=r;
  yB/=r;
  double angleB = angle(yB, xB);
  if (angleB <= _angleFront)
    _s = FrontSide;
  else if (angleB <= _angleLeft)
    _s = LeftSide;
  else if (angleB <= _angleRear)
    _s = RearSide;
  else if (angleB <= _angleRight)
    _s = RightSide;
  else
    _s = FrontSide;
}

void BCollisionEvader::calcPrograms() {
  _output.countElements = 0;
  switch (_s) {
  case FrontSide:
    {
      calcFrontSide();
      break;
    }
  case LeftSide:
    {
      calcLeftSide();
      break;
    }
  case RearSide:
    {
      calcRearSide();
      break;
    }
  case RightSide:
    {
      calcRightSide();
      break;
    }
  }
}

void BCollisionEvader::calcFrontSide() {
  PathProgramData p;
  p.a = _input.pR.maxA;
  p.r = _input.pR.minR;
  p.v = 0;
  p.angle = 0;
  switch (_c) {
  case FrontCourse:
    {
      _output.countElements = 1;
      _output.prgs[0] = p;
      return;
    };
  case RearCourse:
    {
      p.angle = _angleFront;
      _output.countElements = 1;
      _output.prgs[0] = p;
      return;
    };
  case LeftCourse:
    {
      p.angle = Pi/2;
      _output.countElements = 1;
      _output.prgs[0] = p;
      return;
    };
  case RightCourse:
    {
      p.angle = -Pi/2;
      _output.countElements = 1;
      _output.prgs[0] = p;
      return;
    };
  }
}

void BCollisionEvader::calcLeftSide() {
  PathProgramData p;
  p.a = _input.pR.maxA;
  p.r = _input.pR.minR;
  p.v = 0;
  p.angle = 0;
  switch (_c) {
  case FrontCourse:
    {
      return;
    };
  case RearCourse:
    {
      p.angle = _angleFront;
      _output.countElements = 1;
      _output.prgs[0] = p;
      return;
    };
  case LeftCourse:
    {
      return;
    };
  case RightCourse:
    {
      return;
    };
  }
}

void BCollisionEvader::calcRearSide() {
  return;
}

void BCollisionEvader::calcRightSide() {
  PathProgramData p;
  p.a = _input.pR.maxA;
  p.r = _input.pR.minR;
  p.v = 0;
  p.angle = 0;
  switch (_c) {
  case FrontCourse:
    {
      p.angle = -_angleFront;
      _output.countElements = 1;
      _output.prgs[0] = p;
      return;
    };
  case RearCourse:
    {
      p.angle = _angleFront;
      _output.countElements = 1;
      _output.prgs[0] = p;
      return;
    };
  case LeftCourse:
    {
      p.angle = -Pi/2;
      _output.countElements = 1;
      _output.prgs[0] = p;
      return;
    };
  case RightCourse:
    {
      return;
    };
  }
}
