/******************************************************************************/
/* BCollisionEvader -   BMath              */
/******************************************************************************/

#ifndef BCollisionEvader_H
#define BCollisionEvader_H
static const char bcollisionevader_h_id[]="@(#) $Id: bcollisionevader.h,v 1.1 2005/01/15 16:44:52 Oleksiy Balabay Exp $"; static const char* bcollisionevader_h_id_id=bcollisionevader_h_id;
/** \file bcollisionevader.h
  \brief  BCollisionEvader  BMath

     BCollisionEvader  BMath
*/
#include "bmath.h"
#include "bpathfinder.h"

class BCollisionEvader: public BFinder {
public:
  typedef struct CollisionInput {
    double lengthA;
    double lengthB;
    double widthA;
    double widthB;
  } CollisionInput;
  BCollisionEvader();
  virtual ~BCollisionEvader();
  PathFinderOutput evadeCollision(const PathFinderInput& pInput, 
    const CollisionInput& colIn);
private:
  enum Sides {
    FrontSide,
    LeftSide,
    RightSide,
    RearSide
  };
  enum Courses {
    FrontCourse,
    LeftCourse,
    RightCourse,
    RearCourse
  };
private:
  void transformToAO();
  void setAngles();
  void setCourse();
  void setSide();
  void calcPrograms();
  void calcFrontSide();
  void calcLeftSide();
  void calcRearSide();
  void calcRightSide();
private:
  PathFinderInput _input;
  CollisionInput _colIn;
  PathFinderInput _inputAO;
  QWMatrix _mAO;
  Courses _c;
  Sides _s;
  double _angleFront;
  double _angleLeft;
  double _angleRear;
  double _angleRight;
  PathFinderOutput _output;
};

extern "C" {
  BMATH_EXPORT BFinder::PathFinderOutput evadeCollision(
    const BFinder::PathFinderInput& in, 
    const BCollisionEvader::CollisionInput& colIn); 
}

#endif // BCollisionEvader_H
