/******************************************************************************/
/* BCollisionsAlg -      (impl.)            */
/******************************************************************************/
static const char bcollisionsalg_cpp_id[]="@(#) $Id: bcollisionsalg.cpp,v 1.3 2005/01/21 16:25:03 Oleksiy Balabay Exp $"; static const char* bcollisionsalg_cpp_id_id=bcollisionsalg_cpp_id;

#include <qwmatrix.h>
#include <math.h>

#include "bcollisionsalg.h"

/** \file bcollisionsalg.cpp
  \brief     (impl.)

*/

/** ,       A,  X
    A
*/
bool collidesImpl(const BCollisionsData& data) {
  //    A  B
  double aX[4];
  double aY[4];
  double bX[4];
  double bY[4];
//  double cosCA = cos(data.cA);
//  double sinCA = sin(data.cA);
  double cosCB = cos(data.cB);
  double sinCB = sin(data.cB);

  aX[0] = -data.lA/2.0;
  aY[0] = +data.wA/2.0;

  aX[1] = +data.lA/2.0;
  aY[1] = +data.wA/2.0;

  aX[2] = +data.lA/2.0;
  aY[2] = -data.wA/2.0;

  aX[3] = -data.lA/2.0;
  aY[3] = -data.wA/2.0;

  QWMatrix m1(1, 0, 0, 1, data.xB, data.yB);
  //  x   
  QWMatrix m2(cosCB, sinCB, 
              -sinCB, cosCB, 0, 0);
  QWMatrix m = (m2*m1);//.invert();
  m.map(-data.lB/2.0, +data.wB/2.0, &(bX[0]), &(bY[0]));
  m.map(+data.lB/2.0, +data.wB/2.0, &(bX[1]), &(bY[1]));
  m.map(+data.lB/2.0, -data.wB/2.0, &(bX[2]), &(bY[2]));
  m.map(-data.lB/2.0, -data.wB/2.0, &(bX[3]), &(bY[3]));

  for(int i = 0; i < 4; i++) {
    if ((aX[0]<=bX[i])&&(bX[i]<=aX[1])) {
      if ((aY[3]<=bY[i])&&(bY[i]<=aY[0])) { //   
        return true;
      }
    }
  }
  //   . .  3, . 13
  {
    double sAX[4];
    double sAY[4];
    double sBX[4];
    double sBY[4];
    for (int i = 0; i < 4; i++) {
      sAX[i] = aX[(i+1)%4] - aX[i];
      sAY[i] = aY[(i+1)%4] - aY[i];
      sBX[i] = bX[(i+1)%4] - bX[i];
      sBY[i] = bY[(i+1)%4] - bY[i];
    }
    for(int j = 0; j < 4; j++) {
      double det = sBX[j]*(-sAY[j])-sBY[j]*(-sAX[j]);
      double s = (aX[j]-bX[j])*(-sAY[j])-(aY[j]-bY[j])*(-sAX[j]);
      s/=det;
      double t = sBX[j]*(aY[j]-bY[j])-sBY[j]*(aX[j]-bX[j]);
      t/=det;
      if ((s >=0)&&(s<=1)&&(t>=0)&&(t<=1)) {
        return true;
      }
    }
  }
  return false;
}

extern "C" {

  BMATH_EXPORT bool collides(const BCollisionsData& data) {
    //    
    //  . .    
    QWMatrix m1(1, 0, 0, 1, data.xA, data.yA);
    //  x   
    QWMatrix m2(cos(data.cA - Pi/2), -sin(data.cA - Pi/2), 
                sin(data.cA - Pi/2), cos(data.cA - Pi/2), 0, 0);
    QWMatrix mTurn = (m2*m1).invert();
    BCollisionsData d = data;
    d.cA = 0;
    d.cB = data.cA - data.cB;  
    d.xA = 0;
    d.yA = 0;
    mTurn.map(data.xB, data.yB, &(d.xB), &(d.yB));
    // ,    
    // TODO:       
    if (collidesImpl(d)) {
      return true;
    }
    else { 
      BCollisionsData dataB;
      dataB.xA = data.xB; 
      dataB.yA = data.yB;
      dataB.xB = data.xA; 
      dataB.yB = data.yA;
      dataB.lA = data.lB; 
      dataB.wA = data.wB;
      dataB.lB = data.lA; 
      dataB.wB = data.wA;
      dataB.cA = data.cB; 
      dataB.cB = data.cA;
      //    
      //  . .    
      QWMatrix m1(1, 0, 0, 1, dataB.xA, dataB.yA);
      //  x   
      QWMatrix m2(cos(dataB.cA - Pi/2), -sin(dataB.cA - Pi/2), 
                  sin(dataB.cA - Pi/2), cos(dataB.cA - Pi/2), 0, 0);
      QWMatrix mTurn = (m2*m1).invert();
      BCollisionsData d = dataB;
      d.cA = 0;
      d.cB = dataB.cA - dataB.cB;  
      d.xA = 0;
      d.yA = 0;
      mTurn.map(dataB.xB, dataB.yB, &(d.xB), &(d.yB));
      return collidesImpl(d);
    }
  }
}

