/******************************************************************************/
/* BMathTurn -   BMath  (impl.)            */
/******************************************************************************/
static const char bmathturn_cpp_id[]="@(#) $Id: bmathturn.cpp,v 1.2 2005/01/14 16:20:42 Oleksiy Balabay Exp $"; static const char* bmathturn_cpp_id_id=bmathturn_cpp_id;

#include <math.h>

#include "bmathturn.h"
extern "C" {
#ifdef Q_WS_WIN
  bool __stdcall bf_turn_accelerate(
#endif
#ifdef Q_WS_X11
  bool bf_turn_accelerate(
#endif
   double* x ,
   double* y ,
   double* c,
   double* v,
   double* vcomp,
   double* a,
   double* r,
   double t,
   double angle,
   double v_0,
   double v_k,
   double r_c,
   double a_max,
   double k_v /**/ ) {
   BTurnAccelerateInput tai;
   tai.t = t;
   tai.angle = angle;
   tai.v_0 = v_0;
   tai.v_k = v_k;
   tai.r_c = r_c;
   tai.a_max = a_max;
   tai.k_v = k_v;
   try {
     BPoint p = turnAccelerate(tai);
     *x = p.x;
     *y = p.y;
     *c = p.c;
     *v = p.v;
     *vcomp = p.vcomp;
     *a = p.a;
     *r = p.r;
   }
   catch (...) {
     return false;
   }
   return true;

 }
}

BPoint turnAccelerateSameSpeed(const BTurnAccelerateInput& in);
BPoint turnAccelerateZeroEndSpeed(const BTurnAccelerateInput& in);
BPoint turnAccelerateDiffDirections(const BTurnAccelerateInput& in);
BPoint turnAccelerateCommonCase(const BTurnAccelerateInput& in);

BMATH_EXPORT BPoint turnAccelerate(const BTurnAccelerateInput& in) {
  if (in.v_0 == in.v_k)
    return turnAccelerateSameSpeed(in);
  else if (in.v_k == 0) 
    return turnAccelerateZeroEndSpeed(in);
  else if (in.v_0*in.v_k < 0)
    return turnAccelerateDiffDirections(in);
  else 
    return turnAccelerateCommonCase(in);
}

int sgn(double a) {
  return (a>0)?1:((a<0)?-1:0);
}

/**    ,    ll:hl
     ,   ll-1
*/
double root2(double a, double b, double c, double ll, double hl) {
  double d;
  double r1;
  double r2;
  bool r1_good = false;
  bool r2_good = false;
  d = b * b - 4 * a * c;
  if (d < 0) {
    return ll - 1;
  }
  r1 = (-b + sqrt(d)) / (2 * a);
  r2 = (-b - sqrt(d)) / (2 * a);
  if ((r1 >= ll) && (r1 <= hl)) {
    r1_good = true;
  }
  if ((r2 >= ll) && (r2 <= hl)) {
    r2_good = true;
  }
  if (r1_good && r2_good) {
    if (d == 0) {
      return r1;
    }
    return ll - 1;
  }
  if ((!r1_good) && (!r2_good)) {
    return ll - 1;
  }
  if (r1_good) {
    return r1;
  }
  else {
    return r2;
  }
}


BPoint turnAccelerateSameSpeed(const BTurnAccelerateInput& in) {
  double t_a;
  double r;
  double x_t_a;
  double y_t_a;

  BPoint p;
  p.a = 0;

  p.x = p.y = p.v = p.vcomp = p.c = p.a = p.r = 0;
  if (in.v_0 == 0) {
    return p;
  }
  t_a = fabs(in.r_c * in.angle / (in.v_0 * in.k_v));
  r = -sgn(in.v_0 * in.angle) * in.r_c;
  if (r == 0) 
    r = 1;
  if (in.t < t_a) {
    p.x = r * sin(in.k_v * in.v_0 * in.t / r);
 
    p.y = -r * (cos(in.k_v * in.v_0 * in.t / r) - 1);

    p.c = -in.k_v * in.v_0 * in.t / r;

    p.v = in.v_0;

    p.vcomp = in.v_0*in.k_v;

    p.r = in.r_c;
   }
  else {
    x_t_a = r * sin(in.k_v * in.v_0 * t_a / r);
    p.x = x_t_a + in.v_0 * (in.t - t_a) * cos(in.angle);

    y_t_a = -r * (cos(in.k_v * in.v_0 * t_a / r) - 1);
    p.y = y_t_a - in.v_0 * (in.t - t_a) * sin(in.angle);

    p.c = in.angle;    

    p.v = p.vcomp = in.v_0;

    p.r = 0;
  }
  return p;
}


BPoint turnAccelerateZeroEndSpeed(const BTurnAccelerateInput& in) {
  double t_a;
  double t_0;
  bool t_a_exists;
  double r;
  double s;
  double fi;
  double x_t_a, y_t_a;
  double a;
  BPoint p;

  r = -sgn(in.v_0 * in.angle) * in.r_c;
  if (r == 0) 
    r = -sgn(in.v_k * in.angle) * in.r_c;
  if (r == 0) 
    r = -in.r_c;
  a = fabs(in.a_max) * sgn(-in.v_0);
  p.a = a;
 
  t_0 = -in.v_0 / a;

  s = -in.k_v * in.v_0 * in.v_0 / (2 * a);
  if (fabs(s) < fabs(in.angle * r)) { 
    t_a_exists = false;
  }
  else {
    // -angle*r/k_v = v_0*t+a*t*t/2
    t_a = root2(a / 2, in.v_0, in.angle * r / in.k_v, 0, t_0);
    if (t_a < 0) {
      throw BadRoot();
    }
    t_a_exists = true;
  }
  
  //
  if (!t_a_exists) {
    if (in.t < t_0) {
      fi = in.k_v * (in.v_0 * in.t + a * in.t * in.t / 2) / r;
      p.x = r * sin(fi);
      p.y = r * (1 - cos(fi));
      p.v = in.v_0+ in.t*a;
      p.vcomp = p.v*in.k_v;
      p.r = in.r_c;
      p.c = -fi;
    }
    else {
      fi = -in.k_v * in.v_0 * in.v_0 / (2 * r * a);
      p.x = r * sin(fi);
      p.y = r * (1 - cos(fi));
      p.v = 0;
      p.vcomp = p.v*in.k_v;
      p.r = in.r_c;
      p.c = -fi;
    }
  }
  else {
    if (in.t < t_a) {
      fi = in.k_v * (in.v_0 * in.t + a * in.t * in.t / 2) / r;
      p.x = r * sin(fi);
      p.y = r * (1 - cos(fi));
      p.v = in.v_0+in.t*a;
      p.vcomp = p.v*in.k_v;
      p.r = in.r_c;
      p.c = -fi;
    }
    else if ((t_a <= in.t) && (in.t < t_0)) {
      fi = in.k_v * (in.v_0 * t_a + a * t_a * t_a / 2) / r;
      double ss = (in.v_0 + a * t_a) * (in.t - t_a) + a * (in.t - t_a) * (in.t - t_a) / 2;
      x_t_a = r * sin(fi);
      p.x = x_t_a + ss * cos(in.angle);

      y_t_a = r * (1 - cos(fi));
      p.y = y_t_a - ss * sin(in.angle);

      p.v = in.v_0+in.t*a;
      p.vcomp = p.v;
      p.r = 0;
      p.c = -fi;
    }
    else {
      fi = in.k_v * (in.v_0 * t_a + a * t_a * t_a / 2) / r;
      double ss = (in.v_0 + a * t_a) * (t_0 - t_a) + a * (t_0 - t_a) * (t_0 - t_a) / 2;
      x_t_a = r * sin(fi);
      p.x = x_t_a + ss * cos(in.angle);

      y_t_a = r * (1 - cos(fi));
      p.y = y_t_a - ss * sin(in.angle);

      p.v = 0;
      p.vcomp = 0;
      p.r = 0;
      p.c = in.angle;
    }
  }
  return p;
}

BPoint turnAccelerateCommonCase(const BTurnAccelerateInput& in) {
  double t_a;
  double t_al;
  double r;
  double fi;
  double a;
  BPoint p;
  
  r = -sgn(in.v_0 * in.angle) * in.r_c;
  if (r == 0) 
    r = -sgn(in.v_k * in.angle) * in.r_c;
  if (r == 0) 
    r = -in.r_c;
  a = fabs(in.a_max) * sgn(in.v_k - in.v_0);
  t_a = (in.v_k - in.v_0) / a;

  double s_t_a = in.k_v * (in.v_k * in.v_k - in.v_0 * in.v_0) / (2 * a);
  if (fabs(s_t_a) > fabs(in.angle * r)) {
    // -angle*r/k_v = v_0*t+a*t*t/2
    t_al = root2(a / 2, in.v_0, in.angle * r / in.k_v, 0, t_a);
    if (t_al < 0) {
      throw BadRoot();
    }
  }
  else {
    t_al = (-in.angle * r - s_t_a) / (in.k_v * in.v_k) + t_a;
  }
  
  if (t_a < t_al) {
    if ((0 <= in.t) && (in.t < t_a)) {
      fi = in.k_v * (in.v_0 * in.t + a * in.t * in.t / 2) / r;
      p.x = r * sin(fi);
      p.y = r * (1 - cos(fi));
      p.c = -fi;
      p.v = in.v_0+a*in.t;
      p.vcomp = p.v*in.k_v;
      p.a = a;
      p.r = in.r_c;
    }
    else if ((t_a <= in.t) && (in.t < t_al)) {
      fi = in.k_v * (in.v_0 * t_a + a * t_a * t_a / 2) / r + 
        in.k_v * in.v_k * (in.t - t_a) / r;
      p.x = r * sin(fi);
      p.y = r * (1 - cos(fi));
      p.c = -fi;
      p.v = in.v_k;
      p.vcomp = p.v*in.k_v;
      p.a = 0;
      p.r = in.r_c;
    }
    else if (t_al <= in.t) {
      fi = in.k_v * (in.v_0 * t_a + a * t_a * t_a / 2) / r +
        in.k_v * in.v_k * (t_al - t_a) / r;
      p.x = r * sin(fi) + in.v_k * (in.t - t_al) * cos(fi);
      p.y = r * (1 - cos(fi)) - in.v_k * (in.t - t_al) * sin(in.angle);
      p.c = in.angle;
      p.v = in.v_k;
      p.vcomp = p.v;
      p.a = 0;
      p.r = 0;
    }
  }
  else {// t_a >= t_al
    if ((0 <= in.t) && (in.t < t_al)) {
      fi = in.k_v * (in.v_0 * in.t + a * in.t * in.t / 2) / r;
      p.x = r * sin(fi);
      p.y = r * (1 - cos(fi));
      p.c = -fi;
      p.v = in.v_0+a*in.t;
      p.vcomp = p.v*in.k_v;
      p.a = a;
      p.r = in.r_c;
    }
    else if ((t_al <= in.t) && (in.t < t_a)) {
      fi = in.k_v * (in.v_0 * t_al + a * t_al * t_al / 2) / r;
      double ss = (in.v_0 + a * t_al) * (in.t - t_al) + a * (in.t - t_al) * (in.t - t_al) / 2;
      p.x = r * sin(fi) + ss * cos(in.angle);
      p.y = r * (1 - cos(fi)) - ss * sin(in.angle);
      p.c = in.angle;
      p.v = in.v_0+a*in.t;
      p.vcomp = p.v;
      p.a = a;
      p.r = 0;
    }
    else if (t_a <= in.t) {
      fi = in.k_v * (in.v_0 * t_al + a * t_al * t_al / 2) / r;
      double ss = (in.v_0 + a * t_al) * (t_a - t_al) + a * (t_a - t_al) * (t_a - t_al) / 2;
      p.x = r * sin(fi) + ss * cos(in.angle) + in.v_k * (in.t - t_a) * cos(in.angle);
      p.y = r * (1 - cos(fi)) - ss * sin(in.angle) - in.v_k * (in.t - t_a) * sin(in.angle);
      p.c = in.angle;
      p.v = in.v_k;
      p.vcomp = p.v;
      p.a = 0;
      p.r = 0;
    }
  }
  return p;
}

BPoint turnAccelerateDiffDirections(const BTurnAccelerateInput& in) {
  double t_v;
  double a;
  BTurnAccelerateInput in1 = in;
  in1.v_k = 0;
  BPoint p1 = turnAccelerateZeroEndSpeed(in1);

  a = fabs(in.a_max) * sgn(in.v_k - in.v_0);
  t_v = -in.v_0 / a;
  if (in.t <= t_v) {
    return p1;
  }
  else {
    BTurnAccelerateInput in2 = in;
    in2.t=in.t - t_v;
    in2.angle = in.angle - p1.c;
    in2.v_0 = 0;
    BPoint p2 = turnAccelerateCommonCase(in2);
    BPoint p3 = p2;
    p3.x = p2.x*cos(-p1.c) - p2.y*sin(-p1.c)+ p1.x;
    p3.y = p2.x * sin(-p1.c) + p2.y * cos(-p1.c) + p1.y;
    p3.c = p1.c + p2.c;
    return p3;
  }
}
