/******************************************************************************/
/* BPathFinder -     (impl.)            */
/******************************************************************************/
static const char bpathfinder_cpp_id[]="@(#) $Id: bpathfinder.cpp,v 1.6 2005/01/16 15:33:40 Oleksiy Balabay Exp $"; static const char* bpathfinder_cpp_id_id=bpathfinder_cpp_id;

#include <math.h>

#include "bpathfinder.h"
#include "bmathturn.h"

/** \file bpathfinder.cpp
  \brief  BPathFinder   (impl.)

     BPathFinder  
*/

/** \class BPathFinder
  \brief      ""

   .   path_finder.jpg

    :   B   ,    
 B   vk   c ( ).  
 B,     .  : 
 B   r (    ), 
 maxV,   minV,   aMax,
  aMin.

     .      3 . 
   -     B
   -    ,      ,
     
   -       
  
     2   .  ,   
 B,     ,   .    2
 -   .

        (   4), 
 ,        .

          . ,
      ,    
    . 

      ,   ,
    .

      .

    ,   ( ) , 
     .
*/

/** \brief 
*/
BPathFinder::BPathFinder(){

}

/** \brief 
*/
BPathFinder::~BPathFinder(){

}

/**     B  A.   , 
       0.
*/
BPathFinder::PathFinderOutput BPathFinder::findPath(const PathFinderInput& pInput) {
  PathFinderOutput pfOut;
  _input = pInput;
  _inputB = transformToBOrigin(pInput);
  std::vector<PathProgramData> vt1;
  std::vector<PathProgramData> vt2;
  double l1, l2;
  l1 = -1; l2 = -1;
  if (findPathInternal(true, vt1)) {
    l1 = _lengthB + _lengthBA + _lengthA;
  }
  if (findPathInternal(false, vt2)) {
    l2 = _lengthB + _lengthBA + _lengthA;
  }
  if ((l1 == -1)&&(l2 == -1)) {
      pfOut.countElements = 0;
      return pfOut;
  }
  if ((l1 != -1)&&( (l2 == -1) || (l2 > l1) )) {
    pfOut.countElements = vt1.size();
    for(int i = 0; i <= static_cast<int>(vt1.size()); i++) {
      pfOut.prgs[i] = vt1[i];
    }
  }
  else {
    pfOut.countElements = vt2.size();
    for(int i = 0; i <= static_cast<int>(vt2.size()); i++) {
      pfOut.prgs[i] = vt2[i];
    }
  }
  return pfOut;
}

/**     . centerTop == true,   
  (x   )   A .
*/
bool BPathFinder::findPathInternal(bool centerTop, 
  BPathFinder::PathProgramDatas& vt){
  bool speedRez = false;
  _aCenterTop = centerTop;
  _centerB = transformToBCenter(_inputB, _aCenterTop);
  double xB, yB, xA, yA;
  bool rez = pathExists(&xB, &yB, &xA, &yA);
  if (rez) {
    double l = length(xB, yB, xA, yA);

    std::vector<PathProgramData> v;
    SpeedProgramIn spin;
    spin.v0 = _input.b.v;
    spin.vk = _input.a.v;
    spin.T = _input.t;
    spin.l = l;
    spin.aMax = _input.pR.maxA;
    spin.aMin = _input.pR.minA;
    spin.vMax = _input.pR.maxV;
    spin.vMin = _input.pR.minV;
    spin.r = _input.pR.minR;
    speedRez = speedProgram(spin, v);

    if (speedRez) {
      vt = addTurnPrograms(v);
      // debug   
/*      double curS, curT, curV, curA;
      curS = 0; curT = 0; curV = _input.b.v; curA = 0;
      typedef std::vector<PathProgramData>::const_iterator CITP;
      for(CITP it = v.begin(); it != v.end(); it++) {
        double dt = (*it).t - curT;
        double s = curS + curV*dt + curA*dt*dt/2;
        double v = curV+curA*dt;
        curA = (*it).a;
        curS = s;
        curV = v;
        curT = curT+dt;
      }
      double dt = _input.t - curT;
      double s = curS + curV*dt + curA*dt*dt/2;
      double v = curV+curA*dt;
      curS = s;
      curV = v;
      curT = curT+dt;

      if (fabs(curS-l) >= 0.000001) {
//        qDebug ("!");
      }
      if (fabs(curV-_input.a.v) >= 0.000001) {
//        qDebug ("!");
      }
      if (fabs(curT-_input.t) >= 0.000001) {
//        qDebug ("!");
      }*/

    }
  }
  return speedRez;
}

///     , x   
BPathFinder::PathFinderInput 
BPathFinder::transformToBOrigin(const BPathFinder::PathFinderInput& pInput) {
  PathFinderInput rez = pInput;
  //    
  //  .   B
  QWMatrix m1(1, 0, 0, 1, pInput.b.x, pInput.b.y);
  //  x   
  QWMatrix m2(cos(pInput.b.c - Pi/2), -sin(pInput.b.c - Pi/2), 
              sin(pInput.b.c - Pi/2), cos(pInput.b.c - Pi/2), 0, 0);
  _mB = m2*m1;
  rez.b.c = 0;
  rez.b.x = 0;
  rez.b.y = 0;
//  rez.b.v   
  QWMatrix inv = _mB.invert();

  inv.map(pInput.a.x, pInput.a.y, &(rez.a.x), &(rez.a.y));
  rez.a.c = -(pInput.a.c - pInput.b.c); 
  return rez;
}

/**     B  A.   , 
       0.
*/
BMATH_EXPORT BPathFinder::PathFinderOutput findPath(
  const BPathFinder::PathFinderInput& pInput) {
  BPathFinder pf;
  return pf.findPath(pInput);
}

///        in
void BPathFinder::aCenter(const BPathFinder::PathFinderInput& in, bool top, 
  double* x, double* y) {
  //    
  //  .   A
  QWMatrix m1(1, 0, 0, 1, in.a.x, in.a.y);
  //  x   A
  QWMatrix m2(cos(in.a.c), sin(in.a.c), 
              -sin(in.a.c), cos(in.a.c), 0, 0);
  QWMatrix mA = m2*m1;
//  QWMatrix inv = mA.invert();
  mA.map(0,top?(in.pR.minR):(-in.pR.minR),  x, y);
}

///     , x     
BPathFinder::PathFinderInput 
BPathFinder::transformToBCenter(const BPathFinder::PathFinderInput& bOrig,
  bool topA) {
  PathFinderInput rez = bOrig;
  //    B
  double r = bOrig.pR.minR;
  if (bOrig.a.y < 0)
    r = -bOrig.pR.minR;
  //  
  double xA, yA, xV, yV;
  aCenter(bOrig, topA, &xA, &yA);
  _xA = xA;
  _yA = yA;
  double module = sqrt(xA*xA + (yA-r)*(yA-r));
  if (module == 0) {
    //      B
    xV = 0; yV = 0;
  }
  else {
    xV = xA/module;
    yV = (yA-r)/module;
  }
  
  QWMatrix m1(1, 0, 0, 1, 0, r);
  //  x    A
  QWMatrix m2(xV, yV, 
             -yV, xV, 0, 0);
  _mOrigB = m2*m1;
  QWMatrix inv = _mOrigB.invert();
  inv.map(0,0, &(rez.b.x), &(rez.b.y));
  inv.map(bOrig.a.x, bOrig.a.y, &(rez.a.x), &(rez.a.y));
  return rez;
}

///    .  
bool BPathFinder::pathExists(double* xB, double* yB,
  double* xA, double* yA) {
  double x1,y1,x2,y2, xCA, yCA;
  QWMatrix inv = _mOrigB.invert();
  inv.map(_xA,_yA, &xCA, &yCA);
  {
    //  
    x1 = 0; x2 = xCA; y1 = _centerB.pR.minR; y2 = _centerB.pR.minR;
    _mOrigB.map(x1, y1, &x1, &y1);
    _mOrigB.map(x2, y2, &x2, &y2);
    //  
    double xV = x2 - x1;
    double yV = y2 - y1;
    //   B
    double cBX, cBY;
    _mOrigB.map(0,0,&cBX, &cBY);
    //    _xA,_yA
    //   rB (-cBX, -cBY)   B (1, 0)
    // X1Y2-X2Y1
    double rBvB = cBY;
    //   rB (x1-cBX, y1-cBY)   (xV, yV)
    double rBV = (x1-cBX)*yV-xV*(y1-cBY);
    if (rBvB*rBV >= 0) {
      // ,  .   
    //    _xA,_yA
    //   rA (_centerB.a.x-_xA, _centerB.a.y-_yA) 
    //   A 
      double xVA, yVA;
      xVA = _centerB.a.v*cos(_inputB.a.c);
      yVA = _centerB.a.v*sin(_inputB.a.c);
      double rAvA = (_inputB.a.x-_xA)*yVA-xVA*(_inputB.a.y-_yA);
      //   rA (x2-xA, y2-yA)   (xV, yV)
      double rAV = (x2-_xA)*yV-xV*(y2-_yA);
      if (rAvA*rAV >= 0) {
        *xB = x1; *yB = y1; *xA = x2; *yA = y2;
        return true;
      }
    }

    //  
    x1 = 0; x2 = xCA; y1 = -_centerB.pR.minR; y2 = -_centerB.pR.minR;
    _mOrigB.map(x1, y1, &x1, &y1);
    _mOrigB.map(x2, y2, &x2, &y2);
    //  
    xV = x2 - x1;
    yV = y2 - y1;
    //   rB (x1-cBX, y1-cBY)   (xV, yV)
    rBV = (x1-cBX)*yV-xV*(y1-cBY);
    if (rBvB*rBV >= 0) {
      // ,  .   
    //    _xA,_yA
    //   rA (_centerB.a.x-_xA, _centerB.a.y-_yA) 
    //   A 
      double xVA, yVA;
      xVA = _centerB.a.v*cos(_inputB.a.c);
      yVA = _centerB.a.v*sin(_inputB.a.c);
      double rAvA = (_inputB.a.x-_xA)*yVA-xVA*(_inputB.a.y-_yA);
      //   rA (x2-xA, y2-yA)   (xV, yV)
      double rAV = (x2-_xA)*yV-xV*(y2-_yA);
      if (rAvA*rAV >= 0) {
        *xB = x1; *yB = y1; *xA = x2; *yA = y2;
        return true;
      }
    }
    double r = _centerB.pR.minR;
    // ,   "" 
    if (xCA < 2*r) {
      return false;
    }
    double cosA = 2*r/xCA;
    double sinA = sqrt(1 - cosA*cosA);
    x1 = r*cosA; y1 = r*sinA; x2 = xCA-r*cosA; y2 = -r*sinA;
    _mOrigB.map(x1, y1, &x1, &y1);
    _mOrigB.map(x2, y2, &x2, &y2);
    //  
    xV = x2 - x1;
    yV = y2 - y1;
    //   rB (x1-cBX, y1-cBY)   (xV, yV)
    rBV = (x1-cBX)*yV-xV*(y1-cBY);
    if (rBvB*rBV >= 0) {
      // ,  .   
    //    _xA,_yA
    //   rA (_centerB.a.x-_xA, _centerB.a.y-_yA) 
    //   A 
      double xVA, yVA;
      xVA = _centerB.a.v*cos(_inputB.a.c);
      yVA = _centerB.a.v*sin(_inputB.a.c);
      double rAvA = (_inputB.a.x-_xA)*yVA-xVA*(_inputB.a.y-_yA);
      //   rA (x2-xA, y2-yA)   (xV, yV)
      double rAV = (x2-_xA)*yV-xV*(y2-_yA);
      if (rAvA*rAV >= 0) {
        *xB = x1; *yB = y1; *xA = x2; *yA = y2;
        return true;
      }
    }
    x1 = r*cosA; y1 = -r*sinA; x2 = xCA-r*cosA; y2 = r*sinA;
    _mOrigB.map(x1, y1, &x1, &y1);
    _mOrigB.map(x2, y2, &x2, &y2);
    //  
    xV = x2 - x1;
    yV = y2 - y1;
    //   rB (x1-cBX, y1-cBY)   (xV, yV)
    rBV = (x1-cBX)*yV-xV*(y1-cBY);
    if (rBvB*rBV >= 0) {
      // ,  .   
    //    _xA,_yA
    //   rA (_centerB.a.x-_xA, _centerB.a.y-_yA) 
    //   A 
      double xVA, yVA;
      xVA = _centerB.a.v*cos(_inputB.a.c);
      yVA = _centerB.a.v*sin(_inputB.a.c);
      double rAvA = (_inputB.a.x-_xA)*yVA-xVA*(_inputB.a.y-_yA);
      //   rA (x2-xA, y2-yA)   (xV, yV)
      double rAV = (x2-_xA)*yV-xV*(y2-_yA);
      if (rAvA*rAV >= 0) {
        *xB = x1; *yB = y1; *xA = x2; *yA = y2;
        return true;
      }
    }
  }
  return false;
}

///         .  
double BPathFinder::length(double xB, double yB, double xA, double yA) {
  //       
  // B
  //   B
  double r = _inputB.pR.minR;
  double xBO = _centerB.b.x/r;
  double yBO = _centerB.b.y/r;
  double angleB = angle(yBO, xBO);

  QWMatrix inv = _mOrigB.invert();
  double xBC, yBC, xAC, yAC, xACen, yACen;
  inv.map(xB, yB, &xBC, &yBC);
  inv.map(xA, yA, &xAC, &yAC);
  inv.map(_xA, _yA, &xACen, &yACen);
  double tangentX = xAC - xBC;
  double tangentY = yAC - yBC;

  xBC/=r; yBC/=r;
  double angleBC = angle(yBC, xBC);
  //   . +   
  // X1Y2-X2Y1
  double direction = xBC*tangentY-tangentX*yBC;
  double finalAngleB = turnAngle(angleB, angleBC, direction > 0);
//    (angleBC > angleB)?(2*Pi-(angleBC - angleB)):(angleB - angleBC);
  _angleB = finalAngleB;

  double lengthB = finalAngleB*r;
  _lengthB = lengthB/_centerB.pR.lk;
  double lengthBA = sqrt(tangentX*tangentX+tangentY*tangentY);
  _lengthBA = lengthBA;
  double xAO = (_centerB.a.x - xACen)/r;
  double yAO = _centerB.a.y/r;
  double angleA = angle(yAO, xAO);

  xAC=(xAC-xACen)/r; yAC/=r;
  double angleAC = angle(yAC, xAC);
  //   . +   
  // X1Y2-X2Y1
  direction = xAC*tangentY-tangentX*yAC;
  double finalAngleA = turnAngle(angleAC, angleA, direction > 0);
//  if (angleAC > angleA)
//    finalAngleA = (2*Pi-(angleAC - angleA));
//  else
//    finalAngleA = (angleA - angleAC);
  _angleA = finalAngleA;

  double lengthA = finalAngleA*r;
  _lengthA = lengthA/_centerB.pR.lk;

  return lengthB/_centerB.pR.lk+lengthBA+lengthA/_centerB.pR.lk;
}

///      
BFinder::PathProgramDatas BPathFinder::addTurnPrograms(
  const BFinder::PathProgramDatas& vv) {
  PathProgramDatas vt;
  //      _lengthB (   )
  //    _lengthBA
  //    _lengthA
  double angleB = _angleB;
  //    B
  if (_inputB.a.y > 0)
    angleB = -angleB;
  double angleA = _angleA;
  if (_aCenterTop)
    angleA = -angleA;
  int t2Start = vv.size();
  //    -   
  std::vector<PathProgramData> vv1;
  if (angleA != 0) {
    double tA = t_s(_lengthB + _lengthBA, _input.b.v, _input.t, vv);
    for(int i = 0; i < static_cast<int>(vv.size()); i++) {
      double t1 = _input.t;
      if (i != static_cast<int>(vv.size())-1) {
        t1 = vv[i+1].t;
      }
      if ((vv[i].t <= tA)&&(t1>tA)) {
        t2Start = i + 1;
        PathProgramData ppd = vv[i];
        double startV = _input.b.v;
        if ( i != 0) {
          startV = vv[i-1].v;
        }
        ppd.v = startV + ppd.a*(tA - ppd.t);
        vv1.push_back(ppd);
        ppd.v = vv[i].v;
        ppd.t = tA;
        vv1.push_back(ppd);
      }
      else {
        vv1.push_back(vv[i]);
      }
    }
  }
  else {
    vv1 = vv;
  }
  double curS, curT, curAngleB, curAngleA;
  curS = 0; curT = 0; 
  curAngleB = 0; curAngleA = 0;
  double T = 0;
  double V = 0;
  for(int i = 0; i <static_cast<int>(vv1.size()); i++) {
    if ( i == static_cast<int>(vv1.size())) {
      T = _input.t;
    }
    else {
      T = vv1[i+1].t;
    }
    if (i == 0) {
      V = _input.b.v;
    }
    else {
      V = vv1[i-1].v;
    }
    double dt = T - vv1[i].t;
    double ds = V*dt + vv1[i].a*dt*dt/2;
    curS += ds;
    PathProgramData pd = vv1[i];
    if (curS < _lengthB) {
      double da = angleB*ds/_lengthB;
      if (fabs(curAngleB + da) > fabs(angleB)) {
        da = angleB - curAngleB;
      }
      curAngleB += da;
      pd.angle = da;
    }
    else {
      if (curAngleB != angleB) {
        pd.angle = angleB  -  curAngleB;
        curAngleB = angleB;
      }
      else {
        pd.angle = 0;
      }
    }
    if ((_lengthA != 0) && (t2Start <= i)) {
      double da = angleA*ds/_lengthA;
      if (fabs(curAngleA + da) > fabs(angleA)) {
        da = angleA - curAngleA;
      }
      curAngleA += da;
//      curAngleA = angleA*(1- (curS - (_lengthB+_lengthBA) )/_lengthA);
      pd.angle = da;
    }
    vt.push_back(pd);
  }
  return vt;
}

///  ,      ss  . vv
double BFinder::t_s(const double ss, const double v0, const double T,
  const BFinder::PathProgramDatas& vv) {
  double curS, curT, curV, curA;
  curS = 0; curT = 0; curV = v0; curA = 0;
  typedef std::vector<PathProgramData>::const_iterator CITP;
  for(CITP it = vv.begin(); it != vv.end(); it++) {
    double dt = (*it).t - curT;
    double s = curS + curV*dt + curA*dt*dt/2;
    double v = curV+curA*dt;
    if (ss <= s) {
      // t    curT  curT+dt
      if (curA == 0) {
        return (ss-curS)/v + curT;
      }
      else {
        double A = curA/2;
        double B = curV;
        double C = -(ss - curS);
        double ll = 0;
        double hl = dt;
        double rez = root2(A, B, C, ll, hl);
        if (rez < ll) throw MathExc();
        return rez + curT;
      }
    }
    curA = (*it).a;
    curS = s;
    curV = v;
    curT = curT+dt;
  }
  double dt = T - curT;
  double s = curS + curV*dt + curA*dt*dt/2;
  double v = curV+curA*dt;
  if (ss <= s) {
    // t    curT  curT+dt
    if (curA == 0) {
      return (ss-curS)/v + curT;
    }
    else {
      double A = curA/2;
      double B = curV;
      double C = -(ss - curS) ;
      double ll = 0;
      double hl = dt;
      double rez = root2(A, B, C, ll, hl);
      if (rez < ll) throw MathExc();
      return rez+ curT;
    }
  }
  curS = s;
  curV = v;
  curT = curT+dt;
  throw MathExc();
  return -1;
}

/**      sinA   cosA.   
.       0  2*Pi
*/
double BFinder::angle(const double sinA, const double cosA) {
  double angl = acos(cosA);
  if (sinA < 0) 
    angl = -angl;
  if (angl < 0)
    angl+=2*Pi;
  return angl;
}

/**  ,       \a start 
     \a finish.   
     0  2*Pi.  
  \a counter_clockwise - \a true  
  .  \a delta  , 
   \a start  \a finish   0.
   .

        0  2*Pi   
   .
*/
double BFinder::turnAngle(const double start, const double finish,
  bool counter_clockwise, const double delta) {
  if (fabs(start - finish) < delta)
    return 0;
  double angl = (start > finish)?(2*Pi-(start - finish)):(finish - start);
  if (!counter_clockwise)
    angl = 2*Pi - angl;
  return angl;
}

BMATH_EXPORT double timeTo(const BPathFinder::PathFinderInput& pInput, 
  bool* speedUp) {
  BPathFinder pf;
  return pf.timeTo(pInput, speedUp);
}

/**  ,    . speedUp 
,        .
 == -1,    

  \exception      ()
*/
double BPathFinder::timeTo(const PathFinderInput& pInput, bool* speedUp) {
  _input = pInput;
  _inputB = transformToBOrigin(pInput);
  double t1, t2;
  bool b1, b2;
  t1 = -1; t2 = -1;
  b1 = b2 = false;
  bool rez1, rez2;
  rez1 = rez2 = false;
  rez1 = timeToInternal(true, &t1, &b1);
  rez2 = timeToInternal(false, &t2, &b2);

  if ((!rez1)&&(!rez2)) {
      *speedUp = false;
      return -1;
  }
  if ((rez1)&&( (!rez2) || (t2 > t1) )) {
    *speedUp = b1;
    return t1;
  }
  else {
    *speedUp = b2;
    return t2;
  }
}

bool BPathFinder::timeToInternal(bool centerTop, double* time, bool* speedUp) {
  bool speedRez = false;
  _aCenterTop = centerTop;
  _centerB = transformToBCenter(_inputB, _aCenterTop);
  double xB, yB, xA, yA;
  bool rez = pathExists(&xB, &yB, &xA, &yA);
  if (rez) {
    double l = length(xB, yB, xA, yA);
    speedRez = timeToSpeedProgram(_input.b.v, _input.a.v,
      l, _input.pR.maxA, _input.pR.minA,
      _input.pR.maxV, _input.pR.minV, time, speedUp);
  }
  return speedRez;
}

bool BPathFinder::timeToSpeedProgram(double v0, double vk,
  double l, double aMax, double aMin, double vMax, double vMin,
  double* tRez, bool* speedUp) {
  //    ;
  if (vk > vMax) return false;
  if (vk < vMin) return false;
  if (v0 > vMax) return false;
  if (v0 < vMin) return false;
  if (l<0) throw InputData();
  if (aMax < 0) throw InputData();
  if (aMin > 0) throw InputData();
  if (vMax < 0) throw InputData();
  if (vMin > 0) throw InputData();

  //  ,     ta
  double ta;
  double a;
  if (v0<vk)
    a = aMax;
  else
    a = aMin;
  if ((a == 0) && (vk != v0)) throw InputData();
  ta = (vk-v0)/a;
  //     
//  double to = 0;
  //  ,    ta,  l
  double sa = l - (v0*ta + a*ta*ta/2);
  if (sa < 0) { // ,    ,
    // ,  
    *speedUp = (vk > v0);
    *tRez = ta;
    return false;
  }
  // :    sa,    vk
  double vc;
  double ac = aMax;
  vc = sqrt(sa*ac + vk*vk);
  //    ,  ,   
  //        
  double vcMax = vMax;
  double vcc;
  (vc > vcMax)?(vcc = vcMax):(vcc = vc);
  if (vcc == vc) {
    double tc = fabs(2*(vcc - vk)/ac);
    *speedUp = true;
    *tRez = tc + ta;
    return true;
  }
  //  ,      
  //     
  double tc = fabs(2*(vcc - vk)/ac);
  double sm = sa - (vcc*vcc - vk*vk)/ac;
  double tm = (sa - sm)/vcMax;
  *speedUp = true;
  *tRez = ta + tc + tm;
  return true;
}

/** \class BFinder
  \brief      

    ,       ,
     .
  

*/
