/******************************************************************************/
/* BPathFinder -                 */
/******************************************************************************/

#ifndef BPathFinder_H
#define BPathFinder_H
static const char bpathfinder_h_id[]="@(#) $Id: bpathfinder.h,v 1.3 2005/01/10 18:20:57 Oleksiy Balabay Exp $"; static const char* bpathfinder_h_id_id=bpathfinder_h_id;
/** \file bpathfinder.h
  \brief  BPathFinder  

     BPathFinder  
*/
#include <qwmatrix.h>
#include <vector>

#include "bmath.h"

class BMATH_EXPORT BFinder {
public:
  BFinder() {};
  ~BFinder() {};
protected:
  ///       \a l
  typedef struct SpeedProgramIn {
    ///  
    double v0;
    ///  
    double vk;
    /// ,       
    double T;
    /// ,   
    double l;
    ///   
    double aMax;
    ///    (<0)
    double aMin; 
    ///  
    double vMax;
    ///  
    double vMin;
    /**  .     .  
      
    */
    double r;
  };
public:
  typedef struct PathPoint {
    double x;
    double y;
    double v;
    double c;
  } PathPoint;
  typedef struct PathRestrictions {
    double maxA;
    double minA;
    double minR;
    double lk; // lossOfSpeedCoef
    double maxV;
    double minV;
  } PathRestrictions;
  typedef struct PathFinderInput {
    PathPoint a;
    PathPoint b;
    PathRestrictions pR;
    double t; // time
  } PathFinderInput;
  typedef struct PathProgramData {
    double t;
    double v;
    double a;
    double angle;
    double r;
  };
  typedef std::vector<PathProgramData> PathProgramDatas;
  typedef struct PathFinderOutput {
//    PathFinderOutput():countElements(0){};
    int countElements;
    PathProgramData prgs[10];
  } PathFinderOutput;
protected:
  static double angle(const double sinA, const double cosA);
  static double turnAngle(const double start, const double finish,
    bool counter_clockwise, const double delta = 0);
  bool speedProgram(const SpeedProgramIn& spin, PathProgramDatas& spout);
  double t_s(const double ss, const double v0, const double T,
    const PathProgramDatas& v);
private:
  bool speedProgramInternal(const SpeedProgramIn& spin, PathProgramDatas& spout);
  bool speedProgramUpDown(const SpeedProgramIn& spin, PathProgramDatas& spout);
  bool speedProgramDownUp(const SpeedProgramIn& spin, PathProgramDatas& spout);
};

class BMATH_EXPORT BPathFinder: public BFinder {
public:
  BPathFinder();
  PathFinderOutput findPath(const PathFinderInput& pInput);
  double timeTo(const PathFinderInput& pInput, bool* speedUp);
  virtual ~BPathFinder();
private:
  PathFinderInput transformToBOrigin(const PathFinderInput& pInput);
  PathFinderInput transformToBCenter(const PathFinderInput& bOrig, bool topA);
  void aCenter(const PathFinderInput& in, bool top, double* x, double* y);
  bool pathExists(double* xB, double* yB,
    double* xA, double* yA);
  double length(double xB, double yB, double xA, double yA);
  PathProgramDatas addTurnPrograms(
    const PathProgramDatas& v);
  bool findPathInternal(bool centerTop, std::vector<PathProgramData>& vt);
  bool timeToInternal(bool centerTop, double* time, bool* speedUp);
  bool timeToSpeedProgram(double v0, double vk,
    double l, double aMax, double aMin, double vMax, double vMin,
    double* tRez, bool* speedUp);
private:
  PathFinderInput _input;
  PathFinderInput _inputB;
  PathFinderInput _centerB;

  QWMatrix _mB;
  QWMatrix _mOrigB;
  double _xA, _yA; //     
  double _angleB; double _lengthB;
  double _angleA; double _lengthA;
  double _lengthBA;
  bool _aCenterTop;

};

extern "C" {
  BMATH_EXPORT BFinder::PathFinderOutput findPath(
    const BFinder::PathFinderInput& pInput);
}

extern "C" {
  BMATH_EXPORT double timeTo(
    const BFinder::PathFinderInput& pInput, bool* speedUp);
}

#endif // BPathFinder_H
