/******************************************************************************/
/* BPathFinderSP -     (impl.)            */
/******************************************************************************/
static const char bpathfindersp_cpp_id[]="@(#) $Id: bpathfindersp.cpp,v 1.6 2005/03/07 15:34:40 Oleksiy Balabay Exp $"; static const char* bpathfindersp_cpp_id_id=bpathfindersp_cpp_id;

#include <math.h>

#include "bpathfinder.h"
#include "bmathturn.h"

/** \file bpathfindersp.cpp
  \brief  BPathFinder   (impl.)

     BPathFinder     
   
*/
#define FCMP(a, b) (fabs((a)-(b))<0.000001)

/**     \a spout  
  ,  \a spin

   ,  aMax = -aMin, vMax = -vMin.   
    v0, vk

  \return \a true,   ,  - \a false
*/

bool 
BFinder::speedProgram(const BFinder::SpeedProgramIn& spin, 
  BFinder::PathProgramDatas& spout) {
  bool rez = true;
  try {
    rez = speedProgramInternal(spin, spout);
  }
  catch (...) {
    rez = false;
  }
  return rez;
}

bool 
BFinder::speedProgramInternal(const BFinder::SpeedProgramIn& spin, 
  BFinder::PathProgramDatas& spout) {

  //    ;
  double v0 = spin.v0;
  double vk = spin.vk;
  double T = spin.T;
  double l = spin.l;
  double aMax = spin.aMax;
  double aMin = spin.aMin;
  double vMax = spin.vMax;
  double vMin = spin.vMin;
  std::vector<BPathFinder::PathProgramData>& data = spout;

  if (vk > vMax) return false;
  if (vk < vMin) return false;
  if (v0 > vMax) return false;
  if (v0 < vMin) return false;
  if (T<0) throw InputData();
  if (l<0) throw InputData();
  if (aMax < 0) throw InputData();
  if (aMin > 0) throw InputData();
  if (vMax < 0) throw InputData();
  if (vMin > 0) throw InputData();

  if (speedProgramUpDown(spin, data)) {
//    qDebug("UpDown");
    return true;
  }
  if (speedProgramDownUp(spin, data)) {
//    qDebug("DownUp");
    return true;
  }
  data.clear();
  PathProgramData d;
  d.angle = 0;
  d.r = spin.r;
   //  ,     ta
  double ta;
  double a;
  if (v0<vk)
    a = aMax;
  else
    a = aMin;
  if ((a == 0) && (vk != v0)) throw InputData();
  ta = (vk-v0)/a;
  //     
  double to = T - ta;
  if (to < 0) return false;
  //  ,    ta,  l
  double sa = l - (v0*ta + a*ta*ta/2);
  //  
//  double ds = 2*aMax*(to/2)*(to/2);
//  double sMax = l - sa + ds;
//  double sMin = l - sa - ds;
//  qDebug("l=%f, sMax=%f sMin=%f", l, sMax, sMin);

  
  // ,   so    to = T - ta 
  //  
  double so = vk*to;
  //  ,      sc
  double sc = sa - so;
  if (sc == 0) {
    d.t = 0;
    d.a = a;
    d.v = vk;
    data.push_back(d);
    d.t = ta;
    d.a = 0;
    d.v = vk;
    data.push_back(d);
    return true;
  }
  // ,     ,    vk
  //  ,     
  double vc;
  double ac = (sc > 0)?aMax:aMin;
  vc = sqrt(sc*ac);
  if (sc < 0)
    vc = -vc;
  //    ,  ,   
  //        
  double vcMax = vMax - vk;
  double vcMin = vMin - vk;
  double vcc;
  (vc > vcMax)?(vcc = vcMax):(vc < vcMin)?(vcc = vcMin):(vcc = vc);
  if (vcc == vc) {
    //   
    double tc = fabs(2*(vcc)/ac);
    if (tc <= to) {
      d.t = 0;
      d.a = a;
      d.v = vk;
      data.push_back(d);
      d.t = ta;
      d.a = ac;
      d.v = vcc+ vk;
      data.push_back(d);
      d.t = ta+tc/2;
      //    . aMin      aMax
      d.a = -ac;
      d.v = vk;
      data.push_back(d);
      d.t = ta+tc;
      d.a = 0;
      d.v = vk;
      data.push_back(d);
      return true;
    }
    else {
      return false;
    }
  }
  //  ,      
  //     
  double tc = fabs(2*(vcc)/ac);
  double tm = to - tc;
  if (tm < 0)
    return false;
  double sm = sc - vcc*vcc/ac;
  if (tm*vcc < sm)
    return false;
  else {
  // ,    .
    double tmm = sm/vcc;
    d.t = 0;
    d.a = a;
    d.v = vk;
    data.push_back(d);
    d.t = ta;
    d.a = ac;
    d.v = vcc + vk;
    data.push_back(d);
    d.t = ta+tc/2;
    d.a = 0;
    d.v = vcc + vk;
    data.push_back(d);
    d.t = ta+tc/2+tmm;
    d.a = -ac;
    d.v = vk;
    data.push_back(d);
    d.t = ta+tc+tmm;
    d.a = 0;
    d.v = vk;
    data.push_back(d);
    return true;
  }
  return false;
}

/**     l   T, 
:
  -   
  -    
  -    
  -   

      aMax = -aMin, vMax = -vMin, 
   0.
*/

bool BFinder::speedProgramUpDown(
  const BFinder::SpeedProgramIn& spin, 
  BFinder::PathProgramDatas& spout) {
bool rez = true;
try {
  spout.clear();
  PathProgramData d;
  d.angle = 0;
  d.r = spin.r;
  double a = spin.aMax;
  // . ,    ,    
  // 
  double vm = sqrt((2*spin.l*a + spin.v0*spin.v0 + spin.vk*spin.vk)/2);
  double vm1 = (vm > spin.vMax)?spin.vMax:vm;
  double tUp = fabs((vm1 - spin.v0)/a);
  double tDown = fabs((vm1 - spin.vk)/a);
  if (tUp+tDown > spin.T) {
    rez = false;
  }
  else {
    double tRez = tUp+tDown;
    double sUp = (vm1*vm1 - spin.v0*spin.v0)/(2*a);
    double sDown = (vm1*vm1 - spin.vk*spin.vk)/(2*a);
    if (!(sUp >= 0)) throw MathExc();
    if (!(sDown >= 0)) throw MathExc();
    if (!(sUp + sDown <= spin.l)) throw MathExc();
   
    double sRez = spin.l - sUp - sDown;
    double tVm = (sRez - spin.vk*(spin.T - tRez))/(vm1 - spin.vk);
    double sVm = tVm*vm1;
    if (!(tVm >= 0)) throw MathExc();

    double tVk = spin.T - tRez - tVm;
    if (!(tVk >= 0)) throw MathExc();
    double sVk = tVk*spin.vk;
    if (!(FCMP(sVk+sVm+sUp+sDown, spin.l))) throw MathExc();
    if (!(FCMP(tVk+tVm+tUp+tDown, spin.T))) throw MathExc();
    if (!(FCMP(
      (spin.v0*tUp + a*tUp*tUp/2) +
      (vm1*tVm) +
      (vm1*tDown - a*tDown*tDown/2) +
      (spin.vk*tVk), spin.l)
    )) throw MathExc();
    if (!(FCMP(spin.v0 + a*tUp - a*tDown, spin.vk))) throw MathExc();

    d.t = 0;
    d.a = a;
    d.v = vm1;
    spout.push_back(d);
    d.t = tUp;
    d.a = 0;
    d.v = vm1;
    spout.push_back(d);
    d.t = tUp+tVm;
    d.a = -a;
    d.v = spin.vk;
    spout.push_back(d);
    d.t = tUp+tVm+tDown;
    d.a = 0;
    d.v = spin.vk;
    spout.push_back(d);
  }
}
catch(...) {
  rez = false;
}
return rez;
}

bool BFinder::speedProgramDownUp(
  const BFinder::SpeedProgramIn& spin, 
  BFinder::PathProgramDatas& spout) {
bool rez = true;
try {
  spout.clear();
  PathProgramData d;
  d.angle = 0;
  d.r = spin.r;
  double a = spin.aMax;
  // . ,    ,    
  // 
  double vm = sqrt((-2*spin.l*a + spin.v0*spin.v0 + spin.vk*spin.vk)/2);
  double vm1 = (vm > spin.vMax)?spin.vMax:vm;
  double tUp = fabs((vm1 - spin.vk)/a);
  double tDown = fabs((vm1 - spin.v0)/a);
  if (tUp+tDown > spin.T) {
    rez = false;
  }
  else {
    double tRez = tUp+tDown;
    double sUp = (spin.vk*spin.vk - vm1*vm1)/(2*a);
    double sDown = (spin.v0*spin.v0 - vm1*vm1)/(2*a);
    if (!(sUp >= 0)) throw MathExc();
    if (!(sDown >= 0)) throw MathExc();
    if (!(sUp + sDown <= spin.l)) throw MathExc();
   
    double sRez = spin.l - sUp - sDown;
    double tVm = (sRez - spin.vk*(spin.T - tRez))/(vm1 - spin.vk);
    double sVm = tVm*vm1;
    if (!(tVm >= 0)) throw MathExc();

    double tVk = spin.T - tRez - tVm;
    if (!(tVk >= 0)) throw MathExc();
    double sVk = tVk*spin.vk;
    if (!(FCMP(sVk+sVm+sUp+sDown, spin.l))) throw MathExc();
    if (!(FCMP(tVk+tVm+tUp+tDown, spin.T))) throw MathExc();
    if (!(FCMP(
      (spin.v0*tDown - a*tDown*tDown/2) +
      (vm1*tVm) +
      (vm1*tUp + a*tUp*tUp/2) +
      (spin.vk*tVk), spin.l)
    )) throw MathExc();
    if (!(FCMP(spin.v0 + a*tUp - a*tDown, spin.vk))) throw MathExc();

    d.t = 0;
    d.a = -a;
    d.v = vm1;
    spout.push_back(d);
    d.t = tDown;
    d.a = 0;
    d.v = vm1;
    spout.push_back(d);
    d.t = tDown+tVm;
    d.a = a;
    d.v = spin.vk;
    spout.push_back(d);
    d.t = tUp+tVm+tDown;
    d.a = 0;
    d.v = spin.vk;
    spout.push_back(d);
  }
}
catch(...) {
  rez = false;
}
return rez;
}
